<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;

class JoomcrmControllerCategories extends JoomcrmControllerAdmin
{
	/**
	 * Edit a record.
	 *
	 * @return  string  Rendered JView object.
	 */
	public function edit()
	{
		// Set layout
		$view = $this->getView('Categories', 'html');
		$view->setLayout('edit');

		// Add javascript
		$this->doc->addScript(Uri::base() . 'components/com_joomcrm/media/js/category_manager.js');

		// Get stage info
		if ($this->id)
		{
			$category = $this->getModel('Categories')->getCategories($this->id);
			$view->category = $category[0];
			$view->header   = Text::_('COM_JOOMCRM_EDITING_CATEGORY') . $view->category['name'];
		}
		else
		{
			$view->header   = Text::_('COM_JOOMCRM_ADDING_CATEGORY');
		}

		// Display view
		$view->display();
	}

	/**
	 * Cancel current action.
	 *
	 * @return  void
	 */
	public function cancel()
	{
		$msg = Text::_('Category entry cancelled!');
		$this->setRedirect('index.php?option=com_joomcrm&view=categories', $msg);
	}

	/**
	 * Save the edits.
	 *
	 * @return  void
	 */
	public function save()
	{
		$model = $this->getModel('Categories');

		if ($model->store())
		{
			$msg = Text::_('COM_JOOMCRM_SUCCESS');
		}
		else
		{
			$msg = Text::_('COM_JOOMCRM_ERROR');
		}

		$this->setRedirect('index.php?option=com_joomcrm&view=categories', $msg);
	}

	/**
	 * Remove a record or records.
	 *
	 * @return  void
	 */
	public function remove()
	{
		$model = $this->getModel('Categories');

		if (is_array($this->id))
		{
			foreach ($this->id as $id)
			{
				$model->remove($id);
			}
		}
		else
		{
			$model->remove($this->id);
		}

		$msg = Text::_('COM_JOOMCRM_CATEGORY_REMOVED');
		$this->setRedirect('index.php?option=com_joomcrm&view=categories', $msg);
	}

}
