<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */



defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\MVC\Controller\BaseController;

$input = Factory::getApplication()->input;
$views = ['manageuser', 'manageusers', 'manageteam', 'manageteams', 'managestatuse', 'managestatuses', 'loginhistory', 'activities', 'dealsstage', 'dealsstages', 'dealsstatuse', 'dealsstatuses', 'companycategories', 'companycategory', 'notecategories', 'notecategory', 'eventcategories', 'eventcategory', 'documentcategories', 'documentcategory','companycustom','companycustoms','peoplecustom','peoplecustoms','dealcustom','dealcustoms'];


$task = explode('.', $input->get('task', ''));

$task = isset($task[0]) ? $task[0] : true;

$hasNewViews = in_array($input->getWord('view', ''), $views);

$hasNewTasks = in_array($task, $views);

if (!$hasNewViews && !$hasNewTasks)
{


	class JoomcrmController extends BaseController
	{
		/**
		 * The current application instance.
		 *
		 * @var  JApplication
		 */
		protected $app = null;

		/**
		 * The current document instance
		 *
		 * @var  JDocument
		 */
		protected $doc = null;

		/**
		 * Holds the item's id
		 *
		 * @var  integer
		 */
		public $id = null;


		/**
		 * Constructor
		 *
		 * @return  void
		 */
		public function __construct()
		{
			parent::__construct();

			$this->app = Factory::getApplication();
			$this->doc = Factory::getDocument();

			if (!($this->id = $this->app->input->getInt('id')))
			{
				$array = $this->app->input->get('cid', array(), 'array');

				if (is_array($array) && isset($array[0]))
				{
					if (count($array) > 1)
					{
						$this->id = $array;
					}
					else
					{
						$this->id = (int) $array[0];
					}
				}
			}

			$view = $this->app->input->get('view', 'joomcrm');

			// If view doesn't exist, then let's figure it out.
			if (empty($view))
			{
				$class = get_class($this);
				$pos   = strpos($class, 'Controller');

				if ($pos !== false)
				{
					$view = strtolower(substr($class, $pos + 10));

					if (!empty($view))
					{
						$this->app->input->set('view', $view);
					}
				}
			}

			// Set title
			ToolBarHelper::title(Text::_('COM_JOOMCRM_ADMIN'), 'moo');

			// Load menu helper
			JoomcrmHelperMenu::loadNavi();
		}

		/**
		 * Method to load and return a model object.
		 *
		 * @param   string  $name    The name of the model.
		 * @param   string  $prefix  Optional model prefix.
		 * @param   array   $config  Configuration array for the model. Optional.
		 *
		 * @return  mixed   Model object on success; otherwise null failure.
		 *
		 * @since   11.1
		 * @note    Replaces _createModel.
		 */
		protected function createModel($name, $prefix = '', $config = array())
		{
			// Clean the model name
			$modelName   = preg_replace('/[^A-Z0-9_]/i', '', $name);
			$classPrefix = preg_replace('/[^A-Z0-9_]/i', '', $prefix);

			$modelClassName = ucfirst($classPrefix) . ucfirst($modelName);

			if (!class_exists($modelClassName))
			{
				\Joomla\CMS\Log\Log::add(sprintf('Model class %s does not exist.', $modelClassName), \Joomla\CMS\Log\Log::WARNING);

				return false;
			}

			return new $modelClassName($config);
		}

		/**
		 * Method to load and return a view object. This method first looks in the
		 * current template directory for a match and, failing that, uses a default
		 * set path to load the view class file.
		 *
		 * Note the "name, prefix, type" order of parameters, which differs from the
		 * "name, type, prefix" order used in related public methods.
		 *
		 * @param   string  $name    The name of the view.
		 * @param   string  $prefix  Optional prefix for the view class name.
		 * @param   string  $type    The type of view.
		 * @param   array   $config  Configuration array for the view. Optional.
		 *
		 * @return  mixed  View object on success; null or error result on failure.
		 *
		 * @since   11.1
		 */
		protected function createView($name, $prefix = '', $type = 'Html', $config = array())
		{
			// Clean the view name
			$viewName    = preg_replace('/[^A-Z0-9_]/i', '', $name);
			$classPrefix = preg_replace('/[^A-Z0-9_]/i', '', $prefix);
			$viewType    = preg_replace('/[^A-Z0-9_]/i', '', $type);

			// Build the view class name. Ex: JoomcrmViewAccountHtml
			$viewClassName = ucfirst($classPrefix) . ucfirst($viewName) . ucfirst($viewType);

			if (!class_exists($viewClassName))
			{
				\Joomla\CMS\Log\Log::add(sprintf('View class %s does not exist.', $viewClassName), \Joomla\CMS\Log\Log::WARNING);

				return false;
			}

			return new $viewClassName($config);
		}

		/**
		 * [add description]
		 *
		 * @return  void
		 */
		public function add()
		{
			$this->id = 0;
			$this->edit();
		}
	}
}
else
{


	class JoomcrmController extends BaseController
	{
		/**
		 * The default view.
		 *
		 * @var    string
		 * @since  1.6
		 */
		protected $default_view = 'joomcrm';

		/**
		 * Method to display a view.
		 *
		 * @param   boolean  $cachable   If true, the view output will be cached
		 * @param   array    $urlparams  An array of safe URL parameters and their variable types, for valid values see {@link JFilterInput::clean()}.
		 *
		 * @return  JoomcrmController  This object to support chaining.
		 *
		 * @since   1.5
		 */
		public function display($cachable = false, $urlparams = array())
		{

			
			return parent::display();
		}

	}
}
