<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

use Joomla\CMS\Filesystem\File;

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

class JoomcrmControllerDocuments extends JoomcrmController
{
	/**
	 * [edit description]
	 *
	 * @return  [type]  [description]
	 */
	public function edit()
	{
		// Set layout
		$view = $this->getView('Documents', 'html');
		$view->setLayout('edit');

		// Add javascript
		$this->doc->addScript(Uri::root() . 'administrator/components/com_joomcrm/media/js/document_manager.js');

		// Get stage info
		if ($this->id)
		{
			$model    = $this->getModel('Documents');
			$document = $model->getDocuments($this->id);
			$view->document = $document[0];
			$view->header   = Text::_('COM_JOOMCRM_EDITING_DOCUMENT') . $view->document['filename'];
		}
		else
		{
			$view->header   = Text::_('COM_JOOMCRM_ADDING_DOCUMENT');
		}

		// Display view
		$view->display();
	}

	/**
	 * [cancel description]
	 *
	 * @return  [type]  [description]
	 */
	public function cancel()
	{
		$msg = Text::_('Document entry cancelled!');
		$this->setRedirect('index.php?option=com_joomcrm&view=documents', $msg);
	}

	/**
	 * [save description]
	 *
	 * @return  [type]  [description]
	 */
	public function save()
	{
		if ($this->getModel('Documents')->store())
		{
			$msg = Text::_('COM_JOOMCRM_SUCCESS');
		}
		else
		{
			$msg = Text::_('COM_JOOMCRM_ERROR');
		}

		$this->setRedirect('index.php?option=com_joomcrm&view=documents', $msg);
	}

	/**
	 * [remove description]
	 *
	 * @return  [type]  [description]
	 */
	public function remove()
	{
		$model = $this->getModel('Documents');

		if (is_array($this->id))
		{
			foreach ($this->id as $id)
			{
				$model->remove($id);
			}
		}
		else
		{
			$model->remove($this->id);
		}

		$msg = Text::_('COM_JOOMCRM_DOCUMENT_REMOVED');
		$this->setRedirect('index.php?option=com_joomcrm&view=documents', $msg);
	}

	/**
	 * [upload description]
	 *
	 * @return  [type]  [description]
	 */
	public function upload()
	{
		$config     = ComponentHelper::getParams('com_joomcrm');

		// Import joomlas filesystem functions, we will do all the filewriting with joomlas functions,
		// So if the ftp layer is on, joomla will write with that, not the apache user, which might
		// Not have the correct permissions
		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');

		$fieldName = 'document';

		// Any errors the server registered on uploading
		$fileError = $_FILES[$fieldName]['error'];

		if ($fileError > 0)
		{
			switch ($fileError)
			{
				case 1:
					echo Text::_('FILE TO LARGE THAN PHP INI ALLOWS');

					return;
				case 2:
					echo Text::_('FILE TO LARGE THAN HTML FORM ALLOWS');

					return;
				case 3:
					echo Text::_('ERROR PARTIAL UPLOAD');

					return;
				case 4:
					echo Text::_('ERROR NO FILE');

					return;
			}
		}



		// Check for filesize
		$fileSize = $_FILES[$fieldName]['size'];

		if ($fileSize > 2000000)
		{
			echo Text::_('FILE BIGGER THAN 2MB');
		}

		// Check the file extension is ok
		$fileName = $_FILES[$fieldName]['name'];

		$uploadedFileNameParts = explode('.',$fileName);
		$uploadedFileExtension = array_pop($uploadedFileNameParts);
		
		$validFileExts = explode(
			',',
			$config->get('allowed_documents',"png,jpg,jpeg,mp4,mp3,pdf,txt,doc,docx,xsl,xsls,zip")
		);

		// Assume the extension is false until we know its ok
		$extOk = false;

		if (count($validFileExts) === 0 || $validFileExts[0] == '')
		{
			// when none extension is listed, allow all (issue CRM-217)
			$extOk = true;
		}
		else
		{


			if(in_array(strtolower($uploadedFileExtension),$validFileExts))
				$extOk = true;


		}

		if (!$extOk)
		{
			echo Text::_('INVALID EXTENSION');

			return;
		}

		// The name of the file in PHP's temp directory that we are going to move to our folder
		$fileTemp = $_FILES[$fieldName]['tmp_name'];

		// For security purposes, we will also do a getimagesize on the temp file (before we have moved it
		// To the folder) to check the MIME type of the file, and whether it has a width and height
		$imageinfo = getimagesize($fileTemp);

		//we are going to define what file extensions/MIMEs are ok, and only let these ones in (whitelisting), rather than try to scan for bad
		// Types, where we might miss one (whitelisting is always better than blacklisting)
		// $okMIMETypes = 'image/jpeg,image/pjpeg,image/png,image/x-png,image/gif';
		// $validFileTypes = explode(",", $okMIMETypes);

		// If the temp file does not have a width or a height, or it has a non ok MIME, return
		// if( !is_int($imageinfo[0]) || !is_int($imageinfo[1]) ||  !in_array($imageinfo['mime'], $validFileTypes) )
		// {
		// echo Text::_( 'INVALID FILETYPE' );
		// return;
		// }

		// Lose any special characters in the filename
		$fileName = preg_replace("[^A-Za-z0-9.]", "-", $fileName);
		$hash     = md5($fileName) . "." . $uploadedFileExtension;

		// Always use constants when making file paths, to avoid the possibilty of remote file inclusion
		$uploadPath = JPATH_SITE . '/components/com_joomcrm/documents/' . $hash;

		$session = Factory::getSession();

		// check if file already exist, remove it
		if(file_exists($uploadPath))
			File::delete($uploadPath);

		$uploadResult = File::upload($fileTemp, $uploadPath);



		if (!$uploadResult)
		{
			$msg = Text::_('COM_JOOMCRM_DOC_UPLOAD_FAIL');
			$this->setRedirect('index.php?option=com_joomcrm&view=documents&layout=upload&tmpl=component', $msg);
			$session->set("upload_success", false);
		}
		else
		{
			// Update the database

			$data = array('name' => $fileName, 'filename' => $hash, 'filetype' => $uploadedFileExtension, 'size' => $fileSize / 1024, 'created' => date('Y-m-d H:i:s'), 'shared' => 1, 'is_image' =>  is_array(getimagesize($uploadPath)) ? 1 : 0);

			$documentModel = $this->getModel('Documents');


			$storeResult = $documentModel->store($data);

			if ($storeResult)
			{
				$msg = Text::_('COM_JOOMCRM_DOC_UPLOAD_SUCCESS');
				$this->setRedirect('index.php?option=com_joomcrm&view=documents&layout=upload&tmpl=component', $msg);
				$session->set("upload_success", true);
			}
			else
			{

				\Joomla\CMS\Factory::getApplication()->enqueueMessage(implode('<br>',$documentModel->getErrors()));

				$msg = \Joomla\CMS\Language\Text::_('COM_JOOMCRM_DOC_UPLOAD_FAIL');
				$this->setRedirect('index.php?option=com_joomcrm&view=documents&layout=upload&tmpl=component', $msg);
				$session->set("upload_success", false);
			}
		}
	}
}
