<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;

class JoomcrmControllerFormwizard extends JoomcrmController
{
	/**
	 * [edit description]
	 *
	 * @return  [type]  [description]
	 */
	public function edit()
	{
		// Set layout
		$view = $this->getView('Formwizard', 'html');
		$view->setLayout('edit');
		$form_status  = 0;
		$form_source  = 0;
		$form_is_custom = 0;

		// If we get post data generate data for view
		if ($this->id)
		{
			$model        = $this->getModel('Formwizard');
			$view->form   = $model->getForm($this->id);
			$form_type    = $view->form['type'];

			if (isset($view->form['status_id']))
			{
				$form_status  = $view->form['status_id'];
			}

			if (isset($view->form['source_id']))
			{
				$form_source  = $view->form['source_id'];
			}

			if (isset($view->form['is_custom']))
			{
				$form_is_custom  = $view->form['is_custom'];
			}
			
			$view->header = Text::_('COM_JOOMCRM_EDITING_FORM') . $view->form['name'];
		}
		else
		{
			$form_type = null;
			$view->header = Text::_('COM_JOOMCRM_CREATING_FORM');
		}

		// Get joomla users to add
		$user_list = $this->getModel('Users')->getJoomcrmUsers();

		$view->form_types = JoomcrmHelperDropdown::getFormTypes($form_type);
		$view->form_statuses = JoomcrmHelperDropdown::generateDropdown('people_status', $form_status);
		$view->form_sources = JoomcrmHelperDropdown::generateDropdown('source', $form_source);
		$view->form_is_custom = JoomcrmHelperDropdown::generateDropdown('is_custom', $form_is_custom);

		$view->fields = array(
			'lead' => JoomcrmHelperDropdown::getFormFields('people'),
			'contact' => JoomcrmHelperDropdown::getFormFields('people'),
			//'deal' => JoomcrmHelperDropdown::getFormFields('deal'),
			//'company' => JoomcrmHelperDropdown::getFormFields('company')
		);

		$this->doc
			->addScript(Uri::base() . 'components/com_joomcrm/media/js/formwizard.js')
			->addScriptDeclaration('var fields=' . json_encode($view->fields) . ', user_list=' . json_encode($user_list) . ';');

		// Display view
		$view->display();
	}

	/**
	 * [cancel description]
	 *
	 * @return  [type]  [description]
	 */
	public function cancel()
	{
		$msg = Text::_('Form entry cancelled!');
		$this->setRedirect('index.php?option=com_joomcrm&view=formwizard', $msg);
	}

	/**
	 * [save description]
	 *
	 * @return  [type]  [description]
	 */
	public function save()
	{
		if ($this->getModel('Formwizard')->store())
		{
			$msg = "Successfully saved!";
		}
		else
		{
			$msg = "Error Saving!";
		}

		$this->setRedirect('index.php?option=com_joomcrm&view=formwizard', $msg);
	}

	public function regenerate(){

		$id = \Joomla\CMS\Factory::getApplication()->input->get('id',0,'int');

		if ($this->getModel('Formwizard')->regenerate())
		{
			$msg = "HTML code successfully generated!";
		}
		else
		{
			$msg = "Error Saving!";
		}

		$this->setRedirect('index.php?option=com_joomcrm&task=Formwizard.edit&id='.$id.'#htmlEdit', $msg);

	}

	/**
	 * [delete description]
	 *
	 * @return  [type]  [description]
	 */
	public function delete()
	{
		$model = $this->getModel('Formwizard');

		if ($model->delete($this->id))
		{
			$msg = Text::_('COM_JOOMCRM_DELETED_FORM');
		}
		else
		{
			$msg = Text::_('COM_JOOMCRM_ERROR_DELETING_FORM');
		}

		$this->setRedirect('index.php?option=com_joomcrm&view=formwizard', $msg);
	}
}
