<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die;

/**
 * JoomCRM
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */


defined('_JEXEC') or die();

use Joomla\CMS\Table\Table;
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\Archive\Archive;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;use Joomla\Utilities\ArrayHelper;

class JoomcrmControllerImport extends JoomcrmController
{

	public $heads = [];
	public $key = null;
	public $json;
	public $db;

	public function __construct($config = array())
	{
		parent::__construct($config);

		if (!$this->input)
		{
			$this->input = Factory::getApplication()->input;

		}
		Factory::getApplication()->input->set('view', 'import');
		Factory::getApplication()->input->set('layout',  Factory::getApplication()->input->get('layout', 'default'));
	}

	public function downloadImportTemplate()
	{
		$template_type = $this->app->input->getString('template_type');

		$path = JPATH_SITE . '/components/com_joomcrm/media/import_templates/import_' . $template_type . '.csv';

		ob_start();
		header('Content-Description: File Transfer');
		header('Content-Type: application/octet-stream');
		header('Content-Disposition: attachment; filename="import_' . $template_type . '.csv"');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Pragma: public');
		header('Content-Length: ' . filesize($path));
		ob_clean();
		flush();

		readfile($path);

		exit();
	}

	public function installSampleData()
	{
		$config    = ComponentHelper::getParams('com_joomcrm');
		$db        = Factory::getDbo();
		$sampleIds = array();

		$importModel = $this->getModel('Import');

		$sampleCsvFiles = array(
			'company',
			'people',
			'deal',
			// 'event',
			// 'goal'
		);

		$inflector =  \Joomla\String\Inflector::getInstance();
		$inflector->addWord('people');

		foreach ($sampleCsvFiles as $file)
		{
			$table     = $inflector->toPlural($file);
			$modelName = ucfirst($file);

			$importData = $importModel->readCSVFile(JPATH_SITE . '/components/com_joomcrm/sample/sample-' . $file . '.csv', $table, false);

			unset($importData['headers']);

			$ids = $importModel->importCSVData($importData, $modelName, true);

			$sampleIds[$table] = $ids;
		}

		// transform to json
		$config->set('import_sample', serialize($sampleIds));

		// store to component config
		$db->setQuery('UPDATE `#__extensions` SET `params` = ' . $db->Quote($config) . ' WHERE `element` = "com_joomcrm" AND `type` = "component"');
		$db->execute();

		$msg = Text::_('COM_JOOMCRM_SAMPLE_DATA_INSTALLED');
		$this->setRedirect('index.php?option=com_joomcrm&view=import', $msg);
	}

	public function removeSampleData()
	{
		$config = ComponentHelper::getParams('com_joomcrm');

		$sampleIds = unserialize($config->get('import_sample'));

		$db = Factory::getDbo();

		$query = $db->getQuery(true);

		foreach ($sampleIds as $table => $ids)
		{
			if (empty($ids) || $ids === true)
			{
				continue;
			}

			$query
				->clear()
				->delete("#__joomcrm_" . $table)
				->where('id IN(' . implode(',', $ids) . ')');

			$db->setQuery($query)->execute();
		}

		// transform to json
		$config->set('import_sample', 0);

		// store to component config
		$db->setQuery('UPDATE `#__extensions` SET `params` = ' . $db->Quote($config) . ' WHERE `element` = "com_joomcrm" AND `type` = "component"');
		$db->execute();

		$msg = Text::_('COM_JOOMCRM_SAMPLE_DATA_REMOVED');
		$this->setRedirect('index.php?option=com_joomcrm&view=import', $msg);
	}

	public function batch()
	{
		$session = Factory::getSession();
		$success = true;

		if (file_exists(JPATH_BASE . '/tmp/joomcrm_import.php'))
		{
			include_once(JPATH_BASE . '/tmp/joomcrm_import.php');
		}

		if (isset($batchData) && count($batchData) > 0)
		{
			$success = $this->getModel('Import')->importCSVData($batchData, $batchType);
		}

		if ($success === 'redirect' && file_exists(JPATH_BASE . '/tmp/joomcrm_import.php'))
		{
			$this->app->redirect('index.php?option=com_joomcrm&task=import.batch');
		}

		$type = $session->get('batch_type');

		switch ($type)
		{
			case 'company':
				$view = 'companies';
				break;
			case 'deal':
				$view = 'deals';
				break;
			case 'people':
				$view = 'people';
				break;
			default:
				$view = '';
		}

		if (file_exists(JPATH_BASE . '/tmp/joomcrm_import.php'))
		{
			unlink(JPATH_BASE . '/tmp/joomcrm_import.php');
		}

		if ($success === true)
		{
			//$msg = JoomcrmHelperText::_('COM_JOOMCRM_IMPORT_WAS_SUCCESSFUL');
            $msg = Text::_('COM_JOOMCRM_IMPORT_WAS_SUCCESSFUL');
			$this->app->redirect('index.php?option=com_joomcrm&view=' . $view);
		}
		else
		{
			//$msg = JoomcrmHelperText::_('COM_JOOMCRM_ERROR_IMPORTING');
            $msg = Text::_('COM_JOOMCRM_ERROR_IMPORTING');
            $this->app->redirect('index.php?option=com_joomcrm&view=' . $view);
		}

		$view = empty($view) ? null : '&view=' . $view;

		$this->app->redirect('index.php?option=com_joomcrm' . $view);
	}


	public function import()
	{
		$user    = Factory::getApplication()->getIdentity();
		$app     = Factory::getApplication();
		$params  = $this->input->get('import', array(), 'array');
		$imports = Table::getInstance('Import', 'JoomcrmTable');

		$model = BaseDatabaseModel::getInstance('Import', 'JoomcrmModel', array('ignore_request' => true));

		if ($this->input->get('preset') == 'new')
		{

			$save = array(
				'user_id'  => $user->get('id'),
				'type_id'  => $app->input->get('type', 0, 'INT'),
				'params'   => json_encode($params),
				'name'     => $params['name'],
				'crossids' => '[]'
			);
			$imports->save($save);
		}
		else
		{
			$imports->load($this->input->get('preset'));
			$imports->params = json_encode($params);
			$imports->name   = $params['name'];
			$imports->store();
		}

		$db = Factory::getDbo();


		try
		{
			if (function_exists('set_time_limit'))
			{
				set_time_limit(0);
			}
			if (function_exists('ini_set'))
			{
				ini_set('max_execution_time', 0);
			}


			$params     = new Registry($imports->params);
			$import_key = Factory::getSession()->get('key', null);
			$stat = ['new' => 0, 'update' => 0, 'exist' => 0];


			$db->setQuery("SELECT * FROM #__joomcrm_import_rows WHERE `import` = '{$import_key}'");
			$list = $db->loadObjectList();


			if (!$list)
			{
				throw new Exception(\Joomla\CMS\Language\Text::_('COM_JOOMCRM_IMPORTROWSARENOTFOUND'));
			}


			switch ($app->input->get('type', 0, 'INT'))
			{
				case 0:
					$table      = "#__joomcrm_companies";
					$modelTable = "Company";
					break;
				case 1:
					$table      = "#__joomcrm_people";
					$modelTable = "People";
					break;
				case 2:
					$table      = "#__joomcrm_deals";
					$modelTable = "Deal";

					break;

				default  :
					Factory::getApplication()->enqueueMessage(Text::_('COM_JOOMCRM_IMPORT_TYPE_NOT_FOUND'), 'warning');
			}


			$selectedPreset = [];
			$selectedPreset['id'] = Factory::getApplication()->input->get('preset');
			$selectedPreset['type_id'] = isset($imports->type_id) && !empty($imports->type_id) ? $imports->type_id : '';

			Factory::getSession()->set('selectedPreset', $selectedPreset);
			$modelStore = BaseDatabaseModel::getInstance($modelTable, 'JoomcrmModel', array('ignore_request' => true));
			$insertType = Factory::getApplication()->input->get('InsertType', 0, 'INT');
			Factory::getSession()->set('messageInsertType', $insertType);


			foreach ($list as $record)
			{

				$row       = new Registry($record->text);
				$insertRow = new stdClass();

				foreach ($params->get('field') as $column => $mappedColumn)
				{

					if (!empty($row->get($mappedColumn)))
					{
						$insertRow->$column = $row->get($mappedColumn);
					}
				}

				$insertRow = (array) $insertRow;

				if (empty($insertRow)) continue;

				$duplicatedItem = $model->getDuplicatedItem($table, $insertRow, $modelTable);

				$checkName      = $modelTable == "People" ? $insertRow['first_name'] . '' . $insertRow['last_name'] : $insertRow['name'];
				$isDuplicated =
					strtolower($checkName) == strtolower($duplicatedItem->name) ||
					strtolower($insertRow['email']) == strtolower($duplicatedItem->email);

				//set owner id
				$owner_id = $params->get('owner_id','');
				if(!empty($owner_id))
					$insertRow['owner_id'] = (int) $owner_id;

				//set import (used in models to detect importing mode
				$insertRow['import'] = $import_key;


				if ($modelTable == "People")
				{
					if ( // if category is ID and exist
						isset($insertRow['company_id']) &&
						is_numeric($insertRow['company_id'])
					)
					{
						$insertRow['company_id']['value'] = $insertRow['company_id'];

					}

					if( // if category is name and exist
						isset($insertRow['company_id']) &&
						!is_numeric($insertRow['company_id'])
					)
					{
						$insertRow['company_name'] = $insertRow['company_id'];
						unset($insertRow['company_id']);
					}

					// if owner id is name and exist
					if (
						isset($insertRow['owner_id']) &&
						!is_numeric($insertRow['owner_id'])
					)
					{
						$insertRow['owner_name'] = $insertRow['owner_id'];
						unset($insertRow['owner_id']);
					}
				}

				if ($modelTable != "Deal" && $isDuplicated)
				{ // item already exist



					if ($insertType == 0)
					{ // skip item
						$stat['exist'] += 1;
						unset($insertRow);

					}
					elseif ($insertType == 1)
					{ //  Update item

						$insertRow['id'] = $duplicatedItem->id;
						$modelStore->store($insertRow);
						$stat['update'] += 1;
						unset($insertRow);

					}
					elseif ($insertType == 2)
					{ // duplicate item
						$insertRow['import'] = true;
						$modelStore->store($insertRow);
						$stat['new'] += 1;
					}

				}
				else
				{ // new item
					$modelStore->store($insertRow);
					$stat['new'] += 1;
				}

			}

			Factory::getSession()->set('messageState', Text::_('COM_JOOMCRM_IMPORT_WAS_SUCCESSFUL'));

		}
		catch (Exception $e)
		{
			echo $e->getMessage();
			\Joomla\CMS\Factory::getApplication()->enqueueMessage($e->getMessage(), 'warning');
		}

		$layout = Factory::getApplication()->isClient('administrator') ? '&layout=import' : '';

		Factory::getSession()->set('importstat', $stat);
		Factory::getApplication()->redirect(Route::_(
			'index.php?option=com_joomcrm&view=import&step=3&import_type=' . $imports->type_id.$layout, false)
		);
	}


	public function analize()
	{

		$this->key = $this->input->get('json');
		$file      = $this->input->getString('file');

		$ext        = strtolower(File::getExt($file));
		$this->json = JPATH_ROOT . '/tmp/' . $this->key . '.json';

		$upload = JPATH_ROOT . '/tmp/import_uploads/' . urldecode($file);

		if (!File::exists($upload))
		{
			$this->_error('COM_JOOMCRM_IMPORTCANNOTFINDFILE');
		}

		if (!in_array($ext,
			array(
				'zip',
				'csv',
				'json'
			))
		)
		{
			$this->_error('COM_JOOMCRM_IMPORTWRONGEXT');
		}

		if ($ext == 'zip')
		{

			$this->_msg(10, 'COM_JOOMCRM_IMPORTEXTRCT');

			$dir       = JPATH_ROOT . '/tmp/import_extract/' . $this->key;
			$this->dir = $dir;

			if (!Folder::exists($dir))
			{
				Folder::create($dir);
			}

			if (!\Joomla\Archive\Archive::extract($upload, $dir))
			{
				$this->_error('COM_JOOMCRM_IMPORTCANNOTEXTRACT');
			}

			$files = \Joomla\CMS\Filesystem\Folder::files($dir, '\.(csv|json)$', true, true);

			if (count($files) == 0)
			{
				$this->_error('COM_JOOMCRM_IMPORTNOFOUND');
			}

			if (count($files) > 1)
			{
				$this->_error('COM_JOOMCRM_IMPORTMORETHANONE');
			}

			$upload = $files[0];
		}

		$this->_msg(20, 'COM_JOOMCRM_IMPORTPARCE');

		if (!\Joomla\CMS\Filesystem\File::exists($upload))
		{
			$this->_error('COM_JOOMCRM_IMPORTCANNOTFINDFILE');
		}

		$this->db = Factory::getDbo();

		$this->heads = array();

		$this->db->setQuery("DELETE FROM `#__joomcrm_import_rows` WHERE `ctime` < NOW() - INTERVAL 1 DAY OR `import` = {$this->key}");
		$this->db->execute();

		$ext = strtolower(\Joomla\CMS\Filesystem\File::getExt($upload));

		if ($ext == 'csv')
		{
			$this->_load_csv($upload, $this->input->get('delimiter', ','));
		}
		elseif ($ext == 'json')
		{
			$this->_load_json($upload);
		}

		foreach ($this->heads as $h)
		{
			if (strpos($h, '.') !== false)
			{
				$this->_error('COM_JOOMCRM_IMPORTHEADERNAME');
			}
		}

		\Joomla\CMS\Factory::getSession()->set('fileHeaders', json_encode($this->heads));
		\Joomla\CMS\Factory::getSession()->set('key', $this->key);

		$this->_msg(100, 'COM_JOOMCRM_IMPORTPARCE');

		echo json_encode(['success' => true]);


		die;

	}

	/*
	 * Return json file status
	 */
	public function getJsonStatus(){

		// json file name
		$name = Factory::getApplication()->input->get('name',0,'int');

		// get file content
		$content = file_get_contents(JPATH_ROOT.'/tmp/'.$name.'.json');

		header('Content-Type: application/json');

		echo $content;

		die;
	}

	private function _error($msg)
	{
		if (!empty($this->dir))
		{
			Folder::delete($this->dir);
		}
		File::write($this->json, json_encode(array(
			'error' => Text::_($msg)
		)));


		die;
	}

	private function _msg($stat, $msg)
	{
		File::write($this->json, json_encode(array(
			'status' => $stat,
			'msg'    => Text::_($msg)
		)));
	}

	private function _load_csv($filename = '', $delimiter = ',')
	{

		$header = null;

		require JPATH_COMPONENT_SITE.'/libraries/parsecsv/vendor/autoload.php';
		$csv = new ParseCsv\Csv();
		$csv->delimiter = $delimiter;
		$csv->parse($filename);

		$data = [];

		foreach ($csv->data as $row){

			foreach ($csv->titles as $csvColumn){
				$data[$csvColumn] = isset($row[$csvColumn]) ? $row[$csvColumn] : '';
			}

			$this->_row($data);
		}

	}

	private function _row($data)
	{

		$sql = "INSERT INTO #__joomcrm_import_rows (id, `import`,`text`,`ctime`) VALUES (null, %d, '%s', NOW())";

		$sql = sprintf($sql, $this->key, $this->db->escape(json_encode($data)));
		$this->db->setQuery($sql);
		$this->db->execute();

		$columns     = array_keys($data);
		$this->heads = array_unique(array_merge($this->heads, $columns));


	}

	private function _load_json($file)
	{
		$body = json_decode(file_get_contents($file));
		foreach ($body as $row)
		{
			$this->_row($row);
		}
	}

	public function getParams()
	{
		$model  = \Joomla\CMS\MVC\Model\BaseDatabaseModel::getInstance('Import', 'JoomcrmModel', array('ignore_request' => true));
		$jinput = \Joomla\CMS\Factory::getApplication()->input;


		$type   = $jinput->get('type', 0, 'int');
		$preset = $model->getPreset();

		$presetVal = $jinput->get('preset','','string');


		$db        = Factory::getDBO();
		$columns   = [];
		$columnsCf = [];

		$types = [
			0 => [
				'name'           => 'companies',
				'cName'          => "company_custom",
				'columnsExclude' => [
					'id', 'owner_id', 'created', 'modified', 'avatar', 'published', 'public'
				]
			],
			1 => [
				'name'           => 'people',
				'cName'          => "people_custom",
				'columnsExclude' => [
					'id', 'uid', 'owner_id', 'category_id', 'modified', 'avatar', 'published', 'public', 'aim', 'status_id', 'type',
					'assignee_id', 'created', 'source_id'
				]
			],
			2 => [
				'name'           => 'deals',
				'cName'          => "deal_custom",
				'columnsExclude' => [
					'id', 'owner_id', 'company_id', 'stage_id', 'source_id', 'status_id', 'expected_close', 'created', 'category',
					'owner_id', 'modified', 'archived', 'actual_close', 'primary_contact_id', 'published', 'last_viewed','notes'
				]
			]
		];

		if (!array_key_exists($type, $types))
			echo 'Type not found';

		$prefix       = JoomcrmHelperJoomcrm::getIntType($jinput->getInt('type'));
		$fields       = $db->getTableColumns("#__joomcrm_{$types[$type]['name']}");
		$customFields = $this->getCustomfields("#__joomcrm_{$types[$type]['cName']}");

		// include table fields
		foreach ($fields as $fieldName => $fieldType)
		{
			if (!in_array($fieldName, $types[$type]['columnsExclude']))
			{
				$columnInfo        = new stdClass();
				$columnInfo->name  = $fieldName;
				$columnInfo->label = 'COM_JOOMCRM_IMPORT_COLUMN_' . strtoupper($prefix) . '_NAME_' . strtoupper($fieldName);
				$columns[]         = $columnInfo;
			}
		}

		// include note field
		$columnNote = new stdClass();
		$columnNote->name = "note";
		$columnNote->label = 'COM_JOOMCRM_IMPORT_COLUMN_NAME_NOTE';
		$columns[]         = $columnNote;


		// include custom fields
		foreach ($customFields as $fieldName => $fieldType)
		{
			$columnInfo        = new stdClass();
			$name              = preg_replace('/[^A-Za-z0-9\-]/', '', $fieldType[1]);
			$columnInfo->name  = 'custom_' . $fieldType[0];
			$columnInfo->label = $fieldType[1];
			$columnsCf[]       = $columnInfo;

		}

		$columns = array_merge($columns, $columnsCf);

		if ($presetVal != 'new')
		{
			$params = json_decode($preset->params);
			$name     = $preset->name;
			$owner_id = isset($params->owner_id) ? $params->owner_id : '';
			$selected = ArrayHelper::fromObject($preset->params->get('field'));
		}
		else
		{
			$name = $owner_id  = '';
			$selected = 0;
		}

		$heads = json_decode(Factory::getSession()->get('fileHeaders', '{}'));

		$list = array();
		foreach ($heads as $head)
		{
			$list[$head] = $head;
		}

		// load frontend language again
		$lang = Factory::getLanguage();
		$lang->load('com_joomcrm',JPATH_SITE,$lang->getTag(),true);


		echo LayoutHelper::render('import.params',
			[
				'fields'   => $columns,
				'heads'    => $list,
				'selected' => $selected,
				'name'     => $name,
				'owner_id' => $owner_id
			],'', ['client' => 'site']
		);

		jexit();

	}

	public function getCustomfields($table)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true);

		$query->select('id as custom_field_id ,name');
		$query->from($db->quoteName($table));
		$query->order('ordering DESC ');

		$db->setQuery($query);

		return $db->loadRowList();
	}

	public function upload()
	{
		JLoader::register('UploadHandler', JPATH_ROOT . '/components/com_joomcrm/helper/UploadHandler.php');

		$options = array(
			'accept_file_types' => '/\.(zip|json|csv)$/i',
			'upload_dir'        => JPATH_ROOT . '/tmp/import_uploads/'
		);

		Factory::getSession()->set('importprocess', 0);

		if (!Folder::exists($options['upload_dir']))
		{
			Folder::create($options['upload_dir']);
		}

		$upload = new UploadHandler($options);

		die;
	}

	public function map_table($n, $m)
	{
		return (array($n => $m));
	}

	public function deletePreset()
	{
		$model = $this->getModel('import');

		return $model->deletePreset();
	}

	public function cancel(){

		$this->setRedirect('index.php?option=com_joomcrm&view=import');
	}

}