<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

class JoomcrmControllerMenu extends JoomcrmController
{
	protected $text_prefix = 'COM_JOOMCRM';

	public function __construct($config = array())
	{
		parent::__construct($config);

		// Define standard task mappings.

		// Value = 0
		$this->registerTask('unpublish', 'publish');
	}

	/**
	 * [cancel description]
	 *
	 * @return  [type]  [description]
	 */
	public function cancel()
	{
		$msg = Text::_('Menu cancelled!');
		$this->setRedirect('index.php?option=com_joomcrm', $msg);
	}

	public function publish()
	{
		// Check for request forgeries
		Session::checkToken() or die(Text::_('JINVALID_TOKEN'));

		// Get items to publish from the request.
		$cid = Factory::getApplication()->input->get('cid', array(), 'array');
		$data = array('publish' => 1, 'unpublish' => 0, 'archive' => 2, 'trash' => -2, 'report' => -3);
		$task = $this->getTask();
		$value = \Joomla\Utilities\ArrayHelper::getValue($data, $task, 0, 'int');

		if (empty($cid))
		{
			\Joomla\CMS\Log\Log::add(Text::_($this->text_prefix . '_NO_ITEM_SELECTED'), \Joomla\CMS\Log\Log::WARNING, 'jerror');
		}
		else
		{
			// Get the model.
			$model = $this->getModel('Menu');

			// Make sure the item ids are integers
			\Joomla\Utilities\ArrayHelper::toInteger($cid);

			// Publish the items.
			try
			{
				$model->publish($cid, $value);

				if ($value == 1)
				{
					$ntext = $this->text_prefix . '_N_ITEMS_PUBLISHED';
				}
				elseif ($value == 0)
				{
					$ntext = $this->text_prefix . '_N_ITEMS_UNPUBLISHED';
				}
				elseif ($value == 2)
				{
					$ntext = $this->text_prefix . '_N_ITEMS_ARCHIVED';
				}
				else
				{
					$ntext = $this->text_prefix . '_N_ITEMS_TRASHED';
				}
				$this->setMessage(Text::plural($ntext, count($cid)));
			}
			catch (Exception $e)
			{
				$this->setMessage(Text::_('JLIB_DATABASE_ERROR_ANCESTOR_NODES_LOWER_STATE'), 'error');
			}

		}

		$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=menu', false));
	}

	/**
	 * [save description]
	 *
	 * @return  [type]  [description]
	 */
	public function save()
	{
		if ($this->getModel('Menu')->store())
		{
			$msg = "Successfully saved!";
		}
		else
		{
			$msg = "Error Saving!";
		}

		$this->setRedirect('index.php?option=com_joomcrm&view=menu', $msg);
	}

	/**
	 * Method to save the submitted ordering values for records via AJAX.
	 *
	 * @return  void
	 *
	 * @since   3.0
	 */
	public function saveOrderAjax()
	{
		$pks = $this->app->input->post->get('cid', array(), 'array');
		$order = $this->app->input->post->get('order', array(), 'array');

		// Sanitize the input
		\Joomla\Utilities\ArrayHelper::toInteger($pks);
		\Joomla\Utilities\ArrayHelper::toInteger($order);

		// Get the model
		$model = $this->getModel('Menu');

		// Save the ordering
		if ($model->saveorder($pks, $order))
		{
			echo "1";
		}

		// Close the application
		$this->app->close();
	}
}
