<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\HTML\Helpers\Sidebar;
use Joomla\CMS\Uri\Uri;

abstract class JoomcrmHelperMenu
{
	protected static $navLoaded = false;
	protected static $menuModules = false;

	// Load the navigation menu
	public static function loadNavi()
	{
		if (self::$navLoaded === false)
		{
			if (version_compare(JVERSION, '3.0', 'ge'))
			{
				self::load3xNavi();
			}
			else
			{
				// Add home page submenu
				Sidebar::addEntry(Text::_('COM_JOOMCRM_MAIN'), 'index.php?option=com_joomcrm');
				// Add users submenu
				Sidebar::addEntry(Text::_('COM_JOOMCRM_USERS'), 'index.php?option=com_joomcrm&view=manageusers');
			}

			self::$navLoaded = true;
		}
	}

	protected static function load3xNavi()
	{
		$app = Factory::getApplication();

		$links      = self::getMenuLinks();
		$activeLink = $app->input->get('view', '');

		foreach ($links as $link)
		{
			if (!empty($activeLink))
				$active = strtolower($activeLink) === $link['view'];
			else
				$active = false;

			Sidebar::addEntry(
				$link['text'],
				$link['link'],
				$active
			);
		}
	}

	public static function getMenuLinks()
	{

		// load jquery framework
		HTMLHelper::_('jquery.framework');

		//load admin css
		HTMLHelper::stylesheet(Uri::root() . 'media/com_joomcrm/css/menuBuilder.css');
		HTMLHelper::script(Uri::root() . 'media/com_joomcrm/js/menuBuilder.js');
		// load admin js


		$menuItems = array();

		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=joomcrm'),
			'class'   => 'icon-home',
			'text'    => '<i class="fas fa-tachometer-alt isParent"></i> ' . Text::_('COM_JOOMCRM_DASHBOARD'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_DASHBOARD_MENU_TOOLTIP'),
			'id'      => "dashboard_menu_link",
			'view'    => "joomcrm"
		);
		$menuItems[] = array(
			'link'    => Route::_('#'),
			'class'   => 'icon-th-list',
			'text'    => '<i class="fas fa-folder-open isParent"></i> ' . Text::_('COM_JOOMCRM_CATEGORIES'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_COMPANY_CATEGORIES_MENU_TOOLTIP'),
			'id'      => "company_categories_menu_link",
			'view'    => "companycategories"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=companycategories'),
			'class'   => 'icon-th-list',
			'text'    => '<i class="fas fa-building isChild"></i>' . Text::_('COM_JOOMCRM_COMPANY_CATEGORIES'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_COMPANY_CATEGORIES_MENU_TOOLTIP'),
			'id'      => "company_categories_menu_link",
			'view'    => "companycategories"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=notecategories'),
			'class'   => 'icon-th-list',
			'text'    => '<i class="fas fa-sticky-note isChild"></i>' . Text::_('COM_JOOMCRM_NOTE_CATEGORIES'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_NOTES_MENU_TOOLTIP'),
			'id'      => "notes_menu_link",
			'view'    => "notecategories"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=eventcategories'),
			'class'   => 'icon-th-list',
			'text'    => '<i class="fas fa-tags isChild"></i>' . Text::_('COM_JOOMCRM_EVENT_CATEGORIES'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_EVENT_CATEGORIES_MENU_TOOLTIP'),
			'id'      => "eventcategories_menu_link",
			'view'    => "eventcategories"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=documentcategories'),
			'class'   => 'icon-th-list',
			'text'    => '<i class="fas fa-file isChild"></i>' . Text::_('COM_JOOMCRM_DOCUMENT_CATEGORIES'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_DOCUMENT_CATEGORIES_MENU_TOOLTIP'),
			'id'      => "documentcategories_menu_link",
			'view'    => "documentcategories"
		);
		$menuItems[] = array(
			'link'    => Route::_('#'),
			'class'   => 'icon-user',
			'text'    => '<i class="fas fa-users isParent"></i>' . Text::_('COM_JOOMCRM_USERS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_USERS_MENU_TOOLTIP'),
			'id'      => "user_menu_link",
			'view'    => "manageusers"
		);
		/*
		$menuItems[] = array(
			'link' => Route::_('index.php?option=com_joomcrm&view=users'),
			'class' => 'icon-user',
			'text' => '<i class="fas fa-user isChild"></i>'.Text::_('COM_JOOMCRM_USERS_MANAGER'),
			'access' => array(),
			'tooltip' => Text::_('COM_JOOMCRM_USERS_MENU_TOOLTIP'),
			'id' => "user_menu_link",
			'view' => "users"
			);
		if (version_compare(JVERSION, '3.0.0', '>'))
		{
			$menuItems[] = array(
				'link' => Route::_('index.php?option=com_joomcrm&view=teams'),
				'class' => 'icon-team',
				'text' => '<i class="fas fa-users-cog isChild"></i>'.Text::_('COM_JOOMCRM_TEAMS'),
				'access' => array(),
				'tooltip' => Text::_('COM_JOOMCRM_TEAMS_MENU_TOOLTIP'),
				'id' => "team_menu_link",
				'view' => "teams"
				);
		}
        $menuItems[] = array(
            'link' => Route::_('index.php?option=com_joomcrm&view=statuses'),
            'class' => 'icon-thumbs-up',
            'text' => '<i class="fas fa-flag isChild"></i>'.Text::_('COM_JOOMCRM_PEOPLE_STATUSES'),
            'access' => array(),
            'tooltip' => Text::_('COM_JOOMCRM_STATUSES_MENU_TOOLTIP'),
            'id' => "statuses_menu_link",
            'view' => "statuses"
        );
*/
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=manageusers'),
			'class'   => 'icon-user',
			'text'    => '<i class="fas fa-user isChild"></i>' . Text::_('COM_JOOMCRM_USERS_MANAGER'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_USERS_MENU_TOOLTIP'),
			'id'      => "user_menu_link",
			'view'    => "manageusers"
		);

		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=manageteams'),
			'class'   => 'icon-team',
			'text'    => '<i class="fas fa-users-cog isChild"></i>' . Text::_('COM_JOOMCRM_TEAMS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_TEAMS_MENU_TOOLTIP'),
			'id'      => "team_menu_link",
			'view'    => "manageteams"
		);

		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=managestatuses'),
			'class'   => 'icon-thumbs-up',
			'text'    => '<i class="fas fa-flag isChild"></i>' . Text::_('COM_JOOMCRM_PEOPLE_STATUSES'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_STATUSES_MENU_TOOLTIP'),
			'id'      => "statuses_menu_link",
			'view'    => "managestatuses"
		);

		$menuItems[] = array(
			'link'    => Route::_('#'),
			'class'   => 'icon-tasks',
			'text'    => '<i class="fas fa-handshake isParent"></i>' . Text::_('COM_JOOMCRM_DEAL'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_STAGES_MENU_TOOLTIP'),
			'id'      => "stages_menu_link",
			'view'    => "dealsstages"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=dealsstages'),
			'class'   => 'icon-tasks',
			'text'    => '<i class="fas fa-bars isChild"></i>' . Text::_('COM_JOOMCRM_DEAL_STAGES'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_STAGES_MENU_TOOLTIP'),
			'id'      => "stages_menu_link",
			'view'    => "dealsstages"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=dealsstatuses'),
			'class'   => 'icon-tasks',
			'text'    => '<i class="fas fa-flag isChild"></i>' . Text::_('COM_JOOMCRM_DEAL_STATUSES'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_DEAL_STATUSES_MENU_TOOLTIP'),
			'id'      => "deal_statuses_menu_link",
			'view'    => "dealsstatuses"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=companycustoms'),
			'class'   => 'icon-edit',
			'text'    => '<i class="fas fa-cubes isParent"></i>' . Text::_('COM_JOOMCRM_CUSTOM_FIELDS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_CUSTOM_MENU_TOOLTIP'),
			'id'      => "companycustom_menu_link",
			'view'    => "companycustoms"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=companycustoms'),
			'class'   => 'icon-edit',
			'text'    => '<i class="fas fa-building isChild"></i>' . Text::_('COM_JOOMCRM_COMPANY_CUSTOM_FIELDS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_CUSTOM_MENU_TOOLTIP'),
			'id'      => "companycustom_menu_link",
			'view'    => "companycustoms"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=peoplecustoms'),
			'class'   => 'icon-edit',
			'text'    => '<i class="fas fa-users isChild"></i>' . Text::_('COM_JOOMCRM_PEOPLE_CUSTOM_FIELDS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_CUSTOM_MENU_TOOLTIP'),
			'id'      => "peoplecustom_menu_link",
			'view'    => "peoplecustoms"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=dealcustoms'),
			'class'   => 'icon-edit',
			'text'    => '<i class="fas fa-handshake isChild"></i>' . Text::_('COM_JOOMCRM_DEAL_CUSTOM_FIELDS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_CUSTOM_MENU_TOOLTIP'),
			'id'      => "dealcustom_menu_link",
			'view'    => "dealcustoms"
		);

		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=sources'),
			'class'   => 'icon-random',
			'text'    => '<i class="fas fa-power-off isParent"></i>' . Text::_('COM_JOOMCRM_SOURCES'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_SOURCES_MENU_TOOLTIP'),
			'id'      => "sources_menu_link",
			'view'    => "sources"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=templates'),
			'class'   => 'icon-filter',
			'text'    => '<i class="fas fa-tasks isParent"></i>' . Text::_('COM_JOOMCRM_WORKFLOWS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_WORKFLOW_MENU_TOOLTIP'),
			'id'      => "workflow_menu_link",
			'view'    => "templates"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=documents'),
			'class'   => 'icon-folder-open',
			'text'    => '<i class="fas fa-copy isParent"></i>' . Text::_('COM_JOOMCRM_SHARED_DOCUMENTS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_DOCUMENTS_MENU_TOOLTIP'),
			'id'      => "documents_menu_link",
			'view'    => "documents"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=formwizard'),
			'class'   => 'icon-star-empty',
			'text'    => '<i class="fas fa-edit isParent"></i>' . Text::_('COM_JOOMCRM_FORM_WIZARD_HEADER'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_FORMWIZARD_TOOLTIP'),
			'id'      => "formwizard_menu_link",
			'view'    => "formwizard"
		);
		$menuItems[] = array(
			'link'    => Route::_('#'),
			'class'   => 'icon-align-justify',
			'text'    => '<i class="fas fa-wrench isParent"></i>' . Text::_('COM_JOOMCRM_TOOLS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_MENU_MENU_TOOLTIP'),
			'id'      => "menu_menu_link",
			'view'    => "menu"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=menu'),
			'class'   => 'fas fa-ellipsis-h',
			'text'    => '<i class="fas fa-ellipsis-h isChild"></i>' . Text::_('COM_JOOMCRM_MENU'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_MENU_MENU_TOOLTIP'),
			'id'      => "menu_menu_link",
			'view'    => "menu"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_joomcrm&view=import'),
			'class'   => 'icon-upload',
			'text'    => '<i class="fas fa-truck isChild"></i>' . Text::_('COM_JOOMCRM_IMPORT'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_CONFIG_IMPORT_TOOLTIP'),
			'id'      => "import_menu_link",
			'view'    => "import"
		);
		$menuItems[] = array(
			'link'    => Route::_('index.php?option=com_config&view=component&component=com_joomcrm'),
			'class'   => 'icon-cog',
			'text'    => '<i class="fas fa-cog isParent"></i>' . Text::_('COM_JOOMCRM_SETTINGS'),
			'access'  => array(),
			'tooltip' => Text::_('COM_JOOMCRM_CONFIG_MENU_TOOLTIP'),
			'id'      => "config_menu_link",
			'view'    => "config"
		);

		return $menuItems;
	}

	public static function getQuickMenuLinks()
	{
		return array(array('link' => Route::_('index.php?option=com_joomcrm&task=users.add'), 'class' => 'icon-user', 'text' => Text::_('COM_JOOMCRM_ADD_NEW_USER'), 'access' => array()), array('link' => Route::_('index.php?option=com_joomcrm&task=stages.add'), 'class' => 'icon-tasks', 'text' => Text::_('COM_JOOMCRM_ADD_NEW_DEAL_STAGE'), 'access' => array()), array('link' => Route::_('index.php?option=com_joomcrm&task=categories.add'), 'class' => 'icon-th-list', 'text' => Text::_('COM_JOOMCRM_ADD_NEW_NOTE_CATEGORY'), 'access' => array()), array('link' => Route::_('index.php?option=com_joomcrm&task=sources.add'), 'class' => 'icon-random', 'text' => Text::_('COM_JOOMCRM_ADD_NEW_SOURCE'), 'access' => array()), array('link' => Route::_('index.php?option=com_joomcrm&task=companycustom.add'), 'class' => 'icon-edit', 'text' => Text::_('COM_JOOMCRM_ADD_NEW_COMPANY_CUSTOM_FIELD'), 'access' => array()), array('link' => Route::_('index.php?option=com_joomcrm&task=peoplecustom.add'), 'class' => 'icon-edit', 'text' => Text::_('COM_JOOMCRM_ADD_NEW_PEOPLE_CUSTOM_FIELD'), 'access' => array()), array('link' => Route::_('index.php?option=com_joomcrm&task=dealcustom.add'), 'class' => 'icon-edit', 'text' => Text::_('COM_JOOMCRM_ADD_NEW_DEAL_CUSTOM_FIELD'), 'access' => array()), array('link' => Route::_('index.php?option=com_joomcrm&task=statuses.add'), 'class' => 'icon-thumbs-up', 'text' => Text::_('COM_JOOMCRM_ADD_NEW_PERSON_STATUS'), 'access' => array()), array('link' => Route::_('index.php?option=com_joomcrm&task=templates.add'), 'class' => 'icon-filter', 'text' => Text::_('COM_JOOMCRM_CREATE_NEW_WORKFLOW'), 'access' => array()));
	}


	public static function getHelpMenuLinks()
	{
		$config = ComponentHelper::getParams('com_joomcrm');

		// load wizard library , todo: in future move all code related to wizard to it's own class
		JLoader::register('JoomcrmHelperWizard', JPATH_COMPONENT_ADMINISTRATOR . '/helper/wizard.php');

		$types = array(
			array(
				'link'             => Route::_('index.php?option=com_joomcrm&task=users.add&show_fields=uid'),
				'class'            => 'fas fa-users',
				'text'             => Text::_('COM_JOOMCRM_CREATE_NEW_USERS_HELP'),
				'access'           => array(),
				'completed_status' => JoomcrmHelperWizard::checkIfUsersExist(),
			)
		,
			array(
				'link'             => Route::_('index.php?option=com_config&view=component&component=com_joomcrm'),
				'class'            => 'fas fa-cog',
				'text'             => Text::_('COM_JOOMCRM_CREATE_LOCALE_HELP'),
				'access'           => array(),
				'completed_status' => JoomcrmHelperWizard::checkIfConfigAdjusted()
			),
			array(
				'link'             => Route::_('index.php?option=com_joomcrm&task=templates.edit&show_fields=name'),
				'class'            => 'fas fa-tasks',
				'text'             => Text::_('COM_JOOMCRM_CREATE_WORKFLOWS_HELP'),
				'access'           => array(),
				'config'           => 'templates_edit',
				'completed_status' => JoomcrmHelperWizard::checkIfWorkflowAdded()
			),
			array(
				'link'             => Route::_('index.php?option=com_joomcrm&view=menu&layout=default&show_fields=header'),
				'class'            => 'fas fa-ellipsis-h',
				'text'             => Text::_('COM_JOOMCRM_CREATE_MENU_ITEMS_HELP'),
				'access'           => array(),
				'config'           => 'menu_default',
				'completed_status' => $config->get('menu_default', 0)
			),
			array(
				'link'             => Route::_('index.php?option=com_joomcrm&view=import&layout=sample'),
				'class'            => 'fas fa-id-card',
				'text'             => Text::_('COM_JOOMCRM_CREATE_INSTALL_SAMPLE'),
				'access'           => array(),
				'config'           => 'import_sample',
				'completed_status' => JoomcrmHelperWizard::checkIfInstallSampleData()
			),
			array(
				'link'             => Route::_('index.php?option=com_joomcrm&view=import&layout=default'),
				'class'            => 'fas fa-truck',
				'text'             => Text::_('COM_JOOMCRM_CREATE_IMPORT_HELP'),
				'access'           => array(),
				'config'           => 'import_default',
				'completed_status' => $config->get('import_default', 0)
			),
			array(
				'link'             => Route::_('index.php?option=com_joomcrm&view=launch&layout=default'),
				'class'            => 'fas fa-check-circle',
				'text'             => Text::_('COM_JOOMCRM_CREATE_LAUNCH_HELP'),
				'access'           => array(),
				'config'           => 'launch_default',
				'completed_status' => $config->get('launch_default', 0)
			)
		);

		return $types;
	}

	public static function getMenuModules()
	{
		$config = ComponentHelper::getParams('com_joomcrm');

		if (self::$menuModules === false)
		{
			$modules = array();

			/** Main Menu Links **/
			$menu_links       = JoomcrmHelperMenu::getMenuLinks();
			$menu             = JoomcrmHelperView::getView('Joomcrm', 'menu');
			$menu->menu_links = $menu_links;
			$modules['menu']  = $menu;

			/** Quick Menu Links **/
			$quick_menu_links             = JoomcrmHelperMenu::getQuickMenuLinks();
			$quick_menu                   = JoomcrmHelperview::getView('Joomcrm', 'quick_menu');
			$quick_menu->quick_menu_links = $quick_menu_links;
			$modules['quick_menu']        = $quick_menu;

			/** Determine help type on page **/
			$help_type_1         = Factory::getApplication()->input->get('view') != '' || !is_null(Factory::getApplication()->input->get('view')) ? Factory::getApplication()->input->get('view') : Factory::getApplication()->input->get('controller');
			$help_type_2         = Factory::getApplication()->input->get('layout') != '' || !is_null(Factory::getApplication()->input->get('layout')) ? Factory::getApplication()->input->get('layout') : Factory::getApplication()->input->get('task');
			$help_type_1         = ($help_type_1 == "" || is_null($help_type_1)) ? "" : $help_type_1;
			$help_type_2         = ($help_type_2 == "" || is_null($help_type_2)) ? "" : '_' . $help_type_2;
			$help_type           = str_replace(".", "_", $help_type_1 . $help_type_2);
			$help_types          = self::getHelpTypes();
			$show_help           = $config->get('show_help', 1);
			$launch_default      = $config->get('launch_default', 0);
			$step_completed      = $config->get($help_type);
			$show_update_buttons = in_array($help_type, $help_types);

			/** Help Menu Links **/
			$help_menu_links                = JoomcrmHelperMenu::getHelpMenuLinks();
			$help_menu                      = JoomcrmHelperview::getView('joomcrm', 'help_menu');
			$help_menu->help_menu_links     = $help_menu_links;
			$help_menu->help_type           = $help_type;
			$help_menu->show_help           = $show_help;
			$help_menu->launch_default      = $launch_default;
			$help_menu->step_completed      = $step_completed;
			$help_menu->show_update_buttons = $show_update_buttons;
			$modules['help_menu']           = $help_menu;


			$help = JoomcrmHelperview::getView('joomcrm', 'help_info');

			$help->help_menu_links     = $help_menu_links;
			$help->help_type           = $help_type;
			$help->show_help           = $show_help;
			$help->launch_default      = $launch_default;
			$help->step_completed      = $step_completed;
			$help->show_update_buttons = $show_update_buttons;


			$modules['help_info'] = $help;

			$count     = count($help_menu_links) - 1;
			$completed = 0;
			foreach ($help_menu_links as $link)
			{
				if ($link['completed_status'] == 1)
				{
					$completed++;
				}
			}
			$modules['percentage'] = number_format(($completed / $count) * 100, 0);

			/** Auto show input tooltips **/
			$input_fields = explode(',', Factory::getApplication()->input->get('show_fields', ''));

			if (is_array($input_fields) && !empty($input_fields[0]))
			{
				$js = '
/** Tooltips **/
jQuery(document).ready(function ($) {';
				foreach ($input_fields as $input_field)
				{
					$js .= "\tshowTooltip('{$input_field}');";
				}

				$js .= '
});
';
				Factory::getDocument()->addScriptDeclaration($js);
			}


			self::$menuModules = $modules;
		}

		return self::$menuModules;
	}

	public static function getHelpTypes()
	{
		return [];
	}

}
