<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;

class JoomcrmModelDocuments extends JoomcrmModel
{
	public function store($data = null)
	{
		// Load Tables
		$row = new JoomcrmTableDocuments;
		if ($data == null)
		{
			$data = \Joomla\CMS\Factory::getApplication()->input->post->getArray();
		}

		// Date generation
		$date = date('Y-m-d H:i:s');

		if (!array_key_exists('id', $data))
		{
			$data['created'] = $date;
		}

		$data['modified'] = $date;
		$data['shared']   = 1;

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		return true;
	}

	/**
	 * Get list of stages
	 *
	 * @param   int $id  specific search id
	 *
	 * @return  mixed  $results results
	 */
	public function getDocuments($id = null)
	{
		// Database
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->select("d.*")->from("#__joomcrm_documents AS d")->where("d.shared=1")
				 ->order($this->getState('Documents.filter_order') . ' ' . $this->getState('Documents.filter_order_Dir'));

		if ($id)
		{
			$query->where("d.id=$id");
		}

		return $db->setQuery($query)->loadAssocList();
	}

	public function populateState($ordering = null, $direction = null)
	{
		// Get states
		$app              = Factory::getApplication();
		$filter_order     = $app->getUserStateFromRequest('Documents.filter_order', 'filter_order', 'd.filename');
		$filter_order_Dir = $app->getUserStateFromRequest('Documents.filter_order_Dir', 'filter_order_Dir', 'asc');

		// Set states
		$this->setState('Documents.filter_order', $filter_order);
		$this->setState('Documents.filter_order_Dir', $filter_order_Dir);
	}

	public function remove($id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->delete('#__joomcrm_documents')->where('id = ' . $id);

		$db->setQuery($query)->execute();
	}

}
