<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;

JLoader::register('JoomcrmHelperFormwizard',JPATH_SITE.'/components/com_joomcrm/helper/formwizard.php');

class JoomcrmModelFormwizard extends JoomcrmModelJoomcrm
{
	public function populateState($ordering = null, $direction = null)
	{
		// Get states
		$app              = Factory::getApplication();
		$filter_order     = $app->getUserStateFromRequest('Formwizard.filter_order', 'filter_order', 'f.name');
		$filter_order_Dir = $app->getUserStateFromRequest('Formwizard.filter_order_Dir', 'filter_order_Dir', 'asc');

		// Set states
		$this->setState('Formwizard.filter_order', $filter_order);
		$this->setState('Formwizard.filter_order_Dir', $filter_order_Dir);
	}

	public function regenerate(){


		return $this->store(true);
	}

	public function store($generateHTML = false)
	{
		// Load Tables
		$row  = new JoomcrmTableFormwizard;
		$input = \Joomla\CMS\Factory::getApplication()->input;
		$html = $input->get('html','','raw');
		$data = $input->getArray();

		$data['html'] = $html;

		$updateId = false;

		$userId = Factory::getApplication()->getIdentity()->id;

		// Date generation
		$date                = date('Y-m-d H:i:s');
		$data['modified']    = $date;
		$data['modified_by'] = $userId;
		if (!array_key_exists('id', $data))
		{
			$data['created']    = $date;
			$data['created_by'] = $userId;
			$updateId = true;
		}

		if (array_key_exists('fields', $data))
		{
			$data['fields'] = serialize($data['fields']);
		}


		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}


		if ($generateHTML || empty($row->html))
		{
			$row->html = JoomcrmHelperFormwizard::buildForm($row);
			$row->store();
		}

		return true;
	}

	public function __buildQuery()
	{
		return $this->getDbo()->getQuery(true)->select("f.*,CONCAT(user.first_name,' ',user.last_name) AS owner_name")
			   ->from("#__joomcrm_formwizard AS f")->leftJoin("#__joomcrm_users AS user ON user.id = f.owner_id");
	}

	public function getForms()
	{
		$db    = $this->getDbo();
		$query = $this->__buildQuery()
				 ->order($this->getState('Formwizard.filter_order') . ' ' . $this->getState('Formwizard.filter_order_Dir'));

		$results = $db->setQuery($query)->loadAssocList();

		if (count($results) > 0)
		{
			foreach ($results as $key => $result)
			{
				$results[$key]['fields'] = unserialize($result['fields']);
				$results[$key]['html']   = $result['html'];
			}
		}

		return $results;
	}

	public function getForm($formId)
	{
		$db    = $this->getDbo();
		$query = $this->__buildQuery()->where("f.id=" . $formId);

		$result           = $db->setQuery($query)->loadAssoc();
		$result['fields'] = unserialize($result['fields']);

		return $result;
	}

	public function delete($ids)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->delete("#__joomcrm_formwizard");

		if (is_array($ids))
		{
			$query->where("id IN(" . implode(',' . $ids) . ")");
		}
		else
		{
			$query->where("id=" . $ids);
		}

		$db->setQuery($query);

		if ($db->execute())
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public function getTempFormId()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->select('MAX(id)')->from('#__joomcrm_formwizard');

		$lastId = $db->setQuery($query)->loadResult();

		return $lastId + 1;
	}

}
