<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;

jimport('joomla.filesystem.file');

class JoomcrmModelImport extends JoomcrmModel
{
	protected $db = null;
	protected $query = null;

	public function getPresets()
	{
		$this->_db->setQuery("SELECT id as value, name as text FROM #__joomcrm_import WHERE type_id = " .
			Factory::getApplication()->input->get('import_type',0,'INT') . " AND user_id = " . (int)Factory::getApplication()->getIdentity()->get('id', 0));

		return $this->_db->loadObjectList();
	}

	public function getPreset()
	{
		$this->_db->setQuery("SELECT * FROM #__joomcrm_import WHERE type_id = ".(int)Factory::getApplication()->input->get('type')." and id = " . (int)Factory::getApplication()->input->get('preset'));
		$preset = $this->_db->loadObject();

		if(!@$preset)
		{
			return NULL;
		}

		@$preset->params = new Registry(@$preset->params);

		return $preset;
	}
	public function deletePreset(){

		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		$conditions = array(
			$db->quoteName('id') . ' =' .(int)Factory::getApplication()->input->get('preset')

		);

		$query->delete($db->quoteName('#__joomcrm_import'));
		$query->where($conditions);
		$db->setQuery($query);
		$db->execute();
	}


	public function getDuplicatedItem($table,$values,$type){

		// fix enconding problem
		$values = array_map("htmlentities", $values);

		// string to lower
		$values = array_map("strtolower", $values);

		// Get a db connection.
		$db = Factory::getDbo();

		$query = $db->getQuery(true);


		switch ($type)
		{
			case "People":


				$vfirst_name = isset($values['first_name']) ? $values['first_name'] : '';
				$vlast_name =  isset($values['last_name']) ? $values['last_name'] : '';
				$vEmail =  isset($values['email']) ? $values['email'] : '';

				if(empty($vfirst_name.$vlast_name) && empty($vEmail)){
					return false;
				}

				$query->select('*,CONCAT(first_name,last_name) as name');
				$query->from($db->quoteName($table));
				if(!empty($vfirst_name))
					$query->where("LOWER(first_name) = ".$db->quote($vfirst_name));
				if(!empty($vlast_name))
					$query->where("LOWER(last_name)= ".$db->quote($vlast_name));
				if(!empty($vEmail))
					$query->orWhere("LOWER(email)= ".$db->quote($vEmail));
				// $query->where("$Cemail = '$vemail'");
				$query->andWhere('published=1');
				// $query->where("$Cemail = '$vemail'");
				$query->where('published = 1');
				$query->order('created DESC');

				$db->setQuery($query);
				return $db->loadObject();

				break;
			case "Company":

				$vName =  isset($values['name']) ? $values['name'] : '';
				$vEmail =  isset($values['email']) ? $values['email'] : '';

				if(empty($vName)){
					return false;
				}

				$query->select('*');
				$query->from($db->quoteName($table));
				if(!empty($vName))
					$query->where("LOWER(name) = ".$db->quote($vName));
				if(!empty($vEmail))
					$query->where("LOWER(email) = ".$db->quote($vEmail));
				$query->where('published=1');
				$query->order('created DESC');



				$db->setQuery($query);
				return $db->loadObject();

				break;
			case "deal":
				return false;
				break;
		}


	}


	public function importCSVData($data, $modelName, $sample = false)
	{
		$success = false;
		$session = Factory::getSession();
		$session->set("batch_type", $modelName);

		$importedIds = array();

		unset($data['headers']);

		if (count($data) > 0 && isset($modelName))
		{
			$modelClass = 'JoomcrmModel' . ucfirst(strtolower($modelName));
			$model = new $modelClass;
			//$model = BaseDatabaseModel::getInstance($modelName, 'JoomcrmModel');
			$i     = 0;

			foreach ($data as $import)
			{
				if ($i == 1000)
				{
					$file = '<?php $batchType="' . $modelName . '"; $batchData=' . var_export($data, true) . ";";
					if (\Joomla\CMS\Filesystem\File::exists(JPATH_ROOT . "/tmp/joomcrm_import.php"))
					{
						\Joomla\CMS\Filesystem\File::delete(JPATH_ROOT . "/tmp/joomcrm_import.php");
					}
					\Joomla\CMS\Filesystem\File::write(JPATH_ROOT . "/tmp/joomcrm_import.php", $file);

					return "redirect";
				}

				if ($id = $this->checkDuplicate($modelName, $import))
				{
					$import['id'] = $id;
				}

				$import['import'] = true;
				$item             = $model->store($import, $sample);
				$import           = null;
				unset($import);
				$i++;
				$importedIds[] = $item->id;
			}

			if (\Joomla\CMS\Filesystem\File::exists(JPATH_BASE . "/tmp/joomcrm_import.php"))
			{
				\Joomla\CMS\Filesystem\File::delete(JPATH_BASE . "/tmp/joomcrm_import.php");
			}
		}

		return $importedIds;
	}

	private function checkDuplicate($modelName, $import)
	{
		switch ($modelName)
		{
			case "people":
				$db    = $this->getDbo();
				$query = $db->getQuery(true)
					->select("id")
					->from("#__joomcrm_people")
					->where("first_name='" . $db->escape($import['first_name']) . "'")
					->where("last_name='" . $db->escape($import['last_name']) . "'")
					->where("email='" . $db->escape($import['email']) . "'")
					->where("published=1");

				return $db->setQuery($query)->loadResult();
			case "company":
				/*
				$db = $this->getDbo();
				$query = $db->getQuery();
				$query->clear()
					->select("id")
					->from("#__joomcrm_companies")
					->where("name='".$db->escape($import['name'])."'")
					->where("published=1");

				return $db->setQuery($query)->loadResult();
				*/
				return false;
			case "deal":
				return false;
			default:
				return false;
		}

	}


	/**
	 * Read the CSV file
	 *
	 * @param   string $file
	 * @param   string $tableName
	 *
	 * @return  mixed
	 */
	public function readCSVFile($file, $tableName = null)
	{
		ini_set("auto_detect_line_endings", "1");
		$data      = array();
		$line      = 1;
		$headers   = array();
		$i         = -2;
		$db        = $this->getDbo();
		$tableName =  \Joomla\CMS\Factory::getApplication()->input->get('import_type', $tableName);

		if ($tableName == null)
		{
			return false;
		}

		$table           = $db->getTableColumns("#__joomcrm_" . $tableName);
		$special_headers = array('company_id', 'company_name', 'stage_name', 'source_name', 'status_name', 'primary_contact_name', 'assignee_name', 'owner_name', 'type');

		switch ($tableName)
		{
			case "deals":
				$customType = "deal";
				break;
			case "people":
				$customType = "people";
				break;
			case "companies":
				$customType = "company";
				break;
			default:
				$customType = $table;
				break;
		}
		$customFields = JoomcrmHelperCustom::getCustomFields($customType);

		if (($handle = fopen($file, "r")) !== false)
		{
			while (($read = fgetcsv($handle, 1000, ",")) !== false)
			{
				$i++;
				$num = count($read);

				if ($line == 1)
				{
					$headers         = $read;
					$data['headers'] = $headers;
				}
				else
				{
					$line_data = array();

					for ($c = 0; $c < $num; $c++)
					{
						$header_name = array_key_exists($c, $headers) ? $headers[$c] : false;

						if ($header_name)
						{
							if (in_array($header_name, $special_headers))
							{
								switch ($header_name)
								{
									case "company_id":
										$model        = new JoomcrmModelCompany;
										$new_header   = "company_id";
										$company_name = $model->getCompanyName($read[$c]);
										if ($company_name != "")
										{
											$name = $company_name;
										}
										else
										{
											$name = "";
										}
										$special_data = $read[$c];

										break;
									case "company_name":

										$model      = new JoomcrmModelCompany;
										$company_id = $model->getCompanyList($read[$c]);
										if (count($company_id) > 0 && array_key_exists(0, $company_id))
										{
											$new_header = "company_id";
											$name       = $company_id[0]['id'];
										}
										else
										{
											$name       = $read[$c];
											$new_header = "company_name";
										}
										$special_data = $name;

										break;
									case "stage_name":
										$new_header = "stage_id";
										$stage_id   = JoomcrmHelperDeal::getStages($read[$c], true);
										if (count($stage_id))
										{
											$keys     = array_keys($stage_id);
											$stage_id = $keys[0];
										}
										$special_data = $stage_id;

										break;
									case "source_name":

										$new_header = "source_id";
										$source_id  = JoomcrmHelperDeal::getSources($read[$c]);
										if (count($source_id))
										{
											$keys      = array_keys($source_id);
											$source_id = $keys[0];
										}
										$special_data = $source_id;

										break;
									case "status_name":

										$new_header = "status_id";

										if ($tableName == 'people')
										{
											$status_id  = JoomcrmHelperPeople::getStatuses($read[$c], false, true);
										}
										else
										{
											$status_id  = JoomcrmHelperDeal::getStatuses($read[$c], false, true);
										}

										if (count($status_id))
										{
											$keys      = array_keys($status_id);
											$status_id = $keys[0];
										}
										$special_data = $status_id;

										break;
									case "primary_contact_name":

										$model   = new JoomcrmModelPeople;
										$contact = $model->searchForContact($read[$c]);
										if ($contact != "" && count($contact) > 0)
										{
											$new_header   = "primary_contact_id";
											$special_data = array('label' => $contact[0]->label, 'value' => $contact[0]->value);
										}
										else
										{
											$new_header   = "primary_contact_name";
											$special_data = $read[$c];
										}
										$special_data = $read[$c];

										break;
									case "assignee_name":
									case "owner_name":
										$new_header   = str_replace('_name', '_id', $header_name);
										$model        = new JoomcrmModelUser;
										$contact      = $model->searchForUser($read[$c]);
										$special_data = $contact ? $contact : JoomcrmHelperUsers::getUserId();

										break;
									case "type":

										$new_header   = "type";
										$special_data = array('dropdown' => ucwords(JoomcrmHelperDropdown::getContactTypes($read[$c])));
										$special_data = $read[$c];

										break;
								}

								$line_data[$new_header] = $special_data;

							}
							else
							{
								if ($pos = strrpos($header_name, "_custom"))
								{
									$keyName = strtolower(substr($header_name, 0, $pos));
									$key     = array_key_exists($keyName, $customFields) ? $customFields[$keyName] : false;
									if ($key)
									{
										$newFieldName = substr($header_name, $pos + 1) . "_" . $key->id;
										$special_data = array('id' => $key->id, 'label' => $key->name, 'values' => is_array($key->values) ? array_values($key->values) : $key->values, 'type' => "custom", 'custom_type' => $key->type, 'multiple_selections' => $key->multiple_selections);
										if ($key->multiple_selections == 1)
										{
											$exp          = explode(",", $read[$c]);
											$customValues = count($exp) > 0 ? $exp : array($read[$c]);
											// $special_data['value']=JoomcrmHelperCustom::getCustomValue($customType,$key->id,$customValues);
											$special_data = array_keys(array_intersect($key->values, $customValues));
											$arr          = array('-1' => '1');
											if (count($special_data) > 0)
											{
												foreach ($special_data as $key)
												{
													$arr[$key] = 1;
												}
											}
											$special_data = $arr;
										}
										else
										{
											if ($key->type == "picklist")
											{
												// $special_data['value']=JoomcrmHelperCustom::getCustomValue($customType,$key->id,$read[$c]);
												$special_data = array_keys(array_intersect($key->values, array($read[$c])));
												$special_data = array_key_exists(0, $special_data) ? $special_data[0] : "";
											}
											else
											{
												$special_data = $read[$c];
											}
										}

										$line_data[$newFieldName] = $special_data;
									}
								}
								else
								{
									if (array_key_exists($header_name, $table))
									{
										$line_data[$header_name] = $read[$c];
									}
								}
							}
						}
					}

					if (count($line_data) > 0)
					{
						$data[] = $line_data;
					}
				}

				$line++;
			}

			fclose($handle);
		}

		return $data;
	}
}

