<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

jimport('joomla.application.component.modellist');

use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Table\Table;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Categories\Categories;
class JoomcrmModelJoomcrm extends ListModel
{
	public $view = null;

	/**
	 *
	 *
	 * @return  void
	 */
	public function __construct()
	{
		parent::__construct();
		$this->getListLimits();
		$this->view =  \Joomla\CMS\Factory::getApplication()->input->get('view');

	}

	public function getRecentAll(){
	    return array_merge($this->getRecent('deals'), $this->getRecent('events'), $this->getRecent('goals'), $this->getRecent('notes'), $this->getRecent('people'));
    }

    public function getRecent($type)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true)
            ->select("created, DATE_FORMAT(created,{$db->quote('%d')}) AS day, COUNT(*) AS total_$type")
            ->from("#__joomcrm_{$type}")
            ->where('created > (NOW() - INTERVAL 30 DAY)')
            ->group('DATE_FORMAT(created, ' . $db->quote('%d') . ')')
            ->order('created ASC');
        $db->setQuery($query);

        try
        {
            $rows = $db->loadAssocList();
        }
        catch (RuntimeException $e)
        {
            $this->setError($e->getMessage());
            return false;
        }
        return $rows;
    }

	/**
	 * Saves the manually set order of records.
	 *
	 * @param  array   $pks    An array of primary key ids.
	 * @param  integer $order  +1 or -1
	 *
	 * @return  mixed
	 *
	 * @since   11.1
	 */
	public function saveorder($pks = null, $order = null)
	{
		// Initialise variables.
		$view =  Factory::getApplication()->input->get('view',"",'word');

		$table      = Table::getInstance($view, 'JoomcrmTable');
		$conditions = array();

		if (empty($pks))
		{

			return Factory::getApplication()->enqueueMessage( Text::_($this->text_prefix . '_ERROR_NO_ITEMS_SELECTED'),'warning');
		}


		// update ordering values
		foreach ($pks as $i => $pk)
		{
			$table->load((int) $pk);
			if ($table->ordering != $order[$i])
			{
				$table->ordering = $order[$i];

				if (!$table->store())
				{
					$this->setError($table->getError());

					return false;
				}

				// Remember to reorder within position and client_id
				$condition = $this->getReorderConditions($table);
				$found     = false;

				foreach ($conditions as $cond)
				{
					if ($cond[1] == $condition)
					{
						$found = true;
						break;
					}
				}

				if (!$found)
				{
					$key          = $table->getKeyName();
					$conditions[] = array($table->$key, $condition);
				}
			}
		}

		// Execute reorder for each category.
		foreach ($conditions as $cond)
		{
			$table->load($cond[0]);
			$table->reorder($cond[1]);
		}

		return true;
	}

	/**
	 * A protected method to get a set of ordering conditions.
	 *
	 * @param  object $table  A JTable object.
	 *
	 * @return  array  An array of conditions to add to ordering queries.
	 *
	 * @since   11.1
	 */
	protected function getReorderConditions($table)
	{
		return array();
	}

	/**
	 * Method to get a JPagination object for the data set.
	 *
	 * @return  JPagination  A JPagination object for the data set.
	 *
	 * @since   11.1
	 */
	public function getPagination()
	{
		// Get a storage key.
		$store = $this->getStoreId('getPagination');

		// Try to load the data from internal storage.
		if (isset($this->cache[$store]))
		{
			return $this->cache[$store];
		}

		// Create the pagination object.
		jimport('joomla.html.pagination');
		$page = new Pagination($this->getTotal(), $this->getState($this->view . '_limitstart'), $this->getState($this->view . '_limit'));

		// Add the object to the internal cache.
		$this->cache[$store] = $page;

		return $this->cache[$store];
	}

	public function getListLimits()
	{
		$app = Factory::getApplication();

		// Get pagination request variables
		$limit      = $app->getUserStateFromRequest($this->view . '_limit', 'limit', 10);
		$limitstart = $app->getUserStateFromRequest($this->view . '_limitstart', 'limitstart', 0);

		// In case limit has been changed, adjust it
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		$this->setState($this->view . '_limit', $limit);
		$this->setState($this->view . '_limitstart', $limitstart);
	}

	public function getTotal()
	{
		if (empty ($this->_total))
		{
			$query        = $this->__buildQuery();
			$this->_total = $this->_getListCount($query);
			$this->_total = $this->_total ? $this->_total : 0;
		}

		return $this->_total;
	}

	/**
	 * Method to adjust the ordering of a row.
	 *
	 * Returns NULL if the user did not have edit
	 * privileges for any of the selected primary keys.
	 *
	 * @param  integer $pks    The ID of the primary key to move.
	 * @param  integer $delta  Increment, usually +1 or -1
	 *
	 * @return  mixed  False on failure or error, true on success, null if the $pk is empty (no items selected).
	 *
	 * @since   11.1
	 */
	public function reorder($pks, $delta = 0)
	{
		// Initialise variables.
		$data   = Factory::getApplication()->input->get->getArray();
		$table  = Table::getInstance($data['view'], 'JoomcrmTable');
		$pks    = (array) $pks;
		$result = true;

		$allowed = true;

		foreach ($pks as $i => $pk)
		{
			$table->reset();

			if ($table->load($pk))
			{

				$where = array();
				$where = $this->getReorderConditions($table);

				if (!$table->move($delta, $where))
				{
					$this->setError($table->getError());
					unset($pks[$i]);
					$result = false;
				}

			}
			else
			{
				$this->setError($table->getError());
				unset($pks[$i]);
				$result = false;
			}
		}

		if ($allowed === false && empty($pks))
		{
			$result = null;
		}

		return $result;
	}

}
