<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;

class JoomcrmModelStages extends JoomcrmModelJoomcrm
{
	public function store($data = null, $updateOnly = false)
	{
		// Load Tables
		$row  = new JoomcrmTableStages;

		if (!$data)
		{
			$data = \Joomla\CMS\Factory::getApplication()->input->post->getArray();
		}

		if ($updateOnly)
		{
			$row->load($data['id']);
			if (!$row->getDbo()->getAffectedRows())
			{
				return false;
			}
		}

		// Date generation
		$date = date('Y-m-d H:i:s');

		if (!array_key_exists('id', $data))
		{
			$data['created'] = $date;
		}

		$data['modified'] = $date;

		// update only if specified. Do not set default value, it can overwrite wanted value
		if (!$updateOnly)
		{
			$data['won'] = array_key_exists('won', $data) ? 1 : 0;
		}

		// If we are setting this to not be the "won" stage,
		// then we need to double check that it's not the
		// last "won". If there isn't another "won" stage,
		// Then make this the "won".

		if (isset($data['won']) && $data['won'] === 0 && $this->isThereAWonStage() && isset($data['id']) && $this->isLastWonStage($data['id']))
		{
			$data['won'] = 1;
		}

		// Remove the # from the color string.
		if (isset($data['color']))
		{
			$data['color'] = str_replace('#', '', $data['color']);
		}

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		return true;
	}

	public function __buildQuery()
	{
		return $this->getDbo()->getQuery(true)->select("s.*")->from("#__joomcrm_stages AS s")
			   ->order($this->getState('Stages.filter_order') . ' ' . $this->getState('Stages.filter_order_Dir'));
	}

	/**
	 * Get list of stages
	 *
	 * @param   int $id  specific search id
	 *
	 * @return  mixed  $results results
	 */
	public function getStages($id = null)
	{
		// Database
		$db    = $this->getDbo();
		$query = $this->__buildQuery();

		if ($id)
		{
			$query->where("s.id=$id");
		}

		return $db->setQuery($query)->loadAssocList();
	}

	public function populateState($ordering = null, $direction = null)
	{
		// Get states
		$app              = Factory::getApplication();
		$filter_order     = $app->getUserStateFromRequest('Stages.filter_order', 'filter_order', 's.name');
		$filter_order_Dir = $app->getUserStateFromRequest('Stages.filter_order_Dir', 'filter_order_Dir', 'asc');

		// Set states
		$this->setState('Stages.filter_order', $filter_order);
		$this->setState('Stages.filter_order_Dir', $filter_order_Dir);
	}

	public function remove($id)
	{
		if ($this->isLastWonStage($id))
		{
            \Joomla\CMS\Factory::getApplication()->enqueueMessage("Cannot delete stage #{$id}. You are required to have at least 1 'won' stage.",'warning');
			return false;
		}

		$db    = $this->getDbo();
		$query = $db->getQuery(true)->delete('#__joomcrm_stages')->where('id = ' . $id);

		$db->setQuery($query)->execute();
	}

	protected function isThereAWonStage()
	{
		$db = $this->getDbo();
		$query = $db->getQuery(true)
		         ->select('id')
		         ->from('#__joomcrm_stages')
		         ->where('won = 1');

		return $db->setQuery($query)->loadResult();
	}

	protected function isLastWonStage($id)
	{
		$id = (int) $id;

		$db = $this->getDbo();
		$query = $db->getQuery(true)
		         ->select('id')
		         ->from('#__joomcrm_stages')
		         ->where('won = 1');

		$result = $db->setQuery($query)->loadObjectList();

		// If there's only 1 returned, and it's this one return true.
		return (count($result) === 1 && (int) $result[0]->id === $id);
	}

}
