<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;

class JoomcrmModelStatuses extends JoomcrmModelJoomcrm
{
	public function store($data = null, $updateOnly = false)
	{
		// Load Tables
		$row  = new JoomcrmTableStatuses;

		if (!$data)
		{
			$data = Factory::getApplication()->input->post->getArray();
		}

		if ($updateOnly)
		{
			$row->load($data['id']);
			if (!$row->getDbo()->getAffectedRows())
			{
				return false;
			}
		}

		// Date generation
		$date = date('Y-m-d H:i:s');

		if (!array_key_exists('id', $data))
		{
			$data['created'] = $date;
		}

		$data['modified'] = $date;

		// Remove the # from the color string.
		if (isset($data['color']))
		{
			$data['color'] = str_replace('#', '', $data['color']);
		}

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		return true;
	}

	public function __buildQuery()
	{
		return $this->getDbo()->getQuery(true)->select("s.*")->from("#__joomcrm_people_status AS s")
			   ->order($this->getState('Statuses.filter_order') . ' ' . $this->getState('Statuses.filter_order_Dir'));
	}

	/**
	 * Get list of stages
	 *
	 * @param   int $id  specific search id
	 *
	 * @return  mixed  $results results
	 */
	public function getStatuses($id = null)
	{
		$db    = $this->getDbo();
		$query = $this->__buildQuery();

		if ($id)
		{
			$query->where("s.id=$id");
		}

		return $db->setQuery($query)->loadAssocList();
	}

	public function populateState($ordering = null, $direction = null)
	{
		// Get states
		$app              = Factory::getApplication();
		$filter_order     = $app->getUserStateFromRequest('Statuses.filter_order', 'filter_order', 's.name');
		$filter_order_Dir = $app->getUserStateFromRequest('Statuses.filter_order_Dir', 'filter_order_Dir', 'asc');

		// Set states
		$this->setState('Statuses.filter_order', $filter_order);
		$this->setState('Statuses.filter_order_Dir', $filter_order_Dir);
	}

	public function remove($id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->delete('#__joomcrm_people_status')->where('id = ' . $id);

		$db->setQuery($query)->execute();
	}

}
