<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;

class JoomcrmModelTemplates extends JoomcrmModel
{
	public function store()
	{
		// Load Tables
		$row  = new JoomcrmTableTemplates;
		$data = Factory::getApplication()->input->post->getArray();

		// Date generation
		$date = date('Y-m-d H:i:s');
		if (!array_key_exists('id', $data))
		{
			$data['created'] = $date;
		}
		$data['modified'] = $date;

		// Assign default
		// TODO make this a function that updates the database table so there is only ONE default
		$data['default'] = (array_key_exists('default', $data) AND $data['default'] == 'on') ? 1 : 0;

		// Generate custom items for template
		$items = array();
		for ($i = 0; $i < count($data['items']); $i++)
		{
			$id      = $data['items'][$i];
			$name    = $data['names'][$i];
			$day     = $data['days'][$i];
			$type    = $data['types'][$i];
			$items[] = array('name' => $name, 'id' => $id, 'day' => $day, 'type' => $type);
		}
		unset($data['items']);
		unset($data['names']);
		unset($data['days']);
		unset($data['types']);

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		// Get newly inserted template id
		if (!array_key_exists('id', $data))
		{
			$template_id = $this->getDbo()->insertid();
		}
		else
		{
			$template_id = $data['id'];
		}

		
		$db = Factory::getDbo();
		$db->setQuery($db->getQuery(TRUE)
						->delete()
						->from('#__joomcrm_template_data')
						->where($db->qn('template_id') . ' = ' . $db->q($template_id))
					)->execute();

		// Loop through template events and bind the tables to update the database
		for ($i = 0; $i < count($items); $i++)
		{
			$temp_table          = new JoomcrmTableTemplatedata;
			$item                = $items[$i];

			$item['id'] 		 = null;
			$item['template_id'] = $template_id;
			$item['created'] 	 = $date;
			$item['modified'] 	 = $date;

			$temp_table->bind($item);
			$temp_table->check();
			$temp_table->store();
		}

		return true;
	}

	/**
	 * Get list of templates
	 *
	 * @param   int $id  specific search id
	 *
	 * @return  mixed  $results results
	 */
	public function getTemplates($id = null)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("t.*")
			->from("#__joomcrm_templates AS t")
			->order($this->getState('Templates.filter_order') . ' ' . $this->getState('Templates.filter_order_Dir'));

		if ($id)
		{
			$query->where('t.id = ' . (int) $id);
		}

		$results = $db->setQuery($query)->loadAssocList();

		// Left join essential data if we are searching for a specific template
		if ($id)
		{
			$query
				->clear()
				->select("t.*")
				->from("#__joomcrm_template_data AS t")
				->where('t.template_id = ' . (int) $id);

			$results[0]['data'] = $db->setQuery($query)->loadAssocList();
		}

		return $results;
	}

	public function populateState($ordering = null, $direction = null)
	{
		$filter_order     = $this->app->getUserStateFromRequest('Templates.filter_order', 'filter_order', 't.name');
		$filter_order_Dir = $this->app->getUserStateFromRequest('Templates.filter_order_Dir', 'filter_order_Dir', 'asc');

		// Set states
		$this->setState('Templates.filter_order', $filter_order);
		$this->setState('Templates.filter_order_Dir', $filter_order_Dir);
	}

	public function remove($id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->delete('#__joomcrm_templates')
			->where('id = ' . (int) $id);

		$db->setQuery($query)->execute();
	}

}
