<?php

/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;

class JoomcrmModelUsers extends JoomcrmModel
{

	public function store($data = null)
	{
		$db     = Factory::getDbo();
		$config = ComponentHelper::getParams('com_joomcrm');
		
		// Load Tables
		$row    = new JoomcrmTableUsers;



		$data = is_null($data) ? \Joomla\CMS\Factory::getApplication()->input->post->getArray() : $data;

		// Load stored user data
		if (isset($data['id']))
		{
			$row->load($data['id']);
		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onBeforeCRMUserSave', array(&$data));

		// Date generation
		$date = date('Y-m-d H:i:s');

		if (!array_key_exists('id', $data))
		{
			$data['created']     = $date;
			$data['time_zone']   = $config->get('timezone','America/New_York');
			$data['time_format'] = $config->get('time_format','g:i A');
		}

		// Generate team data
		$model  = new JoomcrmModelTeams;
		$teamId = array_key_exists('id', $data) ? $this->getTeamId($data['id']) : 0;

		// Assign user privileges
		$data['modified']   	= $date;
		$data['team_access']	= (array_key_exists('team_access', $data) && $data['team_access'] == 'on') ? 1 : 0;
		$data['admin']      	= (array_key_exists('admin', $data) && $data['admin'] == 'on') ? 1 : 0;
		$data['exports']   		= (array_key_exists('exports', $data) && $data['exports'] == 'on') ? 1 : 0;
		$data['can_delete'] 	= (array_key_exists('can_delete', $data) && $data['can_delete'] == 'on') ? 1 : 0;

		// Republish / register users
		if (array_key_exists('uid', $data) && $data['uid'] != "")
		{
			$db    = $this->getDbo();
			$query = $db->getQuery(true)
				->select("id")
				->from("#__joomcrm_users")
				->where("uid = " . (int) $data['uid']);

			$id = $db->setQuery($query)->loadResult();

			if ($id)
			{
				$data['id']        = $id;
				$data['published'] = 1;
			}

			// Set User API
			$primary_email = $db->setQuery(
								$db->getQuery(TRUE)
								->select('email')
								->from('#__users')
								->where('id = ' . $data['uid']))
							->loadResult();
			$data['api'] = md5($primary_email);

		}

		// When role type is changed, reset information about team
		if (isset($data['role_type']) && isset($row->role_type) && $row->role_type && $row->role_type != $data['role_type'])
		{
			$data['team_id'] = 0;
			$data['team_name'] = '';
		}

		if (array_key_exists('team_id', $data) && $data['team_id'] === "")
		{
			unset($data['team_id']);
		}

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store user table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		if (array_key_exists('role_type', $data) && $data['role_type'] == "manager")
		{
			$teamModel = new JoomcrmModelTeams;
			$teamName  = array_key_exists('team_name', $data) ? $data['team_name'] : "";
			$teamModel->createTeam($row->id, $teamName);
		}

		// If we are downgrading a users priviliges
		if (array_key_exists('manager_assignment', $data) && $data['manager_assignment'] != null && $data['manager_assignment'] != "")
		{
			$newTeamId = $this->getTeamId($data['manager_assignment']);
			$model->updateTeam($teamId, $newTeamId);
		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onAfterCRMUserSave', array(&$data));

		return true;
	}

	public function getUsers($id = null)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("u.*,ju.username,ju.email,ju.lastvisitDate as last_login,
						team_leader.first_name as leader_first_name,team_leader.last_name as leader_last_name,
						team.leader_id as leader_id,
						IF(team.name!='',team.name,CONCAT(team_leader.first_name,' ',team_leader.last_name)) AS team_name")
			->from("#__joomcrm_users AS u")
			->leftJoin("#__users AS ju ON ju.id = u.uid")
			->leftJoin("#__joomcrm_teams AS team ON team.team_id = u.team_id")
			->leftJoin("#__joomcrm_users AS team_leader ON team_leader.id = team.leader_id")
			->order($this->getState('Users.filter_order') . ' ' . $this->getState('Users.filter_order_Dir'));

		if ($id)
		{
			$query->where("u.id=$id");
		}

		$query->where("u.published=1");

		return $db->setQuery($query)->loadAssocList();
	}

	public function populateState($ordering = null, $direction = null)
	{
		$filter_order     = $this->app->getUserStateFromRequest('Users.filter_order', 'filter_order', 'u.last_name');
		$filter_order_Dir = $this->app->getUserStateFromRequest('Users.filter_order_Dir', 'filter_order_Dir', 'asc');

		// Set states
		$this->setState('Users.filter_order', $filter_order);
		$this->setState('Users.filter_order_Dir', $filter_order_Dir);
	}

	public function getJoomlaUsersToAdd()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("ju.id,ju.name,ju.username,ju.email,cu.id as cid,cu.published")
			->from("#__users AS ju")->leftJoin("#__joomcrm_users AS cu ON ju.id = cu.uid");

		$results = $db->setQuery($query)->loadAssocList();
		$users   = array();

		foreach ($results as $key => $user)
		{
			if (!$user['cid'] || $user['published'] == -1)
			{
				$name               = explode(" ", $user['name']);
				$user['first_name'] = array_key_exists(0, $name) ? $name[0] : "";
				$user['last_name']  = array_key_exists(1, $name) ? $name[1] : "";
				$users[$user['id']] = $user;
			}
		}

		return $users;
	}

	public function getJoomcrmUsers($idsOnly = false)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("u.id AS value,CONCAT(u.first_name,' ',u.last_name) AS label")
			->from("#__joomcrm_users AS u")
			->where("u.published=1");

		return $db->setQuery($query)->loadAssocList();
	}

	public function getJoomlaUsersToAddList($namesOnly = false)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("ju.id,ju.name,ju.username,cu.id as cid,cu.published")
			->from("#__users AS ju")->leftJoin("#__joomcrm_users AS cu ON ju.id = cu.uid");

		$results = $db->setQuery($query)->loadAssocList();
		$users   = array();

		foreach ($results as $key => $user)
		{
			if (!$user['cid'] || $user['published'] == -1)
			{
				if ($namesOnly)
				{
					$users[] = $user['name'];
				}
				else
				{
					$users[$user['id']] = $user['name'];
				}
			}
		}

		return $users;
	}

	// Return user team id
	public function getTeamId($user_id)
	{
		if ($user_id > 0)
		{
			$db    = $this->getDbo();
			$query = $db->getQuery(true)
				->select("team_id")
				->from("#__joomcrm_users")
				->where('id = ' . (int) $user_id);

			return $db->setQuery($query)->loadResult();
		}
		else
		{
			return 0;
		}
	}

	public function delete($ids)
	{

		// event trigger before user delete
		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onBeforeCRMUserDelete', array(&$ids));

		$db = Factory::getDbo();

		$query = $db->getQuery(true);

		$where = '';

		if (is_array($ids))
		{

			$where = "id IN(" . implode(',' ,$ids) . ")";
		}
		else
		{
			$where ="id = " . (int) $ids;
		}

		if(!empty($where))
			$query->where($where);

		$query->delete($db->quoteName('#__joomcrm_users'));

		$db->setQuery($query);

		$result = $db->execute();


		if ($result)
		{
			$dispatcher = Factory::getApplication();
			$dispatcher->triggerEvent('onAfterCRMUserDelete', array(&$ids));

			return true;
		}
		else
		{
			return false;
		}
	}



}
