<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;

$user = Factory::getApplication()->getIdentity();
$listOrder = $this->state->get('Sources.filter_order');
$listDirn = $this->state->get('Sources.filter_order_Dir');
$saveOrder = $listOrder == 's.ordering';
$canChange = $user->authorise('core.edit.state', 'com_joomcrm');
$orderKeys = array(0);

if ($saveOrder) {
    $saveOrderingUrl = 'index.php?option=com_joomcrm&task=ajax.saveOrderAjax&tmpl=component';
   HTMLHelper::_('sortablelist.sortable', 'itemList', 'adminForm', strtolower($listDirn), $saveOrderingUrl, false, true);
}

?>

<div class="row">
    <div class="col-md-2">
        <?php echo $this->sidebar; ?>
    </div>
    <div class="col-md-10">
        <?php echo $this->menu['help_info']->display(); ?>
        <div id="system-message-container"></div>
        <form action="<?php echo Route::_('index.php?option=com_joomcrm&view=sources'); ?>" method="post"
              name="adminForm" id="adminForm">
            <table class="table table-striped" id="itemList">
                <thead>
                <tr>
                    <th width="10%" class="hidden-phone">
                        <?php echo HTMLHelper::_('grid.sort', 'JGRID_HEADING_ORDERING', 's.ordering', $listDirn, $listOrder); ?>
                    </th>
                    <th width="1%">
                        <input type="checkbox" name="checkall-toggle" value=""
                               title="<?php echo Text::_('JGLOBAL_CHECK_ALL'); ?>" onclick="Joomla.checkAll(this)"/>
                    </th>
                    <th>
                        <?php echo HTMLHelper::_('grid.sort', 'COM_JOOMCRM_HEADER_SOURCE_NAME', 's.name', $listDirn, $listOrder); ?>
                    </th>
                    <th>
                        <?php echo HTMLHelper::_('grid.sort', 'COM_JOOMCRM_HEADER_SOURCE_COST', 's.cost', $listDirn, $listOrder); ?>
                    </th>
                    <th>
                        <?php echo HTMLHelper::_('grid.sort', 'COM_JOOMCRM_HEADER_SOURCE_TYPE', 's.type', $listDirn, $listOrder); ?>
                    </th>
                    <th width="1%">
                        <?php echo HTMLHelper::_('grid.sort', 'JGLOBAL_FIELD_ID_LABEL', 's.id', $listDirn, $listOrder); ?>
                    </th>
                </tr>
                </thead>
                <tfoot>
                <tr>
                    <td colspan="13">
                        <?php echo $this->pagination->getListFooter(); ?>
                    </td>
                </tr>
                </tfoot>
                <tbody>
                <?php if (count($this->sources)) {
                    $i = 0;
                    foreach ($this->sources as $key => $source) {
                        if (is_numeric($source['ordering']) && !in_array($source['ordering'], $orderKeys)) {
                            $orderKey = $source['ordering'];
                        } else {
                            $orderKey = (max($orderKeys) + 1);
                        }
                        $orderKeys[] = $orderKey;
                        ?>

                        <?php
                        $source['type'] = ($source['type'] == "per") ? "Per Lead/Deal" : "Flat Fee";
                        ?>

                        <tr class="row<?php echo $i % 2; ?>">
                            <td class="order nowrap center hidden-phone">
                                <?php
                                $iconClass = '';
                                if (!$canChange) {
                                    $iconClass = ' inactive';
                                } elseif (!$saveOrder) {
                                    $iconClass = ' inactive tip-top hasTooltip" title="' .HTMLHelper::tooltipText('JORDERINGDISABLED');
                                }
                                ?>
                                <span class="sortable-handler<?php echo $iconClass ?>">
												<i class="icon-menu"></i>
											</span>
                                <?php if ($canChange && $saveOrder) : ?>
                                    <input type="text" style="display:none" name="order[]" size="5"
                                           value="<?php echo $orderKey; ?>"/>
                                <?php endif; ?>
                            </td>
                            <td class="center">
                                <?php echo HTMLHelper::_('grid.id', $key, $source['id']); ?>
                            </td>
                            <td class="order"><?php echo \Joomla\CMS\HTML\HTMLHelper::_('link', 'index.php?option=com_joomcrm&task=Sources.edit&id=' . $source['id'], $source['name']); ?></td>
                            <td class="order"><?php echo JoomcrmHelperUsers::formatAmount($source['cost']); ?></td>
                            <td class="order"><?php echo $source['type']; ?></td>
                            <td class="order"><?php echo $source['id']; ?></td>
                        </tr>

                        <?php $i++;
                    }
                } ?>
                </tbody>
            </table>
            <div>
                <input type="hidden" name="task" value=""/>
                <input type="hidden" name="boxchecked" value="0"/>
                <input type="hidden" name="view" value="sources"/>
                <input type="hidden" name="filter_order" value="<?php echo $listOrder; ?>"/>
                <input type="hidden" name="filter_order_Dir" value="<?php echo $listDirn; ?>"/>
                <?php echo HTMLHelper::_('form.token'); ?>
            </div>
        </form>
    </div>
</div>


<script type="text/javascript">
    Joomla.orderTable = function () {
        table = document.getElementById("sortTable");
        direction = document.getElementById("directionTable");
        order = table.options[table.selectedIndex].value;
        if (order != '<?php echo $listOrder; ?>') {
            dirn = 'asc';
        } else {
            dirn = direction.options[direction.selectedIndex].value;
        }
        Joomla.tableOrdering(order, dirn, '');
    }
</script>
