<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;


use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\HTML\Helpers\Sidebar;

class JoomcrmView extends HtmlView
{
	public $menu;
	public $sidebar;

	public function __construct($config = array())
	{
		if (isset($config['name']))
		{
			$this->_name = $config['name'];
		}
		$config['template_path'] = JPATH_COMPONENT . '/view/' . $this->getName() . '/tmpl/';

		parent::__construct($config);
	}

	/**
	 * Method to get the view name
	 *
	 * The model name by default parsed using the classname, or it can be set
	 * by passing a $config['name'] in the class constructor
	 *
	 * @return  string  The name of the model
	 *
	 * @since   11.1
	 */
	public function getName()
	{
		if (empty($this->_name))
		{
			$class = get_class($this);
			$pos   = strpos($class, 'View');

			if ($pos === false)
			{
              Factory::getApplication()->enqueueMessage(Text::_('JLIB_APPLICATION_ERROR_VIEW_GET_NAME'),'error');
			}

			$tmp = strtolower(substr($class, ($pos + 4)));

			// Remove the format from the end of the string.
			$this->_name = preg_replace('/(raw|html|json)$/', '', $tmp);
		}

		return $this->_name;
	}

	/**
	 * Execute and display a template script. It also sets
	 * common variables that are used in most views.
	 *
	 * @param   string $tpl  The name of the template file to parse.
	 *
	 * @return  void
	 */
	public function display($tpl = null)
	{
		$this->menu = JoomcrmHelperMenu::getMenuModules();

        JoomcrmHelperMenu::loadNavi();
        $this->sidebar = Sidebar::render();

		parent::display($tpl);
	}

}
