<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_users
 *
 * @copyright   (C) 2007 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Helper\ContentHelper;
/**
 * User view class.
 *
 * @since  1.5
 */
class JoomcrmViewDealcustom extends HtmlView
{
	protected $form;

	protected $item;

	protected $grouplist;

	protected $groups;

	protected $state;

	/**
	 * Display the view
	 *
	 * @param   string  $tpl  The name of the template file to parse; automatically searches through the template paths.
	 *
	 * @return  void
	 *
	 * @since   1.5
	 */
	public function display($tpl = null)
	{
		$this->form      = $this->get('Form');
		$this->item      = $this->get('Item');
		$this->state     = $this->get('State');
		$this->menu = JoomcrmHelperMenu::getMenuModules();

		// Check for errors.
		if (count($errors = $this->get('Errors')))
		{
			throw new Exception(implode("\n", $errors), 500);
		}


		$this->addToolbar();
		parent::display($tpl);

	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return void
	 *
	 * @since   1.6
	 */
	protected function addToolbar()
	{
		Factory::getApplication()->input->set('hidemainmenu', true);

		$user      = Factory::getApplication()->getIdentity();
		$canDo     = ContentHelper::getActions('com_joomcrm');

		$isNew     = isset($this->item->id) && $this->item->id > 0 ? false : true;


		ToolbarHelper::title(
			'JoomCRM: Deal Customfields',
			'icon-' . ($isNew ? 'plus' :  'edit')
		);

		if ($canDo->get('core.admin'))
		{
			ToolbarHelper::apply('dealcustom.apply');
			ToolbarHelper::save('dealcustom.save');
		}

		if ($canDo->get('core.admin'))
		{
			ToolbarHelper::save2new('dealcustom.save2new');
		}

		if (empty($this->item->id))
		{
			ToolbarHelper::cancel('dealcustom.cancel');
		}
		else
		{
			ToolbarHelper::cancel('dealcustom.cancel', 'JTOOLBAR_CLOSE');
		}

		ToolbarHelper::divider();
		ToolbarHelper::help(null, null, 'https://www.joomboost.com/support/documentation/76-joomcrm.html?tmpl=component', 'com_joomcrm');
	}
}
