<?php

/**
 * @package     Joomla.Administrator
 * @subpackage  com_users
 *
 * @copyright   (C) 2007 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;




/** @var \Joomla\CMS\WebAsset\WebAssetManager $wa */
$wa = $this->document->getWebAssetManager();
$wa->useScript('table.columns')
    ->useScript('multiselect');

$listOrder  = $this->escape($this->state->get('list.ordering'));

$listDirn   = $this->escape($this->state->get('list.direction'));
$loggeduser = Factory::getApplication()->getIdentity();
$mfa        = PluginHelper::isEnabled('multifactorauth');
$user = Factory::getApplication()->getIdentity();

$saveOrder = $listOrder == 's.ordering';

if ($saveOrder && !empty($this->items)) {
	$saveOrderingUrl = 'index.php?option=com_joomcrm&task=documentcategories.saveOrderAjax&tmpl=component&' . Session::getFormToken() . '=1';
	HTMLHelper::_('draggablelist.draggable');
}
?>
<form action="<?php echo Route::_('index.php?option=com_joomcrm&view=documentcategories'); ?>" method="post" name="adminForm" id="adminForm">
    <div class="row">
        <div class="col-md-2">
         <?php echo $this->sidebar ?>
        </div>
        <div class="col-md-10">
            <div id="j-main-container" class="j-main-container">
                <?php
              // Search tools bar
                 echo LayoutHelper::render('joomla.searchtools.default', array('view' => $this));
                ?>
                <?php if (empty($this->items)) : ?>
                    <div class="alert alert-info">
                        <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
                        <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
                    </div>
                <?php else :?>
                    <table class="table table-striped">
                        <thead>
                        <tr>
                            <td class="w-1">
		                        <?php echo HTMLHelper::_('grid.checkall'); ?>
                            </td>
                            <th scope="col" class="w-1 text-center d-none d-md-table-cell">
		                        <?php echo HTMLHelper::_('searchtools.sort', '', 's.ordering', $listDirn, $listOrder, null, 'asc', 'JGRID_HEADING_ORDERING', 'icon-sort'); ?>
                            </th>
                            <th scope="col">
		                        <?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_HEADER_NAME', 's.name', $listDirn, $listOrder); ?>
                            </th>
                            <th scope="col">
				                <?php echo HTMLHelper::_('searchtools.sort', 'JGLOBAL_FIELD_ID_LABEL', 's.id', $listDirn, $listOrder); ?>
                            </th>
                        </tr>
                        </thead>
                        <tfoot>
                        <tr>
                            <td colspan="13">
                                <!-- pagination -->
                            </td>
                        </tr>
                        </tfoot>
                        <tbody<?php if ($saveOrder) :
		                    ?> class="js-draggable" data-url="<?php echo $saveOrderingUrl; ?>" data-direction="<?php echo strtolower($listDirn); ?>" data-nested="true"<?php
	                    endif; ?>>
                        <?php foreach ($this->items as $key => $item):
	                        $canChange        = $user->authorise('core.edit.state', 'com_joomcrm.documentcategory.' . $item->id);
                            ?>

                                <tr class="row<?php echo $key % 2; ?>" data-draggable-group="1">

                                    <td >
						                <?php echo HTMLHelper::_('grid.id', $key, $item->id); ?>
                                    </td>
                                    <td class="text-center d-none d-md-table-cell">
		                                <?php
		                                $iconClass = '';
		                                if (!$canChange) {
			                                $iconClass = ' inactive';
		                                } elseif (!$saveOrder) {
			                                $iconClass = ' inactive" title="' . Text::_('JORDERINGDISABLED');
		                                }
		                                ?>
                                        <span class="sortable-handler<?php echo $iconClass ?>">
                                        <span class="icon-ellipsis-v" aria-hidden="true"></span>
                                    </span>
		                                <?php if ($canChange && $saveOrder) :?>
                                            <input type="text" name="order[]" size="5" value="<?php echo $item->ordering; ?>" class="width-20 text-area-order hidden">
		                                <?php endif; ?>
                                    </td>
                                    <td>
                                        <a href="index.php?option=com_joomcrm&view=documentcategory&layout=edit&id=<?php echo $item->id ?>">
                                            <?php echo $item->name ?>
                                        </a>
                                    </td>

                                    <td><?php echo $item->id; ?></td>

                                </tr>

			                <?php endforeach; ?>
                        </tbody>
                    </table>

                    <?php // load the pagination. ?>
                    <?php echo $this->pagination->getListFooter(); ?>

                    <?php // Load the batch processing form if user is allowed ?>

                <?php endif; ?>

                <input type="hidden" name="task" value="">
                <input type="hidden" name="boxchecked" value="0">
                <?php echo HTMLHelper::_('form.token'); ?>
            </div>
        </div>
    </div>
</form>
