<?php

/**
 * @package     Joomla.Administrator
 * @subpackage  com_users
 *
 * @copyright   (C) 2009 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;


/** @var Joomla\CMS\WebAsset\WebAssetManager $wa */
$wa = $this->document->getWebAssetManager();
$wa->useScript('core')
	->useScript('multiselect');
	//->useScript('com_content.admin-articles-modal')

$input           = Factory::getApplication()->input;
$field           = $input->getCmd('field');
$listOrder       = $this->escape($this->state->get('list.ordering'));
$listDirn        = $this->escape($this->state->get('list.direction'));
$enabledStates   = array(0 => 'icon-check', 1 => 'icon-times');
$activatedStates = array(0 => 'icon-check', 1 => 'icon-times');
$userRequired    = (int) $input->get('required', 0, 'int');
//$onClick         = "window.parent.jSelectTeam(this);window.parent.Joomla.Modal.getCurrent().close()";
$app = Factory::getApplication();
$function  = $app->input->getCmd('function', 'jSelectTeam');
$onclick   = $this->escape($function);

$script = "

(() => {
  
  window.jSelectTeam = (id, name) => {
  

    if (window.parent.Joomla.Modal) {
      window.parent.Joomla.Modal.getCurrent().close();
    }

    return true;
  };

  document.addEventListener('DOMContentLoaded', () => {
    // Get the elements
    const elements = document.querySelectorAll('.select-link');

    for (let i = 0, l = elements.length; l > i; i += 1) {
      // Listen for click event
      elements[i].addEventListener('click', event => {
        event.preventDefault();
        const {
          target
        } = event;
        const functionName = target.getAttribute('data-function');

        if (functionName === 'jSelectArticle') {
          // Used in xtd_contacts
          window[functionName](target.getAttribute('data-id'), target.getAttribute('data-name'));
        } else {
          // Used in com_menus
          window.parent[functionName](target.getAttribute('data-id'), target.getAttribute('data-name'));
        }

        if (window.parent.Joomla.Modal) {
          window.parent.Joomla.Modal.getCurrent().close();
        }
      });
    }
  });
})();

";
$wa->addInlineScript($script);
?>
<div class="container-popup">
    <form action="<?php echo Route::_('index.php?option=com_joomcrm&view=manageteams&layout=modal&function=' . $function . '&' . Session::getFormToken() . 'tmpl=component'); ?>" method="post" name="adminForm" id="adminForm">

        <?php if (empty($this->items)) : ?>
            <div class="alert alert-info">
                <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
                <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
            </div>
        <?php else : ?>
            <table class="table table-sm">
                <thead>
                <tr>
                    <td class="w-1 text-center">
				        <?php echo HTMLHelper::_('grid.checkall'); ?>
                    </td>
                    <th scope="col">
				        <?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_USERS_HEADER_TEAM', 'u.last_name', $listDirn, $listOrder); ?>
                    </th>
                    <th scope="col">
				        <?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_TEAMS_HEADER_MEMBERS', 'u.team_id', $listDirn, $listOrder); ?>
                    </th>
                    <th scope="col">
				        <?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_ID', 'ju.email', $listDirn, $listOrder); ?>
                    </th>

                </tr>
                </thead>
                <tfoot>
                <tr>
                    <td colspan="13">
                        <!-- pagination -->
                    </td>
                </tr>
                </tfoot>
                <tbody>

		        <?php foreach ($this->items as $key => $item):

                    ?>

                    <tr class="row<?php echo $key % 2; ?>">
                        <td class="center">
					        <?php echo HTMLHelper::_('grid.id', $key, $item->team_id); ?>
                        </td>

                        <td class="order">
	                        <?php $attribs = 'data-function="' . $this->escape($onclick) . '"'
		                        . ' data-id="' . $item->team_id . '"'
		                        . ' data-name="' . $this->escape($item->name) . '"';
	                        ?>

                            <a class="pointer select-link" href="javascript:void(0)" <?php echo $attribs; ?>>
		                        <?php echo $this->escape($item->team_name); ?>
                            </a>
                        </td>

                        <td class="order">
		                    <?php if (is_array($item->members)) : ?>
			                    <?php foreach ($item->members as $member) : ?>
                                    <a class="btn btn-light border rounded btn-sm" href="index.php?option=com_joomcrm&view=manageuser&layout=edit&id=<?php echo $member['id'] ?>">
                                        <i class="icon-user" <?php echo $member['color'] ? 'style="color:' . $member['color'] . '"' : ''; ?>></i>

					                    <?php echo  $member['name'] ?>
                                    </a>
			                    <?php endforeach; ?>
		                    <?php endif; ?>
                        </td>
                        <td><?php echo $item->team_id; ?></td>

                    </tr>

		        <?php endforeach; ?>

                </tbody>
            </table>


            <?php // load the pagination. ?>
            <?php echo $this->pagination->getListFooter(); ?>

        <?php endif; ?>
        <input type="hidden" name="task" value="">
        <input type="hidden" name="field" value="<?php echo $this->escape($field); ?>">
        <input type="hidden" name="boxchecked" value="0">
        <?php echo HTMLHelper::_('form.token'); ?>
    </form>
</div>
