<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_users
 *
 * @copyright   (C) 2007 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Helper\ContentHelper;
/**
 * User view class.
 *
 * @since  1.5
 */
class JoomcrmViewManageuser extends HtmlView
{
	protected $form;

	protected $item;

	protected $grouplist;

	protected $groups;

	protected $state;

	/**
	 * Display the view
	 *
	 * @param   string  $tpl  The name of the template file to parse; automatically searches through the template paths.
	 *
	 * @return  void
	 *
	 * @since   1.5
	 */
	public function display($tpl = null)
	{
		$this->form      = $this->get('Form');
		$this->item      = $this->get('Item');
		$this->state     = $this->get('State');


		// Check for errors.
		if (count($errors = $this->get('Errors')))
		{
			throw new Exception(implode("\n", $errors), 500);
		}


		$this->addToolbar();
		parent::display($tpl);

	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return void
	 *
	 * @since   1.6
	 */
	protected function addToolbar()
	{
		Factory::getApplication()->input->set('hidemainmenu', true);

		$user      = Factory::getApplication()->getIdentity();
		$canDo     = ContentHelper::getActions('com_joomcrm');

		$isNew     = ($this->item->id == 0);
		$isProfile = $this->item->id == $user->id;

		ToolbarHelper::title(
			Text::_(
				$isNew ? 'COM_JOOMCRM_VIEW_NEW_USER_TITLE' : ($isProfile ? 'COM_JOOMCRM_VIEW_EDIT_PROFILE_TITLE' : 'COM_JOOMCRM_VIEW_EDIT_USER_TITLE')
			),
			'user ' . ($isNew ? 'user-add' : ($isProfile ? 'user-profile' : 'user-edit'))
		);

		if ($canDo->get('core.admin'))
		{
			ToolbarHelper::apply('manageuser.apply');
			ToolbarHelper::save('manageuser.save');
		}

		if ($canDo->get('core.admin'))
		{
			ToolbarHelper::save2new('manageuser.save2new');
		}

		if (empty($this->item->id))
		{
			ToolbarHelper::cancel('manageuser.cancel');
		}
		else
		{
			ToolbarHelper::cancel('manageuser.cancel', 'JTOOLBAR_CLOSE');
		}

		ToolbarHelper::divider();
		ToolbarHelper::help('JHELP_USERS_USER_MANAGER_EDIT');
	}
}
