<?php

/**
 * @package         Joomla.Administrator
 * @subpackage      com_users
 *
 * @copyright   (C) 2007 Open Source Matters, Inc. <https://www.joomla.org>
 * @license         GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;


use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;


/** @var \Joomla\Component\Users\Administrator\View\Users\HtmlView $this */


/** @var \Joomla\CMS\WebAsset\WebAssetManager $wa */
$wa = $this->document->getWebAssetManager();
$wa->useScript('table.columns')
	->useScript('multiselect');

HTMLHelper::_('bootstrap.tooltip','.hasTooltip');

$listOrder  = $this->escape($this->state->get('list.ordering'));
$listDirn   = $this->escape($this->state->get('list.direction'));
$loggeduser = Factory::getApplication()->getIdentity();
$mfa        = PluginHelper::isEnabled('multifactorauth');

?>
<form action="<?php echo Route::_('index.php?option=com_joomcrm&view=manageusers'); ?>" method="post" name="adminForm"
      id="adminForm">
    <div class="row">
        <div class="col-md-2">
			<?php echo $this->sidebar ?>
        </div>
        <div class="col-md-10">
            <div id="j-main-container" class="j-main-container">
				<?php
				// Search tools bar
				echo LayoutHelper::render('joomla.searchtools.default', array('view' => $this));
				?>
				<?php if (empty($this->items)) : ?>
                    <div class="alert alert-info">
                        <span class="icon-info-circle" aria-hidden="true"></span><span
                                class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
						<?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
                    </div>
				<?php else : ?>
                    <table class="table table-striped">
                        <thead>
                        <tr>
                            <td class="w-1 text-center">
								<?php echo HTMLHelper::_('grid.checkall'); ?>
                            </td>
                            <th scope="col">
								<?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_USERS_HEADER_NAME', 'u.last_name', $listDirn, $listOrder); ?>
                            </th>
                            <th scope="col">
								<?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_USERS_HEADER_USERNAME', 'ju.username', $listDirn, $listOrder); ?>
                            </th>
                            <th scope="col">
								<?php echo Text::_('COM_JOOMCRM_USERS_HEADER_TEAM'); ?>
                            </th>
                            <th scope="col">
								<?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_USERS_HEADER_EMAIL', 'ju.email', $listDirn, $listOrder); ?>
                            </th>
                            <th scope="col">
								<?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_USERS_HEADER_ROLE', 'u.role_type', $listDirn, $listOrder); ?>
                            </th>
                            <th scope="col">
								<?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_USERS_HEADER_LOGIN', 'ju.lastvisitDate', $listDirn, $listOrder); ?>
                            </th>
                            <th scope="col">
								<?php echo HTMLHelper::_('searchtools.sort', 'COM_JOOMCRM_USERS_HEADER_ID', 'u.id', $listDirn, $listOrder); ?>
                            </th>
                            <th scope="col">
								<?php echo Text::_('COM_JOOMCRM_HISTORY') ?>
                            </th>
                        </tr>
                        </thead>
                        <tfoot>
                        <tr>
                            <td colspan="13">
                                <!-- pagination -->
                            </td>
                        </tr>
                        </tfoot>
                        <tbody>

						<?php foreach ($this->items as $key => $user): ?>

                            <tr class="row<?php echo $key % 2; ?>">
                                <td class="center">
									<?php echo HTMLHelper::_('grid.id', $key, $user->id); ?>
                                </td>
                                <td class="order">
                                    <span class="fas fa-circle me-2" style="color: <?php echo $user->color ?>"></span><a
                                            href="index.php?option=com_joomcrm&view=manageuser&layout=edit&id=<?php echo $user->id ?>">
										<?php echo $user->first_name . ' ' . $user->last_name ?>
                                    </a>
                                </td>
                                <td class="order"><?php echo $user->username; ?></td>
                                <td class="order <?php echo $user->team_id ? '' : 'text-center' ?>">
									<?php
									if ($user->team_id)
									{
										echo $user->team_name . Text::_("COM_JOOMCRM_TEAM_APPEND");
									}
									else
									{
										echo '-';
									}
									?>

                                </td>
                                <td class="order"><?php echo $user->email; ?></td>
                                <td class="order"><small class="bg-light border rounded p-2"><?php echo ucwords($user->role_type); ?></small></td>

                                <td class="order">
									<?php
									if (empty($user->last_login))
									{
										echo ' - ';
									}
									else
									{
										echo HTMLHelper::_('date', $user->last_login, Text::_('DATE_FORMAT_LC6'));
									}
									?>


                                </td>
                                <td class="order"><?php echo $user->id; ?></td>
                                <td>
                                    <button
                                            class="btn btn-info btn-sm hasTooltip"
                                            title="<?php echo  Text::_('COM_JOOMCRM_LOGIN_HISTORY') ?>"
                                            type="button"
                                            data-bs-toggle="modal"
                                            data-bs-target="#ModalSelectuser<?php echo $user->id ?>">
                                        <i class="fas fa-user-lock"></i></button>
                                    <button
                                            class="btn btn-info btn-sm hasTooltip"
                                            title="<?php echo  Text::_('COM_JOOMCRM_ACTIVITIES') ?>"
                                            type="button"
                                            data-bs-toggle="modal"
                                            data-bs-target="#ModalSelectactivities<?php echo $user->id ?>">
                                        <i class="fas fa-history"></i></button>
									<?php echo HTMLHelper::_(
										'bootstrap.renderModal',
										'ModalSelectuser' . $user->id,
										array(
											'title'      => Text::_('COM_JOOMCRM_LOGIN_HISTORY'),
											'url'        => 'index.php?option=com_joomcrm&view=loginhistory&layout=modal&user_id=' . $user->id . '&tmpl=component',
											'height'     => '400px',
											'width'      => '800px',
											'bodyHeight' => 70,
											'modalWidth' => 80,
											'footer'     => '<button type="button" class="btn btn-secondary" data-bs-dismiss="modal">'
												. Text::_('JLIB_HTML_BEHAVIOR_CLOSE') . '</button>',
										)
									); ?>
									<?php echo HTMLHelper::_(
										'bootstrap.renderModal',
										'ModalSelectactivities' . $user->id,
										array(
											'title'      => Text::_('COM_JOOMCRM_ACTIVITIES'),
											'url'        => 'index.php?option=com_joomcrm&view=activities&layout=modal&user_id=' . $user->id . '&tmpl=component',
											'height'     => '400px',
											'width'      => '800px',
											'bodyHeight' => 70,
											'modalWidth' => 80,
											'footer'     => '<button type="button" class="btn btn-secondary" data-bs-dismiss="modal">'
												. Text::_('JLIB_HTML_BEHAVIOR_CLOSE') . '</button>',
										)
									); ?>
                                </td>
                            </tr>

						<?php endforeach; ?>

                        </tbody>
                    </table>

					<?php // load the pagination. ?>
					<?php echo $this->pagination->getListFooter(); ?>

					<?php // Load the batch processing form if user is allowed ?>

				<?php endif; ?>

                <input type="hidden" name="task" value="">
                <input type="hidden" name="boxchecked" value="0">
				<?php echo HTMLHelper::_('form.token'); ?>
            </div>
        </div>
    </div>
</form>
