<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Plugin\PluginHelper;


class com_joomcrmInstallerScript
{
	/**
	 * Install.
	 *
	 * @param    $parent
	 */
	public function install($parent)
	{
		$this->toInstall($parent);
	}

	/**
	 * Update.
	 *
	 * @param    $parent
	 */
	public function update($parent)
	{
		// update extensions
		$this->toInstall($parent, true);

		// remove old files
		$this->updateFiles();

	}

	protected function toInstall($parent, $update = false)
	{
		$manifest    = $parent->getManifest();
		$parent      = $parent->getParent();
		$source      = $parent->getPath("source");
		$module_attr = $manifest->modules->attributes();
		$module_path = isset($module_attr['folder']) ? '/' . $module_attr['folder'] : '';
		$plugin_attr = $manifest->plugins->attributes();
		$plugin_path = isset($plugin_attr['folder']) ? '/' . $plugin_attr['folder'] : '';

		$installer = new Installer();

		// Install modules.
		foreach ($manifest->modules->module as $module)
		{
			$attributes = $module->attributes();
			$path       = $source . $module_path . '/' . $attributes['name'];
			$installer->install($path);
		}

		// Install plugins.
		$plugins = array();
		$db      = Factory::getDbo();

		foreach ($manifest->plugins->plugin as $plugin)
		{
			$attributes = $plugin->attributes();
			$path       = $source . $plugin_path . '/' . $attributes['group'] . '/' . $attributes['name'];

			if ($attributes['enable'])
			{
				$plugins[] = $db->quote($attributes['name']);
			}

			$installer->install($path);
		}

		// Public plugins.
		if (!$update && count($plugins))
		{
			$query = 'UPDATE #__extensions'
				. ' SET enabled = 1'
				. ' WHERE element IN (' . implode(', ', $plugins) . ') AND type = \'plugin\' AND enabled = 0';
			$db->setQuery($query);
			$db->execute();
		}
	}

	public function updateFiles()
	{
		// remove non use files
		$filesToRemove = [
			'/administrator/components/com_joomcrm/model/dealcustom.php',
			'/administrator/components/com_joomcrm/model/peoplecustom.php',
			'/administrator/components/com_joomcrm/model/companycustom.php',
			'/administrator/components/com_joomcrm/model/eventcategories.php',
			'/administrator/components/com_joomcrm/model/companycategories.php',
			'/administrator/components/com_joomcrm/model/documentcategories.php',
			'/administrator/components/com_joomcrm/model/companycategories.php',
			'/administrator/components/com_joomcrm/controller/dealcustom.php',
			'/administrator/components/com_joomcrm/controller/peoplecustom.php',
			'/administrator/components/com_joomcrm/controller/companycustom.php',
			'/administrator/components/com_joomcrm/controller/eventcategories.php',
			'/administrator/components/com_joomcrm/controller/companycategories.php',
			'/administrator/components/com_joomcrm/controller/documentcategories.php',
			'/administrator/components/com_joomcrm/controller/companycategories.php',

		];


		// folders to remove
		$foldersToRemove = [
			'/administrator/components/com_joomcrm/view/dealcustom',
			'/administrator/components/com_joomcrm/view/peoplecustom',
			'/administrator/components/com_joomcrm/view/companycustom',
			'/administrator/components/com_joomcrm/view/eventcategories',
			'/administrator/components/com_joomcrm/view/companycategories',
			'/administrator/components/com_joomcrm/view/documentcategories',
			'/administrator/components/com_joomcrm/view/companycategories',
		];


		// remove files
		foreach ($filesToRemove as $file){

			$filePath = JPATH_ROOT.$file;

			if(file_exists($filePath))
				\Joomla\Filesystem\File::delete($filePath);

		}

		// remove folders
		foreach ($foldersToRemove as $folder){

			$folderPath = JPATH_ROOT.$folder;

			if(is_dir($folderPath))
				\Joomla\Filesystem\Folder::delete($folderPath);

		}


	}


}
