<?php
/**
* JoomCRM Component - Best CRM for Joomla!
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;


class modJoomcrmListHelper
{

	public static function getList($params)
	{
		switch($params->get('list_type'))
		{
			case 'total_deal':
			default:
				$list = self::getTotalDeals();
			break;

			case 'best_closing':
				$list = self::getBestClosings();
			break;

			case 'upcoming_tasks':
				$list = self::getUpcomingTasks();
			break;
		}
	
	return $list;

	}


	public static function getTotalDeals()
	{
		$db = Factory::getDbo();

		return $db->setQuery(
					$db->getQuery(TRUE)
					->select('CONCAT(u.first_name," " , u.last_name) AS name')
					->select('SUM(d.amount) as value')
					->from('#__joomcrm_users AS u')
					->join('left','#__joomcrm_deals AS d ON d.owner_id = u.id')
					->join('left','#__joomcrm_stages AS s ON s.id = d.stage_id')
					->where('d.published = 1')
					->where('s.won = 1')
					->group('d.owner_id')
					->order('value DESC'), 0, 10
				)->loadObjectList();

	}

	public static function getBestClosings()
	{
		$db = Factory::getDbo();

		return $db->setQuery(
					$db->getQuery(TRUE)
					->select('CONCAT(u.first_name, " ", u.last_name) AS name')
					->select('d.id')
					->select('SUM(CASE WHEN s.won = 1 THEN d.published ELSE d.published/(d.published+1) END) AS value')
					->from('#__joomcrm_users AS u')
					->join('left','#__joomcrm_deals AS d ON d.owner_id = u.id')
					->join('left','#__joomcrm_stages AS s ON s.id = d.stage_id')
					->where('d.published = 1')
					->group('d.owner_id')
					->order('value DESC'), 0, 10
				)->loadObjectList();
	}

	public static function getUpcomingTasks()
	{
		$db = Factory::getDbo();
		$user = Factory::getApplication()->getIdentity();

		return $db->setQuery(
				$db->getQuery(TRUE)
				->select('e.name AS name')
				->select('e.due_date as value')
				->from('#__joomcrm_events AS e')
				->join('left','#__joomcrm_users AS u ON u.id = e.owner_id')
				->where('e.due_date > NOW()')
				->where('u.uid = ' . $user->id)
				->order('e.due_date'), 0, 10
			)->loadObjectList();
	}

}