<?php
/**
 * JoomCRM Component - Best CRM for Joomla!
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Application\ApplicationHelper;
use Joomla\CMS\User\UserHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Table\Table;
jimport( 'joomla.plugin.plugin' );

/**
 *
 * @package		Joomla
 * @subpackage	System
 */
class plgJoomcrmUserCreate extends CMSPlugin
{

	function onAfterPersonSave(&$row)
	{


		$lang = Factory::getLanguage();
		$extension = 'com_users';
		$lang->load($extension);

		if($row->status=='created' && $row->email!='') 
		{
			$config = Factory::getConfig();
			$db		= Factory::getDbo();

			$params	= ComponentHelper::getParams('com_users');
			$system	= $params->get('new_usertype', 2);

			$useractivation = $params->get('useractivation');
			$sendpassword = $params->get('sendpassword', 1);

			$data['groups'][] = $system;
			$data['name'] = $row->first_name.' '.$row->last_name;
			$data['email'] = $row->email;
			$data['username'] = $row->email;

			$user = new \Joomla\CMS\User\User(0);

			// Check if the user needs to activate their account.
			if (($useractivation == 1) || ($useractivation == 2)) {
				$data['activation'] = ApplicationHelper::getHash(UserHelper::genRandomPassword());
				$data['block'] = 1;
			}


			// Bind the data.
			if (!$user->bind($data)) {
                Factory::getApplication()->enqueueMessage(Text::sprintf('COM_USERS_REGISTRATION_BIND_FAILED', $user->getError()),'error');
				return false;
			}

			// Load the users plugin group.
			PluginHelper::importPlugin('user');

			// Store the data.
			if (!$user->save()) {
              Factory::getApplication()->enqueueMessage(Text::sprintf('COM_USERS_REGISTRATION_SAVE_FAILED', $user->getError()),"error");
				return false;
			}
        

			$person = Table::getInstance('People','JoomcrmTable');
			$person->load($row->id);
			$person->uid = $user->id;
			$person->store();

			// Compile the notification mail values.
			$data = $user->getProperties();
			$data['fromname']	= $config->get('fromname');
			$data['mailfrom']	= $config->get('mailfrom');
			$data['sitename']	= $config->get('sitename');
			$data['siteurl']	= Uri::root();

			// Handle account activation/confirmation emails.
			if ($useractivation == 2)
			{
				// Set the link to confirm the user email.
				$uri = Uri::getInstance();
				$base = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));
				$data['activate'] = $base.Route::_('index.php?option=com_users&task=registration.activate&token='.$data['activation'], false);

				$emailSubject	= Text::sprintf(
					'COM_USERS_EMAIL_ACCOUNT_DETAILS',
					$data['name'],
					$data['sitename']
				);

				if ($sendpassword)
				{
					$emailBody = Text::sprintf(
						'COM_USERS_EMAIL_REGISTERED_WITH_ADMIN_ACTIVATION_BODY',
						$data['name'],
						$data['sitename'],
						$data['siteurl'].'index.php?option=com_users&task=registration.activate&token='.$data['activation'],
						$data['siteurl'],
						$data['username'],
						$data['password_clear']
					);
				}
				else
				{
					$emailBody = Text::sprintf(
						'COM_USERS_EMAIL_REGISTERED_WITH_ADMIN_ACTIVATION_BODY_NOPW',
						$data['name'],
						$data['sitename'],
						$data['siteurl'].'index.php?option=com_users&task=registration.activate&token='.$data['activation'],
						$data['siteurl'],
						$data['username']
					);
				}
			}
			elseif ($useractivation == 1)
			{
				// Set the link to activate the user account.
				$uri = Uri::getInstance();
				$base = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));
				$data['activate'] = $base.Route::_('index.php?option=com_users&task=registration.activate&token='.$data['activation'], false);

				$emailSubject	= Text::sprintf(
					'COM_USERS_EMAIL_ACCOUNT_DETAILS',
					$data['name'],
					$data['sitename']
				);

				if ($sendpassword)
				{
					$emailBody = Text::sprintf(
						'COM_USERS_EMAIL_REGISTERED_WITH_ACTIVATION_BODY',
						$data['name'],
						$data['sitename'],
						$data['siteurl'].'index.php?option=com_users&task=registration.activate&token='.$data['activation'],
						$data['siteurl'],
						$data['username'],
						$data['password_clear']
					);
				}
				else
				{
					$emailBody = Text::sprintf(
						'COM_USERS_EMAIL_REGISTERED_WITH_ACTIVATION_BODY_NOPW',
						$data['name'],
						$data['sitename'],
						$data['siteurl'].'index.php?option=com_users&task=registration.activate&token='.$data['activation'],
						$data['siteurl'],
						$data['username']
					);
				}
			}
			else
			{

				$emailSubject	= Text::sprintf(
					'COM_USERS_EMAIL_ACCOUNT_DETAILS',
					$data['name'],
					$data['sitename']
				);

				$emailBody = Text::sprintf(
					'COM_USERS_EMAIL_REGISTERED_BODY',
					$data['name'],
					$data['sitename'],
					$data['siteurl']
				);
			}

            
			// Send the registration email.
			if($this->params->get('send_emails',0)) // send email only if enabled in plugin
				Factory::getMailer()->sendMail($data['mailfrom'], $data['fromname'], $data['email'], $emailSubject, $emailBody);
			
		} 

	return true;

	}
}