<?php
/**
 * @package        RSTickets! Pro
 *
 * @copyright  (c) 2010 - 2016 RSJoomla!
 * @link           https://www.rsjoomla.com
 * @license        GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\Component\Fields\Administrator\Helper\FieldsHelper;

defined('_JEXEC') or die('Restricted access');

class plgUserJoomcrm extends CMSPlugin
{
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
	}

	// from joomla user to Joomcrm person
	public function onUserAfterSave($user, $isnew, $success, $msg): void
	{
		$context = Factory::getApplication()->getInput()->get('option');
		$task    = Factory::getApplication()->getInput()->get('task');

		if ($task === 'request')
		{
			return;
		}

		if ($task === 'complete')
		{
			return;
		}

		$jUser = Factory::getUser();

		// processed only on update
		if (!$jUser->id || $isnew)
			return;

		// object of saved user
		$savedUser = Factory::getUser($user['id']);

		// allowed user group
		if(!$this->checkAllowedUserGroups($savedUser))
			return;

		// get person details
		$person = $this->getPerson($user['id']);

		// person not found no need to continue
		if (!$person)
			return;

		// check if user email exists
		if(!isset($user['email']))
			return;

		// person found update its details
		$this->updatePerson(['email' => $user['email'], 'id' => $person->id]);


	}

	// from person to joomla user
	public function onAfterPersonSave($row)
	{


		// no need to continue if person not attached to joomla user or new person
		if (!$row->id || !$row->uid)
			return;

		// get joomla user object
		$user = Factory::getUser($row->uid);


		// allowed user group
		if(!$this->checkAllowedUserGroups($user))
			return;

		// update email
		if(!empty($row->email) && $user->email == $row->email)
			$user->email = $row->email;

		// update name
		if(!empty($row->first_name))
			$user->name = $row->first_name.' '.$row->last_name;



		// update joomla user
		$user->save();

		// update joomla user custom fields
		$this->updateUserCF($row);

		// update profile
		// todo: profile integration to be completed
		//$this->updateProfile($row);

	}

	/*
	 * Update custom fields of joomla user using data from person
	 * person fields are mapped with custom fields true plugin params
	 */
	private function updateUserCF($person){

		// check if integration enabled
		if(!$this->params->get('enable_cf_integration',0))
			return;


		// no need to continue if person has no user id
		if(!$person->uid)
			return;

		// get joomla user object
		$user = Factory::getUser($person->uid);


		// Loading the fields
		$fields = FieldsHelper::getFields('com_users.user');


		// no need to continue if users component has not custom fields yet
		if (!$fields) {
			return;
		}

		// build mapping person fields
		$mappedFields = $this->mapPersonFields();


		// no need to continue if no field mapped
		if (!$mappedFields) {
			return;
		}

		// Loading the model

		/** @var \Joomla\Component\Fields\Administrator\Model\FieldModel $model */
		$model = Factory::getApplication()->bootComponent('com_fields')->getMVCFactory()
			->createModel('Field', 'Administrator', ['ignore_request' => true]);

		// Loop over the fields
		foreach ($fields as $field) {

			// check if field is mapped, skip if not mapped
			if(!in_array($field->id,$mappedFields))
				continue;


			$value = $this->getPersonFieldValue($field->id,$mappedFields,$person);


			// If no value set (empty) remove value from database
			if (\is_array($value) ? !\count($value) : !\strlen($value)) {
				$value = null;
			}

			// JSON encode value for complex fields
			if (\is_array($value) && (\count($value, COUNT_NORMAL) !== \count($value, COUNT_RECURSIVE) || !\count(array_filter(array_keys($value), 'is_numeric')))) {
				$value = json_encode($value);
			}

			// Setting the value for the field and the item
			$model->setFieldValue($field->id, $user->id, $value);
		}
	}


	private function getPersonFieldValue($fieldId,$mappedFields,$person){



		// get value from person
		$personColumnName = array_search($fieldId,$mappedFields);

		// get source name value
		if($personColumnName == 'source_name'){

			// source id
			return $this->getSourceNameFromId($person->source_id);

		}

		// get company name value
		if($personColumnName == 'company_name'){

			// company id
			return $this->getCompanyNameFromId($person->company_id);

		}


		// for other fields
		return $person->$personColumnName;



	}


	public function getSourceNameFromId($sourceId){

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("name")->from("#__joomcrm_sources");
		$query->where("id = " . $db->q($sourceId));

		return $db->setQuery($query)->loadResult();


	}

	public function getCompanyNameFromId($companyId){

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("name")->from("#__joomcrm_companies");
		$query->where("id = " . $db->q($companyId));

		return $db->setQuery($query)->loadResult();


	}



	private function mapPersonFields(){

		// mapped fields variable
		$mappedFields = [];

		// person fields
		$fieldsToMap = [
			'first_name',
			'last_name',
			'company_name',
			'position_name',
			'type',
			'source_name',
			'phone',
			'mobile_phone',
			'home_phone',
			'fax',
			'email',
			'home_email',
			'other_email',
			'home_address_1',
			'home_address_2',
			'home_city',
			'home_state',
			'home_zip',
			'home_country',
			'work_address_1',
			'work_address_1',
			'work_city',
			'work_zip',
			'work_country',
			'website',
			'info',
			'facebook_url',
			'twitter_user',
			'linkedin_url',
		];

		// build mapping
		foreach ($fieldsToMap as $field){

			if(!$this->params->get($field,0))
				continue;

			$mappedFields[$field] = $this->params->get($field,0);

		}


		return count($mappedFields) > 0 ? $mappedFields : false;

	}


	private function updateProfile($person){

		// check if profile plugin enabled
		if(!\Joomla\CMS\Plugin\PluginHelper::isEnabled('user','profile'))
			return;

		// load plugin
		PluginHelper::importPlugin('user','profile');

		// prepare data
		$data = [];
		$data['id'] = $person->uid;

		// update phone
		if(!empty($person->phone))
			$data['profile']['phone'] = $person->phone;

		// trigger event
		$result = Factory::getApplication()->triggerEvent('onUserAfterSave', [$data,false,true,'']);

	}

	private function checkAllowedUserGroups($user){

		$allowedUserGroups = $this->params->get('allowedgroups',[2]);

		foreach ($user->get('groups') as $group){

			if(!in_array($group,$allowedUserGroups))
				continue;

			return true;

		}

		return false;

	}

	public function getPerson($userId)
	{

		$query = Factory::getDbo()->getQuery(true);
		$query->select('*')->from('#__joomcrm_people')->where('uid =' . (int) $userId);
		Factory::getDbo()->setQuery($query);

		$result = Factory::getDbo()->loadObject();

		return is_null($result) ? false : $result;

	}


	public function updatePerson($data)
	{

		$person        = new stdClass();
		$person->id    = $data['id'];
		$person->email = $data['email'];

		return Factory::getDbo()->updateObject('#__joomcrm_people', $person, 'id');

	}


}