<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

class JoomcrmControllerAjax extends JoomcrmController
{
	public function saveAjax()
	{

		$item_id   = $this->app->input->get('item_id');
		$item_type = $this->app->input->get('item_type');
		$field     = $this->app->input->get('field');
		$value     = $this->app->input->get('value');

		$db = Factory::getDbo();

		$data      = array('id' => $item_id, $field => $db->escape($value));
		$post_data = Factory::getApplication()->input->post->getArray();
		$data      = array_merge($data, $post_data);

		$item_type = $item_type == 'person' ? 'people' : $item_type;


		$model = $this->getModel($item_type);

		$returnRow = true;

		$return    = $model->store($data, $returnRow);

		$this->app->close(json_encode($return));
	}

	public function addPersonToDeal()
	{
		$data     = array();
		$model    = $this->getModel('People');
		$personId = $this->app->input->get("person_id");

		if ($personId > 0)
		{
			$data['id'] = $personId;
		}
		else
		{
			$name               = explode(" ", $this->app->input->get("name"));
			$firstName          = array_key_exists(0, $name) ? $name[0] : "";
			$lastName           = array_key_exists(1, $name) ? $name[1] : "";
			$data['first_name'] = $firstName;
			$data['last_name']  = $lastName;

			$id = $model->checkPersonName($data['first_name'] . ' ' . $data['last_name']);

			if ($id)
			{
				$data['id'] = $id;
			}
		}

		$data['deal_id'] = $this->app->input->get("deal_id");

		if ($model->store($data))
		{
			$success = true;
		}
		else
		{
			$success = false;
		}

		$this->app->close(json_encode(array('success' => $success)));
	}

	public function uploadAvatar()
	{
		$model   = $this->getModel('Avatar');
		$item_id = $this->app->input->get('item_id');

		if ($avatar = $model->saveAvatar())
		{
			$response = [
				'success' => true,
				'successMessage' => Text::_('COM_JOOMCRM_SUCCESS_MESSAGE').', '. Text::_('COM_JOOMCRM_AVATAR_UPLOAD_SUCCESS'),
				'itemId' => $item_id,
				'avatarLink' => $avatar
			];


			echo json_encode($response);
			jexit();



		}
		else
		{

			$response = [
				'success' => false,
				'successMessage' => Text::_('COM_JOOMCRM_SUCCESS_MESSAGE').', '. Text::_('COM_JOOMCRM_AVATAR_UPLOAD_SUCCESS')
			];

			echo json_encode($response);
			jexit();
		}
	}



	// Save user profile data
	public function saveProfile()
	{
		$success = true;
        $input = Factory::getApplication()->input;
		$data       = $input->post->getArray();


		$data['id'] = JoomcrmHelperUsers::getUserId();

		$model = $this->getModel('User');

		$msg = '';

		if (!$model->store($data))
		{
			$msg     = $model->getError();
			$success = false;
		}

		// Return results
		$results = array('success' => $success, 'message' => $msg);

        // no ajax call and click on fullscreen button
        $this->app->close(json_encode($results));

	}

	public function saveFullscreen(){

		$success = true;
		$msg = '';
		$input = Factory::getApplication()->input;
		$data       = $input->post->getArray();
		$return = $input->get('return','base64');

		$data['id'] = JoomcrmHelperUsers::getUserId();
		$model = $this->getModel('User');
		$data['url'] = base64_decode($return);
		$data['fullscreen'] = 1;
		if (!$model->store($data))
		{
			$msg     = $model->getError();
			$success = false;
		}

		\Joomla\CMS\Factory::getApplication()->redirect($data['url']);

	}

    private function updateURL($url){
        $tmplPortions = ['&tmpl=component','?tmpl=component','tmpl=component'];

        // remove tmpl portion
        foreach ($tmplPortions as $portion){
            if (strpos($url, $portion) !== false) {
                str_replace($portion,'',$url);
            }
        }

        return $url;
    }

	// Get individual goals
	public function getIndividualGoals()
	{
		$goals = $this->getModel('Goal')->getIndividualGoals($this->app->input->get('id'));

		$view = JoomcrmHelperView::getView('goals', 'filters', array(array('ref' => 'goals', 'data' => $goals)));

		$view->display();
	}

	// Get team goals
	public function getTeamGoals()
	{
		$goals = $this->getModel('Goal')->getTeamGoals($this->app->input->get('id'));

		$view = JoomcrmHelperView::getView('goals', 'filters', array(array('ref' => 'goals', 'data' => $goals)));

		$view->display();
	}

	// Get a leaderboard
	public function getLeaderBoard()
	{
		$leaderboard = $this->getModel('Goal')->getLeaderBoards($this->app->input->get('id'));

		$view = JoomcrmHelperView::getView('goals', 'leaderboard', array(array('ref' => 'leaderboard', 'data' => $leaderboard)));

		$view->display();
	}

	// Update the user filter columns
	public function  updateColumns()
	{
		$loc = $this->app->input->get('loc');

		$column = $this->app->input->get('column');

		$this->getModel('User')->updateColumns($loc, $column);
	}

	/**
	 * Run items through template system
	 *
	 * @return  [type]  [description]
	 */
	public function  createTemplate()
	{
		$return            = array();
		$return['success'] = false;

		if ($this->getModel('Template')->createTemplate())
		{
			$return['success'] = true;
		}

		$this->app->close(json_encode($return));
	}

	public function  getNoteEntry()
	{
		$note_id = $this->app->input->get('note_id');

		$note = $this->getModel('Note')->getNote($note_id);

		$note_view = JoomcrmHelperView::getView('note', 'entry', array(array('ref' => 'note', 'data' => $note[0])));

		$note_view->display();
	}

	public function  getConvoEntry()
	{
		$convo_id = $this->app->input->get('convo_id');

		$convo = $this->getModel('Conversation')->getConversation($convo_id);

		$convo_view = JoomcrmHelperView::getView('deals', 'conversation_entry', array(array('ref' => 'conversation', 'data' => $convo[0])));

		$convo_view->display();
	}

	public function  removePersonFromDeal()
	{
		$person_id = $this->app->input->get('person_id');
		$deal_id   = $this->app->input->get('deal_id');

		$db = Factory::getDbo();

		$query = $db->getQuery(true)->select('COUNT(*)')->from('#__joomcrm_people')->where('id = ' . $person_id);

		$count = $db->setQuery($query)->loadResult();

		if ($count)
		{
			$query->clear()->delete('#__joomcrm_people_cf')->where('association_id = ' . $deal_id)
			->where('association_type = "deal"')->where('person_id=' . $person_id);

			$db->setQuery($query);

			if ($db->execute())
			{
				$success = true;
			}
			else
			{
				$success = false;
			}

			$query->clear()->update('#__joomcrm_deals')->set('primary_contact_id = 0')
			->where('primary_contact_id = ' . $person_id)->where('id=' . $deal_id);

			$db->setQuery($query);

			if ($db->execute())
			{
				$success = true;
			}
			else
			{
				$success = $success ? $success : false;
			}
		}
		else
		{
			$success = false;
		}

		$this->app->close(json_encode(array('success' => $success)));
	}

	public function  getCalendarEvents()
	{
		$data       = Factory::getApplication()->input->get->getArray();
		$start_date = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00", $data['start']));
		$end_date   = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00", $data['end']));

		// ajax mode flag
		Factory::getApplication()->input->set('isAjax',1);

		$model = $this->getModel('Event');
		$model->set('start_date', "$start_date");
		$model->set('end_date', "$end_date");
		$model->set('loc', "calendar");

		$user = JoomcrmHelperUsers::getLoggedInUser();

		if (!$user)
		{
			$model->set('public', 1);
		}

		$this->app->close(json_encode($model->getEvents()));
	}

	/**
	 * Download CSV
	 *
	 * @return [type] [description]
	 */
	public function  downloadCsv()
	{
		ob_start();
		header('Content-Description: File Transfer');
		header('Content-Type: application/octet-stream');
		header('Content-Disposition: attachment; filename="export.csv"');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Pragma: public');
		// header('Content-Length: ' . filesize($path));
		ob_clean();
		flush();

		$this->app->close($this->getModel('Export')->getCsv());
	}

	/**
	 * Download CSV
	 *
	 * @return [type] [description]
	 */
	public function downloadVcard()
	{
		ob_start();
		header('Content-Description: File Transfer');
		header('Content-Type: application/octet-stream');
		header('Content-Disposition: attachment; filename="export.vcf"');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Pragma: public');
		// header('Content-Length: ' . filesize($path));
		ob_clean();
		flush();

		echo $this->getModel('Export')->getVcard();

		exit();
	}

	/**
	 * Download example import templates
	 *
	 * @return [type] [description]
	 */
	public function downloadImportTemplate()
	{
		$template_type = $this->app->input->get('template_type');

		$path = JPATH_SITE . '/components/com_joomcrm/media/import_templates/import_' . $template_type . '.csv';

		ob_start();
		header('Content-Description: File Transfer');
		header('Content-Type: application/octet-stream');
		header('Content-Disposition: attachment; filename="import_' . $template_type . '.csv"');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Pragma: public');
		header('Content-Length: ' . filesize($path));
		ob_clean();
		flush();

		readfile($path);

		$this->app->close();
	}

	public function checkCompanyName()
	{
		$company_name    = $this->app->input->get('company_name', '', 'RAW');
		$existingCompany = $this->getModel('Company')->checkCompanyName($company_name);

		if (!empty($existingCompany))
		{
			/** ucwords(Text::_('COM_JOOMCRM_COMPANY_EXISTS')) **/
			$this->app->close(json_encode(array('success' => true, 'company_id' => $existingCompany, 'message' => "")));
		}
		else
		{
			$this->app->close(json_encode(array('success' => true, 'message' => ucwords(Text::_('COM_JOOMCRM_COMPANY_WILL_BE_CREATED')))));
		}
	}

	public function getCompanyNames()
	{
		$search = $this->app->input->get('search');
		$items = $this->getModel('Company')->getCompanyList($search);

		if (!empty($items))
		{
			$this->app->close(json_encode(array('success' => true, 'items' => $items)));
		}
		else
		{
			$this->app->close(json_encode(array('success' => false)));
		}
	}

	public function getContactNames()
	{
		$search = $this->app->input->get('search');
		$items = $this->getModel('Contacts')->getContacts($search);

		if (!empty($items))
		{
			$this->app->close(json_encode(array('success' => true, 'items' => $items)));
		}
		else
		{
			$this->app->close(json_encode(array('success' => false)));
		}
	}

	public function  checkEmailName()
	{
		$emailExists = JoomcrmHelperJoomcrm::checkEmailName($this->app->input->get('email'));

		if ($emailExists)
		{
			$success = true;
			$msg     = Text::_('COM_JOOMCRM_EMAIL_EXISTS');
		}
		else
		{
			$success = true;
			$msg     = Text::_('COM_JOOMCRM_EMAIL_IS_AVAILABLE');
		}

		$return = array('success' => $success, 'message' => $msg, 'email_exists' => $emailExists);

		$this->app->close(json_encode($return));
	}

	public function  checkDealName()
	{
		$deal_name    = $this->app->input->get('deal_name');
		$existingDeal = $this->getModel('Deal')->checkDealName($deal_name);

		if (!empty($existingDeal))
		{
			$this->app->close(json_encode(array('success' => true, 'deal_id' => $existingDeal, 'message' => "")));
		}
		else
		{
			$this->app->close(json_encode(array('success' => true, 'message' => ucwords(Text::_('COM_JOOMCRM_DEAL_WILL_BE_CREATED')))));
		}
	}

	public function getDeal()
	{
		$dealId    = $this->app->input->getInt('id');
		$deal = $this->getModel('Deal')->getDeals($dealId);

		if (isset($deal[0]))
		{
			$this->app->close(json_encode(array('success' => true, 'deal' => $deal[0], 'message' => "")));
		}
		else
		{
			$this->app->close(json_encode(array('success' => false, 'message' => ucwords(Text::_('COM_JOOMCRM_ERROR')))));
		}
	}

	public function checkPersonName()
	{
		$person_name    = $this->app->input->get('person_name');
		$existingPerson = $this->getModel('People')->checkPersonName($person_name);

		if ($existingPerson)
		{
			$this->app->close(json_encode(array('success' => true, 'person_id' => $existingPerson, 'message' => "")));
		}
		else
		{
			$this->app->close(json_encode(array('success' => true, 'message' => ucwords(Text::_('COM_JOOMCRM_PERSON_WILL_BE_CREATED')))));
		}
	}

	public function previewDocument()
	{
		$document = $this->getModel('Document')->getDocument();

		$filename       = basename($document->name);
		$file_extension = strtolower(substr(strrchr($filename, "."), 1));

		switch ($file_extension)
		{
			case 'gif':
				$ctype = 'image/gif';
				break;
			case 'png':
				$ctype = 'image/png';
				break;
			case 'jpeg':
			case 'jpg':
				$ctype = 'image/jpg';
				break;
			case 'pdf':
				$ctype = 'application/pdf';
				break;
			default:
				$ctype = 'application/' . $file_extension;
				break;
		}

		header('Content-type: ' . $ctype);
		ob_clean();
		flush();
		readfile($document->path);

		$this->app->close();
	}

	public function toggleMailingList()
	{
		$data      = Factory::getApplication()->input->post->getArray();
		$subscribe = $data['subscribe'];
		$success   = false;

		$mailing = Joomcrm\Mailing\Mailing::loadIntegration();

		if (!$subscribe)
		{
			if (($text = $mailing->addMailingList($data)) === true)
			{
				$success = true;
			}
		}
		else
		{	
			if (($text = $mailing->removeMailingList($data)) === true)
			{
				$success = true;
			}
		}

		$response = array('success' => $success, 'text' => $text);

		$this->app->close(json_encode($response));
	}

	public function shareItem()
	{
		$return = array();

		if (JoomcrmHelperJoomcrm::shareItem())
		{
			$return['success'] = true;
		}
		else
		{
			$return['success'] = false;
		}

		$this->app->close(json_encode($return));
	}

	public function unshareItem()
	{
		$return = array();

		if (JoomcrmHelperJoomcrm::unshareItem())
		{
			$return['success'] = true;
		}
		else
		{
			$return['success'] = false;
		}

		$this->app->close(json_encode($return));
	}

	public function saveWizardForm()
	{
		$config = \Joomla\CMS\Factory::getConfig();
		$post   = Factory::getApplication()->input->post->getArray();

		$formReturn = base64_decode($post['formReturn']);

		if ($config->get('captcha') && array_key_exists('g-recaptcha-response', $post))
		{

			if(empty($post['g-recaptcha-response'])){


				Factory::getApplication()->enqueueMessage(\Joomla\CMS\Language\Text::_('Invalid Captcha'),'warning');

				$this->app->redirect($formReturn);
				return false;
			}


			\Joomla\CMS\Plugin\PluginHelper::importPlugin('captcha');
			$dispatcher = Factory::getApplication();
			$res        = $dispatcher->triggerEvent('onCheckAnswer', [$post['g-recaptcha-response']]);

			if (!$res[0])
			{
				Factory::getApplication()->enqueueMessage(\Joomla\CMS\Language\Text::_('Invalid Captcha'),'warning');

				$this->app->redirect($formReturn);
				return false;
			}
		}



		$type = $this->app->input->get('save_type');

		switch ($type)
		{
			case 'lead':
			case 'contact':
				$model = $this->getModel('People');
				break;
			case 'company':
				$model = $this->getModel('Company');
				break;
			case 'deal':
				$model = $this->getModel('Deal');
				break;
		}

		$returnUrl = $this->app->input->getRaw('return', '');

		if (!$returnUrl)
		{
			$returnUrl = 'index.php';
		}


		if (!isset($model) || $model->store() === false)
		{
            Factory::getApplication()->enqueueMessage(\Joomla\CMS\Language\Text::_('COM_JOOMCRM_ERROR_SAVING'),'warning');

			$this->app->redirect($returnUrl, );
		}
		else
		{
			if (empty($returnUrl))
            Factory::getApplication()->enqueueMessage(\Joomla\CMS\Language\Text::_('COM_JOOMCRM_SUCCESSFULLY_SAVED'),'success');
			$this->app->redirect($returnUrl);
		}
	}

	public function toggleCompanyPrivacy()
	{
		$company_id = $this->app->input->get('company_id');
		$model      = $this->getModel('Company');

		$return  = array('success' => false);
		$company = $model->getCompanies($company_id);
		$company = array_key_exists(0, $company) ? $company[0] : false;

		if ($company)
		{
			$privacy = $company['public'] == 0 ? 1 : 0;
			$data    = array('id' => $company['id'], 'public' => $privacy);

			if ($model->store($data))
			{

				$icon = $privacy == 1 ? 'lock' : 'unlock';

				$return['success'] = true;
				$return['text'] = "<i class='fas fa-$icon'></i> ";
				$return['text']    .= $privacy == 1 ? Text::_('COM_JOOMCRM_MAKE_PRIVATE') : Text::_('COM_JOOMCRM_MAKE_PUBLIC');
				$return['color'] = $privacy == 1 ? 'danger' : 'success';

			}
		}

		$this->app->close(json_encode($return));
	}


}
