<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Controller\BaseController;

class JoomcrmController extends BaseController
{
	/**
	 * The current application instance.
	 *
	 * @var  JApplication
	 */
	protected $app = null;

	/**
	 * Constructor
	 *
	 * @return  void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->app = Factory::getApplication();
	}

	/**
	 * Method to load and return a model object.
	 *
	 * @param   string $name    The name of the model.
	 * @param   string $prefix  Optional model prefix.
	 * @param   array  $config  Configuration array for the model. Optional.
	 *
	 * @return  mixed   Model object on success; otherwise null failure.
	 *
	 * @since   11.1
	 * @note    Replaces _createModel.
	 */
	protected function createModel($name, $prefix = '', $config = array())
	{
		// Clean the model name
		$modelName   = preg_replace('/[^A-Z0-9_]/i', '', $name);
		$classPrefix = preg_replace('/[^A-Z0-9_]/i', '', $prefix);

		$modelClassName = ucfirst($classPrefix) . ucfirst($modelName);

		if (!class_exists($modelClassName))
		{
			// \Joomla\CMS\Log\Log::add(sprintf('Model class %s does not exist.', $modelClassName), \Joomla\CMS\Log\Log::WARNING);
			return false;
		}

		return new $modelClassName($config);
	}

	/**
	 * Method to load and return a view object. This method first looks in the
	 * current template directory for a match and, failing that, uses a default
	 * set path to load the view class file.
	 *
	 * Note the "name, prefix, type" order of parameters, which differs from the
	 * "name, type, prefix" order used in related public methods.
	 *
	 * @param   string $name    The name of the view.
	 * @param   string $prefix  Optional prefix for the view class name.
	 * @param   string $type    The type of view.
	 * @param   array  $config  Configuration array for the view. Optional.
	 *
	 * @return  mixed  View object on success; null or error result on failure.
	 *
	 * @since   11.1
	 */
	protected function createView($name, $prefix = '', $type = 'Html', $config = array())
	{
		// Clean the view name
		$viewName    = preg_replace('/[^A-Z0-9_]/i', '', $name);
		$classPrefix = preg_replace('/[^A-Z0-9_]/i', '', $prefix);
		$viewType    = preg_replace('/[^A-Z0-9_]/i', '', $type);

		// Build the view class name. Ex: JoomcrmViewAccountHtml
		$viewClassName = ucfirst($classPrefix) . ucfirst($viewName) . ucfirst($viewType);

		if (!class_exists($viewClassName))
		{
			// \Joomla\CMS\Log\Log::add(sprintf('View class %s does not exist.', $viewClassName), \Joomla\CMS\Log\Log::WARNING);
			return false;
		}

		return new $viewClassName($config);
	}
}
