<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

class JoomcrmControllerDatatables extends JoomcrmController
{
	/**
	 * Main method for executing dataTable API calls
	 * 
	 * @return string JSON
	 */
	public function getData()
	{
		// Define variables
		$model  	= $this->app->input->getString('frommodel');
		$view 		= $this->app->input->getString('toview');
		$layout		= $this->app->input->getString('tolayout');
		$id     	= $this->app->input->getInt('id');
		$draw   	= $this->app->input->getInt('draw');
		$limit		= $this->app->input->getInt('length',10);
		$limitStart	= $this->app->input->getInt('start');
		$orderBy 	= $_REQUEST['order'][0]['column'];
		$orderDir 	= $_REQUEST['order'][0]['dir'];
		$search 	= $_REQUEST['search']['value'];
		$columns	= $_REQUEST['columns'];
		$response 	= new stdClass;
		$getItems 	= 'get' . ucfirst($this->makePlural($model));


		$stateId 	= ucfirst($model) . '.' . $view . '.' . $layout;


		// set user state
		$this->app->setUserState($stateId . '_limitstart', $limitStart); // deprecated
		$this->app->setUserState($stateId . '_limit', $limit); // deprecated
		$this->app->setUserState($stateId . '.limitstart', $limitStart);
		$this->app->setUserState($stateId . '.limit', $limit);
		$this->app->setUserState($stateId . '.filter_order', $orderBy);
		$this->app->setUserState($stateId . '.filter_order_Dir', $orderDir);
		$this->app->setUserState($stateId . '_name', $search); // deprecated
		$this->app->setUserState($stateId . '.name', $search);

		// set fake input variables
		$this->app->input->set('view', $view);

		if ($layout) {
			$this->app->input->set('layout', $layout);
		}
		
		$this->app->input->set('id', null);
		$this->app->input->set('name', $search);
		$this->app->input->set('datatableQuery', true);

		// initialize model when user state and input are ready
		$model 		= $this->getModel($model);

		// configure model
		$model->set('view', $view);
		$model->set('layout',$layout);
		$model->{$this->makeSingular($view) . '_id'} = $id;

		// Get data from model
		$items 		= $model->$getItems();


		$total 		= $model->getTotal();

		// build response JSON
		$response->draw = $draw;
		$response->recordsTotal = $total;
		$response->recordsFiltered = $total;
		$response->data = $model->dataTablefy($items);

		$this->app->close(json_encode($response));
	}

	/**
	 * Method returns singular of some word.
	 * It is quite stupic simple method for 3 words
	 * we need to make signular from. Not solwing all world problems.
	 * 
	 * @param string $name
	 * @return string
	 */
	protected function makeSingular($name)
	{
		if ($name == 'companies')
		{
			$name = 'company';
		}
		elseif ($name == 'people')
		{
			$name = 'person';
		}
		else
		{
			$lastChar = mb_substr($name, -1);

			if ($lastChar == 's')
			{
				$name = mb_substr($name, 0, -1);
			}
			
		}

		return $name;
	}

	/**
	 * Method returns plural of some word.
	 * It is quite stupic simple method for 3 words
	 * we need to make plural from. Not solwing all world problems.
	 * 
	 * @param string $name
	 * @return string
	 */
	protected function makePlural($name)
	{
		if ($name == 'company')
		{
			$name = 'companies';
		}
		elseif ($name == 'person')
		{
			$name = 'people';
		}
		elseif ($name == 'people')
		{
			$name;
		}
		else
		{
			$name = $name . 's';
		}

		return $name;
	}
}