<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

class JoomcrmControllerMail extends JoomcrmController
{
	public function __construct()
	{
		parent::__construct();
		\Joomla\CMS\Factory::getApplication()->input->set('format', 'raw');
	}

	/**
	 *  Remove emails
	 */
	public function removeEmail()
	{
		$email_id = $this->app->input->getString('id');

		$this->getModel('Mail')->removeEmail($email_id);
	}

	public function saveEmail()
	{
		$email_id = $this->app->input->getString('id');

		$this->getModel('Mail')->saveEmail($email_id);
	}

	public function brewCoffee()
	{
		$db    = Factory::getDBO();
		$query = $db->getQuery(true)->select('u.id, u.first_name, u.last_name, user.email')
				 ->from('#__joomcrm_users AS u')->leftJoin('#__users AS user ON u.uid = user.id')
				 ->where('u.morning_coffee=1')->where("u.published=1");

		$people = $db->setQuery($query)->loadObjectList();

		foreach ($people as $person)
		{
			$layout = $this->loadStats($person->id);
			$layout->user = $person;
			$this->sendMail($layout, $person->email);
		}
	}

	public function mailDailyAgenda()
	{

	}

	public function mailWeeklyTeamUSE()
	{

	}

	public function mailWeeklyPersonalUSE()
	{

	}

	public function loadStats($person_id)
	{
		// Set the user ID for helpers
		\Joomla\CMS\Factory::getApplication()->input->set('user_id', $person_id);

		$statsModel = $this->getModel('Stats');
		$statsModel->set('person_id', $person_id);

		// Load view
		$coffeeView = JoomcrmHelperView::getView('emails', 'coffee.report');

		// Get Total Deals Amount
		$activeDeals                  = $statsModel->getActiveDealsAmount();
		$coffeeView->totalDealsAmount = $activeDeals;

		// Get Stage Details
		$stages             = $statsModel->getStages();
		$coffeeView->stages = $stages;

		// Get Number of Converted Leads
		$leads                         = $statsModel->getLeads();
		$coffeeView->numConvertedLeads = $leads['contact'];

		// Get Number of New Leads
		$coffeeView->numNewLeads = $leads['lead'];

		// Get Note Details
		$notes             = $statsModel->getNotes();
		$coffeeView->notes = $notes;

		// Get ToDo Details
		$todos             = $statsModel->getTodos();
		$coffeeView->todos = $todos;

		// Get Deal Activity
		$dealActivity             = $statsModel->getDealActivity();
		$coffeeView->dealActivity = $dealActivity;

		// Get Lead Activity
		// $coffeeView->leadActivity = $leadActivity;

		// Get Contact Activity
		// $coffeeView->contactActivity = $contactActivity;

		return $coffeeView;
	}

	public function sendMail($layout, $recipient)
	{
		$mailer = Factory::getMailer();
		$mailer->isHTML(true);
		$mailer->Encoding = 'base64';

		$config = Factory::getConfig();
		$sender = array($config->get('config.mailfrom'), $config->get('config.fromname'));

		$mailer->setSender($sender);
		$mailer->addRecipient($recipient);
		$mailer->setSubject(Text::_('COM_JOOMCRM_COFFEE_REPORT_SUBJECT') . ' ' . JoomcrmHelperDate::formatDate(date('Y-m-d')));

		ob_start();
		$layout->display();
		$body = ob_get_clean();

		$mailer->setBody($body);
		$send = $mailer->send();

		if ($send !== true)
		{
			$this->app->close('Error sending email: ' . $send->getMessage());
		}
	}
}
