<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/



defined('_JEXEC') or die;

use Joomla\CMS\Table\Table;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;

class JoomcrmControllerMain extends JoomcrmController
{
	public function trash()
	{
		$item_ids  = $this->app->input->get('item_id', null, 'array');
		$item_type = $this->app->input->get('item_type');

		switch ($item_type)
		{
			case "deals":
				$table = "deal";
				break;
			case "events":
				$table = "event";
				break;
			case "companies":
				$table = "company";
				break;
			case "documents":
				$table = "document";
				break;
			case "conversations":
				$table = "conversation";
				break;
			case "goals":
				$table = "goal";
				break;
			case "notes":
				$table = "note";
				break;
			case "people":
				$table = "people";
				break;
			case "reports":
				$table = "report";
				break;
			default:
				$table = $item_type;
				break;
		}

		// ADD TO MODELS * trash model *
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
			->update('#__joomcrm_' . $item_type)
			->set('published = -1')
			->where("id IN(" . implode(',', $item_ids) . ")");

		$db->setQuery($query);

		$data = array();
        try
        {
            $db->execute();

            $data['success'] = true;

        }
        catch (RuntimeException $e)
        {
            $data['success']   = false;
            $data['error_msg'] = $e->getMessage();
        }



		$redirect = $this->app->input->get('page_redirect');

		if ($redirect)
		{
			$msg = ($data['success']) ? Text::_('COM_JOOMCRM_SUCCESSULLY_REMOVED_ITEM') : Text::_('COM_JOOMCRM_ERROR_REMOVING_ITEM');
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=' . $redirect), $msg);
		}
		else
		{
			echo json_encode($data);
		}

		foreach ($item_ids as $id)
		{
			$oldSaveRow = Table::getInstance($table, 'JoomcrmTable');
			$oldSaveRow->load($id);

			$newSaveRow = Table::getInstance($table, 'JoomcrmTable');
			$newSaveRow->load($id);

			JoomcrmHelperActivity::saveActivity($oldSaveRow, $newSaveRow, $table, 'deleted');
		}
	}

	/*
	 * main method to save to tables
	 */
	public function save()
	{
		$model = $this->getModel($this->app->input->get('model'));

		// If we are requesting a return redirect set up redirect link
		if ($this->app->input->get('return'))
		{
			$link = Route::_('index.php?option=com_joomcrm&view=' . $this->app->input->get('return'));
		}

		if ($db_id = $model->store())
		{
			$msg = Text::_('COM_JOOMCRM_SUCCESSFULLY_SAVED');

			// Redirect if set else return json
			if ($this->app->input->get('return'))
			{
				$this->setRedirect($link, $msg);
			}
			else
			{
				// If deal information is being edited
				if ($this->app->input->get('model') == 'deal')
				{
					$model = $this->getModel('Deal');
					$model->set('_id', $db_id);
					$model->set('user_override', 'all');
					$deal   = $model->getDeals();
					$return = $deal[0];
				}

				// If people information is being edited
				if ($this->app->input->get('model') == 'people')
				{
					$person = $this->getModel('People')->getPerson($this->app->input->get('id'));
					$return = $person[0];
				}
				
				// If company information is being edited
				if ($this->app->input->get('model') == 'company')
				{
					// $company = $this->getModel('Company')->getCompany($this->app->input->get('id'));
					$return       = array('id'=>$db_id);
				}


				// If conversation information is being edited
				if ($this->app->input->get('model') == 'conversation')
				{
					$conversation = $this->getModel('Conversation')->getConversation($db_id);
					$return       = $conversation[0];
				}

				// If note information is being edited
				if ($this->app->input->get('model') == 'note')
				{
					$note   = $this->getModel('Note')->getNote($db_id);
					$return = $note[0];
				}

				// If event information is being edited
				if ($this->app->input->get('model') == 'event')
				{
					$return = $this->getModel('Event')->getEvent($db_id);
				}

				// Document info
				if ($this->app->input->get('model') == 'document')
				{
					$this->app->close(json_encode($model->getDocument($db_id)));
				}

				// Return reqeusts
				if (isset($return))
				{
					$return['error'] = false;
					$this->app->close(json_encode($return));
				}
			}

		}
		else
		{
			$msg = Text::_('COM_JOOMCRM_ERROR_SAVING').': '.$model->getError();

			// Redirect if set else return json info
			if ($this->app->input->get('return'))
			{
				$this->setRedirect($link, $msg);

			}
			else
			{

				$data = Factory::getApplication()->input->post->getArray();
				$data['error'] = true;
				$data['msg'] = $msg;
				$this->app->close(json_encode($data));
			}
		}
	}

	public function addPersonToCompany()
	{
		$person_id   = $this->app->input->getInt("person_id");
		$company_id  = $this->app->input->getInt('company_id');
		$person_name = $this->app->input->getString('person_name');

		if ($person_id > 0)
		{
			$data = array('id' => $person_id, 'company_id' => $company_id);
		}
		else
		{
			$name_string = explode(" ", $person_name);
			$first_name  = array_key_exists(0, $name_string) ? $name_string[0] : "";
			$last_name   = array_key_exists(1, $name_string) ? $name_string[1] : "";

			if (empty($first_name) && empty($last_name))
			{
				$this->app->close(json_encode(array('success' => false)));
			}

			$data = array('first_name' => $first_name, 'last_name' => $last_name, 'company_id' => $company_id);
		}

		$success = (boolean) $this->getModel('People')->store($data);

		$this->app->close(json_encode(array('success' => $success)));
	}

	/**
	*	Method assign a deal to person or company.
	*/
	public function addDeal()
	{
		$type 		= $this->app->input->getString('type');
		$deal_id   	= $this->app->input->getInt("deal_id");
		$id  		= $this->app->input->getInt('id');
		$deal_name 	= $this->app->input->getString('deal_name');

		if ($deal_id > 0)
		{
			// assign existing deal
			$data = array('id' => $deal_id);
		}
		else
		{
			// deal does not exist, create it with provided name and assign it
			$data = array('name' => $deal_name);
		}

		if ($type == 'person' && $id)
		{
			$data['person_id'] = $id;
		}
		elseif ($type == 'company' && $id)
		{
			$data['company_id'] = $id;
		}

		$success = (boolean) $this->getModel('Deal')->store($data);

		$this->app->close(json_encode(array('success' => $success)));
	}

	/*
	 * Method to save CF tables via AJAX
	 */
	public function saveCf()
	{
		$return = array();

		// Get post data
		$data = \Joomla\CMS\Factory::getApplication()->input->post->getArray();

		// Get db Object
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		$table = $data['table'];
		$loc   = $data['loc'];

		unset($data['table']);
		unset($data['loc']);

		//write to tables if there is no association already in cf tables
		$query->select('* FROM #__joomcrm_' . $table . '_cf');

		// Loop to see if we have matches in database
		$overrides = array('tmpl','view');

		foreach ($data as $key => $value)
		{
			if (!in_array($key, $overrides))
			{
				$query->where($key . " = '" . $value . "'");
			}
		}

		$results = $db->setQuery($query)->loadAssocList();

		// Determine if we found any results
		if (count($results) == 0)
		{
			$query->insert('#__joomcrm_' . $table . '_cf');

			// Timestamp
			$data['created'] = date('Y-m-d H:i:s');
			$date            = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

			// Loop through data to get query string
			foreach ($data as $key => $value)
			{
				if (!in_array($key, $overrides))
				{
					// determine key and key values
					$query->set($key . " = '" . $value . "'");
				}
			}

			$db->setQuery($query)->execute();

			// If return data requested
			if ($table === 'people')
			{
				// Determine which page we want are wanting to send information back to
				if ($loc == 'deal')
				{
					$return = $this->getModel('People')
							  ->getPerson(array_key_exists('person_id', $data) ? $data['person_id'] : "");
					$return = $return[0];
				}

				if ($loc == 'person')
				{
					$return = $this->getModel('Deal')
							  ->getDeals(array_key_exists('deal_id', $data) ? $data['deal_id'] : '');
					$return = $return[0];
				}
			}
		}
		else
		{
			$return = array('error' => true);
		}

		// Return json data
		$this->app->close(json_encode($return));
	}

	// Ajax function to retrieve all people for autocomplete searches
	public function getPeople()
	{
		$people = $this->getModel('People')->getPeopleList();

		$this->app->close(json_encode($people));

	}

	// Ajax function to retrieve all deals for autocomplete searches
	public function getDeals()
	{
		$deals = $this->getModel('Deal')->getDealList();

		$this->app->close(json_encode($deals));
	}

	// Ajax function to retrieve all event & task associations for autocomplete searches
	public function getTaskAssociations()
	{
		$people = $this->getModel('People')->getPeopleList();

		if (count($people))
		{
			foreach ($people as $index => $row)
			{
				$people[$index]['type']             = "person";
				$people[$index]['association_link'] = Route::_('index.php?option=com_joomcrm&view=people&layout=person&id=' . $row['id']);
			}
		}

		$deals = $this->getModel('Deal')->getDealList();

		if (count($deals))
		{
			foreach ($deals as $index => $row)
			{
				$deals[$index]['type']             = 'deal';
				$deals[$index]['association_link'] = Route::_('index.php?option=com_joomcrm&view=deals&layout=deal&id=' . $row['id']);
			}
		}

		$companies = $this->getModel('Company')->getCompanyList();

		if (count($companies))
		{
			foreach ($companies as $index => $row)
			{
				$companies[$index]['type']             = 'company';
				$companies[$index]['association_link'] = Route::_('index.php?option=com_joomcrm&view=companies&layout=company&id=' . $row['id']);
			}
		}

		// Merge our results to a grand object
		$results = array_merge($people, $deals, $companies);

		// Return results as json object
		$this->app->close(json_encode($results));
	}

	// Ajax function to populate note dropdowns
	public function getNoteDropdowns()
	{
		$data = array(
			'people' => $this->getModel('People')->getPeopleList(),
			'deals' => $this->getModel('Deal')->getDealList(),
			'categories' => $this->getModel('Note')->getNoteCategories()
		);

		$this->app->close(json_encode($data));
	}
}
