<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */


use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;


defined('_JEXEC') or die;

JLoader::register('JoomcrmHelperUsers', JPATH_COMPONENT . '/helpers/users.php');
BaseDatabaseModel::addIncludePath(JPATH_SITE . '/components/com_joomcrm/model', 'JoomcrmModel');

class JoomcrmControllerMyteam extends JoomcrmController
{

	// invite member to user team
	public function inviteMember()
	{

		// check CSRF
		Session::checkToken() or die('Invalid Token');

		// check if user authorized to invite to his team
		if(!Factory::getApplication()->getIdentity()->authorise('core.inviteto.own.team','com_joomcrm')){
			die('Not Allowed');
		}

		// Inits
		$currentUser = Factory::getApplication()->getIdentity();

		// get model to save invite
		$model = BaseDatabaseModel::getInstance('Myteam', 'JoomcrmModel', array('ignore_request' => true));

		// get inputs
		$email = $this->input->get('email', '', 'string');

		// check if email or role is empty
		if (empty($email))
		{
			$this->returnRequest(['fail', 'COM_JOOMCRM_MYTEAM_EMPTY_FORM']);
		}

		// check if email is not valid
		if (!filter_var($email, FILTER_VALIDATE_EMAIL))
		{
			$this->returnRequest(['fail', 'COM_JOOMCRM_MYTEAM_INVALID_EMAIL']);
		}

		// check if email already invited
		if ($model->alreadyInvited($email))
		{
			$this->returnRequest(['fail', 'COM_JOOMCRM_MYTEAM_EMAIL_ALREADY_INVITED']);
		}

		// check if team owner has a team
		if (!$model->isTeamOwner($currentUser->id))
		{
			$this->returnRequest(['fail', 'COM_JOOMCRM_MYTEAM_NOT_ALLOWED']);
		}

		// check if email already has a team, if already has a team should not send invite.
		if ($model->hasTeam($email))
		{
			$this->returnRequest(['fail', 'COM_JOOMCRM_MYTEAM_EMAIL_ALREADY_HAS_TEAM']);
		}

		// build data for saving and email notification
		$data = [
			'team_id' => JoomcrmHelperUsers::getTeamId($currentUser->id, 'uid'),
			'email'   => $email,
			'token'   => $this->generateToken(),
			'params'  => '' // like role, can delete deals ... not yet completed
		];

		// get team name
		$data['team_name'] = JoomcrmHelperUsers::getTeamName($data['team_id']);

		// check saving invite in database
		if (!$model->saveInvite($data))
		{
			$this->returnRequest(['fail', 'COM_JOOMCRM_MYTEAM_PROBLEM']);
		}

		// get last inserted invite id
		$data['invite_id'] = Factory::getDbo()->insertid();

		// check sending email, status is success cuz invite was saved there is only problem in sending email , owner of team can resend it again later in pending users tab of my team view
		if (!$this->sendInviteEmail($data))
		{
			$this->returnRequest(['fail', 'COM_JOOMCRM_MYTEAM_PROBLEM_SENDING_EMAIL']);
		}

		// all checks passed , everything works fine
		$this->returnRequest(['success', 'COM_JOOMCRM_MYTEAM_INVITE_SENT']);
	}

	// accept invite from email or notification
	public function acceptInvite()
	{
		$input = Factory::getApplication()->input;

		$model = BaseDatabaseModel::getInstance('Myteam', 'JoomcrmModel', array('ignore_request' => true));

		$token = $input->get('token', '', 'ALNUM');
		$id    = $input->get('inviteId', 0, 'INT');

		// check token and invite id
		if (empty($token) && $id <= 0)
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=dashboard'), Text::_('COM_JOOMCRM_INVALID_INVITE'), 'warning');

			return false;
		}

		// get invite object
		$invite = $model->getInvite($token, $id);

		// check if invite still exist in database , by token and id
		if (!$invite)
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=dashboard'), Text::_('COM_JOOMCRM_INVALID_INVITE'), 'warning');

			return false;
		}

		// add user to team
		if (!$model->addUserToTeam($invite))
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=dashboard'), Text::_('COM_JOOMCRM_PROBLEM_ACCEPTING_INVITE'), 'warning');

			return false;
		}

		// remove invite from database
		$model->removeInvite($invite->id);

		// get owner joomla id from team id
		$team_owner = JoomcrmHelperUsers::getOwnerFromTeamId($invite->team_id);

		// send email to team owner
		$this->sendAcceptedInviteEmail(['invited_email'    => $invite->email,
		                                'team_owner_email' => Factory::getUser($team_owner)->email]);

		// redirect to dashboard with success message
		$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=dashboard'), Text::_('COM_JOOMCRM_SUCCESS_INVITE'), 'success');
	}

	// change user team role
	public function changeRole()
	{

		// csrf check

		// check owner has a team
		// check if user has a team
		// check if user exist in his team
		// change role
	}

	/*
	 * Delete user team
	 */
	public function deleteMember()
	{

		// check CSRF
		Session::checkToken('get') or die('Invalid Token');

		// get id
		$id = Factory::getApplication()->input->get('memberId', 0, 'int');

		// get myteam model
		$model = BaseDatabaseModel::getInstance('Myteam', 'JoomcrmModel', array('ignore_request' => true));

		// check invite id
		if ($id == 0)
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_PROBLEM_DELETING_MEMBER'), 'warning');

			return false;
		}

		// check if team owner has a team
		if (!$model->isTeamOwner(Factory::getApplication()->getIdentity()->id))
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_MYTEAM_NOT_ALLOWED'), 'warning');

			return false;
		}

		// get team id
		$team_id = JoomcrmHelperUsers::getTeamId();

		// check if user exist in his team
		if (!$model->userExistInTeam($id, $team_id))
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_PROBLEM_DELETING_MEMBER'), 'warning');

			return false;
		}

		// delete member, we don't remove user completely only we unpublish it cuz has will have many data (peoples, companies .. and we don't want to loose it)
		if (!$model->deleteMember($id))
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_PROBLEM_DELETING_MEMBER'), 'warning');

			return false;
		}

		$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_MYTEAM_MEMBER_DELETED'), 'success');

	}

	/*
	 * Delete invite
	 */
	public function deleteInvite()
	{
		// check CSRF
		Session::checkToken('get') or die('Invalid Token');

		// get id
		$id = Factory::getApplication()->input->get('inviteId', 0, 'int');

		// get myteam model
		$model = BaseDatabaseModel::getInstance('Myteam', 'JoomcrmModel', array('ignore_request' => true));

		// check invite id
		if ($id == 0)
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_PROBLEM_DELETING_INVITE'), 'warning');

			return false;
		}

		// check if team owner has a team
		if (!$model->isTeamOwner(Factory::getApplication()->getIdentity()->id))
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_MYTEAM_NOT_ALLOWED'), 'warning');

			return false;
		}

		// get team id
		$team_id = JoomcrmHelperUsers::getTeamId();

		// remove invite from database
		$model->removeInvite($id, $team_id);

		// return to team view
		$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_MYTEAM_INVITE_REMOVED'), 'success');

	}

	// resend email to invited member
	public function resendEmail()
	{

		// check CSRF
		Session::checkToken('get') or die('Invalid Token');

		// get id
		$id = Factory::getApplication()->input->get('inviteId', 0, 'int');

		// get myteam model
		$model = BaseDatabaseModel::getInstance('Myteam', 'JoomcrmModel', array('ignore_request' => true));

		// check invite id
		if ($id == 0)
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_PROBLEM_RESENDING_INVITE'), 'warning');

			return false;
		}

		// check if team owner has a team
		if (!$model->isTeamOwner(Factory::getApplication()->getIdentity()->id))
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_MYTEAM_NOT_ALLOWED'), 'warning');

			return false;
		}

		// get team id
		$team_id = JoomcrmHelperUsers::getTeamId();

		// get invite
		$invite = $model->isMyInvite($id, $team_id);

		// check if is my invite
		if (!$invite)
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_MYTEAM_NOT_ALLOWED'), 'warning');

			return false;
		}

		// build data for email
		$data['invite_id'] = $invite->id;
		$data['token']     = $invite->token;
		$data['team_name'] = JoomcrmHelperUsers::getTeamName($invite->team_id); // get team name
		$data['email']     = $invite->email;

		// check sending email, status is success cuz invite was saved there is only problem in sending email , owner of team can resend it again later in pending users tab of my team view
		if (!$this->sendInviteEmail($data))
		{
			$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_PROBLEM_RESENDING_INVITE'), 'warning');
		}

		// success return
		$this->setRedirect(Route::_('index.php?option=com_joomcrm&view=myteam'), Text::_('COM_JOOMCRM_MYTEAM_EMAIL_RESENT'), 'success');

	}

	private function sendInviteEmail($data)
	{

		$config = Factory::getConfig();

		$mailer = Factory::getMailer();

		// Assemble the invitation accept link.
		$link = "index.php?option=com_joomcrm&task=myteam.acceptInvite&token={$data['token']}&inviteId={$data['invite_id']}";
		$mode = $config->get('force_ssl', 0) == 2 ? 1 : (-1);

		// Put together the email template data.
		$data['fromname']  = $config->get('fromname');
		$data['mailfrom']  = $config->get('mailfrom');
		$data['sitename']  = $config->get('sitename');
		$data['link_text'] = Route::_($link, false, $mode);
		$data['link_html'] = Route::_($link, true, $mode);

		$subject = Text::sprintf(
			'COM_JOOMCRM_INVITE_SUBJECT',
			$data['team_name'],
			$data['sitename']
		);

		$body = Text::sprintf(
			'COM_JOOMCRM_INVITE_BODY',
			$data['team_name'],
			$data['link_text']
		);

		// build mail
		$mailer->setSubject($subject);
		$mailer->setBody($body);
		$mailer->addRecipient($data['email']);
		$mailer->isHTML(true);
		$mailer->setFrom($data['mailfrom'], $data['fromname']);

		// Send the password reset request email.
		return $mailer->send();

	}

	private function sendAcceptedInviteEmail($data)
	{

		$config = Factory::getConfig();
		$mailer = Factory::getMailer();


		// Put together the email template data.
		$data['fromname'] = $config->get('fromname');
		$data['mailfrom'] = $config->get('mailfrom');
		$data['sitename'] = $config->get('sitename');

		$subject = Text::_('COM_JOOMCRM_ACCEPTED_INVITE_SUBJECT');

		$body = Text::sprintf(
			'COM_JOOMCRM_ACCEPTED_INVITE_BODY',
			$data['invited_email']
		);

		// build mail
		$mailer->setSubject($subject);
		$mailer->setBody($body);
		$mailer->addRecipient($data['team_owner_email']);
		$mailer->isHTML(true);
		$mailer->setFrom($data['mailfrom'], $data['fromname']);

		// Send the password reset request email.
		return $mailer->send();

	}

	private function generateToken()
	{
		return md5(uniqid(rand(), true));
	}

	/*
	 * Return json request
	 */
	private function returnRequest($data)
	{

		$return['status'] = $data[0];

		// load alert message
		$return['message'] = LayoutHelper::render('common.ajaxalert', ['data' => $data]);

		echo json_encode($return);
		jexit();

	}


	/*
	 * Create current user team
	 */
	public function autoCreateTeam()
	{
		// inits
		$app = Factory::getApplication();
		$allowTeamCreate = ComponentHelper::getParams('com_joomcrm')->get('allow_team_create', 0);
		$canManageTeam = Factory::getApplication()->getIdentity()->authorise('core.manage.own.team', 'com_joomcrm');

		// if user is not logged
		if (Factory::getApplication()->getIdentity()->id == 0)
		{
			$app->enqueueMessage(Text::_('COM_JOOMCRM_USER_WITH_NO_TEAM'), 'warning');
			$app->redirect(JoomcrmHelperConfig::getLoginRedirect());
		}



		// if user not allowed to create team/manage team (ACL+feature enabled)
		if(!$allowTeamCreate && !$canManageTeam){
			$app->enqueueMessage(Text::_('COM_JOOMCRM_USER_WITH_NO_TEAM'), 'warning');
			$app->redirect(JoomcrmHelperConfig::getLoginRedirect());
		}

		// refresh current user info
		$cuser = JoomcrmHelperUsers::getLoggedInUser();

		// if user not a part of any team
		if($cuser == null || $cuser == 0){

			// Get an instance of the generic users model
			BaseDatabaseModel::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_joomcrm/model', 'JoomcrmModel');
			$usersModel = BaseDatabaseModel::getInstance('Users', 'JoomcrmModel');

			// check if current user group already has a team.
			$firstname = empty(Factory::getApplication()->getIdentity()->name) ? Factory::getApplication()->getIdentity()->username : Factory::getApplication()->getIdentity()->name;

			// create user team with manager
			$usersModel->store([
				'uid'         => Factory::getApplication()->getIdentity()->id,
				'first_name'  => $firstname,
				'last_name'   => '',
				'role_type'   => 'manager',
				'team_name'   => $firstname,
				'team_access' => 'on',
				'exports'     => 'on',
				'can_delete'  => 'on',
				'color'       => ''
			]);

			$app->redirect(Route::_('index.php?option=com_joomcrm&view=dashboard'));
		}



	}


}
