<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/


use Joomla\CMS\Factory;


defined('_JEXEC') or die;

/**
 * Class for various activity functions
 */
abstract class JoomcrmHelperActivity
{
	public static $limit = null;
	public static $type = null;
	public static $type_id = null;

	/**
	 * Save User Activity
	 * @param  Array $old_info    [description]
	 * @param  Array $new_info    [description]
	 * @param  Object $model       [description]
	 * @param  String $action_type [description]
	 */
	public static function saveActivity($old_info, $new_info, $model, $action_type)
	{
		$db      = Factory::getDBO();
		$user_id = JoomcrmHelperUsers::getUserId();

        $model = $model == 'people' ? 'person' : $model;

		if ($action_type == 'created' || $action_type == "deleted" || $action_type == "uploaded")
		{

            $activity = new stdClass();
            $activity->type = $model;
            $activity->type_id = (int) $new_info->id;
            $activity->user_id = (int) $user_id;
            $activity->date = Factory::getDate()->toSql();
            $activity->new_value = (int) $new_info->id;
            $activity->action_type = $action_type;
            $activity->field = 'id';

            $db->insertObject('#__joomcrm_history',$activity,'id');

		}
		else
		{
			$differences = self::recursive_array_diff((array) $old_info, (array) $new_info);

			if (count($differences) > 0)
			{
				foreach ($differences as $key => $old_value)
				{
                    if(empty($new_info->$key))
                        continue;

                    $activity = new stdClass();
                    $activity->type = $model;
                    $activity->type_id = (int) $new_info->id;
                    $activity->user_id = (int) $user_id;
                    $activity->date = Factory::getDate()->toSql();
                    $activity->old_value = $old_value;
                    $activity->new_value = $new_info->$key;
                    $activity->action_type = $action_type;
                    $activity->field = $key;

                    $db->insertObject('#__joomcrm_history',$activity,'id');


				}
			}
		}
	}

	public static function saveUserLoginHistory()
	{
		$user_id = JoomcrmHelperUsers::getUserId();

		// If there is a currently logged in user.
		if ($user_id > 0)
		{
			$db      = Factory::getDbo();
			$query   = $db->getQuery(true);
			$today   = Factory::getDate()->format('Y-m-d');

			$query
				->select("COUNT(id)")
				->from("#__joomcrm_login_history")
				->where("date = '" . $today . "'")
				->where("user_id=" . $user_id);

			$existing = $db->setQuery($query)->loadResult();


			if (!$existing)
			{
				$query
					->clear()
					->insert("#__joomcrm_login_history")
					->set("user_id=" . $user_id . ",date = '" . $today . "'");

				$db->setQuery($query)->execute();
			}
		}
	}

	public static function getActivity()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		$db->setQuery('SET SQL_BIG_SELECTS = 1');
		$db->execute();

		$query
			->clear()
			->select('h.*, CONCAT(u.first_name," ", u.last_name) AS owner_name, c.name as company_name, CONCAT(p.first_name," ", p.last_name) AS person_name,
						d.name as deal_name, e.name as event_name, note_cat.name as notes_category_name,
						event_cat.name as events_category_name, old_event_cat.name as events_category_name_old, old_note_cat.name AS notes_category_name_old,
						doc.name AS document_name,status.name AS deal_status_name_old, status2.name AS deal_status_name,deal_source.name AS deal_source_name_old,deal_source_2.name AS deal_source_name,
						deal_stage.name AS deal_stage_name_old,deal_stage_2.name AS deal_stage_name,CONCAT(deal_owner.first_name," ",deal_owner.last_name) AS deal_owner_name_old,
						CONCAT(deal_owner_2.first_name," ",deal_owner_2.last_name) AS deal_owner_name');

		if (JoomcrmHelperQuote::hasCRMQuote())
		{
			JoomcrmHelperQuote::buildActivitySelect($query);
		}

		$query
			->from('#__joomcrm_history AS h')
			->leftJoin('#__joomcrm_users AS u ON u.id = h.user_id')
			->leftJoin('#__joomcrm_companies AS c ON c.id = h.type_id AND h.type="company"')
			->leftJoin('#__joomcrm_notes AS n ON n.id = h.type_id AND h.type="note"')
			->leftJoin('#__joomcrm_deals AS d on d.id = h.type_id AND h.type="deal"')
			->leftJoin('#__joomcrm_people AS p on p.id = h.type_id AND h.type="person"')
			->leftJoin('#__joomcrm_goals AS g on g.id = h.type_id AND h.type="goal"')
			->leftJoin('#__joomcrm_events AS e on e.id = h.type_id AND h.type="event"')
			->leftJoin("#__joomcrm_events_cf AS ecf on e.id = ecf.event_id")
			->leftJoin('#__joomcrm_reports AS r on r.id = h.type_id AND h.type="report"')
			->leftJoin('#__joomcrm_documents AS doc ON doc.id = h.type_id AND h.type="document"')
			->leftJoin('#__joomcrm_notes_categories as note_cat ON note_cat.id = h.new_value AND h.field="category_id" AND h.type="notes"')
			->leftJoin('#__joomcrm_events_categories as event_cat ON event_cat.id = h.new_value AND h.field="category_id" AND h.type="events"')
			->leftJoin('#__joomcrm_notes_categories as old_note_cat ON old_note_cat.id = h.old_value AND h.field="category_id" AND h.type="notes"')
			->leftJoin('#__joomcrm_events_categories as old_event_cat ON old_event_cat.id = h.old_value AND h.field="category_id" AND h.type="events"')
			->leftJoin("#__joomcrm_deal_status AS status ON status.id = h.old_value AND h.type='deal'")
			->leftJoin("#__joomcrm_deal_status AS status2 ON status2.id = h.new_value AND h.type='deal'")
			->leftJoin("#__joomcrm_sources AS deal_source ON deal_source.id = h.old_value AND h.type='deal'")
			->leftJoin("#__joomcrm_sources AS deal_source_2 ON deal_source_2.id = h.new_value AND h.type='deal'")
			->leftJoin("#__joomcrm_stages AS deal_stage ON deal_stage.id = h.old_value AND h.type='deal'")
			->leftJoin("#__joomcrm_stages AS deal_stage_2 ON deal_stage_2.id = h.new_value AND h.type='deal'")
			->leftJoin("#__joomcrm_users AS deal_owner ON deal_owner.id = h.old_value AND h.type='deal'")
			->leftJoin("#__joomcrm_users AS deal_owner_2 ON deal_owner_2.id = h.new_value AND h.type='deal'");

		if (JoomcrmHelperQuote::hasCRMQuote())
		{
			JoomcrmHelperQuote::buildActivityJoin($query);
		}

		if (isset(self::$type) && strlen(self::$type) > 0 && isset(self::$type_id) && self::$type_id > 0)
		{
			if (self::$type == "quote")
			{
				JoomcrmHelperQuote::buildActivityWhere($query, self::$type_id);
			}
			else
			{
				$query->where("h.type_id=" . self::$type_id);
				$query->where("h.type='" . self::$type . "'");
			}
		}

		$member_id   = JoomcrmHelperUsers::getUserId();
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();

		if ($member_role != 'exec')
		{
			// Manager filter
			if ($member_role == 'manager')
			{
				$query->where('u.team_id = ' . $team_id);
			}
			// Basic user filter
			else
			{
				$query->where(array('h.user_id = ' . $member_id));
			}
		}

		// TODO: Add assignees to the display (massive left join)
		$query->where('h.field!="assignee_id" AND h.field!="repeats"');
		$query->order('h.date DESC');

		if (!isset(self::$type) && !isset(self::$type_id))
		{
			$typeId =  Factory::getApplication()->input->get('id');
			$type   =  Factory::getApplication()->input->get('layout');

			if ($typeId && $type)
			{
				switch ($type)
				{
					case "deal":
						$query->where("( (h.type_id=" . $typeId . " AND h.type='deal' ) OR
	        							 (n.deal_id=" . $typeId . ") OR
	        							 (ecf.association_id=" . $typeId . " AND ecf.association_type='deal') OR
	        							 (doc.association_id=" . $typeId . " AND doc.association_type='deal')
	        							)");
						break;
					case "company":
						$query->where("( (h.type_id=" . $typeId . " AND h.type='company' ) OR
	        							 (n.company_id=" . $typeId . ") OR
	        							 (ecf.association_id=" . $typeId . " AND ecf.association_type='company') OR
	        							 (doc.association_id=" . $typeId . " AND doc.association_type='company')
	        							)");
						break;
					case "person":
						$query->where("( (h.type_id=" . $typeId . " AND h.type='person' ) OR
	        							 (n.person_id=" . $typeId . ") OR
	        							 (ecf.association_id=" . $typeId . " AND ecf.association_type='person') OR
	        							 (doc.association_id=" . $typeId . " AND doc.association_type='person')
	        							)");
						break;
				}
			}
		}

		if (self::$limit != null)
		{
			$query .= " LIMIT " . self::$limit;
		}
		else
		{
			$query .= " LIMIT 10";
		}

		return $db->setQuery($query)->loadObjectList();
	}

	public static function recursive_array_diff($a1, $a2)
	{
		$r = array();

		foreach ($a1 as $k => $v)
		{
			if ($k != '_' && $k != 'modified')
			{
				if (array_key_exists($k, $a2))
				{
					if (is_array($v))
					{
						$rad = self::recursive_array_diff($v, $a2[$k]);
						
						if (count($rad))
						{
							$r[$k] = $rad;
						}
					}
					else
					{
						if ($v != $a2[$k])
						{
							$r[$k] = $v;
						}
					}
				}
				else
				{
					$r[$k] = $v;
				}
			}
		}

		return $r;
	}
}
