<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

abstract class JoomcrmHelperCompany
{
	// Get a specific
	public static function getCompany($id)
	{
		// Get db object
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select('name, id FROM #__joomcrm_companies')->where('id = ' . $id);

		return $db->setQuery($query)->loadAssocList();
	}

	// Get company filter types
	public static function getTypes()
	{
		return array('all' => Text::_('COM_JOOMCRM_ALL_COMPANIES'), 'today' => Text::_('COM_JOOMCRM_COMPANIES_TASKS_TODAY'), 'tomorrow' => Text::_('COM_JOOMCRM_COMPANIES_TASKS_TOMORROW'), 'updated_thirty' => Text::_('COM_JOOMCRM_COMPANIES_UPDATED_LAST_MONTH'), 'recent' => Text::_('COM_JOOMCRM_RECENTLY_ADDED'), 'past' => Text::_('COM_JOOMCRM_CONTACTED_LONG_AGO'));
	}

	// Get column filters
	public static function getColumnFilters()
	{
		return array('description' => ucwords(Text::_('COM_JOOMCRM_EDIT_TASK_DESCRIPTION')), 'owner' => ucwords(Text::_('COM_JOOMCRM_COMPANIES_OWNER')), 'phone' => ucwords(Text::_('COM_JOOMCRM_PEOPLE_PHONE')), 'fax' => ucwords(Text::_('COM_JOOMCRM_COMPANY_FAX')), 'email' => ucwords(Text::_('COM_JOOMCRM_COMPANY_EMAIL')), 'address' => ucwords(Text::_('COM_JOOMCRM_PERSON_ADDRESS')), 'country' => ucwords(Text::_('COM_JOOMCRM_PEOPLE_COUNTRY')), 'next_task' => ucwords(Text::_('COM_JOOMCRM_PEOPLE_TASK')), 'notes' => ucwords(Text::_('COM_JOOMCRM_PEOPLE_NOTES')), 'added' => ucwords(Text::_('COM_JOOMCRM_COMPANIES_ADDED')), 'updated' => ucwords(Text::_('COM_JOOMCRM_COMPANIES_UPDATED')));
	}

	// Get selected column filters
	public static function getSelectedColumnFilters()
	{
		// Get the user session data
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("companies_columns")->from("#__joomcrm_users")
				 ->where('id = ' . JoomcrmHelperUsers::getUserId());

		$results = $db->setQuery($query)->loadResult();

		// Unserialize columns
		$columns =  !is_null($results) ? unserialize($results) : [];

		if (is_array($columns) && !empty($columns))
		{
			return $columns;
		}
		else
		{
			// If it is empty then load a default set
			return JoomcrmHelperCompany::getDefaultColumnFilters();
		}
	}

	// Get default column filters
	public static function getDefaultColumnFilters()
	{
		return array('avatar', 'phone', 'notes', 'added', 'updated', 'category');
	}

	public static function getCategories()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("c.id,c.name")->from("#__joomcrm_company_categories AS c")
				 ->order("c.ordering");

		$categories = $db->setQuery($query)->loadObjectList();

		$return = array('0' => \Joomla\CMS\Language\Text::_('COM_JOOMCRM_ANY_CATEGORY'));

		if (count($categories) > 0)
		{
			foreach ($categories as $category)
			{
				$return[$category->id] = $category->name;
			}
		}

		return $return;
	}

	public static function getCategoryDropdown($selected = null, $name = 'category_id', $class = 'class="inputbox"')
	{
		$categories = self::getCategories();
		$options    = array();
		$arr        = array();

		if (count($categories) > 0)
		{
			foreach ($categories as $id => $category)
			{
				$options[$id] = $category;
			}
		}

		foreach ($options as $value => $label)
		{
			$arr[] = HTMLHelper::_('select.option', $value, $label);
		}

		return HTMLHelper::_('select.genericlist', $arr, $name, $class, 'value', 'text', $selected);
	}

	public static function categoryExist($value){

		// inits
		$db = Factory::getDbo();

		// if is numeric filter by id , if not filter by name
		$whereKey = !is_numeric($value) ? 'name' : 'id';
		// if is numeric make sure value is string, if not make sure value is integer
		$value = !is_numeric($value) ? $db->quote($value) : (int) $value;

		$query = $db->getQuery(true);
		$query->select('id')->from('#__joomcrm_company_categories')->where("$whereKey = $value");
		$db->setQuery($query);
		$result = $db->loadResult();

		return is_null($result) ? 0 : (int) $result;
	}

	/*
	 * This method add new category if $value is the name of category and doesn't exist yet (only for import)
	 */
	public static function updateCategoryForImport($value){

		// check if category exist
		$catId = self::categoryExist($value);

		// cat id exist
		if($catId > 0)
			return $catId;

		// cat id doesn't exist and $value is name of category add this new category
		if($catId == 0 && !is_numeric($value))
			return self::newCategory($value);

		return 0;
	}

	public static function newCategory($name){

		$newCategory = new stdClass();
		$newCategory->name = $name;
		$newCategory->created = Factory::getDate()->toSql();
		$newCategory->modified = Factory::getDate()->toSql();

		Factory::getDbo()->insertObject('#__joomcrm_company_categories',$newCategory,'id');

		return (int) Factory::getDbo()->insertid();


	}

}
