<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

abstract class JoomcrmHelperCustom
{
	public static function generateCustom($type, $id = null)
	{
		// Base html
		$return = array();

		// Grab db
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("cf.* FROM #__joomcrm_" . $type . "_custom AS cf")->order("cf.ordering");

		$row = $db->setQuery($query)->loadAssocList();

		// Determine selected values
		if ($id)
		{
			$custom_data = self::getCustomData($id, $type);
		}

		if (is_array($row) && count($row) > 0)
		{
			// Loop for explosion delims
			foreach ($row as $custom)
			{
				// Retrieve custom values
				$custom['values'] = json_decode($custom['values']);

				// Determine selected values
				if ($id && $custom['type'] != 'forecast')
				{
					$custom['selected'] = (array_key_exists($custom['id'], $custom_data)) ? $custom_data[$custom['id']] : '';// Text::_('COM_JOOMCRM_CLICK_TO_EDIT');
				}

				// Append items to array
				$return[] = $custom;
			}
		}

		return $return;
	}

	public static function getCustomFields($type)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("cf.*")->from("#__joomcrm_" . $type . "_custom AS cf");

		$results = $db->setQuery($query)->loadObjectList("name");

		$fields = array();

		if (count($results) > 0)
		{
			foreach ($results as $key => $field)
			{
				$fields[strtolower($key)] = $field;

				$field->values = json_decode($field->values);
			}
		}

		return $fields;
	}

	public static function getCustomCsvData($itemId, $itemType)
	{
		switch ($itemType)
		{
			case "people":
				$table = "people";
				break;
			case "companies":
				$table = "company";
				break;
			case "deals":
				$table = "deal";
				break;
			default:
				$table = $itemType;
				break;
		}

		return self::getCustomData($itemId, $table, "name", "values", true, true);
	}

	// Get custom data to prefill dropdowns
	public static function getCustomData($id, $type, $key = "id", $label = "keys", $delimitArray = false, $displayData = false)
	{

		$task = Factory::getApplication()->input->get('task', '', 'word');

		// Get dbo
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
			->select("cf.*,f.name,f.values,f.type,f.multiple_selections,cf.custom_field_id FROM #__joomcrm_" . $type . "_custom AS f")
			->leftJoin("#__joomcrm_" . $type . "_custom_cf AS cf ON f.id=cf.custom_field_id AND cf." . $type . "_id=" . $id);

		$db_results = $db->setQuery($query)->loadAssocList();

		$results = array();

		if (count($db_results) != 0)
		{
			foreach ($db_results as $id => $row)
			{
				switch ($key)
				{
					case "id":
						$keyName = $row['custom_field_id'];
						break;
					case "name":
						$keyName = strtolower($row['name']);
						break;
					default:
						$keyName = $key;
						break;
				}

				if ($row[$type . '_id'] > 0)
				{
					$valueData = self::getCustomValue($type, $row['custom_field_id'], $row['value'], null, $displayData);
				}
				else
				{
					$valueData = $row['multiple_selections'] == 1 ? array() : "";
				}

				if ($delimitArray)
				{
					$values = $row['multiple_selections'] == 1 ? implode(",", ($valueData)) : $valueData;
				}
				else
				{
					$values = $valueData;
				}

				$results[$keyName] = $values;

			}
		}

		return $results;
	}

	/**
	 * Get custom field values from picklists // forecasts // otherwise return the value as it was an input field
	 */
	public static function getCustomValue($customType, $customNameOrId, $customValue, $itemId = null, $displayValues = false)
	{
		$db    = Factory::getDBO();
		$query = $db->getQuery(true);

		$id = strpos($customNameOrId, "custom_") !== false ? str_replace("custom_", "", $customNameOrId) : $customNameOrId;

		$query
			->select("c.name,c.type,c.values,c.multiple_selections")
			->from("#__joomcrm_" . $customType . "_custom AS c")
			->where("c.id=" . $id);

		$custom = $db->setQuery($query)->loadObject();

		switch ($custom->type)
		{
			case "forecast":
				if ($itemId > 0)
				{
					$query
						->clear()
						->select("(d.amount * ( d.probability / 100 )) AS amount")
						->from("#__joomcrm_deals AS d")
						->where("d.id=" . $itemId);

					return $db->setQuery($query)->loadResult();
				}
				else
				{
					return $customValue;
				}
				break;
			case "currency":
				return $customValue;
				break;
			case "picklist":

				$values = json_decode($custom->values);

				if ($custom->multiple_selections == 0)
				{
					$key      = array_key_exists($customValue, $values) ? $values[$customValue] : Text::_('COM_JOOMCRM_NONE');
					$valueKey = array_search($customValue, $values);

					if ($displayValues)
					{
						return array_key_exists($customValue, $values) ? $values[$customValue] : "";
					}
					else
					{
						return $key;
					}
				}
				else
				{
					if (!is_array($customValue))
					{
						$unserialized = @unserialize($customValue);
					}

					// if was unserialized successufully
					if ($unserialized)
						$customValue = $unserialized;

					// check if customValue was not unserialized successfully and has comma separator then explode it and update it
					if (!$unserialized)
					{

						$listValues = explode(',', $customValue);
						if (count($listValues) > 0)
							$customValue = self::updatePickListIndexes($values, $listValues); // update to match picklist keys with custom values keys
					}

					$keys      = array();
					$keyValues = array();

					if (is_array($customValue))
					{
						foreach ($customValue as $key => $value)
						{
							if (array_key_exists($value, $values) && $value == 1)
							{
								$keys[$key] = 1;
							}

							if (in_array($value, $customValue))
							{
								$keyValues[$key] = $value;
							}
						}
					}

					if ($displayValues == true)
					{
						$returnValues = array();

						foreach ($keyValues as $valueKey => $toggled)
						{
							if ($toggled == 1 && array_key_exists($valueKey, $values))
							{
								$returnValues[] = $values[$valueKey];
							}
						}

						return $returnValues;
					}
					else
					{
						return count($keys) > 0 ? $keys : $keyValues;
					}
				}
				break;
			case "date":
				return $customValue;
				break;
			default:
				return $customValue;
				break;
		}

		return $customValue;
	}

	public static function updatePickListIndexes($fieldValues, $values)
	{

		$newArray = [];


		foreach ($fieldValues as $fkey => $fvalue)
		{

			foreach ($values as $value)
			{

				if ($value == $fvalue)
					$newArray[$fkey] = 1;


			}


		}


		return $newArray;

	}

	public static function cleanCustomForm($data)
	{
		if (count($data) > 0)
		{
			foreach ($data as $key => $field)
			{
				if ($field['type'] == "forecast")
				{
					unset($data[$key]);
				}
				else
				{
					$data[$key]['display'] = $field['name'];
					$data[$key]['name']    = "custom_" . $field['id'];
				}
			}
		}

		return $data;
	}

	public static function getCustomTypes($type)
	{
		switch ($type)
		{
			case "deal":
				$arr = [
					'number'   => Text::_('COM_JOOMCRM_NUMBER'),
					'text'     => Text::_('COM_JOOMCRM_ADMIN_GENERIC_TEXT'),
					'textarea'     => Text::_('COM_JOOMCRM_ADMIN_GENERIC_TEXTAREA'),
					'currency' => Text::_('COM_JOOMCRM_CURRENCY'),
					'picklist' => Text::_('COM_JOOMCRM_PICKLIST'),
					'forecast' => Text::_('COM_JOOMCRM_FORECAST'), 'date' => Text::_('COM_JOOMCRM_DATE')
				];
				break;
			case "company":
				$arr = [
					'number'   => Text::_('COM_JOOMCRM_NUMBER'),
					'text'     => Text::_('COM_JOOMCRM_ADMIN_GENERIC_TEXT'),
					'textarea'     => Text::_('COM_JOOMCRM_ADMIN_GENERIC_TEXTAREA'),
					'currency' => Text::_('COM_JOOMCRM_CURRENCY'),
					'picklist' => Text::_('COM_JOOMCRM_PICKLIST'),
					'date'     => Text::_('COM_JOOMCRM_DATE'),
					'person'   => Text::_('COM_JOOMCRM_PERSON')
				];
				break;
			case "people":
				$arr = ['number'   => Text::_('COM_JOOMCRM_NUMBER'),
				        'text'     => Text::_('COM_JOOMCRM_ADMIN_GENERIC_TEXT'),
				        'textarea'     => Text::_('COM_JOOMCRM_ADMIN_GENERIC_TEXTAREA'),
				        'currency' => Text::_('COM_JOOMCRM_CURRENCY'),
				        'picklist' => Text::_('COM_JOOMCRM_PICKLIST'),
				        'date'     => Text::_('COM_JOOMCRM_DATE'),
				        'company'  => Text::_('COM_JOOMCRM_COMPANY')
				];
				break;
		}

		return $arr;
	}

	public static function isReported($reportId)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("reported")->from("#__joomcrm_deal_custom")->where("id=" . $reportId);

		return $db->setQuery($query)->loadResult();
	}
}
