<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;

abstract class JoomcrmHelperDate
{
	/**
	 * Get timezone dropdown info for profile
	 */
	public static function getTimezones()
	{
		$list  = timezone_identifiers_list();
		$zones = array();

		foreach ($list as $zone)
		{
			$zones[$zone] = $zone;
		}

		return $zones;
	}

	/**
	 * Get date format for profile
	 * @return Array An array of date options
	 */
	public static function getDateFormats()
	{
		return array('m/d/y' => 'mm/dd/yy', 'd/m/y' => 'dd/mm/yy');
	}

	/**
	 * Get time formates for profile
	 */
	public static function getTimeFormats()
	{
		return array('g:i A' => '7:30 PM', 'H:i' => '19:30');
	}

	/**
	 * Get goal dates for filtering
	 */
	public static function getGoalDates()
	{
		return array('this_week'    => Text::_("COM_JOOMCRM_THIS_WEEK"),
		             'next_week'    => Text::_("COM_JOOMCRM_NEXT_WEEK"),
		             'this_month'   => Text::_("COM_JOOMCRM_THIS_MONTH"),
		             'next_month'   => Text::_("COM_JOOMCRM_NEXT_MONTH"),
		             'this_quarter' => Text::_("COM_JOOMCRM_THIS_QUARTER"),
		             'next_quarter' => Text::_("COM_JOOMCRM_NEXT_QUARTER"),
		             'this_year'    => Text::_("COM_JOOMCRM_THIS_YEAR"), 'custom' => Text::_("COM_JOOMCRM_CUSTOM"));
	}

	public static function getRangeNames()
	{
		return array('all'            => strtolower(Text::_('COM_JOOMCRM_ALL')),
		             'this_month'     => strtolower(Text::_('COM_JOOMCRM_THIS_MONTH')),
		             'this_quarter'   => strtolower(Text::_('COM_JOOMCRM_THIS_QUARTER')),
		             'next_month'     => strtolower(Text::_('COM_JOOMCRM_NEXT_MONTH')),
		             'next_quarter'   => strtolower(Text::_('COM_JOOMCRM_NEXT_QUARTER')),
		             'year_remainder' => strtolower(Text::_('COM_JOOMCRM_THIS_YEAR')));
	}

	/**
	 * Get Created dates for filtering
	 */
	public static function getCreatedDates()
	{
		return array(
			'all'        => strtolower(Text::_('COM_JOOMCRM_ALL')),
			'this_week'  => Text::_("COM_JOOMCRM_THIS_WEEK"),
			'last_week'  => Text::_("COM_JOOMCRM_LAST_WEEK"),
			'this_month' => Text::_("COM_JOOMCRM_THIS_MONTH"),
			'last_month' => Text::_("COM_JOOMCRM_LAST_MONTH"),
			'today'      => Text::_("COM_JOOMCRM_TODAY"),
			'yesterday'  => Text::_("COM_JOOMCRM_YESTERDAY")
		);
	}

	/**
	 * Get the weeks in a month for report page data creation
	 */
	public static function getWeeksInMonth($current_month)
	{
		$month      = intval(date('m', strtotime($current_month))); // Force month to single integer if '0x'
		$year       = intval(date('Y', strtotime($current_month)));
		$suff       = array('st', 'nd', 'rd', 'th', 'th', 'th'); //week suffixes
		$end        = date('t', mktime(0, 0, 0, $month, 1, $year)); // Last date day of month: 28 - 31
		$start      = date('w', mktime(0, 0, 0, $month, 1, $year)); //1st day of month: 0 - 6 (Sun - Sat)
		$last       = 7 - $start; // Get last day date (Sat) of first week
		$noweeks    = ceil((($end - ($last + 1)) / 7) + 1); // Total no. weeks in month
		$output     = ""; // Initialize string
		$weeks      = array();
		$monthlabel = str_pad($month, 2, '0', STR_PAD_LEFT);

		for ($x = 1; $x < $noweeks + 1; $x++)
		{
			if ($x == 1)
			{
				$startdate = "$year-$monthlabel-01";
				$day       = $last - 6;
			}
			else
			{
				$day       = $last + 1 + (($x - 2) * 7);
				$day       = str_pad($day, 2, '0', STR_PAD_LEFT);
				$startdate = "$year-$monthlabel-$day";
			}

			if ($x == $noweeks)
			{
				$enddate = "$year-$monthlabel-$end";
			}
			else
			{
				$dayend  = $day + 6;
				$dayend  = str_pad($dayend, 2, '0', STR_PAD_LEFT);
				$enddate = "$year-$monthlabel-$dayend";
			}
			$output  .= "{$x}{$suff[$x - 1]} week -> Start date=$startdate End date=$enddate <br />";
			$weeks[] = array('start_date' => $startdate, 'end_date' => $enddate);
		}

		return $weeks;
	}

	/**
	 * Get month names for report page charts and data creation
	 */
	public static function getMonthNames()
	{
		return array(
			Text::_('COM_JOOMCRM_JANUARY'),
			Text::_('COM_JOOMCRM_FEBRUARY'),
			Text::_('COM_JOOMCRM_MARCH'),
			Text::_('COM_JOOMCRM_APRIL'),
			Text::_('COM_JOOMCRM_MAY'),
			Text::_('COM_JOOMCRM_JUNE'),
			Text::_('COM_JOOMCRM_JULY'),
			Text::_('COM_JOOMCRM_AUGUST'),
			Text::_('COM_JOOMCRM_SEPTEMBER'),
			Text::_('COM_JOOMCRM_OCTOBER'),
			Text::_('COM_JOOMCRM_NOVEMBER'),
			Text::_('COM_JOOMCRM_DECEMBER')
		);
	}

	/**
	 * Get abbreviated month names for report page charts and data creation
	 */
	public static function getMonthNamesShort($startDate = null, $endDate = null)
	{
		$dates = array(
			Text::_('COM_JOOMCRM_JAN'),
			Text::_('COM_JOOMCRM_FEB'),
			Text::_('COM_JOOMCRM_MAR'),
			Text::_('COM_JOOMCRM_APR'),
			Text::_('COM_JOOMCRM_MAY'),
			Text::_('COM_JOOMCRM_JUN'),
			Text::_('COM_JOOMCRM_JUL'),
			Text::_('COM_JOOMCRM_AUG'),
			Text::_('COM_JOOMCRM_SEP'),
			Text::_('COM_JOOMCRM_OCT'),
			Text::_('COM_JOOMCRM_NOV'),
			Text::_('COM_JOOMCRM_DEC')
		);

		$startDate = $startDate == null ? date("Y-01-01") : $startDate;
		$endDate   = $endDate == null ? date("Y-12-31") : $endDate;

		$total  = self::getMonths($startDate, $endDate);
		$return = array();

		for ($i = 0; $i < $total; $i++)
		{
			$key      = ((int) date("m", strtotime("$startDate + $i months")) - 1) % 12;
			$return[] = $dates[$key];
		}

		return $return;
	}

	/**
	 * Get month start and end dates and names for report page charts and data creation
	 */
	public static function getMonthDates($startDate = null, $endDate = null)
	{
		$startDate   = $startDate == null ? date('Y-01-01 00:00:00') : $startDate;
		$endDate     = $endDate == null ? date('Y-12-31 00:00:00') : $endDate;
		$month_names = JoomcrmHelperDate::getMonthNames();
		$months      = array();

		$total = self::getMonths($startDate, $endDate);

		for ($i = 0; $i < $total; $i++)
		{
			$key        = ((int) date("m", strtotime("$startDate + $i months")) - 1) % 12;
			$months[$i] = array('name' => $month_names[$key],
			                    'date' => date('Y-m-d 00:00:00', strtotime("$startDate + $i months")));
		}

		return $months;
	}

	public static function getMonths($start, $end)
	{
		$startParsed = date_parse_from_format('Y-m-d', $start);
		$startMonth  = $startParsed['month'];
		$startYear   = $startParsed['year'];

		$endParsed = date_parse_from_format('Y-m-d', $end);
		$endMonth  = $endParsed['month'];
		$endYear   = $endParsed['year'];

		return ($endYear - $startYear) * 12 + ($endMonth - $startMonth) + 1;
	}

	public static function getTimeIntervals()
	{


		$predefined_time_slot = (int) ComponentHelper::getParams('com_joomcrm')->get('predefine_time_slot', 5);

		$starttime = "7:00:00";
		$temptime  = strtotime($starttime);
		$nextday   = strtotime($starttime . " + 1 day");

		$times = array();
		do
		{
			$times[date("H:i:s", $temptime)] = date(JoomcrmHelperUsers::getTimeFormat(), $temptime);
			$temptime                        = date("Y-m-d H:i:s", $temptime + ($predefined_time_slot * 60));
			$temptime                        = strtotime($temptime);
		} while ($temptime < $nextday);

		return $times;

	}

	/**
	 * Method converts date to UTC from user timezone
	 *
	 * @param string  $date
	 * @param boolean $time
	 *
	 * @return  string  $time
	 */
	public static function formatDBDate($date = null, $time = true)
	{
		// missing date = now
		if (!$date)
		{
			$date = date('Y-m-d H:i:s');
		}

		$userTz   = JoomcrmHelperUsers::getTimezone();
		$dateTime = new DateTime($date, new DateTimeZone($userTz));

		$utc = "UTC";
		$dateTime->setTimeZone(new DateTimeZone($utc));

		if (!$time)
		{
			$mysql = "Y-m-d";
		}
		else
		{
			$mysql = "Y-m-d H:i:s";
		}

		$time = $dateTime->format($mysql);

		return $time;
	}

	public static function formatDate($date, $time = false, $useUserDateFormat = true, $convertToUserTimezone = true)
	{

		if(empty($date) || is_null($date))
			return '';

		if (strtotime($date) <= 0)
		{
			return "";
		}

		try
		{
			if ($convertToUserTimezone)
			{
				$dateTime = self::convertDateToUserTimezone($date);
			}
			else
			{
				$dateTime = new DateTime($date);
			}


			$date_format = JoomcrmHelperUsers::getDateFormat();

			// Always use a 4 digit year.
			if (strpos($date_format, 'y'))
			{
				$date_format = str_replace('y', 'Y', $date_format);
			}

			if ($useUserDateFormat)
			{
				$date = $dateTime->format($date_format);
			}
			else
			{
				$date = $dateTime->format("Y-m-d H:i:s");
			}



			if ($time)
			{

				if (!(date("H:i:s", strtotime($date)) == "00:00:00"))
				{

					if (is_string($time))
					{
						$time_format = date($time, strtotime($date));
					}
					else
					{
						$time_format = self::formatTime($date);
					}
					$date .= ' ' . $time_format;
				}
			}


			return $date;
		}
		catch (Exception $e)
		{
			return "";
		}

	}

	public static function formatDateString($date, $fourYear = false)
	{
		if ($fourYear)
		{
			$dateFormat = 'm/d/Y';
		}
		else
		{
			$dateFormat = JoomcrmHelperUsers::getDateFormat();
		}

		return date($dateFormat, strtotime((string)$date));
	}

	public static function formatTimeString($time, $timeFormatOverride = null)
	{
		$timeFormat = $timeFormatOverride ? $timeFormatOverride : JoomcrmHelperUsers::getTimeFormat();

		return date($timeFormat, strtotime($time));
	}

	public static function formatTime($time, $timeFormatOverride = null)
	{


		$exp = explode(" ", $time);
		if (count($exp) < 1)
		{
			$time = "0000-00-00 " . $time;
		}

		$time_format = $timeFormatOverride ? $timeFormatOverride : JoomcrmHelperUsers::getTimeFormat();

		$dateTime    = new DateTime($time,new DateTimeZone('utc'));

		return $dateTime->format($time_format);

	}

	/**
	 * Method converts date from UTC to user timezone
	 *
	 * @param string  $date
	 * @param boolean $time
	 *
	 * @return  string  $time
	 */
	public static function convertDateToUserTimezone($date, $returnString = false)
	{

		$userTz = JoomcrmHelperUsers::getTimezone();


		$utc    = "UTC";

		$dateTime = new DateTime($date, new DateTimeZone($utc));


		$dateTime->setTimezone(new DateTimeZone($userTz));


		return $returnString ? $dateTime->format("Y-m-d H:i:s") : $dateTime;
	}

	public static function getCurrentTime($string = false, $showTime = false)
	{
		$timezone = JoomcrmHelperUsers::getTimezone();
		$current  = new DateTime();
		$current->setTimezone(new DateTimeZone($timezone));

		if ($string)
		{
			$format = $showTime ? "Y-m-d H:i:s" : "Y-m-d";

			return $current->format($format);
		}

		return $current;
	}

	public static function getElapsedTime($date, $showDays = true, $showMonths = true, $showYears = true, $showHours = true, $showMinutes = true, $showSeconds = false)
	{
		$date = self::convertDateToUserTimezone($date, true);
		$time = time() - strtotime($date); // to get the time since that moment

		$tokens = [
			31536000 => 'COM_JOOMCRM_YEAR',
			2592000  => 'COM_JOOMCRM_MONTH',
			604800   => 'COM_JOOMCRM_WEEK',
			86400    => 'COM_JOOMCRM_DAY',
			3600     => 'COM_JOOMCRM_HOUR',
			60       => 'COM_JOOMCRM_MINUTE',
			1        => 'COM_JOOMCRM_SECOND'
		];

		foreach ($tokens as $unit => $text)
		{
			if ($time < $unit)
			{
				continue;
			}
			$numberOfUnits = floor($time / $unit);

			$text = Text::_($text . (($numberOfUnits > 1) ? 'S' : ''));

			return $numberOfUnits . ' ' . $text;
		}

		return self::formatDate($date);
	}

	public static function pluralize($count, $text)
	{
		return $count . (($count == 1) ? (" $text ") : (" $text"."s "));
	}

	public static function x_week_range(&$start_date, &$end_date, $date)
	{
		$ts         = strtotime($date);
		$start      = (date('w', $ts) == 0) ? $ts : strtotime('last sunday', $ts);
		$start_date = date('Y-m-d H:i:s', $start);
		$end_date   = date('Y-m-d H:i:s', strtotime('next saturday', $start));
	}

	public static function getRelativeDate($display_date)
	{
		$now                = self::getCurrentTime(true, false);
		$tomorrow           = self::getCurrentTime(true, false) . " + 1 days";
		$day_after_tomorrow = self::getCurrentTime(true, false) . " + 2 days";

		$this_week = null;
		$next_week = null;
		self::x_week_range($this_week, $next_week, self::getCurrentTime(true, false));

		$week_after_next      = null;
		$week_after_next_week = null;
		self::x_week_range($week_after_next, $week_after_next_week, self::getCurrentTime(true, false) . " + 1 week");

		$next_month      = date("Y-m-1", strtotime(self::getCurrentTime(true, false) . " +1 month"));
		$next_next_month = date("Y-m-1", strtotime(self::getCurrentTime(true, false) . " +2 months"));

		if (strtotime($display_date) < strtotime($now))
		{
			$current_heading = Text::_('COM_JOOMCRM_LATE_ITEMS');
		}
		elseif (strtotime($display_date) >= strtotime($now) && strtotime($display_date) < strtotime($tomorrow))
		{
			$current_heading = Text::_('COM_JOOMCRM_TODAY');
		}
		elseif (strtotime($display_date) >= strtotime($tomorrow) && strtotime($display_date) < strtotime($day_after_tomorrow))
		{
			$current_heading = Text::_('COM_JOOMCRM_TOMORROW');
		}
		elseif (strtotime($display_date) >= strtotime($day_after_tomorrow) && strtotime($display_date) < strtotime($week_after_next))
		{
			$current_heading = Text::_('COM_JOOMCRM_THIS_WEEK');
		}
		elseif (strtotime($display_date) >= strtotime($week_after_next) && strtotime($display_date) < strtotime($week_after_next_week))
		{
			$current_heading = Text::_('COM_JOOMCRM_NEXT_WEEK');
		}
		elseif (strtotime($display_date) >= strtotime($week_after_next_week) && strtotime($display_date) < strtotime($next_month))
		{
			$current_heading = Text::_('COM_JOOMCRM_THIS_MONTH');
		}
		elseif (strtotime($display_date) >= strtotime($next_month) && strtotime($display_date) < strtotime($next_next_month))
		{
			$current_heading = Text::_('COM_JOOMCRM_NEXT_MONTH');
		}
		else
		{
			$current_heading = Text::_('COM_JOOMCRM_IN_THE_FUTURE');
		}

		return $current_heading;
	}
}
