<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;


use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;


abstract class JoomcrmHelperDeal
{
	public static function getDeal($id)
	{
		// Get db object
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("name,id FROM #__joomcrm_deals")->where('id=' . $id);

		return $db->setQuery($query)->loadAssocList();
	}

	/**
	 * function to return filter types for deals
	 *
	 * @return  array  Deal Types
	 */
	public static function getDealTypes()
	{
		return array('all' => Text::_('COM_JOOMCRM_ALL_DEALS'), 'today' => Text::_('COM_JOOMCRM_DEALS_TASKS_TODAY'), 'tomorrow' => Text::_('COM_JOOMCRM_DEALS_TASKS_TOMORROW'), 'updated_thirty' => Text::_('COM_JOOMCRM_DEALS_UPDATED_LAST_MONTH'), 'valuable' => Text::_('COM_JOOMCRM_DEALS_MOST_VALUABLE'), 'past' => Text::_('COM_JOOMCRM_PAST_DUE_DEALS'), 'not_updated_thirty' => Text::_('COM_JOOMCRM_DEALS_NOT_UPDATED'), 'shared' => Text::_('COM_JOOMCRM_SHARED_DEALS'), 'archived' => Text::_('COM_JOOMCRM_ARCHIVED_DEALS'));
	}

	public static function checkStage($stageId)
	{
		if ($stageId == "all")
		{
			return false;
		}

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("id")->from("#__joomcrm_stages")->where("id=" . $stageId);

		$id = $db->setQuery($query)->loadResult();

		return ($id > 0) ? true : false;
	}

	/**
	 * function to return deal stages
	 *
	 * @param   string  $stage_name  [description]
	 * @param   boolean $stagesOnly  [description]
	 * @param   boolean $idsOnly     [description]
	 *
	 * @return  array                 Deal Stages
	 */
	public static function getStages($stage_name = null, $stagesOnly = false, $idsOnly = true)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_stages");

		if ($stage_name)
		{
			$query->where("name=" . $db->q($stage_name));
		}

		if (!$stagesOnly)
		{
			$base = array('all' => Text::_('COM_JOOMCRM_ALL_STAGES'), 'active' => Text::_('COM_JOOMCRM_ACTIVE_STAGES'));
		}
		else
		{
			$base = array();
		}

		$query->order('ordering');

		$results = $db->setQuery($query)->loadAssocList();

		if ($idsOnly)
		{
			$stages = array();

			foreach ($results as $key => $stage)
			{
				$stages[$stage['id']] = $stage['name'];
			}
		}
		else
		{
			$stages = $results;
		}

		return $base + $stages;
	}

	/**
	 * function to return deal stages
	 *
	 * @return  array  Inactive deal stages
	 */
	public static function getNonInactiveStages()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("id")->from("#__joomcrm_stages")->order('ordering')->where('percent > 0');

		return $db->setQuery($query)->loadColumn();
	}

	/**
	 * [getPrimaryContact description]
	 *
	 * @param   [type]  $deal_id  [description]
	 *
	 * @return  array             [description]
	 */
	public static function getPrimaryContact($deal_id)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("primary_contact_id")->from("#__joomcrm_deals")->where("id=" . $deal_id);

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * get stages for sorting sources
	 *
	 * @return  array  [description]
	 */
	public static function getSourceStages()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_stages");

		// Filter by active and closed stages
		$inactive_stage_ids = self::getInactiveStages();
		$query->where("id NOT IN(" . implode(',', $inactive_stage_ids) . ")")->order('ordering');

		// Merge arrays
		$base = array('all' => 'all stages', 'active' => 'active stages');

		$results = $db->setQuery($query)->loadAssocList();
		$stages  = array();

		if (count($results) > 0)
		{
			foreach ($results as $key => $stage)
			{
				$stages[$stage['id']] = $stage['name'];
			}
		}

		return $base + $stages;
	}

	/**
	 * function to return active stages
	 *
	 * @param   boolean $idsOnly  [description]
	 *
	 * @return  array              [description]
	 */
	public static function getActiveStages($idsOnly = false)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_stages")->where("percent > 0")
				 ->where("percent < 100")->where("won=0")->order('ordering');

		$results = $db->setQuery($query)->loadAssocList();
		$stages  = array();

		if ($idsOnly)
		{
			if (count($results) > 0)
			{
				foreach ($results as $key => $stage)
				{
					$stages[$stage['id']] = $stage['name'];
				}
			}
		}
		else
		{
			return $results;
		}

		return $stages;
	}

	/**
	 * [getActiveStageIds description]
	 *
	 * @return  array  Active stage ids
	 */
	public static function getActiveStageIds()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("id")->from("#__joomcrm_stages")->where("percent > 0")
				 ->where("percent < 100")->where("won=0")->order('ordering');

		return $db->setQuery($query)->loadColumn();
	}

	/**
	 * function to return deal stages
	 *
	 * @return  array  [description]
	 */
	public static function getGoalStages()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_stages")->order('ordering');

		$results = $db->setQuery($query)->loadAssocList();
		$stages  = array();

		foreach ($results as $key => $stage)
		{
			$stages[$stage['id']] = $stage['name'];
		}

		return $stages;
	}

	public static function getDealFilters()
	{
		return array('all' => strtolower(Text::_('COM_JOOMCRM_ALL')), 'this_week' => strtolower(Text::_('COM_JOOMCRM_THIS_WEEK')), 'next_week' => strtolower(Text::_('COM_JOOMCRM_NEXT_WEEK')), 'this_month' => strtolower(Text::_('COM_JOOMCRM_THIS_MONTH')), 'next_month' => strtolower(Text::_('COM_JOOMCRM_NEXT_MONTH')));
	}

	/**
	 * get closing filters for deals
	 *
	 * @return  array  [description]
	 */
	public static function getClosing()
	{
		return self::getDealFilters();
	}

	/**
	 * [getExpectedClosingDates description]
	 *
	 * @return  array  [description]
	 */
	public static function getExpectedClosingDates()
	{
		return array('all' => strtolower(Text::_('COM_JOOMCRM_ALL')), 'this_week' => strtolower(Text::_('COM_JOOMCRM_THIS_WEEK')), 'next_week' => strtolower(Text::_('COM_JOOMCRM_NEXT_WEEK')), 'this_month' => strtolower(Text::_('COM_JOOMCRM_THIS_MONTH')), 'next_month' => strtolower(Text::_('COM_JOOMCRM_NEXT_MONTH')));
	}

	/**
	 * [getActualClosingDates description]
	 *
	 * @return  array  [description]
	 */
	public static function getActualClosingDates()
	{
		return array('all' => strtolower(Text::_('COM_JOOMCRM_ALL')), 'this_week' => strtolower(Text::_('COM_JOOMCRM_THIS_WEEK')), 'last_week' => strtolower(Text::_('COM_JOOMCRM_LAST_WEEK')), 'last_month' => strtolower(Text::_('COM_JOOMCRM_LAST_MONTH')));
	}

	/**
	 * get closing filters for deals
	 *
	 * @return  array  [description]
	 */
	public static function getModified()
	{
		return JoomcrmHelperDate::getCreatedDates();
	}

	/**
	 * get closing filters for deals
	 *
	 * @return  array  [description]
	 */
	public static function getCreated()
	{
		return self::getDealFilters();
	}

	/**
	 * Get amounts for dropdowns
	 *
	 * @return  array  [description]
	 */
	public static function getAmounts()
	{
		return array('small' => Text::_('COM_JOOMCRM_SMALL'), 'medium' => Text::_('COM_JOOMCRM_MEDIUM'), 'large' => Text::_('COM_JOOMCRM_LARGE'));
	}

	/**
	 * get won stage
	 *
	 * @return  array  [description]
	 */
	public static function getWonStages()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
			->select("s.id")
			->from("#__joomcrm_stages AS s")
			->where("s.won = 1");

		return $db->setQuery($query)->loadColumn();
	}

	/**
	 * get lost stage
	 *
	 * @return  array  [description]
	 */
	public static function getInactiveStages()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("s.id")->from("#__joomcrm_stages AS s")->where('s.percent=0');

		$stages = $db->setQuery($query)->loadColumn();
		$base   = array(0);

		return $stages + $base;
	}

	/**
	 * [getClosedStages description]
	 *
	 * @return  array  [description]
	 */
	public static function getClosedStages()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("s.id")->from("#__joomcrm_stages AS s")->where('s.percent=100');

		return $db->setQuery($query)->loadColumn();
	}

	/**
	 * get deal statuses
	 *
	 * @param   string  $status_name   [description]
	 * @param   boolean $returnColors  [description]
	 *
	 * @return  array                   [description]
	 */
	public static function getStatuses($status_name = null, $returnColors = false, $unique = false)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_deal_status");

		if ($status_name)
		{
			$query->where("name=" . $db->q($status_name));
		}

		$query->order('ordering');

		$results  = $db->setQuery($query)->loadAssocList();
		$statuses = array();

		if ($returnColors)
		{
			$statuses[0] = array('id' => 0, 'label' => Text::_('COM_JOOMCRM_NONE_STATUS'), 'color' => 'ffffff');
		}
		else
		{
			if (!$unique)
			{
				$statuses[0] = ucwords(Text::_('COM_JOOMCRM_NONE_STATUS'));
			}
		}

		if (count($results) > 0)
		{
			foreach ($results as $key => $status)
			{
				if ($returnColors)
				{
					$statuses[$status['id']] = array('id' => $status['id'], 'label' => $status['name'], 'color' => $status['color']);
				}
				else
				{
					$statuses[$status['id']] = $status['name'];
				}
			}
		}

		return $statuses;
	}

	/**
	 * get deal sources
	 *
	 * @param   string $source_name  [description]
	 *
	 * @return  array                 [description]
	 */
	public static function getSources($source_name = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_sources")->order('ordering');

		if ($source_name)
		{
			$query->where("name=" . $db->q($source_name));
		}

		$results = $db->setQuery($query)->loadAssocList();
		$sources = array();

		foreach ($results as $key => $source)
		{
			$sources[$source['id']] = $source['name'];
		}

		return $sources;
	}

	/**
	 * get user created custom fields from database
	 *
	 * @param   integer $id  [description]
	 *
	 * @return  array         [description]
	 */
	public static function getUserCustomFields($id = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_deal_custom");

		if ($id)
		{
			$query->where('id = ' . (int) $id);
		}

		return $db->setQuery($query)->loadAssocList();
	}

	/**
	 * get all custom fields for reports
	 *
	 * @return  array  [description]
	 */
	public static function getAllCustomFields()
	{
		$base = array("summary" => ucwords(Text::_("COM_JOOMCRM_SUMMARY")), "amount" => ucwords(Text::_("COM_JOOMCRM_AMOUNT")), "name" => ucwords(Text::_("COM_JOOMCRM_DEALS_NAME")), "owner_id" => ucwords(Text::_("COM_JOOMCRM_DEAL_OWNER")), "stage_id" => ucwords(Text::_("COM_JOOMCRM_DEAL_STAGE")), "probability" => ucwords(Text::_("COM_JOOMCRM_DEAL_PROBABILITY")), "status_id" => ucwords(Text::_("COM_JOOMCRM_DEAL_STATUS")), "expected_close" => ucwords(Text::_("COM_JOOMCRM_DEAL_CLOSE")), "modified" => ucwords(Text::_("COM_JOOMCRM_MODIFIED")), "created" => ucwords(Text::_("COM_JOOMCRM_CREATED")), "source_id" => ucwords(Text::_("COM_JOOMCRM_REPORTS_SOURCE")), "actual_close" => ucwords(Text::_("COM_JOOMCRM_DEALS_ACTUAL_CLOSE")), "primary_contact_name" => ucwords(Text::_("COM_JOOMCRM_PRIMARY_CONTACT_NAME")), "primary_contact_email" => ucwords(Text::_("COM_JOOMCRM_PRIMARY_CONTACT_EMAIL")), "primary_contact_phone" => ucwords(Text::_("COM_JOOMCRM_PRIMARY_CONTACT_PHONE")), "primary_contact_city" => ucwords(Text::_("COM_JOOMCRM_PRIMARY_CONTACT_CITY")), "primary_contact_state" => ucwords(Text::_("COM_JOOMCRM_PRIMARY_CONTACT_STATE")), "primary_contact_company_name" => ucwords(Text::_("COM_JOOMCRM_PRIMARY_CONTACT_COMPANY_NAME")), "company_name" => ucwords(Text::_("COM_JOOMCRM_DEAL_COMPANY")), "company_category" => ucwords(Text::_('COM_JOOMCRM_COMPANY_CATEGORY_NAME')));

		$custom = self::getUserCustomFields();

		for ($i = 0; $i < count($custom); $i++)
		{
			$field                          = $custom[$i];
			$base["custom_" . $field['id']] = $field['name'];
		}

		return $base + array('calculate_field' => ucwords(Text::_('COM_JOOMCRM_CALCULATE_FIELD')));
	}

	/**
	 * get column filters
	 *
	 * @return  array  [description]
	 */
	public static function getColumnFilters()
	{
		$arr = array('company' => ucwords(Text::_('COM_JOOMCRM_DEALS_COMPANY')), 'primary_contact' => ucwords(Text::_('COM_JOOMCRM_PRIMARY_CONTACT')), 'contacts' => ucwords(Text::_('COM_JOOMCRM_DEALS_CONTACTS')), 'summary' => ucwords(Text::_('COM_JOOMCRM_DEALS_SUMMARY')), 'amount' => ucwords(Text::_('COM_JOOMCRM_DEALS_AMOUNT')), 'status' => ucwords(Text::_('COM_JOOMCRM_DEALS_STATUS')), 'stage' => ucwords(Text::_('COM_JOOMCRM_DEALS_STAGE')), 'source' => ucwords(Text::_('COM_JOOMCRM_DEAL_SOURCE')), 'expected_close' => ucwords(Text::_('COM_JOOMCRM_DEALS_EXPECTED_CLOSE')), 'actual_close' => ucwords(Text::_('COM_JOOMCRM_DEALS_ACTUAL_CLOSE')), 'owner' => ucwords(Text::_('COM_JOOMCRM_DEALS_OWNER')), 'next_action' => ucwords(Text::_('COM_JOOMCRM_DEALS_NEXT')), 'deals_due' => ucwords(Text::_('COM_JOOMCRM_DEALS_DUE')), 'notes' => ucwords(Text::_('COM_JOOMCRM_DEALS_NOTES')), 'created' => ucwords(Text::_('COM_JOOMCRM_PEOPLE_ADDED')), 'modified' => ucwords(Text::_('COM_JOOMCRM_PEOPLE_UPDATED')));

		return JoomcrmHelperQuote::hasCrmQuote() ? $arr + JoomcrmHelperQuote::getColumnFilters() : $arr;
	}

	/**
	 * get selected column filters
	 *
	 * @return  array  [description]
	 */
	public static function getSelectedColumnFilters()
	{
		$db    = Factory::getDbo();

		$query = $db->getQuery(true)->select("deals_columns")->from("#__joomcrm_users")
				 ->where("id=" . JoomcrmHelperUsers::getUserId());

		$results = $db->setQuery($query)->loadResult();

		$columns = !is_null($results) ? unserialize($results) : [];

		if (is_array($columns) && !empty($columns))
		{
			return $columns;
		}
		else
		{
			// If it is empty then load a default set
			return JoomcrmHelperDeal::getDefaultColumnFilters();
		}
	}

	/**
	 * get default column filters
	 *
	 * @return  array  [description]
	 */
	public static function getDefaultColumnFilters()
	{
		return array('company', 'primary_contact', 'amount', 'stage', 'expected_close', 'next_action', 'deals_due', 'notes', 'created', 'modified');
	}

	/**
	 * [downloadDocument description]
	 *
	 * @return  void
	 */
	public static function downloadDocument()
	{
		$model    = new JoomcrmModelDocument;
		$document = $model->getDocument();

		header('Content-Description: File Transfer');
		header('Content-Type: application/octet-stream');
		header('Content-Disposition: attachment; filename=' . basename($document->name));
		header('Content-Transfer-Encoding: binary');
		header('Expires: 0');
		header('Cache-Control: must-revalidate');
		header('Pragma: public');
		header('Content-Length: ' . filesize($document->path));
		ob_clean();
		flush();
		readfile($document->path);
		exit;
	}

}
