<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

abstract class JoomcrmHelperDocument
{
	/**
	 * get users total associated documents
	 *
	 * @return  integer  [description]
	 */
	public static function getTotalDocuments()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("count(*)")->from("#__joomcrm_documents AS d");

		// Filter depending on user access
		$user_id = JoomcrmHelperUsers::getUserId();

		$query->where(array("d.owner_id=" . $user_id, 'd.shared=1'), 'OR')->where("d.published=1");

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * function to get possible document association types
	 *
	 * @return  array  [description]
	 */
	public static function getAssocTypes()
	{
		return array('all' => Text::_('COM_JOOMCRM_ALL_DOCUMENTS'),
					'deals' => Text::_('COM_JOOMCRM_DOCUMENTS_DEALS'),
					'people' => Text::_('COM_CMRERY_DOCUMENTS_PEOPLE'),
					'companies' => Text::_('COM_JOOMCRM_DOCUMENTS_COMPANIES'),
					'emails' => Text::_('COM_JOOMCRM_DOCUMENTS_EMAILS'),
					'shared' => Text::_('COM_JOOMCRM_DOCUMENTS_SHARED'));
	}

	/**
	 * get different document doctypes
	 *
	 * @return  array  [description]
	 */
	public static function getDocTypes()
	{
		return array('all' => Text::_('COM_JOOMCRM_ALL_TYPES'),
			'spreadsheets' => Text::_('COM_JOOMCRM_SPREADSHEETS'),
			'images' => Text::_('COM_JOOMCRM_IMAGES'),
			'documents' => Text::_('COM_JOOMCRM_DOCUMENTS'),
			'pdfs' => Text::_('COM_JOOMCRM_PDFS'),
			'presentations' => Text::_('COM_JOOMCRM_PRESENTATIONS'),
			'others' => Text::_('COM_JOOMCRM_OTHERS'));
	}


	public static function getCategories()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("c.id,c.name")->from("#__joomcrm_document_categories AS c")
			->order("c.ordering");

		$categories = $db->setQuery($query)->loadObjectList();

		$return = array('0' => \Joomla\CMS\Language\Text::_('COM_JOOMCRM_ANY_CATEGORY'));

		if (count($categories) > 0)
		{
			foreach ($categories as $category)
			{
				$return[$category->id] = $category->name;
			}
		}

		return $return;
	}

}
