<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

abstract class JoomcrmHelperDropdown
{
	/**
	 * [generateDropdown description]
	 *
	 * @param   [type]   $type       [description]
	 * @param   [type]   $selection  [description]
	 * @param   [type]   $name       [description]
	 * @param   boolean $raw        [description]
	 *
	 * @return  string               [description]
	 *
	 * @todo    REFACTOR
	 */
	public static function generateDropdown($type, $selection = null, $name = null, $raw = false,$extraClass = '')
	{
		// Base html
		$html = '';

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		// Generate query based on type

		switch ($type)
		{
			case "company":
				$query->select('c.id, c.name')->from('#__joomcrm_companies AS c')->where('c.published > 0')->order('name ASC');
				break;
			case "stage":
				$query->select('s.id, s.name')->from('#__joomcrm_stages AS s')->order('s.ordering');
				break;
			case "source":
				$query->select('s.id, s.name')->from('#__joomcrm_sources AS s')->order('s.ordering');
				break;
			case "deal_status":
				$query->select('d.id, d.name')->from('#__joomcrm_deal_status AS d')->order('d.ordering');
				break;
			case "people_status":
				$query->select('p.id, p.name')->from('#__joomcrm_people_status AS p')->order('p.ordering');
				break;
			case "deal":
				$query->select('d.id,d.name')->from("#__joomcrm_deals AS d")->where("d.published > 0")
				->leftJoin('#__joomcrm_users AS user ON user.id = d.owner_id');

				// Filter data using member role permissions
				$member_id   = JoomcrmHelperUsers::getUserId();
				$member_role = JoomcrmHelperUsers::getRole();
				$team_id     = JoomcrmHelperUsers::getTeamId();

				if ($member_role != 'exec')
				{
					if ($member_role == 'manager')
					{
						// Manager filter
						$query->where('user.team_id = ' . $team_id);
					}
					else
					{
						// Basic user filter
						$query->where(array('d.owner_id = ' . $member_id));
					}
				}

		}

		// Run query and grab results
		if ($query != "")
		{
			$row = $db->setQuery($query)->loadAssocList();
		}

		if (!isset($row))
		{
			$row = array();
		}
		else
		{
			if (!is_array($row) && !(count($row) > 0))
			{
				$row = array();
			}
		}

		if ($raw)
		{
			return $row;
		}

		// Determine which kind of dropdown we are generating
		$selected = ($selection == null) ? "selected='selected'" : '';
		switch ($type)
		{
			case 'is_custom':
				$name = $name ? $name : "name=is_custom";

				$isCustomList = [0 => Text::_('JNO'), 1 => Text::_('JYES')];

				$html = '
			 		<select onChange="htmlEditField();" class="form-select '.$extraClass.'" ' . $name . ' id="is_custom">';
				foreach ($isCustomList as $key => $value)
				{
					$selected = ($key == $selection) ? "selected='selected'" : '';
					$html .= '<option value="' . $key . '" ' . $selected . ' ' . $name . ' >' . $value . '</option>';
				}

				$html .= '</select>';
				break;

			case "company":
				$name = $name ? $name : "name=company_id";
				$html = '
					<select class="form-select '.$extraClass.'" ' . $name . ' id="company_id">';
				$html .= "<option value='0' " . $selected . ">".Text::_('COM_JOOMCRM_SELECT_COMPANY');
				foreach ($row as $company => $info)
				{
					$selected = ($info['id'] == $selection) ? "selected='selected'" : '';
					$html .= '<option value="' . $info['id'] . '" ' . $selected . ' >' . $info['name'] . '</option>';
				}
				$html .= '</select>';
				break;
			case "stage":
				$name = $name ? $name : "name=stage_id";
				$html = '
			 		<select class="form-select '.$extraClass.'" ' . $name . ' id="stage_id">';
				$html .= "<option value='0' " . $selected . ">".Text::_('COM_JOOMCRM_SELECT_STAGE');
				foreach ($row as $stage => $info)
				{
					$selected = ($info['id'] == $selection) ? "selected='selected'" : '';
					$html .= '<option value="' . $info['id'] . '" ' . $selected . ' ' . $name . ' >' . $info['name'] . '</option>';
				}

				$html .= '</select>';
				break;
			case "source":
				$name = $name ? $name : "name=source_id";
				$html = '<select class="form-select '.$extraClass.'" ' . $name . ' id="source_id">';
				$html .= "<option value='0' " . $selected . ">".Text::_('COM_JOOMCRM_SELECT_SOURCE');
				if (count($row) > 0)
				{
					foreach ($row as $source => $info)
					{
						$selected = ($info['id'] == $selection) ? "selected='selected'" : '';
						$html .= '<option value="' . $info['id'] . '" ' . $selected . ' ' . $name . ' >' . $info['name'] . '</option>';
					}
				}
				$html .= '</select>';
				break;
			case "probability":
				$name = $name ? $name : "name=probability";
				$html = '
					<select class="form-select '.$extraClass.'" ' . $name . ' id="probability_id">';
				$html .= "<option value='0' " . $selected . ">".Text::_('COM_JOOMCRM_SELECT_PROBABILITY');
				for ($i = 5; $i <= 95; $i += 5)
				{
					$selected = ($i == $selection) ? "selected='selected'" : '';
					$html .= '<option value="' . $i . '" ' . $selected . ' ' . $name . ' >' . $i . '%</option>';
				}
				$html .= '</select>';
				break;
			case "deal_status":
				$name = $name ? $name : "name=status_id";
				$html = '
					<select class="form-select '.$extraClass.'" ' . $name . ' id="status_id">';
				$html .= "<option value='0' " . $selected . ">".Text::_('COM_JOOMCRM_SELECT_STATUS');
				foreach ($row as $status => $info)
				{
					$selected = ($info['id'] == $selection) ? "selected='selected'" : '';
					$html .= '<option value="' . $info['id'] . '" ' . $selected . ' ' . $name . ' >' . $info['name'] . '</option>';
				}

				$html .= '</select>';
				break;
			case "people_status":
				$name = $name ? $name : "name=status_id";
				$html = '
					<select class="form-select '.$extraClass.'" ' . $name . ' id="status_id">';
				$html .= "<option value='0' " . $selected . ">".Text::_('COM_JOOMCRM_SELECT_STATUS');
				foreach ($row as $status => $info)
				{
					$selected = ($info['id'] == $selection) ? "selected='selected'" : '';
					$html .= '<option value="' . $info['id'] . '" ' . $selected . ' ' . $name . ' >' . $info['name'] . '</option>';
				}

				$html .= '</select>';
				break;
			case "deal":
				$name = $name ? $name : "name=deal_id";
				$html = '
					<select class="form-select '.$extraClass.'" ' . $name . ' id="deal_id">';
				$html .= "<option value='0' " . $selected . ">".Text::_('COM_JOOMCRM_SELECT_DEAL');
				foreach ($row as $deal => $info)
				{
					$selected = ($info['id'] == $selection) ? "selected='selected'" : '';
					$html .= '<option value="' . $info['id'] . '" ' . $selected . ' ' . $name . ' >' . $info['name'] . '</option>';
				}

				$html .= '</select>';
				break;

			default:

				$model = JoomCRMHelperDropdown::getModelFromType($type);

				$html = '<ul>';
				$html .= '<li><a href="javascript:void(0)" onclick="saveAjax(\'' . $type . '\',\'' . $model . '\',\'Lead\')">' . Text::_('COM_JOOMCRM_PERSON_LEAD') . '</a></li>';
				$html .= '<li><a href="javascript:void(0)" onclick="saveAjax(\'' . $type . '\',\'' . $model . '\',\'Contact\')">' . Text::_('COM_JOOMCRM_PEOPLE_CONTACT') . '</a></li>';
				$html .= "</ul>";

				break;
		}

		return $html;
	}

	/**
	 * [getModelFromType description]
	 *
	 * @param   string $type  [description]
	 *
	 * @return  string         [description]
	 */
	public static function getModelFromType($type)
	{
		if (stripos($type, 'person') !== false)
		{
			$model = 'people';
		}

		return $model;

	}

	/**
	 * Get team names for dropdowns
	 *
	 * @param 	string 	Prepend id with prefix
	 * @return  array  	Team names for dropdown
	 */
	public static function getTeamNames($prefix = '')
	{
		// Get all teams
		$teams = JoomcrmHelperUsers::getTeams();

		// Generate array
		$return      = array();
		$managerTeam = JoomcrmHelperUsers::getTeamId();
		if (is_array($teams) && count($teams) > 0)
		{
			foreach ($teams as $key => $value)
			{
				$return[$prefix . $value['team_id']] = $value['team_name'] . Text::_('COM_JOOMCRM_TEAM_APPEND');
			}
		}
		unset($return[$managerTeam]);

		return $return;
	}

	/**
	 * Get user names for dropdowns
	 *
	 * @return  array  User names for dropdown
	 */
	public static function getUserNames()
	{
		// Get all teams
		$users = JoomcrmHelperUsers::getUsers();

		// Generate array
		$return = array();

		if (is_array($users) && count($users) > 0)
		{
			foreach ($users as $key => $value)
			{
				$return[$value['id']] = $value['first_name'] . ' ' . $value['last_name'];
			}
		}

		return $return;
	}

	/**
	 * [getOwnerDropdown description]
	 *
	 * @param   mixed  $selected  [description]
	 * @param   string $name      [description]
	 * @param   string $class     [description]
	 *
	 * @return  string  HTML Select List
	 */
	public static function getOwnerDropdown($selected = null, $name = "owner_id", $class = "class='form-control'")
	{
		$users = self::getUserNames();

		$arr = array();
		foreach ($users as $value => $label)
		{
			$arr[] = HTMLHelper::_('select.option', $value, $label);
		}

		return HTMLHelper::_('select.genericlist', $arr, $name, $class, 'value', 'text', $selected);
	}

	/**
	 * Get person contact types
	 *
	 * @param   string $contact_type_name  [description]
	 *
	 * @return  string  HTML Select List
	 */
	public static function getContactTypes($contact_type_name = null)
	{
		$contact_types = array('contact' => Text::_('COM_JOOMCRM_CONTACT'), 'lead' => Text::_('COM_JOOMCRM_LEAD'));

		if (!in_array($contact_type_name, $contact_types))
		{
			$currentValue = '0'; // Set this value from DB, etc.
			$arr          = array();

			foreach ($contact_types as $name => $value)
			{
				$arr[] = HTMLHelper::_('select.option', $name, $value);
			}

			return HTMLHelper::_('select.genericlist', $arr, 'type', 'class="form-control"', 'value', 'text', $currentValue);
		}
		else
		{
			return $contact_type_name;
		}
	}

	/**
	 * [getPeopleList description]
	 *
	 * @return  array  [description]
	 */
	public static function getPeopleList()
	{
		// Open model
		$model       = new JoomcrmModelPeople;
		$people      = $model->getPeopleList();
		$people_list = array();

		if (count($people))
		{
			foreach ($people as $index => $row)
			{
				$people_list[$row['id']] = $row['first_name'] . ' ' . $row['last_name'];
			}
		}

		return $people_list;
	}


	public static function getCompanyList(){
		// Open model
		$model       = new JoomcrmModelCompany;
		$company     = $model->getCompanyList();
		$company_list = array();

		if (count($company))
		{
			foreach ($company as $index => $row)
			{
				$company_list[$row['id']] = $row['name'];
			}
		}

		return $company_list;
	}

	/**
	 * load the navigation menu
	 *
	 * @return  array
	 */
	public static function getMemberRoles()
	{
		return array('exec' => 'Executive', 'manager' => 'Manager', 'basic' => 'Basic');
	}

	/**
	 * load teams to assign to users
	 *
	 * @param   [type]  $team  [description]
	 *
	 * @return  [type]         [description]
	 */
	public static function getTeams($team = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
				 ->select("t.team_id AS id,u.first_name,u.last_name,u.team_id,IF(t.name!='',t.name,CONCAT(u.first_name,' ',u.last_name)) AS team_name")
				 ->from("#__joomcrm_users AS u")->leftJoin("#__joomcrm_teams AS t on t.leader_id = u.id")
				 ->where("u.role_type='manager'");

		// $query->where("u.published=1");

		$results = $db->setQuery($query)->loadAssocList();
		$users   = array();

		if (is_array($results) && count($results) > 0)
		{
			foreach ($results as $key => $user)
			{
				$users[$user['id']] = $user['team_name'];
			}
		}

		if ($team > 0)
		{
			// unset($users[$team]);
		}

		return $users;
	}

	/**
	 * [getManagers description]
	 *
	 * @param   [type]  $remove  [description]
	 *
	 * @return  [type]           [description]
	 */
	public static function getManagers($remove = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("u.id,u.first_name,u.last_name,u.team_id")->from("#__joomcrm_users AS u")
				 ->where("u.role_type='manager'");

		// $query->where("u.published=1");

		$results = $db->setQuery($query)->loadAssocList();
		$users   = array();

		foreach ($results as $key => $user)
		{
			if ($user['id'] == $remove)
			{
				unset($results[$key]);
			}
			else
			{
				$users[$user['id']] = $user['first_name'] . ' ' . $user['last_name'];
			}
		}

		return $users;
	}

	/**
	 * [getSources description]
	 *
	 * @return  array  [description]
	 */
	public static function getSources()
	{
		return array('per' => 'Per Lead/Deal', 'flat' => 'Flat Fee');
	}

	/**
	 * [getTemplateTypes description]
	 *
	 * @return  array  [description]
	 */
	public static function getTemplateTypes()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("id,name")->from("#__joomcrm_events_categories")->order('ordering');

		$types  = $db->setQuery($query)->loadObjectList();
		$return = array();

		if ($types)
		{
			foreach ($types as $type)
			{
				$return[$type->id] = $type->name;
			}
		}

		return $return;
	}

	/**
	 * [showImportTypes description]
	 *
	 * @param   string $selected  [description]
	 * @param   string $name      [description]
	 * @param   string $class     [description]
	 *
	 * @return  string  HTML Select List
	 */
	public static function showImportTypes($selected = "deals", $name = "import_type", $class = 'class="form-control"')
	{
		$import_types = array('deals' => Text::_('COM_JOOMCRM_DEALS'), 'people' => Text::_('COM_JOOMCRM_PEOPLE'), 'companies' => Text::_('COM_JOOMCRM_COMPANIES'));

		$arr = array();

		foreach ($import_types as $value => $label)
		{
			$arr[] = HTMLHelper::_('select.option', $value, $label);
		}

		return HTMLHelper::_('select.genericlist', $arr, $name, $class, 'value', 'text', $selected);
	}

	/**
	 * [getFormTypes description]
	 *
	 * @param   string $selected  [description]
	 * @param   string $name      [description]
	 * @param   string $class     [description]
	 *
	 * @return  string  HTML Select List
	 */
	public static function getFormTypes($selected = "lead", $name = "type", $class = "class='form-select'")
	{
		$import_types = array(
			'lead' => Text::_('COM_JOOMCRM_LEAD'),
			'contact' => Text::_('COM_JOOMCRM_CONTACT'),
			// 'company' => Text::_('COM_JOOMCRM_COMPANY')
			// 'deal' => Text::_('COM_JOOMCRM_DEAL')
		);

		$arr = array();

		foreach ($import_types as $value => $label)
		{
			$arr[] = HTMLHelper::_('select.option', $value, $label);
		}

		return HTMLHelper::_('select.genericlist', $arr, $name, $class, 'value', 'text', $selected);
	}

	/**
	 * [getFormFields description]
	 *
	 * @param   string $type  [description]
	 *
	 * @return  array          [description]
	 */
	public static function getFormFields($type)
	{
		$arr = array();

		switch ($type)
		{
			case "people":
				$base   = array(
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_FIRST')),
								'name' => 'first_name', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_LAST')),
								'name' => 'last_name', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_COMPANY')),
								'name' => 'company_name', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_POSITION')),
								'name' => 'position', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_PHONE')),
								'name' => 'phone', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_EMAIL')),
								'name' => 'email', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_ADDRESS_1_NULL')),
								'name' => 'home_address_1', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_ADDRESS_2_NULL')),
								'name' => 'home_address_2', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_CITY_NULL')),
								'name' => 'home_city', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_STATE_NULL')),
								'name' => 'home_state', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_ZIP_NULL')),
								'name' => 'home_zip', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_COUNTRY_NULL')),
								'name' => 'home_country', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_FAX')),
								'name' => 'fax', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_WEBSITE')),
								'name' => 'website', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_FACEBOOK_URL')),
								'name' => 'facebook_url', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_TWITTER_USER')),
								'name' => 'twitter_user', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_LINKEDIN_URL')),
								'name' => 'linkedin_url', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_ADDRESS_1_NULL')),
								'name' => 'work_address_1', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_ADDRESS_2_NULL')),
								'name' => 'work_address_2', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_CITY_NULL')),
								'name' => 'work_city', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_STATE_NULL')),
								'name' => 'work_state', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_ZIP_NULL')),
								'name' => 'work_zip', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_COUNTRY_NULL')),
								'name' => 'work_country', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_MOBILE_PHONE')),
								'name' => 'mobile_phone', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_HOME_EMAIL')),
								'name' => 'home_email', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_OTHER_EMAIL')),
								'name' => 'other_email', 
								'type' => 'text'), 
							array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_HOME_PHONE')),
								'name' => 'home_phone', 
								'type' => 'text'),);
				$custom = JoomcrmHelperCustom::generateCustom('people');
				if (count($custom) > 0)
				{
					$custom = JoomcrmHelperCustom::cleanCustomForm($custom);
				}
				$arr = array_merge($base, $custom);
				break;
			case "deal":
			case "company":
				$base = array();
				break;
		}

		return $arr;
	}

	/**
	 * [generateDealStatuses description]
	 *
	 * @param   mixed  $selected  [description]
	 * @param   string $name      [description]
	 * @param   string $class     [description]
	 *
	 * @return  string  HTML Select List
	 */
	public static function generateDealStatuses($selected = null, $name = "status_id", $class = "class='form-control'")
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("id,name")->from("#__joomcrm_deal_status")->order('ordering');

		$statuses = $db->setQuery($query)->loadAssocList();

		$options    = array();
		$options[0] = Text::_('COM_JOOMCRM_NONE_STATUS');

		if (count($statuses) > 0)
		{
			foreach ($statuses as $status)
			{
				$options[$status['id']] = $status['name'];
			}
		}

		$arr = array();

		foreach ($options as $value => $label)
		{
			$arr[] = HTMLHelper::_('select.option', $value, $label);
		}

		return HTMLHelper::_('select.genericlist', $arr, $name, $class, 'value', 'text', $selected);
	}

	/**
	 * [getCustomCalculationFieldOptions description]
	 *
	 * @return  array  [description]
	 */
	public static function getCustomCalculationFieldOptions()
	{
		$base = array("d.amount" => ucwords(Text::_("COM_JOOMCRM_AMOUNT")), "d.probability" => ucwords(Text::_("COM_JOOMCRM_DEAL_PROBABILITY")), "d.expected_close" => ucwords(Text::_("COM_JOOMCRM_DEAL_CLOSE")), "d.modified" => ucwords(Text::_("COM_JOOMCRM_MODIFIED")), "d.created" => ucwords(Text::_("COM_JOOMCRM_CREATED")), "d.actual_close" => ucwords(Text::_("COM_JOOMCRM_DEALS_ACTUAL_CLOSE")),);

		$custom = JoomcrmHelperDeal::getUserCustomFields();

		for ($i = 0; $i < count($custom); $i++)
		{
			$field                                     = $custom[$i];
			$base["custom_" . $field['id'] . ".value"] = $field['name'];
		}

		return $base;
	}

	/**
	 * [getCustomOperationFieldOptions description]
	 *
	 * @return  array  [description]
	 */
	public static function getCustomOperationFieldOptions()
	{
		return array('+' => '+', '-' => '-', '*' => '*', '%' => '%', '^' => '^', '>' => '>', '<' => '<', '/' => '/');
	}

	/**
	 * [getCustomCalculationFields description]
	 *
	 * @param   mixed  $selected  [description]
	 * @param   string $name      [description]
	 * @param   string $class     [description]
	 *
	 * @return  string  HTML Select List
	 */
	public static function getCustomCalculationFields($selected = null, $name = "calculation_field", $class = "class='span2 no-chosen'")
	{
		$base = self::getCustomCalculationFieldOptions();

		$arr = array();

		foreach ($base as $value => $label)
		{
			$arr[] = HTMLHelper::_('select.option', $value, $label);
		}

		return HTMLHelper::_('select.genericlist', $arr, $name, $class, 'value', 'text', $selected);
	}

	/**
	 * [getCustomOperationFields description]
	 *
	 * @param   mixed  $selected  [description]
	 * @param   string $name      [description]
	 * @param   string $class     [description]
	 *
	 * @return  string  HTML Select List
	 */
	public static function getCustomOperationFields($selected = null, $name = "operation_field", $class = "class='span1 no-chosen'")
	{
		$base = self::getCustomOperationFieldOptions();

		$arr = array();

		foreach ($base as $value => $label)
		{
			$arr[] = HTMLHelper::_('select.option', $value, $label);
		}

		return HTMLHelper::_('select.genericlist', $arr, $name, $class, 'value', 'text', $selected);
	}

}
