<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die('Restricted access');

use Joomla\Registry\Registry;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

abstract class JoomcrmHelperEmail
{
    /*
     * @type : add , delete, edit,
     * @user : id of user to send email if 0 means will send to executive or team leader ..
     */
	static public function sendCompanyEmail($validData,$type,$ownerOnly = false){

     	$app	= Factory::getApplication();
		$params = self::getSubformParams('companyemail');
		$currentUser = Factory::getUser();
		$enabled = $params->get('enable_company_'.$type.'_notif',0);
		$sendToExecutives   = $params->get("send_company_'.$type.'_notif_toexecutives",1);
		$sendToTeamLeader   = $params->get('send_company_'.$type.'_notif_toleader',1);
		$sendToTeamMembers  =  $params->get('send_company_'.$type.'_notif_toteam',0);



		if(!$enabled) return;

		$owner  = self::getUserInfo($validData->owner_id);
	    $author = self::getUserInfo($currentUser->id,1);

		$uri = Uri::getInstance();
		$base = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));

	    $itemId = self::getItemId('companies');

		$companyLink = $base.Route::_("index.php?option=com_joomcrm&view=companies&layout=company&id=$validData->id$itemId");

		$email_subject = $params->get('company_'.$type.'_notif_subject');
		$email_body    = $params->get('company_'.$type.'_notif_body');

		$placeholders = [
			'{username}',
			'{company-owner}',
			'{company-email}',
			'{company-name}',
			'{company-address}',
			'{company-website}',
			'{company-link}'
		];

		$replace = [
			$currentUser->name,
			$owner->first_name.' '.$owner->last_name,
			$validData->email,
			$validData->name,
			$validData->address_1,
			$validData->website,
			$companyLink
		];

		$emailData['email_from']		= $app->get('mailfrom');
		$emailData['email_fromname']	= $app->get('fromname');
		$emailData['email_body']	    = str_replace($placeholders, $replace, $email_body);
		$emailData['email_subject']	    = str_replace($placeholders, $replace, $email_subject);

		if($ownerOnly){
			$emailData['users'] = self::getUserID((int) $validData->owner_id);
			self::sendEmail($emailData);
			return true;
		}

		// send to all executives
	     if($sendToExecutives){
		     $emailData['users'] = self::getExecutives();
		     self::sendEmail($emailData);
	     }

	    // send to team leader
	    if($sendToTeamLeader){
		    $emailData['users'] = self::getTeamMembers($author,1);
		    self::sendEmail($emailData);
	    }

	    // send to team members
	    if($sendToTeamMembers){
		    $emailData['users'] = self::getTeamMembers($author);
		    self::sendEmail($emailData);
	    }

	    return true;

	}

	static public function sendTaskEmail($validData,$type,$ownerOnly = false){

		$app	= Factory::getApplication();
		$params = self::getSubformParams('taskemail');
		$currentUser = Factory::getUser();
		$enabled = $params->get('enable_task_'.$type.'_notif',0);

		if(!$enabled) return;

		$owner  = self::getUserInfo($validData->owner_id);
		$assignedUser = self::getUserInfo($validData->assignee_id);

		$username = Factory::getUser($owner->uid)->username;

		$uri = Uri::getInstance();
		$base = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));

		$itemId = self::getItemId('events');

		$tasksLink = $base.Route::_("index.php?option=com_joomcrm&view=events$itemId");

		$email_subject = $params->get('task_'.$type.'_notif_subject');
		$email_body    = $params->get('task_'.$type.'_notif_body');

		$placeholders = [
			'{username}',
			'{task-assignee}',
			'{task-name}',
			'{task-type}',
			'{task-description}',
			'{task-duedate}',
			'{task-link}'
		];

		$replace = [
			$username,
			$assignedUser->first_name.' '.$assignedUser->last_name,
			$validData->name,
			JoomcrmHelperEvent::getCategoryName($validData->category_id),
			$validData->description,
			$validData->due_date,
			$tasksLink
		];

		$emailData['email_from']		= $app->get('mailfrom');
		$emailData['email_fromname']	= $app->get('fromname');
		$emailData['email_body']	    = str_replace($placeholders, $replace, $email_body);
		$emailData['email_subject']	    = str_replace($placeholders, $replace, $email_subject);

		if($ownerOnly){
			$emailData['users'] = self::getUserID((int) $validData->assignee_id);
			self::sendEmail($emailData);
			return true;
		}

		return true;

	}

	static public function sendEventEmail($validData,$type,$ownerOnly = false){

		$app	= Factory::getApplication();
		$params = self::getSubformParams('eventemail');
		$currentUser = Factory::getUser();
		$enabled = $params->get('enable_event_'.$type.'_notif',0);

		if(!$enabled) return;

		$owner  = self::getUserInfo($validData->owner_id);
		$assignedUser = self::getUserInfo($validData->assignee_id);

		$username = Factory::getUser($owner->uid)->username;

		$uri = Uri::getInstance();
		$base = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));

		$itemId = self::getItemId('events');

		$eventsLink = $base.Route::_("index.php?option=com_joomcrm&view=events$itemId");

		$email_subject = $params->get('event_'.$type.'_notif_subject');
		$email_body    = $params->get('event_'.$type.'_notif_body');

		$placeholders = [
			'{username}',
			'{event-assignee}',
			'{event-name}',
			'{event-type}',
			'{event-description}',
			'{event-startdate}',
			'{event-enddate}',
			'{event-link}'
		];

		$replace = [
			$username,
			$assignedUser->first_name.' '.$assignedUser->last_name,
			$validData->name,
			JoomcrmHelperEvent::getCategoryName($validData->category_id),
			$validData->description,
			$validData->start_time,
			$validData->end_time,
			$eventsLink
		];

		$emailData['email_from']		= $app->get('mailfrom');
		$emailData['email_fromname']	= $app->get('fromname');
		$emailData['email_body']	    = str_replace($placeholders, $replace, $email_body);
		$emailData['email_subject']	    = str_replace($placeholders, $replace, $email_subject);

		if($ownerOnly){
			$emailData['users'] = self::getUserID((int) $validData->assignee_id);
			self::sendEmail($emailData);
			return true;
		}

		return true;

	}

    static private function sendEmail($data) {

     	 if(!$data['users']) return;

	     if(!empty($data['users']) or !is_null($data['users'])){

		     foreach ($data['users'] as $user){

			     $email_to = Factory::getUser($user->uid)->email;

			     if(!is_null($email_to)){

				     Factory::getMailer()->sendMail(
					     $data['email_from'],
					     $data['email_fromname'],
					     $email_to,
					     $data['email_subject'],
					     $data['email_body'],
					     true
				     );

			     }

		     }
	     }

	}

    static private function getItemId($view){

     	$menus = Factory::getApplication()->getMenu('site')->getItems('component_id',\Joomla\CMS\Component\ComponentHelper::getComponent('com_joomcrm')->id);

     	// search item id of given view
     	foreach ($menus as $menu){
     		if($menu->query['view'] == $view){
		        return "&itemId=$menu->id";
	        }
        }

     	// search itemid of dashboard view
	     foreach ($menus as $menu){
		     if($menu->query['view'] == 'dashboard'){
			     return "&itemId=$menu->id";

		     }
	     }

	     // else
	     return "";

	}

     // get all users IDs with role executive
	 static private function getExecutives(){

	        $db = Factory::getDbo();
			$query = $db->getQuery(true);
			$query->select("uid")->from('#__joomcrm_users')->where("role_type = 'exec'");
			$db->setQuery($query);
			return $db->loadObjectList();

	 }

	 static private function getUserID($id){

			 $db = Factory::getDbo();
			 $query = $db->getQuery(true);
			 $query->select("*")->from('#__joomcrm_users')->where("id = $id");

			 $db->setQuery($query);

			 return $db->loadObjectList();
	 }

	 static private function getUserInfo($id,$type = 0){

			$where = $type ? "uid = $id" : "id = $id";

		    $db = Factory::getDbo();
		    $query = $db->getQuery(true);

		    $query->select("*")->from('#__joomcrm_users')->where("$where");

		    $db->setQuery($query);

		    return $db->loadObject();
	 }

	 static private function getTeamMembers($userInfo,$teamLeaderOnly = 0){

     	 if($userInfo->team_id > 0){

	         $db = Factory::getDbo();
	         $query = $db->getQuery(true);
	         $andWhere="AND role_type != 'manager'";

	         if($teamLeaderOnly){
				$andWhere = "AND role_type = 'manager'";
	         }

	         $query->select("uid,role_type")->from('#__joomcrm_users')->where("team_id = $userInfo->team_id $andWhere");

	         $db->setQuery($query);
	         return $db->loadObjectList();

         }
     	 else{
     	 	return false;
         }
     }


     static public function getSubformParams($name){
     	return  new Registry(ComponentHelper::getParams('com_joomcrm')->get($name));
     }
}