<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

abstract class JoomcrmHelperEvent
{
	/**
	 * get category listings for events
	 *
	 * @param   boolean $appendLanguage  [description]
	 *
	 * @return  array                     [description]
	 */
	public static function getCategories($appendLanguage = false)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		$query->select("name,id")->from("#__joomcrm_events_categories")->order("ordering");

		$results    = $db->setQuery($query)->loadAssocList();
		$categories = array();

		if ($results && is_array($results) && count($results) > 0)
		{
			foreach ($results as $key => $category)
			{
				$categories[$category['id']] = $appendLanguage ? Text::_('COM_JOOMCRM_OF_TYPE') . ' ' . $category['name'] : $category['name'];
			}
		}

		return $categories;
	}


	public static function getCategoryName($id){

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		$query->select("name")->from("#__joomcrm_events_categories")->where("id = ".(int)$id);

		return $db->setQuery($query)->loadResult();

	}

	/**
	 * get repeat intervals
	 *
	 * @return  array  [description]
	 */
	public static function getRepeatIntervals()
	{
		return array('none' => Text::_("COM_JOOMCRM_DOESNT_REPEAT"), 'daily' => Text::_("COM_JOOMCRM_DAILY"), 'weekdays' => Text::_("COM_JOOMCRM_EVERY_WEEKDAY"), 'weekly' => Text::_("COM_JOOMCRM_WEEKLY"), 'weekly-mwf' => Text::_("COM_JOOMCRM_WEEKLY_MWF"), 'weekly-tr' => Text::_("COM_JOOMCRM_WEEKLY_TTH"), 'monthly' => Text::_("COM_JOOMCRM_MONTHLY"), 'yearly' => Text::_("COM_JOOMCRM_YEARLY"));
	}

	/**
	 * [getEventStatuses description]
	 *
	 * @return  array  [description]
	 */
	public static function getEventStatuses()
	{
		return array('0' => Text::_('COM_JOOMCRM_INCOMPLETE'), '1' => Text::_('COM_JOOMCRM_COMPLETED'));
	}

	/**
	 * [getEventTypes description]
	 *
	 * @return  array  [description]
	 */
	public static function getEventTypes()
	{
		return array('all' => Text::_('COM_JOOMCRM_TASKS_SLASH_EVENTS'), 'task' => Text::_('COM_JOOMCRM_TASKS'), 'event' => Text::_('COM_JOOMCRM_EVENTS'));
	}

	/**
	 * [getEventDueDates description]
	 *
	 * @return  array  [description]
	 */
	public static function getEventDueDates()
	{
		return array('any' => Text::_('COM_JOOMCRM_DUE_ANY_TIME'), 'today' => Text::_('COM_JOOMCRM_DUE_TODAY'), 'tomorrow' => Text::_('COM_JOOMCRM_DUE_TOMORROW'), 'this_week' => Text::_('COM_JOOMCRM_DUE_THIS_WEEK'), 'past_due' => Text::_('COM_JOOMCRM_PAST_DUE'), 'not_past_due' => Text::_('COM_JOOMCRM_NOT_PAST_DUE'));
	}

	/**
	 * [getEventAssociations description]
	 *
	 * @return  array  [description]
	 */
	public static function getEventAssociations()
	{
		return array('any' => Text::_('COM_JOOMCRM_ANYTHING'), 'person' => Text::_('COM_JOOMCRM_PEOPLE'), 'deal' => Text::_('COM_JOOMCRM_DEALS'), 'company' => Text::_('COM_JOOMCRM_COMPANIES'));
	}

}
