<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Layout\LayoutHelper;


JLoader::register('JoomcrmHelperCustom',JPATH_SITE.'/components/com_joomcrm/helper/custom.php');


class JoomcrmHelperFormwizard
{

	public static function buildForm($form){

		$defaultFields = self::getFormFields('people');


		// load JoomCRM language file
		$lang = Factory::getLanguage();
		$lang->load('com_joomcrm',JPATH_SITE,$lang->getTag(),true);

		// set default layout path
		LayoutHelper::$defaultBasePath = JPATH_SITE.'/components/com_joomcrm/layouts';

		$htmlForm = [];

		// unserialise fields
		$form->fields = unserialize($form->fields);


		// build fields
		if(is_array($form->fields) && count($form->fields) > 0){

			foreach ($defaultFields as $field){

				if(in_array($field['name'],$form->fields)){

					// set some field data
					$field['display'] = Text::_($field['display']);
					$field['id'] = $field['type'].'_'.$field['name'];
					$field['required'] = $field['required'] ? 'required': '';

					$fieldLayout = LayoutHelper::render("formwizard.fields.{$field['type']}",$field);

					$htmlForm[] = LayoutHelper::render('formwizard.fieldContainer',['field' => $fieldLayout]);

				}
			}

		}


		// build captcha
		$fieldLayoutCaptcha = LayoutHelper::render("formwizard.fields.recaptcha",[]);
		$htmlForm[] = LayoutHelper::render('formwizard.fieldContainer',['field' => $fieldLayoutCaptcha]);

		// build hidden fields
		if($form->owner_id > 0)
			$htmlForm[] = LayoutHelper::render("formwizard.fields.hidden",['name' => 'owner_id','value' => $form->owner_id ]); // owner field
		if($form->status_id > 0)
			$htmlForm[] = LayoutHelper::render("formwizard.fields.hidden",['name' => 'status_id','value' => $form->status_id ]); // status field

		if($form->source_id > 0)
			$htmlForm[] = LayoutHelper::render("formwizard.fields.hidden",['name' => 'source_id','value' => $form->source_id ]); // source field

		$htmlForm[] = LayoutHelper::render("formwizard.fields.hidden",['name' => 'save_type','value' => $form->type ]); // save type field

		$htmlForm[] = LayoutHelper::render("formwizard.fields.hidden",['name' => 'type','value' => $form->type ]); // type field

		$htmlForm[] = LayoutHelper::render("formwizard.fields.hidden",['name' => 'return','value' => $form->return_url ]); // return url field

		$htmlForm[] = LayoutHelper::render("formwizard.fields.hidden",['name' => 'form_id','value' => $form->id ]); // form id field

		// build button field
		$fieldLayoutButton = LayoutHelper::render("formwizard.fields.submit",[]);
		$htmlForm[] = LayoutHelper::render('formwizard.fieldContainer',['field' => $fieldLayoutButton]);

		// build form
		$form = LayoutHelper::render('formwizard.formContainer',['fields' => implode('',$htmlForm)]);



		return $form;
	}





	/**
	 * [getFormFields description]
	 *
	 * @param   string $type  [description]
	 *
	 * @return  array          [description]
	 */
	public static function getFormFields($type)
	{
		$arr = array();

		switch ($type)
		{
			case "people":
				$base   = array(
					[
						'display' => ucwords(Text::_('COM_JOOMCRM_PERSON_FIRST')),
					 'name'    => 'first_name',
					 'type'    => 'text',
						'required' => 1
					],
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_LAST')),
					      'name' => 'last_name',
					      'type' => 'text',
					      'required' => 1),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_COMPANY')),
					      'name' => 'company_name',
					      'type' => 'text',
					      'required' => 1),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_POSITION')),
					      'name' => 'position',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_PHONE')),
					      'name' => 'phone',
					      'type' => 'text',
					      'required' => 1),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_EMAIL')),
					      'name' => 'email',
					      'type' => 'text',
					      'required' => 1),
					array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_ADDRESS_1')),
					      'name' => 'home_address_1',
					      'type' => 'text',
					      'required' => 1),
					array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_ADDRESS_2')),
					      'name' => 'home_address_2',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_CITY')),
					      'name' => 'home_city',
					      'type' => 'text',
					      'required' => 1),
					array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_STATE')),
					      'name' => 'home_state',
					      'type' => 'text',
					      'required' => 1),
					array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_ZIP')),
					      'name' => 'home_zip',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_HOME_COUNTRY')),
					      'name' => 'home_country',
					      'type' => 'text',
					      'required' => 1),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_FAX')),
					      'name' => 'fax',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_WEBSITE')),
					      'name' => 'website',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_FACEBOOK_URL')),
					      'name' => 'facebook_url',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_TWITTER_USER')),
					      'name' => 'twitter_user',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_LINKEDIN_URL')),
					      'name' => 'linkedin_url',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_ADDRESS_1')),
					      'name' => 'work_address_1',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_ADDRESS_2')),
					      'name' => 'work_address_2',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_CITY')),
					      'name' => 'work_city',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_STATE')),
					      'name' => 'work_state',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_ZIP')),
					      'name' => 'work_zip',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_WORK_COUNTRY')),
					      'name' => 'work_country',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_MOBILE_PHONE')),
					      'name' => 'mobile_phone',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_HOME_EMAIL')),
					      'name' => 'home_email',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_OTHER_EMAIL')),
					      'name' => 'other_email',
					      'type' => 'text',
					      'required' => 0),
					array('display' => ucwords(Text::_('COM_JOOMCRM_PERSON_HOME_PHONE')),
					      'name' => 'home_phone',
					      'type' => 'text',
					      'required' => 0),);
				$custom = JoomcrmHelperCustom::generateCustom('people');
				if (count($custom) > 0)
				{
					$custom = JoomcrmHelperCustom::cleanCustomForm($custom);
				}
				$arr = array_merge($base, $custom);
				break;
			case "deal":
			case "company":
				$base = array();
				break;
		}

		return $arr;
	}




}