<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Layout\LayoutHelper;

abstract class JoomcrmHelperJoomcrm
{
    public static function percent2Color($value, $brightness = 255, $max = 100, $min = 0, $thirdColorHex = '00')
    {
        // Calculate first and second color (Inverse relationship)
        $value = number_format($value);
        $first = (1 - ($value / $max)) * $brightness;
        $second = ($value / $max) * $brightness;

        // Find the influence of the middle color (yellow if 1st and 2nd are red and green)
        $diff = abs($first - $second);
        $influence = ($brightness - $diff) / 2;
        $first = intval($first + $influence);
        $second = intval($second + $influence);

        // Convert to HEX, format and return
        $firstHex = str_pad(dechex($first), 2, 0, STR_PAD_LEFT);
        $secondHex = str_pad(dechex($second), 2, 0, STR_PAD_LEFT);

        return $firstHex . $secondHex . $thirdColorHex;

        // alternatives:
        // return $thirdColorHex . $firstHex . $secondHex;
        // return $firstHex . $thirdColorHex . $secondHex;
    }

    /**
     * Get task and event templates
     *
     * @param   [String]  $type  ["deal","person"]
     * @param   [int]     $id    [Optional ID to get all events with a template]
     *
     * @return  [mixed]   $results
     */
    public static function getTaskTemplates($type, $id = null)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true)->select('t.*')->from('#__joomcrm_templates AS t')
            ->where('t.type=' . $db->quote($type));

        return $db->setQuery($query)->loadAssocList();
    }

    /**
     * Method to store custom field cf data associated with items
     *
     * @param int $id : The id of the item we wish to store associated data
     * @param mixed $cf_data : The data to be stored
     *
     * @return  void
     */
    /* public static function storeCustomCf($id, $cf_data, $type)
    {

        // Get DBO
        $db = Factory::getDbo();
        $query = $db->getQuery(true);
        $table = '#__joomcrm_' . $type . '_custom_cf';

        // Date generation
        $date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

        // Loop through $cf_data array to update/insert

        foreach ($cf_data as $row) {
            $typeOf = $type . "_id";
            $existId = self::getPersonID($id,$row['custom_field_id'], $table,$type);
            if ((int)$existId->$typeOf == (int)$id && (int)$existId->custom_field_id  == (int)$row['custom_field_id']) {
               $query->clear()->update($table)
                    ->set($type . "_id=" . (int)$id)
                    ->set("custom_field_id=" . (int)$row['custom_field_id'])
                    ->set("value=" . $db->quote($row['custom_field_value']))
                    ->set("modified='$date'")
                    ->where("$typeOf = {$id} AND custom_field_id = " . (int)$row['custom_field_id']);
                $db->setQuery($query);
                $db->execute();
            } else {
                // If we do not successfully update any data then perform an insert
                $object = new stdClass();
                $object->$typeOf = (int)$id;
                $object->custom_field_id = (int)$row['custom_field_id'];
                $object->value = $row['custom_field_value'];
                $object->modified = $date;
                $db->insertObject($table, $object);

            }

        }

    } */

    public static function storeCustomCf($id, $cf_data, $type)
    {


        // Get DBO
        $db    = Factory::getDbo();
        $query = $db->getQuery(true);
        $table = '#__joomcrm_' . $type . '_custom_cf';

        // Date generation
        $date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

        // Loop through $cf_data array to update/insert
        foreach ($cf_data as $row)
        {
            if (is_array($row['custom_field_value']))
            {
                $row['custom_field_value'] = serialize($row['custom_field_value']);
            }

            // update query
            $query->clear()->update($table)
                ->set($type . "_id=" . (int) $id)
                ->set("custom_field_id=" . (int) $row['custom_field_id'])
                ->set("value=" . $db->quote($row['custom_field_value']))
                ->set("modified='$date'")
                ->where($type . "_id = {$id} AND custom_field_id = " . (int) $row['custom_field_id']);

            $db->setQuery($query)->execute();

            // If we do not successfully update any data then perform an insert
            if ($db->getAffectedRows() <= 0)
            {
                $query->clear()->insert($table)
                    ->set($type . "_id=" . (int) $id)
                    ->set("custom_field_id=" . (int) $row['custom_field_id'])
                    ->set("value=" . $db->quote($row['custom_field_value']))
                    ->set("modified='$date'");

                $db->setQuery($query)->execute();
            }
        }
    }

    public static function getPersonID($id,$custom_field_id, $table,$type)
    {
        $typeOf = $type . "_id";
        $id = (int)$id;
        $custom_field_id = (int)$custom_field_id;
        $db = Factory::getDbo();
        $query = $db->getQuery(true);
        $query->select("$typeOf,custom_field_id");
        $query->from($table);
        $query->where("$typeOf = {$id}");
        $query->where("custom_field_id = {$custom_field_id}");

        $db->setQuery($query);
        return $db->loadObject();

    }

    /**
     * Checks whether email address already exists at #__joomcrm_users_email_cf table
     * or #__joomcrm_users table.
     *
     * @param string $email
     * @return boolean
     */
    public static function checkEmailName($email)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true);

        $query->select("email")->from("#__joomcrm_users_email_cf")->where("email='" . $email . "'");

        $results = $db->setQuery($query)->loadObjectList();

        if (count($results) > 0) {
            return true;
        } else {
            $query->clear()->select("j.email")->from("#__joomcrm_users AS u")->leftJoin("#__users AS j ON j.id = u.uid")
                ->where("j.email='" . $email . "'");

            $results = $db->setQuery($query)->loadObjectList();

            if (count($results) > 0) {
                return true;
            }
        }

        return false;
    }

    public static function getBytes($val)
    {
        $val = trim($val);
        $last = strtolower($val[strlen($val) - 1]);

        switch ($last) {
            // The 'G' modifier is available since PHP 5.1.0
            case 'g':
                $val *= 1024;
            case 'm':
                $val *= 1024;
            case 'k':
                $val *= 1024;
        }

        return $val;
    }

    public static function shareItem($itemId = null, $itemType = null, $userId = null)
    {
        $itemId = $itemId ? $itemId :  \Joomla\CMS\Factory::getApplication()->input->get('item_id');
        $itemType = $itemType ? $itemType :  \Joomla\CMS\Factory::getApplication()->input->get('item_type');
        $userId = $userId ? $userId :  \Joomla\CMS\Factory::getApplication()->input->get('user_id');

        $db = Factory::getDbo();
        $query = $db->getQuery(true)->insert("#__joomcrm_shared")->columns('item_id,item_type,user_id')
            ->values($itemId . "," . $db->Quote($itemType) . "," . $userId);

        $db->setQuery($query)->execute();

        return true;
    }

    public static function unshareItem($itemId = null, $itemType = null, $userId = null)
    {
        $itemId = $itemId ? $itemId :  \Joomla\CMS\Factory::getApplication()->input->get('item_id');
        $itemType = $itemType ? $itemType :  \Joomla\CMS\Factory::getApplication()->input->get('item_type');
        $userId = $userId ? $userId :  \Joomla\CMS\Factory::getApplication()->input->get('user_id');

        $db = Factory::getDbo();
        $query = $db->getQuery(true)->delete("#__joomcrm_shared")->where('item_id=' . $itemId)
            ->where('item_type=' . $db->Quote($itemType))->where('user_id=' . $userId);

        $db->setQuery($query)->execute();

        return true;
    }

    public static function showShareDialog()
    {

        return LayoutHelper::render('common.sharemodal', []);

    }

    public static function getAssociationName($associationType = null, $associationId = null)
    {
        $associationType = $associationType ? $associationType :  \Joomla\CMS\Factory::getApplication()->input->get('association_type');
        $associationId = $associationId ? $associationId :  \Joomla\CMS\Factory::getApplication()->input->get('association_id');

        $db = Factory::getDbo();
        $query = $db->getQuery(true);

        switch ($associationType) {
            case "company":
                $select = "name";
                $table = "companies";
                break;
            case "person":
                $select = "CONCAT(first_name,' ',last_name)";
                $table = "people";
                break;
            case "deal":
                $select = "name";
                $table = "deals";
                break;
        }

        $query->select($select)->from("#__joomcrm_" . $table)->where("id=" . $associationId);

        return $db->setQuery($query)->loadResult();
    }

    public static function splitExpr($expression)
    {
        // Original expression copied and used for calculations
        $exp = $expression;

        // Terms to be returned
        $terms = array();

        // Get fields and operations
        $fields = JoomcrmHelperDropdown::getCustomCalculationFieldOptions();
        $ops = JoomcrmHelperDropdown::getCustomOperationFieldOptions();

        // Split the string into characters
        $split = str_split($exp);

        // First detect any operators or parenthesis
        if (count($split) > 0) {
            foreach ($split as $pos => $term) {
                if (array_key_exists($term, $ops)) {
                    $exp = substr_replace($exp, "#", $pos, 1);
                    $terms['operations'][$pos] = $term;
                    $terms['expression'][$pos] = $term;
                    $terms['expression_details'][$pos] = array('type' => 'operation', 'value' => $term);
                } elseif ($term == "(" || $term == ")") {
                    $exp = substr_replace($exp, "#", $pos, 1);
                    $terms['parenthesis'][$pos] = $term;
                    $terms['expression'][$pos] = $term;
                    $terms['expression_details'][$pos] = array('type' => 'parenthesis', 'value' => $term);
                }
            }
        }

        // Next we detect any fields from the system
        if (count($fields) > 0) {
            foreach ($fields as $key => $field) {
                if (strstr($exp, $key)) {
                    while (strstr($exp, $key)) {
                        $pos = strpos($exp, $key);
                        $length = strlen($key);
                        $exp = substr_replace($exp, "#", $pos, $length);
                        $pos = strpos($expression, $key);
                        $terms['found'][$pos] = $key;
                        $terms['terms'][$pos] = $key;
                        $terms['expression'][$pos] = $key;
                        $terms['expression_details'][$pos] = array('type' => 'field', 'value' => $key);
                        $terms['strings'][] = $exp;
                    }
                }
            }
        }

        // Finaly we extract any custom user input
        $exp = str_replace("#", " ", $exp);
        $split = explode(" ", $exp);
        $terms['split'] = $split;
        $terms['string'] = $exp;

        if (count($split) > 0) {
            foreach ($split as $term) {
                if (strlen(trim($term)) > 0) {
                    $pos = strpos($expression, $term);
                    $terms['input'][$pos] = $term;
                    $terms['expression'][$pos] = $term;
                    $terms['expression_details'][$pos] = array('type' => 'input', 'value' => $term);
                }
            }
        }

        // Sort the expression terms for priority
        ksort($terms['expression']);
        ksort($terms['expression_details']);

        // Here we reconstruct the expression from the above operations to perform validity
        $terms['original_expression'] = "";

        foreach ($terms['expression'] as $term) {
            $terms['original_expression'] .= $term;
        }

        $terms['input_expression'] = $expression;

        // Validity
        $terms['string_validated'] = ($terms['original_expression'] == $expression) ? true : false;

        return $terms;
    }

	    public static function getStringType($type)
	    {
	        $result = [];
	        switch ($type) {
	            case "companies":
	                $result['type_id'] = 0;
	                $result['type'] = 'company';
	                break;
	            case "people":
	                $result['type_id'] = 1;
	                $result['type'] = 'people';
	                break;
	            case "deals":
	                $result['type_id'] = 2;
	                $result['type'] = 'deals';
	                break;
	        }
	        return $result;
	    }

    public static function getIntType($type)
    {
        switch ($type) {
            case 0:
                $type = 'company';
                break;
            case 1:
                ;
                $type = 'people';
                break;
            case 2:
                $type = 'deals';
                break;
        }
        return $type;
    }

    public static function RemoveSpecialCharacters($word)
    {
        return strtolower(preg_replace('/[^a-zA-Z0-9]/s', '', $word));
    }

    public static function replacePlaceholders($text,$placeholders){

	    // replace placeholders
	    foreach ($placeholders as $key => $placeholder){
		    $text = str_replace('{'.$key.'}',$placeholder,$text);
	    }

	    return $text;
    }

}
