<?php
/**
 * @package     Joomla.Libraries
 * @subpackage  HTML
 *
 * @copyright   Copyright (C) 2005 - 2013 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('JPATH_PLATFORM') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Categories\Categories;
use Joomla\CMS\Pagination\Pagination;

/**
 * Utility class for jQuery JavaScript behaviors
 *
 * @package     Joomla.Libraries
 * @subpackage  HTML
 * @since       3.0
 */
class JoomcrmPagination extends Pagination {

	protected $_viewall = false;

	/*
	 * This method update pagination when filter (cuz use AJAX) is used in table people, table deals ...
	 */
	public static function ajaxPagination($current){

		$input = \Joomla\CMS\Factory::getApplication()->input;


		if($input->get('format','','word') != 'raw') // not ajax request return nothing
			return '';

		$ajaxLimit = $input->get('ajaxLimit',0,'int');
		$limit = $ajaxLimit > 0 ? $ajaxLimit : $current->pagination->limit;


		$paginationLayout = \Joomla\CMS\Layout\LayoutHelper::render('common.pagination',['current' => $current]);


		return '
			<tr class="ajaxPagination" style="display:none;">
				<td>'.$paginationLayout.'</td>
			</tr>
			<script>
	            jQuery("form > div.joomcrmPagination").replaceWith(jQuery("tr.ajaxPagination div.joomcrmPagination")); // update pagination
	            jQuery("tr.ajaxPagination").remove(); // remove tr (container of new pagination)
	        </script>
	        <input type="hidden" name="ajaxLimit" value="'.$limit.'">
		';

	}

	public function __construct($total, $limitstart, $limit, $prefix = '')
	{
		parent::__construct($total, $limitstart, $limit, $prefix);
	}

	public function getLimitBox()
	{
		$app = Factory::getApplication();

		// Initialise variables.
		$limits = array();
		$limit_config = ComponentHelper::getParams('com_joomcrm')->get('companies_limit',10);


		// add default from config
		$limits[] =HTMLHelper::_('select.option', "$limit_config",Text::_('JDEFAULT').' ('.$limit_config.')');

		// Make the option list.
		for ($i = 5; $i <= 30; $i += 5)
		{
			$limits[] =HTMLHelper::_('select.option', "$i");
		}
		$limits[] =HTMLHelper::_('select.option', '50', Text::_('J50'));
		$limits[] =HTMLHelper::_('select.option', '100', Text::_('J100'));
		$limits[] =HTMLHelper::_('select.option', '0', Text::_('JALL'));

		$selected = $this->_viewall ? 0 : $this->limit;

		// Build the select list.
		if ($app->isClient('administrator'))
		{
			$html =HTMLHelper::_(
				'select.genericlist',
				$limits,
				$this->prefix . 'limit',
				'class="inputbox" size="1" onchange="Joomla.submitform();"',
				'value',
				'text',
				$selected
			);
		}
		else
		{
			$html =HTMLHelper::_(
				'select.genericlist',
				$limits,
				$this->prefix . 'limit',
				'class="inputbox" size="1" onchange="this.form.submit()"',
				'value',
				'text',
				$selected
			);
		}
		return $html;
	}

	/**
	 * Create and return the pagination page list string, ie. Previous, Next, 1 2 3 ... x.
	 *
	 * @return  string  Pagination page list string.
	 *
	 * @since   1.5
	 */
	public function getPagesLinks()
	{
		// Build the page navigation list.
		$data = $this->_buildDataObject();


		$list           = array();
		$list['prefix'] = $this->prefix;

		$itemOverride = false;
		$listOverride = false;

		// Build the select list
		if ($data->all->base !== null)
		{
			$list['all']['active'] = true;
			$list['all']['data']   = $itemOverride ? pagination_item_active($data->all) : $this->_item_active($data->all);
		}
		else
		{
			$list['all']['active'] = false;
			$list['all']['data']   = $itemOverride ? pagination_item_inactive($data->all) : $this->_item_inactive($data->all);
		}

		if ($data->start->base !== null)
		{
			$list['start']['active'] = true;
			$list['start']['data']   = $itemOverride ? pagination_item_active($data->start) : $this->_item_active($data->start);
		}
		else
		{
			$list['start']['active'] = false;
			$list['start']['data']   = $itemOverride ? pagination_item_inactive($data->start) : $this->_item_inactive($data->start);
		}

		if ($data->previous->base !== null)
		{
			$list['previous']['active'] = true;
			$list['previous']['data']   = $itemOverride ? pagination_item_active($data->previous) : $this->_item_active($data->previous);
		}
		else
		{
			$list['previous']['active'] = false;
			$list['previous']['data']   = $itemOverride ? pagination_item_inactive($data->previous) : $this->_item_inactive($data->previous);
		}

		// Make sure it exists
		$list['pages'] = array();

		foreach ($data->pages as $i => $page)
		{
			if ($page->base !== null)
			{
				$list['pages'][$i]['active'] = true;
				$list['pages'][$i]['data']   = $itemOverride ? pagination_item_active($page) : $this->_item_active($page);
			}
			else
			{
				$list['pages'][$i]['active'] = false;
				$list['pages'][$i]['data']   = $itemOverride ? pagination_item_inactive($page) : $this->_item_inactive($page);
			}
		}

		if ($data->next->base !== null)
		{
			$list['next']['active'] = true;
			$list['next']['data']   = $itemOverride ? pagination_item_active($data->next) : $this->_item_active($data->next);
		}
		else
		{
			$list['next']['active'] = false;
			$list['next']['data']   = $itemOverride ? pagination_item_inactive($data->next) : $this->_item_inactive($data->next);
		}

		if ($data->end->base !== null)
		{
			$list['end']['active'] = true;
			$list['end']['data']   = $itemOverride ? pagination_item_active($data->end) : $this->_item_active($data->end);
		}
		else
		{
			$list['end']['active'] = false;
			$list['end']['data']   = $itemOverride ? pagination_item_inactive($data->end) : $this->_item_inactive($data->end);
		}

		if ($this->total > $this->limit)
		{
			return LayoutHelper::render('joomla.pagination.list',['list' => $list]);
		}
		else
		{
			return '';
		}


	}

	/**
	 * Method to create an active pagination link to the item
	 *
	 * @param   PaginationObject  $item  The object with which to make an active link.
	 *
	 * @return  string  HTML link
	 *
	 * @since   1.5
	 * @note    As of 4.0 this method will proxy to `\LayoutHelper::render('joomla.pagination.link', ['data' => $item, 'active' => true])`
	 */
	protected function _item_active($item)
	{
		$cls = '';

		if ($item->text == Text::_('Next')) { $item->text = '&raquo;'; $cls = "next";}
		if ($item->text == Text::_('Prev')) { $item->text = '&laquo;'; $cls = "previous";}

		if ($item->text == Text::_('First')) { $cls = "first";}
		if ($item->text == Text::_('Last'))   { $cls = "last";}

		return '<li class="page-item"><a class="page-link ' . $cls . '" href="' . $item->link . '" title="' . $item->text . '">' . $item->text . '</a></li>';
	}

	/**
	 * Method to create an inactive pagination string
	 *
	 * @param   PaginationObject  $item  The item to be processed
	 *
	 * @return  string
	 *
	 * @since   1.5
	 * @note    As of 4.0 this method will proxy to `\LayoutHelper::render('joomla.pagination.link', ['data' => $item, 'active' => false])`
	 */
	protected function _item_inactive($item)
	{
		$cls = (int)$item->text > 0 ? 'active': 'disabled';
		return '<li class="page-item ' . $cls . '"><a class="page-link">' . $item->text . '</a></li>';
	}

}