<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

abstract class JoomcrmHelperPeople
{
	/**
	 * get an individual person
	 *
	 * @param   integer $id  [description]
	 *
	 * @return  array         [description]
	 */
	public static function getPerson($id)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
				 ->select("p.first_name,p.last_name,p.id,p.company_id,c.name as company_name FROM #__joomcrm_people AS p")
				 ->leftJoin("#__joomcrm_companies AS c ON c.id = p.company_id")->where('p.id=' . $id);

		return $db->setQuery($query)->loadAssocList();
	}

	/**
	 * return statuses
	 *
	 * @param   boolean $idsOnly  [description]
	 *
	 * @return  [type]             [description]
	 */
	public static function getStatusList($idsOnly = false)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_people_status")->order("ordering");

		return $db->setQuery($query)->loadAssocList();
	}

	/**
	 * return people filter types
	 *
	 * @param   boolean $filters  [description]
	 *
	 * @return  array              [description]
	 */
	public static function getPeopleTypes($filters = true)
	{
		if ($filters)
		{
			return array('all' => Text::_('COM_JOOMCRM_ALL_PEOPLE'), 'leads' => Text::_('COM_JOOMCRM_ALL_LEADS'), 'not_leads' => Text::_('COM_JOOMCRM_ALL_PEOPLE_WHO_ARE_NOT_LEADS'), 'shared' => Text::_('COM_JOOMCRM_ALL_PEOPLE_WHO_ARE_SHARED'));
		}
		else
		{
			return array('contact' => Text::_('COM_JOOMCRM_CONTACT'), 'lead' => Text::_('COM_JOOMCRM_LEAD'));
		}
	}

	/**
	 * return stages
	 *
	 * @return  array  [description]
	 */
	public static function getStages()
	{
		return array('all' => Text::_('COM_JOOMCRM_INCLUDE_EVERYTHING'), 'today' => Text::_('COM_JOOMCRM_TASKS_DUE_TODAY'), 'tomorrow' => Text::_('COM_JOOMCRM_TASKS_DUE_TOMORROW'), 'past_thirty' => Text::_('COM_JOOMCRM_UPDATED_PAST_THIRTY'), 'recently_added' => Text::_('COM_JOOMCRM_RECENTLY_ADDED'), 'last_import' => Text::_('COM_JOOMCRM_PART_OF_LAST_IMPORT'));
	}

	/**
	 * get column filters
	 *
	 * @return  array  [description]
	 */
	public static function getColumnFilters()
	{
		return array(
			'avatar' => ucwords(Text::_('COM_JOOMCRM_AVATAR')),
			'company' => ucwords(Text::_('COM_JOOMCRM_COMPANY')),
			'email' => ucwords(Text::_('COM_JOOMCRM_EMAIL')),
			'phone' => ucwords(Text::_('COM_JOOMCRM_PHONE')),
			'mobile_phone' => ucwords(Text::_('COM_JOOMCRM_MOBILE_PHONE')),
			'position' => ucwords(Text::_('COM_JOOMCRM_TITLE')),
			'owner' => ucwords(Text::_('COM_JOOMCRM_OWNER')),
			'assigned' => ucwords(Text::_('COM_JOOMCRM_ASSIGNED_TO')),
			'status' => ucwords(Text::_('COM_JOOMCRM_STATUS')),
			'source' => ucwords(Text::_('COM_JOOMCRM_SOURCE')),
			'type' => ucwords(Text::_('COM_JOOMCRM_TYPE')),
			'next_task' => ucwords(Text::_('COM_JOOMCRM_NEXT_TASK')),
			'notes' => ucwords(Text::_('COM_JOOMCRM_NOTES')),
			'city' => ucwords(Text::_('COM_JOOMCRM_CITY')),
			'state' => ucwords(Text::_('COM_JOOMCRM_STATE')),
			'postal_code' => ucwords(Text::_('COM_JOOMCRM_POSTAL_CODE')),
			'country' => ucwords(Text::_('COM_JOOMCRM_COUNTRY')),
			'added' => ucwords(Text::_('COM_JOOMCRM_ADDED')),
			'updated' => ucwords(Text::_('COM_JOOMCRM_UPDATED'))
		);
	}

	/**
	 * get selected column filters
	 *
	 * @return  array  [description]
	 */
	public static function getSelectedColumnFilters()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
			->select("people_columns")
			->from("#__joomcrm_users")
			->where("id=" . (int) JoomcrmHelperUsers::getUserId());

		$results = $db->setQuery($query)->loadResult();

		$columns = !is_null($results) ? unserialize($results) : [];

		if (is_array($columns) && !empty($columns))
		{
			return $columns;
		}
		else
		{
			// If it is empty then load a default set
			return self::getDefaultColumnFilters();
		}
	}

	/**
	 * get default column filters
	 *
	 * @return  array  [description]
	 */
	public static function getDefaultColumnFilters()
	{
		return array('avatar', 'company', 'email', 'phone', 'position', 'type', 'status', 'source', 'notes');
	}


	/**
	 * get people statuses
	 *
	 * @param   string  $status_name   [description]
	 * @param   boolean $returnColors  [description]
	 *
	 * @return  array                   [description]
	 */
	public static function getStatuses($status_name = null, $returnColors = false, $unique = false)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_people_status");

		if ($status_name)
		{
			$query->where("name=" . $db->q($status_name));
		}

		$query->order('ordering');

		$results  = $db->setQuery($query)->loadAssocList();
		$statuses = array();

		if ($returnColors)
		{
			$statuses[0] = array('id' => 0, 'label' => Text::_('COM_JOOMCRM_NONE_STATUS'), 'color' => 'ffffff');
		}
		else
		{
			if (!$unique)
			{
				$statuses[0] = ucwords(Text::_('COM_JOOMCRM_NONE_STATUS'));
			}
		}

		if (count($results) > 0)
		{
			foreach ($results as $key => $status)
			{
				if ($returnColors)
				{
					$statuses[$status['id']] = array('id' => $status['id'], 'label' => $status['name'], 'color' => $status['color']);
				}
				else
				{
					$statuses[$status['id']] = $status['name'];
				}
			}
		}

		return $statuses;
	}

}
