<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Environment\Browser;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;

abstract class JoomcrmHelperStyles
{
	/**
	 * get base stylesheets
	 *
	 * @return  string  [description]
	 */
	public static function getBaseStyle()
	{
		return Uri::base() . "components/com_joomcrm/media/css/default.css";
	}

	/**
	 * load all styles
	 *
	 * @return  void
	 */
	public static function loadStyleSheets()
	{
		$app  = Factory::getApplication();
		$doc  = Factory::getDocument();
		$view = $app->input->get('view');

		if ($view === "print")
		{
			$doc->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/print.css');
		}

		if ($view === 'publiccalendar')
		{
			$doc->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/public.css');
		}



			$base_style = self::getBaseStyle();

			$doc
				->addStyleSheet($base_style, 'text/css', array('data-id' => "joomcrmStylesheet"));

			// Add IE styles....
			$ie    = 'components/com_joomcrm/media/css/ie.css';
			$ieCss = file_get_contents($ie);

			$doc->addCustomTag("<!--[if IE]><style type='text/css'>" . $ieCss . "</style><![endif]--> ");

			// Misc styles
			$doc->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/jquery.cluetip.css');
			$doc->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/datepicker.css');
			$doc->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/jquery.dataTables.css');

			if (JoomcrmHelperUsers::isFullscreen())
			{
				$doc->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/fullscreen.css');

				// If we're in the Beez 2.5 template, force override the template width.
				if (version_compare(JVERSION, '3.0', 'lt') && substr($app->getTemplate(), 0, 4) === 'beez')
				{
					$doc->addStyleDeclaration('body #all {max-width:1200px;font-size:12px}');
				}
			}

			$doc->addStyleSheet(Uri::base() . 'administrator/components/com_joomcrm/media/css/icon-override.css');


		jimport('joomla.environment.browser');
		$browser        = Browser::getInstance();
		$browserType    = $browser->getBrowser();
		$browserVersion = $browser->getMajor();

		if (($browserType == 'msie') && ($browserVersion < 8))
		{
			$doc->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/ie.css');
		}
	}

}
