<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/


defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

abstract class JoomcrmHelperTemplate
{
	public static function startCompWrap()
	{
		$user = JoomcrmHelperUsers::getLoggedInUser();
		$config = ComponentHelper::getParams('com_joomcrm');


		if ($user)
		{
			if (JoomcrmHelperUsers::isFullscreen() && !( Factory::getApplication()->input->get('view') == "print"))
			{
				echo '<div id="logged_in_user" class="d-flex justify-content-center align-items-center mb-3"><span class="me-3 fw-bold">' . $config->get('welcome_message',"Hello") . ' ' . JoomcrmHelperUsers::getFirstName() . "! </span>";
				echo self::displayLogout();
				echo '</div>';
			}
		}

		echo '<div id="com_joomcrm">';
		echo '<div id="message" style="display:none;"></div>';
		echo '<div id="google-map" style="display:none;"></div>';

		if ($user)
		{
			echo '<div id="edit_note_entry" style="display:none;"></div>';
			echo '<div id="edit_convo_entry" style="display:none;"></div>';
			echo '<div id="document_preview_modal" style="display:none;"></div>';
			echo self::getEventDialog();
			echo '<script type="text/javascript">var userDateFormat = "' . JoomcrmHelperUsers::getDateFormat(false) . '";</script>';
			echo '<script type="text/javascript">var user_id = "' . JoomcrmHelperUsers::getUserId() . '";</script>';
		}



		echo '<div id="edit_task" style="display:none;"></div>
				<div id="edit_event" style="display:none;"></div>';

	}

	public static function displayLogout()
	{
		$returnURL = base64_encode(Uri::root());
		$string    = '<form class="inline-form" action="index.php?option=com_users&task=user.logout" method="post">';
		$string .= '<input type="hidden" name="return" value="' . $returnURL . '" />';
		$string .= '<button type="submit" class="btn btn-danger btn-sm"><i class="fas fa-sign-out-alt"></i>'.Text::_('COM_JOOMCRM_LOGOUT').'</button>';
		$string .=HTMLHelper::_('form.token');
		$string .= '</form>';

		return $string;
	}

	public static function loadToolbar()
	{

		// Load menu
		$menu_model = new JoomcrmModelMenu;
		$menu_model->setState('filter.published', 1);
		$list = $menu_model->getMenu();

		echo LayoutHelper::render('common.maintoolbar',['list' => $list]);
	}

	public static function endCompWrap()
	{
		$app = Factory::getApplication();


		// load ajax modal layout
        echo LayoutHelper::render('common.ajaxmodal',[]);
		?>
            <div class="clearfix"></div>
			</div>
	<?php
	
	}

	public static function loadReportMenu()
	{
		$str = "<div id='reports_menu'>";
		$str .= "<ul>";
		$str .= '<li><a href="' . Route::_('index.php?option=com_joomcrm&view=reports') . '" >' . ucwords(Text::_('COM_JOOMCRM_DASHBOARD')) . '</a></li>';
		$str .= "<li class='seperator'></li>";
		$str .= '<li><a href="' . Route::_('index.php?option=com_joomcrm&view=reports&layout=sales_pipeline') . '" >' . ucwords(Text::_('COM_JOOMCRM_SALES_PIPELINE')) . '</a></li>';
		$str .= "<li class='seperator'></li>";
		$str .= '<li><a href="' . Route::_('index.php?option=com_joomcrm&view=reports&layout=source_report') . '" >' . ucwords(Text::_('COM_JOOMCRM_SOURCE_REPORT')) . '</a></li>';
		$str .= "<li class='seperator'></li>";
		$str .= '<li><a href="' . Route::_('index.php?option=com_joomcrm&view=reports&layout=roi_report') . '" >' . ucwords(Text::_('COM_JOOMCRM_ROI_REPORT')) . '</a></li>';
		$str .= "<li class='seperator'></li>";
		$str .= '<li><a href="' . Route::_('index.php?option=com_joomcrm&view=reports&layout=deal_milestones') . '" >' . ucwords(Text::_('COM_JOOMCRM_DEAL_MILESTONES')) . '</a></li>';
		$str .= "<li class='seperator'></li>";
		$str .= '<li><a href="' . Route::_('index.php?option=com_joomcrm&view=reports&layout=notes') . '" >' . ucwords(Text::_('COM_JOOMCRM_NOTES')) . '</a></li>';
		$str .= "<li class='seperator'></li>";
		$str .= '<li><a href="' . Route::_('index.php?option=com_joomcrm&view=reports&layout=custom_reports') . '" >' . ucwords(Text::_('COM_JOOMCRM_CUSTOM_REPORTS')) . '</a></li>';
		$str .= "</ul>";
		$str .= "</div>";

		return $str;
	}

	public static function getEventDialog()
	{
		$html = "";
		$html .= '<div id="new_event_dialog" style="display:none;">';
		$html .= '<div class="new_events">';
		$html .= '<a href="javascript:void(0);" class="task" onclick="addTaskEvent(\'task\')">' . Text::_('COM_JOOMCRM_ADD_TASK') . '</a><a  href="javascript:void(0);" class="event" onclick="addTaskEvent(\'event\')">' . Text::_('COM_JOOMCRM_ADD_EVENT') . '</a><a href="javascript:void(0);" class="complete" onclick="jQuery(\'#new_event_dialog\').dialog(\'close\');">' . Text::_('COM_JOOMCRM_DONE') . '</a>';
		$html .= '</div>';
		$html .= '</div>';

		return $html;
	}


	public static function loadJavascriptLanguage()
	{
		Text::script("COM_JOOMCRM_NAME_YOUR_FIELD");
		Text::script("COM_JOOMCRM_NO_FIELDS_DEFINED");
		Text::script("COM_JOOMCRM_INVALID_OPERATIONS");
		Text::script("COM_JOOMCRM_INVALID_PARENTHESES");
		Text::script("COM_JOOMCRM_INVALID_FIELD_LENGTH");
		Text::script("COM_JOOMCRM_ARE_YOU_SURE_DELETE_REPORT");
		Text::script("COM_JOOMCRM_DRAG_FIELD_HERE");
		Text::script("COM_JOOMCRM_ARE_YOU_SURE_DELETE_REPORT");
		Text::script('COM_JOOMCRM_PLEASE_SELECT_A_USER');
		Text::script('COM_JOOMCRM_SUCCESS_MESSAGE');
		Text::script('COM_JOOMCRM_ADD_NEW_NOTE');
		Text::script('COM_JOOMCRM_ADD');
		Text::script('COM_JOOMCRM_SUCCESS_MESSAGE');
		Text::script('COM_JOOMCRM_GENERIC_UPDATED');
		Text::script('COM_JOOMCRM_DEALS_BY_STAGE');
		Text::script('COM_JOOMCRM_TOTAL');
		Text::script('COM_JOOMCRM_DEALS_BY_STATUS');
		Text::script('COM_JOOMCRM_REVENUE_FROM_LEAD_SOURCES');
		Text::script('COM_JOOMCRM_CURRENCY');
		Text::script('COM_JOOMCRM_YEARLY_COMMISSIONS');
		Text::script('COM_JOOMCRM_MONTHLY_COMMISSIONS');
		Text::script('COM_JOOMCRM_YEARLY_REVENUE');
		Text::script('COM_JOOMCRM_MONTHLY_REVENUE');
		Text::script('COM_JOOMCRM_WEEK');
		Text::script('COM_JOOMCRM_DELETE_GOALS');
		Text::script('COM_JOOMCRM_DELETE_GOAL_CONFIRMATION');
		Text::script('COM_JOOMCRM_TODAY');
		Text::script('COM_JOOMCRM_MONTH');
		Text::script('COM_JOOMCRM_WEEK');
		Text::script('COM_JOOMCRM_DAY');
		Text::script('COM_JOOMCRM_DAYS');
		Text::script('COM_JOOMCRM_EVENT_MARKED_COMPLETE');
		Text::script('COM_JOOMCRM_EVENT_MARKED_INCOMPLETE');
		Text::script('COM_JOOMCRM_EVENTS_SUCCESSFULLY_UPDATED');
		Text::script('COM_JOOMCRM_EVENT_HAS_BEEN_POSTPONED');
		Text::script('COM_JOOMCRM_EDIT_NOTE');
		Text::script('COM_JOOMCRM_JANUARY');
		Text::script('COM_JOOMCRM_FEBRUARY');
		Text::script('COM_JOOMCRM_MARCH');
		Text::script('COM_JOOMCRM_APRIL');
		Text::script('COM_JOOMCRM_MAY');
		Text::script('COM_JOOMCRM_JUNE');
		Text::script('COM_JOOMCRM_JULY');
		Text::script('COM_JOOMCRM_AUGUST');
		Text::script('COM_JOOMCRM_SEPTEMBER');
		Text::script('COM_JOOMCRM_OCTOBER');
		Text::script('COM_JOOMCRM_NOVEMBER');
		Text::script('COM_JOOMCRM_DECEMBER');
		Text::script('COM_JOOMCRM_JAN');
		Text::script('COM_JOOMCRM_FEB');
		Text::script('COM_JOOMCRM_MAR');
		Text::script('COM_JOOMCRM_APR');
		Text::script('COM_JOOMCRM_MAY');
		Text::script('COM_JOOMCRM_JUN');
		Text::script('COM_JOOMCRM_JUL');
		Text::script('COM_JOOMCRM_AUG');
		Text::script('COM_JOOMCRM_SEP');
		Text::script('COM_JOOMCRM_OCT');
		Text::script('COM_JOOMCRM_NOV');
		Text::script('COM_JOOMCRM_DEC');
		Text::script('COM_JOOMCRM_SUNDAY');
		Text::script('COM_JOOMCRM_MONDAY');
		Text::script('COM_JOOMCRM_TUESDAY');
		Text::script('COM_JOOMCRM_WEDNESDAY');
		Text::script('COM_JOOMCRM_THURSDAY');
		Text::script('COM_JOOMCRM_FRIDAY');
		Text::script('COM_JOOMCRM_SATURDAY');
		Text::script('COM_JOOMCRM_SUN');
		Text::script('COM_JOOMCRM_MON');
		Text::script('COM_JOOMCRM_TUE');
		Text::script('COM_JOOMCRM_WED');
		Text::script('COM_JOOMCRM_THU');
		Text::script('COM_JOOMCRM_FRI');
		Text::script('COM_JOOMCRM_SAT');
		Text::script('COM_JOOMCRM_EDIT');
		Text::script('COM_JOOMCRM_DELETE_CONFIRMATION');
		Text::script('COM_JOOMCRM_DELETE_LIST_CONFIRMATION');
		Text::script('COM_JOOMCRM_DELETE_PERSON_FROM_DEAL_CONFIRM');
		Text::script('COM_JOOMCRM_MARK_COMPLETE');
		Text::script('COM_JOOMCRM_MARK_INCOMPLETE');
		Text::script('COM_JOOMCRM_SUCCESSFULLY_REMOVED_EVENT');
		Text::script('COM_JOOMCRM_ITEM_SUCCESSFULLY_UPDATED');
		Text::script('COM_JOOMCRM_OK');
		Text::script('COM_JOOMCRM_ADDED');
		Text::script('COM_JOOMCRM_VERIFY_ALERT');
		Text::script('COM_CMERY_ARE_YOU_SURE_DELETE_REPORT');
		Text::script('COM_JOOMCRM_DEALS_BY_STAGE');
		Text::script('COM_JOOMCRM_DEALS_BY_STATUS');
		Text::script('COM_JOOMCRM_TOTAL');
		Text::script('COM_JOOMCRM_UPDATE');
		Text::script('COM_JOOMCRM_UPLOADING');
		Text::script('COM_JOOMCRM_YOUR_DOCUMENT_IS_BEING_UPLOADED');
		Text::script('COM_JOOMCRM_MESSAGE_DETAILS');
		Text::script('COM_JOOMCRM_CLICK_TO_EDIT');
		Text::script('COM_JOOMCRM_UPDATED_PRIMARY_CONTACT');
		Text::script('COM_JOOMCRM_MANAGE_MAILING_LISTS');
		Text::script('COM_JOOMCRM_MAILING_LIST_LINKS');
		Text::script('COM_JOOMCRM_REMOVE');
		Text::script('COM_JOOMCRM_ADD');
		Text::script('COM_JOOMCRM_EMAIL');
		Text::script('COM_JOOMCRM_ACTIVE_DEAL');
		Text::script('COM_JOOMCRM_START_TYPING_DEAL');
		Text::script('COM_JOOMCRM_START_TYPING_PERSON');
		Text::script('COM_JOOMCRM_SUCCESSFULLY_SHARED_ITEM');
		Text::script('COM_JOOMCRM_SUCCESSFULLY_UNSHARED_ITEM');
		Text::script('COM_JOOMCRM_SHARE_ITEM');
		Text::script('COM_JOOMCRM_OUT_OF');
		Text::script('COM_JOOMCRM_FILENAME');
		Text::script('COM_JOOMCRM_DUPLICATE_COMPANY');
		Text::script('COM_JOOMCRM_DOCUMENT_INFO');
		Text::script('COM_JOOMCRM_SHOW_FILTERS');
		Text::script('COM_JOOMCRM_RESET_FILTERS');
		Text::script('COM_JOOMCRM_CREATE_TASK');
		Text::script('COM_JOOMCRM_CREATE_EVENT');
		Text::script('COM_JOOMCRM_CONTACTS');
		Text::script('COM_JOOMCRM_CLOSE');
		Text::script('COM_JOOMCRM_READ_MORE');
		Text::script('COM_JOOMCRM_ERROR_LOADING');
		Text::script('COM_JOOMCRM_EMAIL_SAVED');
		Text::script('COM_JOOMCRM_FAILURE');

		/** Here we load custom javascript variables that are not loaded through Joomla! language **/
		Factory::getDocument()
			->addScriptDeclaration("var comJoomcrmCurrency='" . JoomcrmHelperConfig::getCurrency() . "';");
	}

	public static function getListEditActions()
	{
		$list_html = "";

		if (JoomcrmHelperUsers::canDelete())
		{
			$list_html .= "<div id='list_edit_actions'>";
			$list_html .= '<div class="dropdown" id="list_edit_actions_dropdown">';
			$list_html .= Text::_('COM_JOOMCRM_PERFORM') . "<a class='dropdown btn btn-light border btn-sm' data-bs-toggle='dropdown' id='list_edit_actions_dropdown_link'>" . Text::_('COM_JOOMCRM_ACTIONS') . "</a>" . Text::_('COM_JOOMCRM_ON_THE') . "<span id='items_checked'></span> " . Text::_('COM_JOOMCRM_ITEMS');
			$list_html .= '<ul class="dropdown-menu float-start">';
			$list_html .= '<li><a class="dropdown-item" onclick="deleteListItems()">' . Text::_('COM_JOOMCRM_DELETE') . '</a></li>';
			$list_html .= '</ul>';
			$list_html .= "</div>";
			$list_html .= "</div>";
		}

		return $list_html;
	}

	public static function getEventListEditActions()
	{
		$list_html = "";

		$list_html .= "<div id='list_edit_actions'>";
		$list_html .= '<div class="dropdown" id="list_edit_actions_dropdown">';
		$list_html .= Text::_('COM_JOOMCRM_PERFORM') . "<a class='dropdown-toggle btn btn-sm btn-light' data-bs-toggle='dropdown' id='list_edit_actions_dropdown_link'>" . Text::_('COM_JOOMCRM_ACTIONS') . "</a>" . Text::_('COM_JOOMCRM_ON_THE') . "<span id='items_checked'></span> " . Text::_('COM_JOOMCRM_ITEMS');
		$list_html .= '<ul class="dropdown-menu float-start">';
		$list_html .= '<li><a onclick="deleteListItems()">' . Text::_('COM_JOOMCRM_DELETE') . '</a></li>';
		$list_html .= '</ul>';
		$list_html .= "</div>";
		$list_html .= "</div>";

		return $list_html;
	}

	public static function adjustWebsiteUrl($website)
	{
		$website = trim($website);

		if (strpos($website, 'http') !== 0)
		{
			$website = 'http://' . $website;
		}

		return $website;
	}

	/*
	 * This method force JoomCRM to load on a specified theme
	 */
	public static function forceTemplateTheme($theme_id = 0){

	    // get selected theme from config if 0
		$theme_id = ($theme_id > 0 ) ? $theme_id : ComponentHelper::getParams('com_joomcrm')->get('force_template_theme','');

		// return if theme id is 0
		if(!$theme_id) return false;

		// get template data
		$app = Factory::getApplication();
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('*')
			->from('`#__template_styles`')
			->where("`id`= $theme_id");
		$db->setQuery($query);
		$theme = $db->loadObject();

		// load template
		$app->setTemplate($theme);

        // make sure to load template asset

        $asset = 'templates/'.$theme->template.'/joomla.asset.json';


        if(!file_exists(JPATH_ROOT.'/'.$asset))
            return true;


		$doc = Factory::getApplication()->getDocument();
		$wa = $doc->getWebAssetManager();
		$wa->getRegistry()->addRegistryFile($asset);

		return true;
    }


}
