<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;

	abstract class JoomcrmHelperUsers
{
	/*
	 * Return array of all JoomCRM users as objects
	 */
	public static function getAllUsers(){

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		$query->select("*")->from("#__joomcrm_users");

		$query->where("published=1");

		return $db->setQuery($query)->loadObjectList();

	}

	/*
	 * This method return list of all users ready for jhtml select use.
	 */
	public static function getAllUsersForSelect(){

		$users = self::getAllUsers();
		$list = [];
		$list[''] = Text::_('COM_JOOMCRM_SELECT_OWNER');

		if(is_null($users)) return ;

		foreach ($users as $user){

			$isMe = $user->uid == Factory::getApplication()->getIdentity()->id ? '(Me)' : '';

			$list[$user->id] = "$user->first_name $user->last_name $isMe";
		}

		return $list;


	}

	/**
	 * get users depending on logged in member type
	 *
	 * @param   integer $id       [description]
	 * @param   boolean $idsOnly  [description]
	 *
	 * @return  array              [description]
	 */
	public static function getUsers($id = null, $idsOnly = false)
	{
		// Filter based on current logged in user
		$user      = self::getUserId();
		$user_role = self::getRole();
		$results   = array();
		$team_access = self::getTeamAccess($user);

		// User role filters
		if ($user_role != 'basic' || $team_access)
		{
			$db    = Factory::getDbo();
			$query = $db->getQuery(true);

			$select = ($idsOnly) ? "id AS value, CONCAT(first_name, ' ', last_name) AS label" : "*";

			$query->select($select)->from("#__joomcrm_users");

			if ($id)
			{
				$query->where('id = ' . (int) $id);
			}
			else
			{
				if ($user_role == 'exec')
				{
					// $query->where("id<>".$user);
				}
				else
				{
					if ($user_role == 'manager' || $team_access)
					{
						$team_id = self::getTeamId();
						$query->where('team_id=' . $team_id /*.' AND id <> '.$user*/);
					}
				}
			}

			$query->where("published=1");

			$results = $db->setQuery($query)->loadAssocList();
		}

		// Assign other user info
		if (!$idsOnly)
		{
			if (count($results) > 0)
			{
				foreach ($results as $key => $user)
				{
					$results[$key]['emails'] = self::getEmails($user['id']);
				}
			}
		}

		return $results;
	}

	/**
	 * Get specified users first name.
	 *
	 * @param   integer $id  User ID to use in query.
	 *
	 * @return  string        First name for specified user id.
	 */
	public static function getFirstName($id = null)
	{
		if (is_null($id))
		{
			$id = self::getLoggedInUser()->id;
		}

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("first_name")->from("#__joomcrm_users")->where('id=' . (int) $id);

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Get all company users
	 *
	 * @return  array       [description]
	 */
	public static function getCompanyUsers()
	{
		// Filter based on current logged in user
		$user      = Factory::getApplication()->getIdentity();
		$user_role = self::getRole();
		$results   = array();

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_users")->where("published=1");

		$results = $db->setQuery($query)->loadAssocList();

		// Assign other user info
		foreach ($results as $key => $user)
		{
			$results[$key]['emails'] = self::getEmails($user['id']);
		}

		return $results;
	}

	/**
	 * Get user email address for a user
	 *
	 * @param   int $id  user id to get emails for
	 *
	 * @return  mixed  $results db results
	 */
	public static function getEmails($id = null)
	{
		// Joomcrm User ID
		if (is_null($id))
		{
			$id = self::getUserId();
		}

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_users_email_cf")->where("member_id=" . (int) $id);

		$email_cf = $db->setQuery($query)->loadAssocList();

		$query->clear()->select("j.email, u.id AS member_id")->from("#__joomcrm_users AS u")
		->leftJoin("#__users AS j ON j.id = u.uid")->where("u.id=" . (int) $id);

		$primary = $db->setQuery($query)->loadAssocList();

		return array_merge($email_cf, $primary);
	}

	/**
	 * return current logged in JoomCRM user ID based on Joomla Id
	 *
	 * @return  string  JoomCRM User ID for currently logged in Joomla user.
	 */
	public static function getUserId($uid = null)
	{
		$app = Factory::getApplication();

		if(is_null($uid)){
			$uid = $app->input->get('uid', NULL);
		}

		$user = Factory::getUser($uid);

		if ($user->id > 0)
		{
			$db    = Factory::getDbo();
			$query = $db->getQuery(true)
				->select("id")
				->from("#__joomcrm_users")
				->where('uid = ' . (int) $user->id);

			return $db->setQuery($query)->loadResult();
		}

		return 0;
	}

	/**
	 * return user role
	 *
	 * @param   integer $user_id  [description]
	 *
	 * @return  string             [description]
	 */
	public static function getRole($user_id = null)
	{
		if (is_null($user_id))
		{
			$user_id = Factory::getApplication()->getIdentity()->id;
		}

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("role_type")->from("#__joomcrm_users")->where('uid = ' . (int) $user_id);

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * return user team id
	 *
	 * @param   integer $user_id  [description]
	 *
	 * @return  string             [description]
	 */
	public static function getTeamId($user_id = null,$columnIdentifier = 'id')
	{

		if (is_null($user_id))
		{
			// Return early if we still don't have a user id.
			if (!($user_id = self::getUserId()))
			{
				return 0;
			}
		}

		$user_id = (int) $user_id;

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("team_id")->from("#__joomcrm_users")->where("$columnIdentifier = $user_id");

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * return user team id
	 *
	 * @param   integer $user_id  [description]
	 *
	 * @return  string             [description]
	 */
	public static function getTeamAccess($user_id = null)
	{
		if (is_null($user_id))
		{
			// Return early if we still don't have a user id.
			if (!($user_id = self::getUserId()))
			{
				return 0;
			}
		}

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("team_access")->from("#__joomcrm_users")->where('id = ' . (int) $user_id);

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * return team name
	 *
	 * @param   integer $id  [team id]
	 *
	 * @return  string [team name]
	 */
	public static function getTeamName($id){

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$query->select("name")
			->from("#__joomcrm_teams")
			->where("team_id = $id");
		$db->setQuery($query);

		$result = $db->loadResult();

		return $result;
	}

	public static function getOwnerFromTeamId($team_id){

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$query->select("u.uid as team_owner_id")
			->from("#__joomcrm_teams t")
			->join('left',"#__joomcrm_users u on u.id = t.leader_id")
			->where("t.team_id = $team_id");

		$db->setQuery($query);

		$result = $db->loadResult();

		return $result;

	}

	/*
	 * Check of user given id has a team
	 */
	public static function isTeamOwner($id)
	{

		$id = (int) $id;

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$query
			->select("t.leader_id")
			->from("#__joomcrm_users u")
			->join("left", "#__joomcrm_teams t on u.id = t.leader_id")
			->where("u.uid = $id");

		$db->setQuery($query);

		$result = $db->loadResult();

		return !is_null($result) && $result > 0 ? true : false;

	}

	/**
	 * return teams to execs
	 *
	 * @param   integer $id  [description]
	 *
	 * @return  array         [description]
	 */
	public static function getTeams($id = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
				 ->select("t.*,u.first_name,u.last_name,IF(t.name!='',t.name,CONCAT(u.first_name,' ',u.last_name)) AS team_name")
				 ->from("#__joomcrm_teams AS t")
				 ->leftJoin("#__joomcrm_users AS u ON u.id = t.leader_id AND u.published=1");

		// Search for specific team
		if ($id)
		{
			$query->where("t.team_id=$id");
		}

		$user_role = self::getRole();
		$user_id   = self::getUserId();

		if ($user_role == 'manager')
		{
			$team_id = self::getTeamId();
			$query->where('t.team_id = ' . (int) $team_id);
		}

		$query->where('u.published = 1');

		return $db->setQuery($query)->loadAssocList();
	}

	/**
	 * Get users associated with a specific team
	 *
	 * @param   integer $id  specific team id requested
	 *
	 * @return  mixed    results from database
	 */
	public static function getTeamUsers($id = null, $idsOnly = false)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		$select = ($idsOnly) ? "u.id" : "u.*";

		if (is_null($id))
		{
			$id = self::getTeamId();
		}

		$query->select($select)->from("#__joomcrm_users AS u")->where('u.team_id = ' . (int) $id)
		->where('u.published = 1');

		$db->setQuery($query);

		if ($idsOnly)
		{
			$users = $db->loadColumn();
		}
		else
		{
			$users = $db->loadAssocList();
		}

		return $users;
	}

	/**
	 * [getAllSharedUsers description]
	 *
	 * @return  array  [description]
	 */
	public static function getAllSharedUsers()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("id AS value,CONCAT(first_name,' ',last_name) AS label")
				 ->from("#__joomcrm_users")->where("published=1");

		$role = self::getRole();

		switch ($role)
		{
			case "manager":
			case "basic":
				$query->where('team_id = ' . (int) self::getTeamId());
				break;
		}

		return $db->setQuery($query)->loadObjectList();
	}

	/**
	 * [getItemSharedUsers description]
	 *
	 * @param   [type]  $itemId    [description]
	 * @param   [type]  $itemType  [description]
	 *
	 * @return  array              [description]
	 */
	public static function getItemSharedUsers($itemId, $itemType)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select("s.user_id AS value,CONCAT(u.first_name,' ',u.last_name) AS label")
				 ->from("#__joomcrm_shared AS s")->leftJoin("#__joomcrm_users AS u ON u.id = s.user_id AND u.published=1")
				 ->where("s.item_id=" . $itemId)->where("s.item_type=" . $db->quote($itemType));

		return $db->setQuery($query)->loadObjectList();
	}

	/**
	 * Get deal count associated with users
	 *
	 * @param  $id   int User Id to filter for
	 * @param  $team int Team Id associated to user
	 * @param  $role String User role to filter for
	 *
	 * @return integer  Count of deals returned from database
	 */
	public static function getDealCount($id, $team, $role)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select('count(*)')->from('#__joomcrm_deals AS d')
				 ->leftJoin("#__joomcrm_users AS u ON u.id = d.owner_id AND u.published=1");

		// Filter based on id and role
		if ($role != 'exec')
		{
			if ($role == 'manager')
			{
				$query->where("u.team_id=$team");
			}
			else
			{
				$query->where("d.owner_id=$id");
			}
		}

		$query->where("d.published=1");

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Get deal count associated with users
	 *
	 * @param $id   int User Id to filter for
	 * @param $team int Team Id associated to user
	 * @param $role String User role to filter for
	 *
	 * @return  int  Count of deals returned from database
	 */
	public static function getEventCount($id, $team, $role)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select('count(*)')->from('#__joomcrm_events AS e')
				 ->leftJoin("#__joomcrm_users AS u ON ( u.id = e.owner_id OR u.id = e.assignee_id ) AND u.published=1");

		// Filter based on id and role
		if ($role != 'exec')
		{
			if ($role == 'manager')
			{
				$query->where("u.team_id=$team");
			}
			else
			{
				$query->where("e.owner_id=$id");
			}
		}
		$query->where("e.published=1");

		// Return results
		$db->setQuery($query);
		$count = $db->loadResult();

		return $count;
	}

	/**
	 * Get people count associated with users
	 *
	 * @param $id   int User Id to filter for
	 * @param $team int Team Id associated to user
	 * @param $role String User role to filter for
	 *
	 * @return  int  Count of people returned from database
	 */
	public static function getPeopleCount($id = null, $team = null, $role = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		if (!$id)
		{
			$id = self::getUserId();
		}
		if (!$team)
		{
			$team = self::getTeamId();
		}
		if (!$role)
		{
			$role = self::getRole();
		}

		// Query
		$query->select('count(*)')->from('#__joomcrm_people AS p')
		->leftJoin("#__joomcrm_users AS u ON ( u.id = p.owner_id OR u.id = p.assignee_id ) AND u.published=1");

		// Filter based on id and role
		if ($role != 'exec')
		{
			if ($role == 'manager')
			{
				$query->where("u.team_id=$team");
			}
			else
			{
				$query->where("( p.owner_id=$id OR p.assignee_id=$id )");
			}
		}

		$query->where("p.published=1");

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Get people count associated with users
	 *
	 * @param $id   int User Id to filter for
	 * @param $team int Team Id associated to user
	 * @param $role String User role to filter for
	 *
	 * @return  int  Count of people returned from database
	 */
	public static function getLeadCount($id = null, $team = null, $role = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		if (!$id)
		{
			$id = self::getUserId();
		}
		if (!$team)
		{
			$team = self::getTeamId();
		}
		if (!$role)
		{
			$role = self::getRole();
		}

		$query->select('count(*)')->from('#__joomcrm_people AS p')
		->leftJoin("#__joomcrm_users AS u ON ( u.id = p.owner_id OR u.id = p.assignee_id ) AND u.published=1");

		// Filter based on id and role
		if ($role != 'exec')
		{
			if ($role == 'manager')
			{
				$query->where("u.team_id=$team");
			}
			else
			{
				$query->where("( p.owner_id=$id OR p.assignee_id=$id )");
			}
		}

		$query->where("p.type='lead'")->where("p.published=1");

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Get people count associated with users
	 *
	 * @param $id   int User Id to filter for
	 * @param $team int Team Id associated to user
	 * @param $role String User role to filter for
	 *
	 * @return  int  Count of people returned from database
	 */
	public static function getContactCount($id = null, $team = null, $role = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		if (!$id)
		{
			$id = self::getUserId();
		}
		if (!$team)
		{
			$team = self::getTeamId();
		}
		if (!$role)
		{
			$role = self::getRole();
		}

		$query->select('count(*)')->from('#__joomcrm_people AS p')
		->leftJoin("#__joomcrm_users AS u ON ( u.id = p.owner_id OR u.id = p.assignee_id ) AND u.published=1");

		// Filter based on id and role
		if ($role != 'exec')
		{
			if ($role == 'manager')
			{
				$query->where("u.team_id=$team");
			}
			else
			{
				$query->where("( p.owner_id=$id OR p.assignee_id=$id )");
			}
		}

		$query->where("p.type='contact'")->where("p.published=1");

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Get people emails associated with users
	 *
	 * @param $id   int User Id to filter for
	 * @param $team int Team Id associated to user
	 * @param $role String User role to filter for
	 *
	 * @return  int  Count of people returned from database
	 */
	public static function getPeopleEmails($id = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		if (!$id)
		{
			$id = self::getUserId();
		}

		$query->select('p.id,p.email')->from('#__joomcrm_people AS p')
		->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=p.id AND shared.item_type='person'")
		->where("(p.owner_id=$id OR p.assignee_id=$id OR shared.user_id=$id)");

		$results = $db->setQuery($query)->loadAssocList();

		// Clean results
		$return = array();

		if ($results)
		{
			foreach ($results as $key => $user)
			{
				$return[$user['id']] = $user['email'];
			}
		}

		return $return;
	}

	/**
	 * Get array of people IDs by their e-mail address
	 *
	 * @param 	string 	$email   email address for search
	 *
	 * @return  array 	
	 */
	public static function getPeopleIdsByEmail($email)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		$query->select('p.id')
			->from('#__joomcrm_people AS p')
			->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=p.id AND shared.item_type='person'")
			->where('p.email = ' . $db->quote($email));

		$ids = $db->setQuery($query)->loadColumn();

		return $ids;
	}

	/**
	 * Get company count associated with users
	 *
	 * @param $id   int User Id to filter for
	 * @param $team int Team Id associated to user
	 * @param $role String User role to filter for
	 *
	 * @return  int  Count of companies returned from database
	 */
	public static function getCompanyCount($id, $team, $role)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select('count(*)')->from('#__joomcrm_companies AS c')
				 ->leftJoin("#__joomcrm_users AS u ON u.id = c.owner_id AND u.published=1")->where("c.published=1");

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Get commission rates for users
	 *
	 * @param   int $id  user id requested else logged in user id is used
	 *
	 * @return  int  commission rate
	 */
	public static function getCommissionRate($id = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		// Logged in user
		if ($id == null)
		{
			$user    = Factory::getApplication()->getIdentity();
			$user_id = self::getUserId();
		}
		else
		{
			$user_id = $id;
		}

		$query->select("commission_rate")->from("#__joomcrm_users")->where('id=' . $user_id)->where("published=1");

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * [isFullscreen description]
	 *
	 * @return  boolean  [description]
	 */
	public static function isFullscreen()
	{
		$config     = ComponentHelper::getParams('com_joomcrm');

		if ($config->get("fullscreen",0) == 1)
		{
			return true;
		}

        $user_id = self::getUserId();

        if ($user_id > 0)
        {
            $db    = Factory::getDbo();
            $query = $db->getQuery(true)
                ->select('fullscreen')
                ->from('#__joomcrm_users')
                ->where('id=' . (int) $user_id);

            return $db->setQuery($query)->loadResult();
        }

        return false;
	}

	/**
	 * [getDateFormat description]
	 *
	 * @param   boolean $php  [description]
	 *
	 * @return  [type]         [description]
	 */
	public static function getDateFormat($php = true)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		// Logged in user
		$user    = Factory::getApplication()->getIdentity();
		$user_id = self::getUserId();

		if ($user_id > 0)
		{
			$query->select("date_format")->from("#__joomcrm_users")->where('id=' . (int) $user_id);

			if (!($format = $db->setQuery($query)->loadResult()))
			{
				$format = 'm/d/y';
			}
		}
		else
		{
			return "m/d/y";

		}

		if (!$php)
		{
			$format = str_replace(array('m', 'd', 'y'), array('mm', 'dd', 'yy'), $format);
		}

		return $format;
	}

	/**
	 * [getTimeFormat description]
	 *
	 * @param   [type]  $id  [description]
	 *
	 * @return  [type]       [description]
	 */
	public static function getTimeFormat($id = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$config     = ComponentHelper::getParams('com_joomcrm');

		// Logged in user
		if ($id == null)
		{
			$user    = Factory::getApplication()->getIdentity();
			$user_id = self::getUserId();
		}
		else
		{
			$user_id = $id;
		}

		if ($user_id > 0)
		{
			$query->select("time_format")->from("#__joomcrm_users")->where('id=' . (int) $user_id);

			if (!($time = $db->setQuery($query)->loadResult()))
			{
				$time = $config->get("time_format",'g:i A');
			}

			return $time;
		}
		else
		{
			return $config->get("time_format",'g:i A');
		}

	}


	public static function getTimezone($id = null)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$config     = ComponentHelper::getParams('com_joomcrm');

		// Logged in user
		if ($id == null)
		{
			$user    = Factory::getApplication()->getIdentity();
			$user_id = self::getUserId();
		}
		else
		{
			$user_id = $id;
		}

		if ($user_id > 0)
		{
			$query->select("time_zone")->from("#__joomcrm_users")->where('id=' . (int) $user_id);

			if (!($tz = $db->setQuery($query)->loadResult()))
			{
				$tz = $config->get('timezone','America/New_York');
			}

			return $tz;
		}
		else
		{
			return $config->get('timezone','America/New_York');
		}

	}

	/**
	 * [getLoggedInUser description]
	 *
	 * @return  [type]  [description]
	 */
	public static function getLoggedInUser()
	{
		$user_id = Factory::getApplication()->getIdentity()->id;
		$db      = Factory::getDbo();
		$query   = $db->getQuery(true)
			->select('c.*, u.email')
			->from('#__joomcrm_users AS c')
			->where('c.uid = ' . $db->quote($user_id))
			->where("c.published=1")
			->leftJoin("#__users AS u ON u.id=c.uid");

		$user = $db->setQuery($query)->loadObject();

		if ($user !== null && $user->id > 0)
		{
			$user->emails          = self::getEmails($user->id);
			$user->document_bypass = !is_null($user->document_bypass) ? unserialize($user->document_bypass) : '';
		}

		return $user;
	}

	/**
	 * [getDocumentBypass description]
	 *
	 * @return  [type]  [description]
	 */
	public static function getDocumentBypass()
	{
		$userId = self::getLoggedInUser()->id;
		$db     = Factory::getDbo();
		$query  = $db->getQuery(true)->select("document_bypass")->from("#__joomcrm_users")->where("id=" . (int) $userId);

		$result = $db->setQuery($query)->loadResult();

		if (!empty($result))
		{
			try
			{
				$return = unserialize($result);
			}
			catch (Exception $e)
			{
				$return = array();
			}
		}
		else
		{
			$return = array();
		}

		return $return;
	}

	/**
	 * [getUser description]
	 *
	 * @param   integer $user_id  [description]
	 * @param   boolean $array    [description]
	 *
	 * @return  mixed              [description]
	 */
	public static function getUser($user_id, $array = false)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select('c.*')->from('#__joomcrm_users AS c')
				 ->where('c.id = ' . $db->quote($user_id))->where("c.published=1");

		$db->setQuery($query);

		if (!$array)
		{
			return $db->loadObject();
		}
		else
		{
			return $db->loadColumn();
		}
	}

	/**
	 * Get user id from given email
	 *
	 * @param   string $email  user email
	 *
	 * @return  integer or bool
	 */
	public static function getUserIdFromEmail($email){
		$db = Factory::getDbo();
		$query = $db
			->getQuery(true)
			->select('id')
			->from($db->quoteName('#__users'))
			->where($db->quoteName('email') . " = " . $db->quote($email));

		$db->setQuery($query);

		return $db->loadResult();
	}

	/**
	 * Determine if logged in user ( or specified user ) is an administrator
	 *
	 * @param   integer $user_id  [description]
	 *
	 * @return  boolean            [description]
	 */
	public static function isAdmin($user_id = null)
	{
		if (is_null($user_id))
		{
			$user_id = self::getUserId();
		}

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select('c.admin')->from('#__joomcrm_users AS c')
				 ->where('c.id = ' . $db->quote($user_id));

		$user = $db->setQuery($query)->loadObject();

		return $user->admin == 1;
	}

	/**
	 * Determine if logged in user ( or specified user ) can delete items
	 *
	 * @param   integer $user_id  [description]
	 *
	 * @return  boolean            [description]
	 */
	public static function canDelete($user_id = null)
	{
		if (is_null($user_id))
		{
			$user_id = self::getUserId();
		}

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select('c.admin,c.can_delete')->from('#__joomcrm_users AS c')
				 ->where('c.id = ' . $db->quote($user_id));

		$user = $db->setQuery($query)->loadObject();

		return ($user->admin == 1 || $user->can_delete == 1);
	}

	/**
	 * Determine if logged in user ( or specified user ) can export items
	 *
	 * @param   [type]  $user_id  [description]
	 *
	 * @return  boolean           [description]
	 */
	public static function canExport($user_id = null)
	{
		if (is_null($user_id))
		{
			$user_id = self::getUserId();
		}

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select('c.exports,c.admin')->from('#__joomcrm_users AS c')
				 ->where('c.id = ' . $db->quote($user_id));

		$user = $db->setQuery($query)->loadObject();

		return ($user->exports == 1 || $user->admin == 1);
	}

	/**
	 * [formatAmount description]
	 *
	 * @param   float $amount  [description]
	 *
	 * @return  float           [description]
	 */
	public static function formatAmount($amount)
	{

		// get joomcrm config
		$config     = \Joomla\CMS\Component\ComponentHelper::getParams('com_joomcrm');

		// get currency params
		$currency = $config->get('currency','$');
		$currencyPosition = (bool) $config->get('currency_position',0);
		$decimalDelimiter = $config->get('decimal_delimiter','.');
		$thousandsDelimiter = $config->get('thousands_delimiter',',');

		// format amount
		$amount = number_format((float) $amount, 2,$decimalDelimiter,$thousandsDelimiter);

		// add currency position and return
		return !$currencyPosition ? $currency.' '.$amount : $amount.' '.$currency;
	}

	/**
	 * [getGraphDate description]
	 *
	 * @param   [type]   $graphField  [description]
	 * @param   boolean $format      [description]
	 *
	 * @return  [type]                [description]
	 */
	public static function getGraphDate($graphField, $format = false)
	{
		$userId = self::getUserId();
		$db     = Factory::getDbo();
		$query  = $db->getQuery(true)->select($graphField)->from("#__joomcrm_users")->where("id=" . (int) $userId);

		$result = $db->setQuery($query)->loadResult();

		if (strtotime((int)$result) <= 0)
		{
			switch ($graphField)
			{
				case "dashboard_graph_date_start":
				case "report_graph_date_start":
					$result = date("Y-01-01");
					break;
				case "dashboard_graph_date_end":
				case "report_graph_date_end":
					$result = date("Y-12-31");
					break;
			}
		}

		return $format ? JoomcrmHelperDate::formatDateString($result) : $result;
	}

	/**
	 * [getHomePageChart description]
	 *
	 * @return  string  [description]
	 */
	public static function getHomePageChart()
	{
		$user_id = self::getUserId();
		$db      = Factory::getDbo();
		$query   = $db->getQuery(true)->select("home_page_chart")->from("#__joomcrm_users")
				   ->where("id=" . (int) $user_id);

		$chart = $db->setQuery($query)->loadResult();

		return !empty($chart) ? $chart : "dealStagePie";
	}

	/**
	 * [getFloats description]
	 *
	 * @param   [type]  $type  [description]
	 *
	 * @return  [type]         [description]
	 */
	public static function getFloats($type)
	{
		$userId = self::getUserId();

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)->select($type . "_floats_right," . $type . "_floats_left")->from("#__joomcrm_users")
				 ->where("id=" . $userId);

		$floats = $db->setQuery($query)->loadAssoc();

		if ($floats)
		{
			$return          = array();
			$return["left"]  = explode(",", $floats[$type . "_floats_left"]);
			$return["right"] = explode(",", $floats[$type . "_floats_right"]);

			if ((count($return['left']) == 0 && count($return['right']) == 0) || ($floats[$type . '_floats_left'] == "" && $floats[$type . '_floats_right'] == ""))
			{
				return self::getDefaultFloats($type);
			}

			return $return;
		}
		else
		{
			return self::getDefaultFloats($type);
		}
	}

	/**
	 * [getDefaultFloats description]
	 *
	 * @param   [type]  $type  [description]
	 *
	 * @return  array          [description]
	 */
	public static function getDefaultFloats($type)
	{
		switch ($type)
		{
			case "dashboard":
				$return          = array();
				$return['left']  = explode(",", "tasks_events_float,deals_float");
				$return['right'] = explode(",", "sales_float,inbox_float,latest_float");

				return $return;
				break;
			case "goal":
				$return          = array();
				$return['left']  = explode(",", "leaderboard_float");
				$return['right'] = explode(",", "individual_goals_float,team_goals_float,company_goals_float");

				return $return;
				break;
			case "sales_dashboard":
				$return          = array();
				$return['left']  = explode(",", "deal_stage_float,deal_status_float");
				$return['right'] = explode(",", "yearly_commission_float,yearly_revenue_float,monthly_commission_float,monthly_revenue_float");

				return $return;
				break;
		}

		return array('left' => array(), 'right' => array());
	}

	/**
	 * Validate users API key
	 * @param  string $key The hash value of the user
	 * @return boolean     True if the value provided is equal
	 */
	public static function validateAPIKey($uid, $key)
	{
		$db = Factory::getDbo();

		$truth = $db->setQuery(
			$db->getQuery(TRUE)
			->select('api')
			->from('#__joomcrm_users')
			->where('uid = ' . $uid))
		->loadResult();

		if($key === $truth)
		{
			return true;
		}

		return false;
	}



}
