<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */
// No direct access
defined('_JEXEC') or die;

use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\MVC\Controller\BaseController;

JLoader::register('JoomcrmHelperVersion',JPATH_ADMINISTRATOR . '/components/com_joomcrm/helper/version.php');
// check if compatiblity plugin is enabled
//JoomcrmHelperVersion::compatiblityPlugin();
HTMLHelper::_('bootstrap.dropdown');
HTMLHelper::_('bootstrap.collapse');
HTMLHelper::_('bootstrap.tab','#myTab');
HTMLHelper::_('bootstrap.collapse','.navbar-toggler');
HTMLHelper::_('bootstrap.popover');
HTMLHelper::_('bootstrap.toast');


// load Joomcrm library
require_once __DIR__.'/libraries/vendor/autoload.php';

// Set to 1,000,000. This should be enough.
ini_set('pcre.backtrack_limit', 1000000);

// Sessions
//jimport('joomla.session.session');
jimport('joomla.application.component.helper');
HTMLHelper::_('formbehavior.chosen', 'select:not(.no-chosen)');


JLoader::register('JoomcrmHelperRoute', JPATH_SITE . '/components/com_joomcrm/helper/route.php');
JLoader::register('JoomcrmPagination',JPATH_SITE.'/components/com_joomcrm/helper/pagination.php');
JLoader::register('JoomcrmHelperPluginEvent',JPATH_SITE.'/components/com_joomcrm/helper/pluginEvent.php');

// migrate legacy config to new joomla config
//JoomcrmHelperMigrate::migrateConfig();

// load custom css code
$customCssCode = ComponentHelper::getParams('com_joomcrm')->get('custom_css','');

if(!empty($customCssCode)){
	Factory::getDocument()->addStyleDeclaration($customCssCode);
}

Factory::getDocument()->addScriptDeclaration("
let base_url = '" .Uri::base() . "';
");

JLoader::registerPrefix('Joomcrm', __DIR__);
JLoader::registerPrefix('Joomcrm', JPATH_ADMINISTRATOR . '/components/com_joomcrm');

if (JoomcrmHelperQuote::hasCrmQuote())
{
	JoomcrmHelperQuote::loadLanguage();
}

// Require helper files
// ths helper is removed in joomla 4
//require_once JPATH_ADMINISTRATOR . '/modules/mod_quickicon/helper.php';

// Load plugins
PluginHelper::importPlugin('joomcrm');

$app    = Factory::getApplication();
$view   = $app->input->get('view', 'dashboard');
$format = $app->input->get('format', 'html');
$task   = $app->input->get('task', 'display');
$config = ComponentHelper::getParams('com_joomcrm');

// Load only of doc type is HTML
if (\Joomla\CMS\Factory::getDocument()->getType() == 'html' && \Joomla\CMS\Component\ComponentHelper::getParams('com_joomcrm')->get('force_bs',0)){


	HTMLHelper::_('bootstrap.framework');
	HTMLHelper::_('bootstrap.loadcss');
}




$controller = null;
if (strpos($task, '.') !== false)
{
	list($controller, $t) = explode('.', $task);
}
else
{
	$noController = array('note', 'print', 'activities', 'profile', 'contacts', 'mailing');

	if (!in_array($view, $noController))
	{
		Factory::getApplication()->input->set('task', $view . '.' . $task);
		$app->input->set('task', $view . '.' . $task);
	}
}



// Set the default view if none passed.
Factory::getApplication()->input->set('view', $view);
$app->input->set('view', $view);


// See if we should load the fullscreen version.
if (JoomcrmHelperUsers::isFullscreen())
{
	Factory::getApplication()->input->set('tmpl', 'component');
	$app->input->set('tmpl', 'component');
}

// force JoomCRM to load on a specified template theme
JoomcrmHelperTemplate::forceTemplateTheme();

$publicViews   = array('publiccalendar');
$overrides     = array('mail');
$taskOverrides = array(
	'ajax.saveWizardForm',
	'calendar.ical',
	'myteam.acceptInvite',
	'myteam.autoCreateTeam',
	'import.analize',
	'import.upload',
	'import.getJsonStatus'
);

// check if users allowed to manage/create it's own team
$allowTeamCreate = $config->get('allow_team_create', 0);

// can manage team
$canManageTeam = Factory::getApplication()->getIdentity()->authorise('core.manage.own.team', 'com_joomcrm');


if (($format == 'raw' && in_array($controller, $overrides)) || in_array($task, $taskOverrides))
{
	// Todo verify public page
}
/*elseif (Factory::getApplication()->getIdentity()->guest) // redirect if guest
{
	$app->enqueueMessage(Text::_('COM_JOOMCRM_PLEASE_LOGIN_FIRST'));
	$app->redirect(JoomcrmHelperConfig::getLoginRedirect());
}*/
else
{

	// refresh current user info
	$cuser = JoomcrmHelperUsers::getLoggedInUser();

	// If this is a public view, or if the user is logged in, continue. Otherwise redirect to joomla login.
	if (in_array($view, $publicViews) || ($cuser !== null && $cuser->uid > 0))
	{
		if ($format != 'raw')
		{
			JoomcrmHelperTemplate::startCompWrap();


			$doc            = Factory::getDocument();
			$currentLangTag = Factory::getLanguage()->getTag();

			JoomcrmHelperStyles::loadStyleSheets();

			// make sure to load jquery, bootstrap and chosen libs
			\Joomla\CMS\HTML\HTMLHelper::_('jquery.framework');
			//\Joomla\CMS\HTML\HTMLHelper::_('formbehavior.chosen', 'select');


			// make sure to laod fontawesome
			$wa  = Factory::getDocument()->getWebAssetManager();
			$wa->useStyle('fontawesome');

			// force desktop mode , disable responsive by removing viewport meta tag
			if($config->get('disable_responsive_mode',0)){

					$doc->addScriptDeclaration("jQuery('meta[name=\"viewport\"]').attr('content', 'width=1250')");
			}

			HTMLHelper::_('bootstrap.dropdown', '.dropdown-toggle');

			$doc
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/jquery-ui.js')
               // ->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/jquery-ui.css')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/iziModal.min.js')
				->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/iziModal.min.css')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/autogrow.js')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/jquery.cluetip.min.js')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/jquery.dataTables.min.js')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/readmore.min.js');


			$doc->addScript(Uri::base() . 'components/com_joomcrm/media/js/bootstrap-datepicker.js')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/underscore.min.js')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/clipboard.min.js')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/joomcrm.js')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/filters.js')
				->addScript(Uri::base() . 'components/com_joomcrm/media/js/people_manager.js');




            // Start component div wrapper
			if ($view !== "print")
			{
				JoomcrmHelperTemplate::loadToolbar();
			}

		}

		JoomcrmHelperActivity::saveUserLoginHistory();
	}
	// display create team message if user not a part of team
	if($allowTeamCreate && $canManageTeam  && Factory::getApplication()->getIdentity()->id > 0 && $cuser == null){

		$createLayout = LayoutHelper::render('myteam.createteam');
		$app->enqueueMessage($createLayout,'info');
	}
	// display warning if user not part of team or not logged
	if($cuser == null or Factory::getApplication()->getIdentity()->id == 0){
		$app->enqueueMessage(Text::_('COM_JOOMCRM_USER_WITH_NO_TEAM'),'warning');
		$app->redirect(JoomcrmHelperConfig::getLoginRedirect());
	}

}

JoomcrmHelperTemplate::loadJavascriptLanguage();

// Create the controller
$controller = BaseController::getInstance('Joomcrm');


// Perform the Request task
$controller->execute($app->input->get('task'));

// End component wrapper
if ($format !== 'raw' && $controller !== 'ajax')
{
	JoomcrmHelperTemplate::endCompWrap();
}

// Redirect if set by the controller


$controller->redirect();
