<?php
/**
 * JoomCRM
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die();

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;

extract($displayData);

// get current selected preset id by type
$selectedPreset = Factory::getSession()->get("selectedPreset");

if(isset($selectedPreset['type_id']) && $current->input->get('import_type') == $selectedPreset['type_id']){
	$currentPreset = $selectedPreset['id'];
}else{
	$currentPreset = '';
}

$layout = \Joomla\CMS\Factory::getApplication()->isClient('administrator') ? '&layout=import' : '';

switch ($current->input->get('import_type'))
{
	case 0:
		$val    = "companies";
		break;
	case 1:
		$val    = "people";
		break;
	case 2:
		$val    = "deals";
		break;

	default  : Factory::getApplication()->enqueueMessage(Text::_('COM_JOOMCRM_IMPORT_TYPE_NOT_FOUND'),'warning');
}


?>
<div class="nav nav-tabs mb-3"  id="nav-tab" role="tablist">
	<a class="nav-link nav-link" href="<?php echo Route::_('index.php?view=import&import_type='.$current->input->get('import_type').'&layout=import&option=com_joomcrm'); ?>"><?php echo Text::_('COM_JOOMCRM_IMPORTUPLOAD')?>
	</a>
	<a class="nav-link nav-link active "><?php echo Text::_('COM_JOOMCRM_IMPORTCONFIG')?></a>
	<a class="nav-link nav-link"><?php echo Text::_('COM_JOOMCRM_IMPORTFINISH')?></a>
</div>
<form action="<?php echo Uri::getInstance()->toString(); ?>" method="post" name="importForm" id="importForm">
	<div class="form-group my-3">
		<div class="row gx-2">
			<div class="col-sm-10">
				<?php echo HTMLHelper::_('select.genericlist', $current->presets, 'preset', 'class="form-control"','value','text',$currentPreset);?>

			</div>
			<div class="col-sm-2">
				<button class="btn btn-danger" id="deletePreset" title="<?php echo Text::_('COM_JOOMCRM_DELETE_PRESET')?>" data-bs-toggle="tooltip"><i class="fas fa-times"></i></button>

			</div>
		</div>
	</div>


	<div class="hide" id="preset-form">

	</div>

	<div class="rounded p-2 border d-flex justify-content-between bg-light w-100 mt-3">
        <a  class="btn btn-light border bg-light" href="index.php?option=com_joomcrm&view=import<?php echo $layout ?>&step=1&import_type=<?php echo $current->input->get('import_type') ?>">
            <i class="fas fa-arrow-left"></i> <?php echo Text::_('COM_JOOMCRM_BACK');?>
        </a>
		<button class="btn btn-primary" type="button" id="next-step"><?php echo Text::_('COM_JOOMCRM_NEXT')?> <i class="fas fa-arrow-right"></i> (<?php echo Text::_('COM_JOOMCRM_IMPORTFINISH') ?>)</button>
	</div>
	<input type="hidden" name="task" value="import.import">
	<input type="hidden" name="step" value="3">
	<input type="hidden" name="type" value="<?php echo $current->input->get('import_type'); ?>">
	<input type="hidden" name="key" value="<?php echo $current->input->get('key'); ?>">
</form>
<script>
    (function($) {
        var preset = $('#preset');

        if(preset.val() == '') {
            $('#deletePreset').css('display','none')
        }
        $('#deletePreset').click(function(e) {
            e.preventDefault();
            if(preset.val() != '') {
                $.get(Joomla.getOptions('system.paths').base+'/index.php?option=com_joomcrm&task=import.deletePreset&format=raw',
                    {'preset': preset.val()})
                    .done(function(data) {
                        preset.val('');
                        $('#preset-form').html(data).slideDown('fast', function() {
                            location.reload();
                        });
                    });
            }
        });

        $('#next-step').bind('click', function(event) {
            
            var submit = true;

            if(!preset.val()) {
                alert('<?php echo Text::_('COM_JOOMCRM_PLEASE_SELECT_IMPORT_SETTINGS') ?>');
                submit = false;
                return false;
            }
            if($('#importname').val() == 0) {
                alert('<?php echo Text::_('COM_JOOMCRM_ENTER_PRESET_NAME') ?>');
                submit = false;
                return false;
            }

            <?php if(\Joomla\CMS\Factory::getApplication()->isClient('administrator')): ?>
            if($('#importowner_id').val() == 0) {
                alert('<?php echo Text::_('COM_JOOMCRM_PLEASE_SELECT_OWNER') ?>');
                submit = false;
                return false;
            }
            <?php endif; ?>

            $.each($('div.required select'), function() {
                if($(this).val() == 0) {
                    alert('<?php echo Text::_('COM_JOOMCRM_REQUIRED_FIELDS_NOT_SET') ?>');
                    submit = false;
                    return false;
                }
            });

            if($('#importfieldid').val() == 0) {
                alert('Set ID');
                submit = false;
                return false;
            }

            $(this).attr('disabled', 'disabled');

            if(submit) {
                $('#importForm').submit();
            }
        });

        if(preset.val() != '') {
            $('#preset-form').html('').slideUp('fast', function(){
                $.get(Joomla.getOptions('system.paths').base+'/index.php?option=com_joomcrm&task=import.getParams&format=raw',
                    {'preset': preset.val(), 'type': $('input[name="type"]').val(), 'section_id': 0 })
                    .done(function(data) {
                        $('#preset-form').html(data).slideDown('fast', function() {
                            //$('#importfieldcategory').bind('change', categoryload);
                        });
                    });
            });
        }

        preset.bind('change', function() {
            if($(this).val() != '') {
                $('#preset-form').html('').slideUp('fast', function(){
                    $.get(Joomla.getOptions('system.paths').base+'/index.php?option=com_joomcrm&task=import.getParams&format=raw',
                        {'preset': preset.val(), 'type': $('input[name="type"]').val(), 'section_id': 0 })
                        .done(function(data) {
                            $('#preset-form').html(data).slideDown('fast', function() {
                                //$('#importfieldcategory').bind('change', categoryload);
                            });
                        });
                });
            } else {
                $('#preset-form').html('').slideUp('fast');
            }
        });

    }(jQuery))
</script>