<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

namespace Joomcrm\Mailing\Integrations;

use AcyMailing\Controllers\UsersController;
use Joomcrm\Mailing\Mailing;
use Joomcrm\Patterns\Singleton;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;

defined('_JEXEC') or die();

/*
 * This integration support Acymailing v7+
 */
class Acymailing extends Mailing
{
	use Singleton;


	public function __construct()
	{
		// load construct parent first , if return false means disabled
		$result = parent::__construct();

		// means disabled
		if(!$result){
			self::$invalid = true;
			return false;
		}

		// check if component enabled if not no need to continue;
		if(!ComponentHelper::isEnabled('com_acym')){
			self::$invalid = true;
			return false;
		}

		// get subscriber id
		$this->getSubscriberId();

		// get mailing lists
		$this->getMailingLists();

		// get newsletters
		$this->getNewsletters();

		return true;


	}

	public function getSubscriberId(){

		$db = Factory::getDbo();

		// acymailing v6+ support
		$query = $db->getQuery(true)->select("id as subid")->from('#__acym_user')
			->where("email='" . $this->subscriberEmail . "'");

		$this->subscriberId = $db->setQuery($query)->loadResult();

	}


	public function getMailingLists($all = false){

		$db            = Factory::getDbo();

		$query         = $db->getQuery(true);

		$query->select("DISTINCT id as listid,list.name,list.description,list.color,list.access")
			->from("#__acym_list AS list");

		if ($this->subscriberId)
		{
			$query->select("subscribed.list_id AS isSubscribed")
				->leftJoin("#__acym_user_has_list AS subscribed ON subscribed.list_id = list.id AND subscribed.status=1 AND subscribed.user_id = " . $this->subscriberId);
		}

		if (!$all && $this->subscriberId)
		{
			$query->where('subscribed.list_id != "NULL"');
		}

		$query->where("list.active=1")->where("list.visible=1")->order("creation_date ASC");

		$this->lists = $db->setQuery($query)->loadObjectList();


		if (count($this->lists) > 0)
		{
			foreach ($this->lists as $klist => $list)
			{

				if (!isset($list->isSubscribed))
				{
					$list->isSubscribed = 0;
				}

				// check if user can access to list, if not remove list from mailing lists
				if (!$this->canAccessToList($list->access))
				{
					unset($this->lists[$klist]);
				}

			}
		}

		return $this->lists;

	}


	public function canAccessToList($access)
	{

		// check if everyone can access to list
		if (empty($access) or $access == 'all')
			return true; // user can access list

		// convert to array
		$listAccessGroups = explode(',', $access);

		// check each access if exist in user authourized groups
		foreach ($listAccessGroups as $access)
		{
			if (in_array($access, Factory::getApplication()->getIdentity()->getAuthorisedGroups()))
				return true; // user can access list
		}

		return false; // user can't access

	}

	/**
	 * Get Acymailing List Newsletters
	 * @return  array
	 */
	public function getNewsletters($listId = null)
	{
		// inits
		$db = Factory::getDbo();


		// get list id
		if (is_null($listId))
		{
			$this->listId = (isset($this->listId) && $this->listId > 0) ? $this->listId : Factory::getApplication()->input->getInt('listid', null);
		}

		// if list id still null , get the first list id
		if(is_null($this->listId)){
            $this->listId = isset($this->lists[0]) ? $this->lists[0]->listid : '';
		}


		// subscriber id not found
		if (is_null($this->subscriberId))
			return [];

        // check if subscribe id and list id exists
        if (!$this->subscriberId || !$this->listId || empty($this->listId))
            return [];

		$query = $db->getQuery(true)
			->select("mail.id as mailid, mail.subject, mailstat.send_date as senddate, user.open, user.open_date as opendate")
			->from("#__acym_mail AS mail")
			->leftJoin("#__acym_mail_has_list AS listmail ON listmail.mail_id = mail.id")
			->leftJoin("#__acym_user_stat AS user ON mail.id = user.mail_id")
			->leftJoin("#__acym_mail_stat AS mailstat ON mailstat.mail_id = mail.id")
			->where("listmail.list_id = " . $this->listId)->where("user.user_id=" . $this->subscriberId);

		//->where("mail.library = 0"); this column was removed from v7.6+

		return $db->setQuery($query)->loadObjectList();

	}

	/**
	 * Add JoomCRM People to Acymailing Lists
	 */
	public function addMailingList($data)
	{

		$item = parent::getItem($data);

		$listid = $data['listid'];


		if (!isset($item['email']) && isset($item[0]['email']))
		{
			$item = $item[0];
		}

		if (!isset($item['first_name']) && isset($item['owner_first_name']))
		{
			$item['first_name'] = $item['owner_first_name'];
		}

		if (!isset($item['last_name']) && isset($item['owner_last_name']))
		{
			$item['last_name'] = $item['owner_last_name'];
		}


		// load controller for user subscription
		\JLoader::register('UsersController', JPATH_ADMINISTRATOR . '\components\com_acym\controllers\users.php');

		// get new instance
		$frontUsers = new \AcyMailing\Controllers\UsersController();

		$subid = $this->getSubscriberIdFromEmail($item['email']);


		if (!$subid) //  if user doesn't exist add user to subscribers list
		{

			$subscriber = new \stdClass();
			$subscriber->name = $item['first_name'].' '.$item['last_name'];
			$subscriber->email = $item['email'];
			$subscriber->active = 1;
			$subscriber->confirmed = 1;
			$subscriber->tracking = 1;
			$subscriber->active = 1;
			$subscriber->creation_date = Factory::getDate()->toSql();
			$subscriber->source = 'JoomCRM on '.$subscriber->creation_date;

			Factory::getDbo()->insertObject('#__acym_user',$subscriber,'id');

		}

		$subid = $this->getSubscriberIdFromEmail($item['email']);


		// subscribe user to list
		$frontUsers->currentClass->subscribe($subid, $listid);

		return true;



	}

	public function getSubscriberIdFromEmail($email){

		$db         = Factory::getDbo();
		$query      = $db->getQuery(true);

		// get subscriber id from email
		$query->select("id")->from("#__acym_user as user")->where("email='" . $email . "'");

		return $db->setQuery($query)->loadResult();

	}

	/**
	 * Remove JoomCRM People from Acymailing Lists
	 */
	public function removeMailingList($data)
	{
		$id           = $data['id'];
		$listid       = $data['listid'];
		$subscriberId = $this->subscriberId;

		if ($subscriberId)
		{
			$db    = Factory::getDbo();
			$query = $db->getQuery(true);

			$time = time();

			// load controller for user subscription
			\JLoader::register('UsersController', JPATH_ADMINISTRATOR . '\components\com_acym\controllers\users.php');
			// get new instance
			$frontUsers = new UsersController();
			// subscribe user
			$frontUsers->currentClass->unsubscribe($subscriberId, $listid);

		}

		return true;
	}

	public function getLinks()
	{
		$input = Factory::getApplication()->input;
		$db         = Factory::getDbo();

		$mailid     = $input->get('mailid',0,'int');

		$query = $db->getQuery(true)
			->select("click.click,url.name,url.url")
			->from("#__acym_url_click AS click")
			->leftJoin("#__acym_url AS url ON url.id = click.url_id")
			->where("click.user_id = " . $this->subscriberId)
			->where("click.mail_id = " . $mailid);


		return $db->setQuery($query)->loadObjectList();
	}




}