<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

namespace Joomcrm\Mailing;

use Joomcrm\Mailing\Integrations;

use Joomcrm\Patterns\Singleton;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;

defined('_JEXEC') or die();

// register all models
BaseDatabaseModel::addIncludePath(JPATH_SITE . '/components/com_joomcrm/model', 'JoomcrmModel');
// register model override
\JLoader::register('JoomcrmModel', JPATH_SITE . '/components/com_joomcrm/model/model.php');


class Mailing
{

	public $listId = null;
	public $peopleIds = null;
	public $subscriber = null;
	public $subscriberEmail = false;
	public $subscriberId = 0;
	public $lists = [];
	public $newsletters = [];
	public static $invalid = false;

	public function __construct()
	{

		// get integration name
		$integration = ComponentHelper::getParams('com_joomcrm')->get('mailinglists_integration', 0);

		// if disabled no need to continue;
		if (!$integration)
		{
			return false;
		}


		$input = Factory::getApplication()->input;

		$this->listId    = $input->get('list_id', 0, 'int');
		$this->peopleIds = $input->get('people_ids', [], 'array');

		// get subscriber email
		$this->getSubscriberEmail();

		return true;


	}

	public function getItem($data)
	{
		$input = Factory::getApplication()->input;

		$id  = $data['id'];
		$loc = $input->getWord('loc', $input->getWord('view'));

		switch ($loc)
		{
			case "company":
				$modelName = "company";
				$funcName  = "Companies";
				break;
			case "person":
				$modelName = "people";
				$funcName  = "Person";
				break;
			default:
				$model    = $loc;
				$funcName = ucwords($loc);
				break;
		}


		$model = BaseDatabaseModel::getInstance($modelName, 'JoomcrmModel');


		$func = "get" . $funcName;

		return $model->$func($id);
	}

	public static function isEnabled($type)
	{

		$config = ComponentHelper::getParams('com_joomcrm');

		if(self::$invalid)
			return false;

		if($config->get('mailinglists_integration', 0) && $config->get('acy_'.$type, 0))
			return true;

		return false;


	}

	/*
	 * Get mailing instance by integration
	 */
	public static function loadIntegration()
	{

		$config = ComponentHelper::getParams('com_joomcrm');

		// get integration name, don't continue if disabled
		if (!$integration = $config->get('mailinglists_integration', 0))
			return false;

		$class = "\Joomcrm\\Mailing\\Integrations\\" . ucfirst($integration);

		$mailing = $class::getInstance();

		if(self::$invalid)
			return false;

		// get integration instance
		return $mailing;

	}

	public function getSubscriberEmail()
	{

		$input = Factory::getApplication()->input;


		$id = $input->getInt('id', 0);

		$view = $input->getWord('loc', $input->getWord('view'));

		switch ($view)
		{
			case "companies":
				$modelName = "company";
				break;
			case "person":
				$modelName = "people";
				break;
			default:
				$modelName = $view;
				break;
		}

		$model = BaseDatabaseModel::getInstance($modelName, 'JoomcrmModel', array('ignore_request' => true));

		$this->subscriberEmail = $model->getEmail($id);


	}


}