/**
 * Globals
 */
let chart = null;
let charts = ['dealStagePie',
    'dealStageBar',
    'dealStatusPie',
    'dealStatusBar',
    'yearlyCommissions',
    'monthlyCommissions',
    'yearlyRevenue',
    'monthlyRevenue'];

let currentChart = null;
let colors = Highcharts.getOptions().colors;

jQuery(document).ready(function () {
    var chart = ( typeof homePageChart !== 'undefined' ) ? homePageChart : "dealStagePie";
    showAllCharts();
    showChart(chart);
});

function showAllCharts(data) {

    if (data != null)
        graphData = data;

    if (typeof graphData !== 'undefined' && graphData != null) {

        // Loop through deal stages to assign certain values
        if (graphData.deal_stage != null) {
            for (i = 0; i < graphData.deal_stage.length; i++) {
                graphData.deal_stage[i].color = colors[i];
            }
        }
        // Loop through deal statuss to assign color values
        if (graphData.deal_status != null) {
            for (i = 0; i < graphData.deal_status.length; i++) {
                graphData.deal_status[i].color = colors[i];
            }
        }
        // Loop through commission months to assign colors
        if (graphData.yearly_commissions != null) {
            for (i = 0; i < graphData.yearly_commissions.length; i++) {
                graphData.yearly_commissions[i].color = colors[i];
            }
        }
        // Loop through commission weeks to assign colors
        if (graphData.monthly_commissions != null) {
            for (i = 0; i < graphData.monthly_commissions.length; i++) {
                graphData.monthly_commissions[i].color = colors[i];
            }
        }
        // Loop through revenue weeks to assign colors
        if (graphData.monthly_revenue != null) {
            for (i = 0; i < graphData.monthly_revenue.length; i++) {
                graphData.monthly_revenue[i].color = colors[i];
            }
        }
        // Loop through revenue months to assign colors
        if (graphData.yearly_revenue != null) {
            for (i = 0; i < graphData.yearly_revenue.length; i++) {
                graphData.yearly_revenue[i].color = colors[i];
            }
        }

        // Show initial pie graph
        currentChart = currentChart != null ? currentChart : 0;
        showChart(charts[currentChart]);

        // Loop through charts
        jQuery("#chart_select_next").bind('click', function () {
            chartLoop('next');
        });
        jQuery("#chart_select_prev").bind('click', function () {
            chartLoop('prev');
        });

    }

}

// Shows different kinds of charts
function showChart(type) {

    switch (type) {

        case "dealStagePie" :
            chart = new Highcharts.Chart({
                chart: {
                    renderTo: 'sales_graphs',
                    plotBackgroundColor: null,
                    plotBorderWidth: null,
                    plotShadow: true
                },
                title: {
                    text: Joomla.JText._('COM_JOOMCRM_DEALS_BY_STAGE')
                },
                tooltip: {
                    formatter: function () {
                        return '<b>' + this.point.name + '</b>: ' + this.percentage.toFixed(2) + ' %<br />( ' + this.y + ' ' + Joomla.JText._('COM_JOOMCRM_OUT_OF') + ' ' + this.total + ' ) <br />(<b> ' + formatMoney(this.point.amount) + "</b> " + Joomla.JText._('COM_JOOMCRM_OUT_OF') + " <b>" + formatMoney(this.point.deal_total) + "</b> )";
                    }
                },
                plotOptions: {
                    pie: {
                        allowPointSelect: true,
                        cursor: 'pointer',
                        dataLabels: {
                            enabled: true,
                            color: '#000000',
                            connectorColor: '#000000',
                            formatter: function () {
                                return '<b>' + this.point.name + '</b>: ' + this.percentage.toFixed(2) + ' %<br />( ' + this.y + ' ' + Joomla.JText._('COM_JOOMCRM_OUT_OF') + ' ' + this.total + ' ) ';
                            }
                        }
                    }
                },
                series: [
                    {
                        type: 'pie',
                        name: Joomla.JText._('COM_JOOMCRM_DEALS_BY_STAGE'),
                        data: graphData.deal_stage
                    }
                ]
            });
            break;

        case 'dealStageBar' :
            chart = new Highcharts.Chart({
                chart: {
                    renderTo: 'sales_graphs',
                    defaultSeriesType: 'column'
                },
                title: {
                    text: Joomla.JText._('COM_JOOMCRM_DEALS_BY_STAGE')
                },
                xAxis: {
                    categories: graphData.stage_names
                },
                yAxis: {
                    min: 0,
                    title: {
                        text: Joomla.JText._('COM_JOOMCRM_TOTAL')
                    }
                },
                legend: {
                    enabled: false
                },
                tooltip: {
                    formatter: function () {
                        return '<b>' + this.point.name + '</b>: <br />(<b> ' + formatMoney(this.point.amount) + "</b> " + Joomla.JText._('COM_JOOMCRM_OUT_OF') + " <b>" + formatMoney(this.point.deal_total) + "</b> )";
                    }
                },
                plotOptions: {
                    column: {
                        pointPadding: 0.2,
                        borderWidth: 0
                    }
                },
                series: [
                    {
                        name: Joomla.JText._('COM_JOOMCRM_DEALS_BY_STAGE'),
                        data: graphData.deal_stage
                    }
                ]
            });
            break;

        case 'dealStatusPie' :
            chart = new Highcharts.Chart({
                chart: {
                    renderTo: 'sales_graphs',
                    plotBackgroundColor: null,
                    plotBorderWidth: null,
                    plotShadow: true
                },
                title: {
                    text: Joomla.JText._('COM_JOOMCRM_DEALS_BY_STATUS')
                },
                tooltip: {
                    formatter: function () {
                        return '<b>' + this.point.name + '</b>: ' + this.percentage.toFixed(2) + ' %<br />( ' + this.y + ' ' + Joomla.JText._('COM_JOOMCRM_OUT_OF') + ' ' + this.total + ' ) <br />(<b> ' + formatMoney(this.point.amount) + "</b> " + Joomla.JText._('COM_JOOMCRM_OUT_OF') + " <b>" + formatMoney(this.point.deal_total) + "</b> )";
                    }
                },
                plotOptions: {
                    pie: {
                        allowPointSelect: true,
                        cursor: 'pointer',
                        dataLabels: {
                            enabled: true,
                            color: '#000000',
                            connectorColor: '#000000',
                            formatter: function () {
                                return '<b>' + this.point.name + '</b>: ' + this.percentage.toFixed(2) + ' %<br />( ' + this.y + ' ' + Joomla.JText._('COM_JOOMCRM_OUT_OF') + ' ' + this.total + ' ) ';
                            }
                        }
                    }
                },
                series: [
                    {
                        type: 'pie',
                        name: Joomla.JText._('COM_JOOMCRM_DEALS_BY_STATUS'),
                        data: graphData.deal_status
                    }
                ]
            });
            break;

        case 'dealStatusBar' :
            chart = new Highcharts.Chart({
                chart: {
                    renderTo: 'sales_graphs',
                    defaultSeriesType: 'column'
                },
                title: {
                    text: Joomla.JText._('COM_JOOMCRM_DEALS_BY_STATUS')
                },
                xAxis: {
                    categories: graphData.status_names
                },
                yAxis: {
                    min: 0,
                    title: {
                        text: Joomla.JText._('COM_JOOMCRM_TOTAL')
                    }
                },
                legend: {
                    enabled: false
                },
                tooltip: {
                    formatter: function () {
                        return '<b>' + this.point.name + '</b>: <br />(<b> ' + formatMoney(this.point.amount) + "</b> " + Joomla.JText._('COM_JOOMCRM_OUT_OF') + " <b>" + formatMoney(this.point.deal_total) + "</b> )";
                    }
                },
                plotOptions: {
                    column: {
                        pointPadding: 0.2,
                        borderWidth: 0
                    }
                },
                series: [
                    {
                        name: Joomla.JText._('COM_JOOMCRM_DEALS_BY_STATUS'),
                        data: graphData.deal_status
                    }
                ]
            });
            break;

        case 'leadRevenue' :
            chart = new Highcharts.Chart({
                chart: {
                    renderTo: 'sales_graphs',
                    plotBackgroundColor: null,
                    plotBorderWidth: null,
                    plotShadow: true
                },
                title: {
                    text: Joomla.JText._('COM_JOOMCRM_REVENUE_FROM_LEAD_SOURCES')
                },
                tooltip: {
                    formatter: function () {
                        return '<b>' + this.point.name + '</b>: ' + comJoomcrmCurrency + this.point.data;
                    }
                },
                plotOptions: {
                    pie: {
                        allowPointSelect: true,
                        cursor: 'pointer',
                        dataLabels: {
                            enabled: true,
                            color: '#000000',
                            connectorColor: '#000000',
                            formatter: function () {
                                return '<b>' + this.point.name + '</b>: ' + comJoomcrmCurrency + this.point.data;
                            }
                        }
                    }
                },
                series: [
                    {
                        type: 'pie',
                        name: Joomla.JText._('COM_JOOMCRM_REVENUE_FROM_LEAD_SOURCES'),
                        data: graphData.lead_sources
                    }
                ]
            });

            break;

        // TODO
        case 'leadCloseTime' :


            break;

        case 'yearlyCommissions' :
            chart = new Highcharts.Chart({
                chart: {
                    renderTo: 'sales_graphs',
                    defaultSeriesType: 'column'
                },
                title: {
                    text: Joomla.JText._('COM_JOOMCRM_YEARLY_COMMISSIONS')
                },
                xAxis: {
                    categories: graphData.months
                },
                yAxis: {
                    min: 0,
                    title: {
                        text: Joomla.JText._('COM_JOOMCRM_TOTAL')
                    },
                    labels: {formatter: function () {
                        return comJoomcrmCurrency + this.value;
                    }}
                },
                legend: {
                    enabled: false
                },
                tooltip: {
                    formatter: function () {
                        return '' +
                            this.x + ': ' + comJoomcrmCurrency + this.y;
                    }
                },
                plotOptions: {
                    column: {
                        pointPadding: 0.2,
                        borderWidth: 0
                    }
                },
                series: [
                    {
                        name: Joomla.JText._('COM_JOOMCRM_YEARLY_COMMISSIONS'),
                        data: graphData.yearly_commissions
                    }
                ]
            });
            break;

        case 'monthlyCommissions' :
            chart = new Highcharts.Chart({
                chart: {
                    renderTo: 'sales_graphs',
                    defaultSeriesType: 'column'
                },
                title: {
                    text: Joomla.JText._('COM_JOOMCRM_MONTHLY_COMMISSIONS')
                },
                xAxis: {
                    categories: graphData.weeks
                },
                yAxis: {
                    min: 0,
                    title: {
                        text: Joomla.JText._("COM_JOOMCRM_TOTAL")
                    },
                    labels: {formatter: function () {
                        return comJoomcrmCurrency + this.value;
                    }}
                },
                legend: {
                    enabled: false
                },
                tooltip: {
                    formatter: function () {
                        return '' +
                            this.x + ': ' + comJoomcrmCurrency + this.y;
                    }
                },
                plotOptions: {
                    column: {
                        pointPadding: 0.2,
                        borderWidth: 0
                    }
                },
                series: [
                    {
                        name: Joomla.JText._('COM_JOOMCRM_MONTHLY_COMMISSIONS'),
                        data: graphData.monthly_commissions
                    }
                ]
            });
            break;

        case 'yearlyRevenue' :
            chart = new Highcharts.Chart({
                chart: {
                    renderTo: 'sales_graphs',
                    defaultSeriesType: 'column'
                },
                title: {
                    text: Joomla.JText._('COM_JOOMCRM_YEARLY_REVENUE')
                },
                xAxis: {
                    categories: graphData.months
                },
                yAxis: {
                    min: 0,
                    title: {
                        text: Joomla.JText._('COM_JOOMCRM_TOTAL')
                    },
                    labels: {formatter: function () {
                        return comJoomcrmCurrency + this.value;
                    }}
                },
                legend: {
                    enabled: false
                },
                tooltip: {
                    formatter: function () {
                        return '' +
                            this.x + ': ' + comJoomcrmCurrency + this.y;
                    }
                },
                plotOptions: {
                    column: {
                        pointPadding: 0.2,
                        borderWidth: 0
                    }
                },
                series: [
                    {
                        name: Joomla.JText._('COM_JOOMCRM_YEARLY_REVENUE'),
                        data: graphData.yearly_revenue
                    }
                ]
            });
            break;

        case 'monthlyRevenue' :
            chart = new Highcharts.Chart({
                chart: {
                    renderTo: 'sales_graphs',
                    defaultSeriesType: 'column'
                },
                title: {
                    text: Joomla.JText._('COM_JOOMCRM_MONTHLY_REVENUE')
                },
                xAxis: {
                    categories: graphData.weeks
                },
                yAxis: {
                    min: 0,
                    title: {
                        text: Joomla.JText._('COM_JOOMCRM_TOTAL')
                    },
                    labels: {formatter: function () {
                        return comJoomcrmCurrency + this.value;
                    }}
                },
                legend: {
                    enabled: false
                },
                tooltip: {
                    formatter: function () {
                        return '' +
                            this.x + ': ' + comJoomcrmCurrency + this.y;
                    }
                },
                plotOptions: {
                    column: {
                        pointPadding: 0.2,
                        borderWidth: 0
                    }
                },
                series: [
                    {
                        name: Joomla.JText._('COM_JOOMCRM_MONTHLY_REVENUE'),
                        data: graphData.monthly_revenue
                    }
                ]
            });
            break;

    }
}

// Loops through charts on dashboard
function chartLoop(position) {
    // Get next position
    if (position == 'next') {
        currentChart = ( currentChart == charts.length - 1 ) ? 0 : currentChart + 1;
    }
    if (position == 'prev') {
        currentChart = ( currentChart == 0 ) ? charts.length : currentChart - 1;
    }
    // Show the next chart
    showChart(charts[currentChart]);
}