/**
 * Globals
 */
var current_area = null;
var order_url = null;
var order_col = null;
var order_dir = null;
var header_refs = new Array();
var row_refs = new Array();
var joomcrmStylesheet = null;
var graphFilterType = null;
var graphFilterId = null;

// Document ready
jQuery(document).ready(function ($) {

    // Find unique joomcrm stylesheet
    findStylesheet();

    // Bind joomcrm dropdowns
    bindDropdownItems();

    // Bind column filters for page
    $('#column_filter').find('input[type=checkbox]').each(function (index, el) {
        $(el).bind('change', function (event) {
            event.stopPropagation();
            column(el.id);
        });
    });

    // Assign the correct sort direction
    assignFilterOrder();

    // Assign a company name search filter input box
    bindFilters();

    // Determine which columns should be shown on the page
    if (typeof(loc) !== 'undefined') {
        if (loc == 'deals' || loc == 'companies' || loc == 'people') {
            showColumns();
        }
    }

    // keyword filter
    $(".keyword_filter").keydown(
        _.debounce(
            function(){
                let base = $(this);

                // check if placeholder value in value attr and remove
                if(base.attr('placeholder') == base.attr('value'))
                    base.attr('value','');

                filterTable(base.val(), base.attr('name'), base);
            },
            500
        )
    );

});

function bindFilters() {
    var $ = jQuery,
        filterInput = $('.filter_input');

    filterInput.unbind();

    filterInput.bind('change', function () {
        var base = $(this);
        filterTable(base.val(), base.attr('name'), base);
    });

}


function findStylesheet() {

    var stylesheets = document.styleSheets;
    var stylesheets = jQuery("head").children('link');

    jQuery.each(stylesheets, function (index, stylesheet) {
        if (jQuery(stylesheet).data('id') == "joomcrmStylesheet") {
            joomcrmStylesheet = index;
        }
    });

}

function bindDropdownItems() {
    var $ = jQuery,
        items = $('.dropdown_item');

    if (items.length) {
        items.each(function(index, el) {
            var base = $(el);

            base.click(function () {
                var wrapper = base.closest('div.dropdown')
                var toggle = wrapper.find('.dropdown-toggle');
                toggle.html(base.html());

                if (typeof base.data('value') !== 'undefined') {
                    ajaxSaveModal(base);
                }
            });
        });
    }
}


function ajaxSaveModal(ele) {

    var $ = jQuery,
        base = $(ele),
        parentDiv = base.closest('div.dropdown');

    //



    var item_id = parentDiv.data('item-id');
    var item_type = parentDiv.data('item');
    var value_type = parentDiv.data('field');
    var new_value = base.data('value');

    console.log(new_value)

    dataString = "item_id=" + item_id + "&item_type=" + item_type + "&field=" + value_type + "&value=" + new_value;

    $.ajax({
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&task=ajax.saveAjax&format=raw&tmpl=component',
        type: 'POST',
        data: dataString,
        dataType: 'JSON',
        success: function (data) {
            modalMessage(Joomla.JText._('COM_JOOMCRM_SUCCESS_MESSAGE', 'Success'), Joomla.JText._('COM_JOOMCRM_GENERIC_UPDATED', 'Successfully updated'));
            if (item_type == "deal" && value_type == "stage_id") {
                if (data.closed == true) {
                    actual_close = data.actual_close;
                    actual_close_formatted = data.actual_close_formatted;
                    $("#actual_close").val(actual_close_formatted);
                    $("#actual_close_hidden").val(actual_close);
                    $("#expected_close_container").hide();
                    $("#actual_close_container").show();
                } else {
                    expeced_close = data.expeced_close;
                    expeced_close_formatted = data.expected_close_formatted;
                    $("#expeced_close").val(expeced_close_formatted);
                    $("#expected_close_hidden").val(expeced_close);
                    $("#actual_close_container").hide();
                    $("#expected_close_container").show();
                }
            }
            if (loc == "deals") {
                expected_close = data.expected_close_formatted;
                actual_close = data.actual_close_formatted;
                if (data.closed == true) {
                    $("#expected_close_" + data.id).html(expected_close);
                    $("#actual_close" + data.id).html(actual_close);
                } else {
                    $("#actual_close" + data.id).html(Joomla.JText._('COM_JOOMCRM_ACTIVE_DEAL'));
                }
            }
        }
    });
}


function hideDropdown(current_area) {
    if (current_area != null) {

        if (jQuery("#" + current_area).hasClass('editable')) {
            jQuery("#" + current_area).children('div.editable_info').hide();
            current_area = null;
        } else {

            // Unbind area
            jQuery("#" + current_area).unbind();
            // Hide area
            var div = current_area.replace('_link', '');

            jQuery("#" + div).hide();

            // Rebind area
            var bind_area = current_area;
            jQuery("#" + bind_area).bind('click', function () {
                showDropdown(bind_area);
            });

        }

    }
}

// Function to bind filter dropdowns
function showDropdown(area) {

    var link = area;
    var div = area.replace('_link', '');

    // if there is already a filter box open we close it and reassign our bind functionalities
    if (current_area != null) {

        if (jQuery("#" + current_area).hasClass('editable')) {

            jQuery("#" + current_area).children('div.editable_info').hide();
            current_area = null;

        } else {

            // Unbind area
            jQuery("#" + current_area).unbind();
            // Hide area
            jQuery("#" + current_area.replace('_link', '')).hide();

            // Rebind area
            var bind_area = current_area;
            jQuery("#" + bind_area).bind('click', function () {
                showDropdown(bind_area);
            });

        }

    }

    // Assign new area
    if (current_area != div) {

        // Fade in area
        jQuery("#" + div).show();

        // Unbind any prior functionalities
        jQuery("#" + link).unbind();

        // Set css
        jQuery("#" + div).css('left', jQuery("#" + link).position().left + "px");
        jQuery("#" + div).css('top', jQuery("#" + link).position().top + parseInt(jQuery("#" + link).css('height')) + "px");
        // jQuery("#"+div).css('top','100px');`
        // TODO bind area to fade out when losing focus or other clicks
        jQuery("#" + div).focus();
        jQuery("#" + div).focusout(function () {

        });

        // Bind area to hide when clicked
        jQuery("#" + link).bind('click', function () {
            // Hide area
            jQuery("#" + div).hide();
            // Rebind area
            jQuery("#" + link).bind('click', function () {
                showDropdown(link);
            });
        });

        // Update new current area
        current_area = link;

    } else {
        // Hide/show area
        if (jQuery("#" + div).is(':visible')) {
            // Hide
            jQuery("#" + div).hide();
        } else {
            // Fade in area
            jQuery("#" + div).show();
        }

    }

}

// Function to show initial page load columns
function initColumn(area) {
    var $ = jQuery,
        base = $('#' + area),
        show_area = base[0].id.replace('show_', '');

    if (base.is(':checked')) {
        // Insert the new rule
        addRule('div#com_joomcrm th.' + show_area + '{ display:table-cell; }');
        // Insert reference to the new rule to our global references object
        if (document.styleSheets[joomcrmStylesheet].cssRules) {
            header_refs[show_area] = document.styleSheets[joomcrmStylesheet].cssRules[0];
        } else {
            header_refs[show_area] = document.styleSheets[joomcrmStylesheet].rules[0];
        }

        // Insert the new rule
        addRule('div#com_joomcrm td.' + show_area + '{ display:table-cell; }');
        // Insert reference to the new rule to our global references object
        if (document.styleSheets[joomcrmStylesheet].cssRules) {
            row_refs[show_area] = document.styleSheets[joomcrmStylesheet].cssRules[0];
        } else {
            row_refs[show_area] = document.styleSheets[joomcrmStylesheet].rules[0];
        }

    } else {
        addRule('div#com_joomcrm th.' + show_area + '{ display:none; }');
        // insert reference to the new rule to our global references object
        if (document.styleSheets[joomcrmStylesheet].cssRules) {
            header_refs[show_area] = document.styleSheets[joomcrmStylesheet].cssRules[0];
        } else {
            header_refs[show_area] = document.styleSheets[joomcrmStylesheet].rules[0];
        }

        // Insert the new rule
        addRule('div#com_joomcrm td.' + show_area + '{ display:none; }');
        // Insert reference to the new rule to our global references object
        if (document.styleSheets[joomcrmStylesheet].cssRules) {
            row_refs[show_area] = document.styleSheets[joomcrmStylesheet].cssRules[0];
        } else {
            row_refs[show_area] = document.styleSheets[joomcrmStylesheet].rules[0];
        }
    }
}

function addRule(rule) {
    if (document.styleSheets && document.styleSheets.length) {
        var a = document.styleSheets[joomcrmStylesheet], arr = rule.replace(/}.*/, '').split('{'), arr2, i;
        if (a.insertRule)a.insertRule(rule, 0);
        else if (a.addRule) {
            arr2 = arr[0].split(/,/);
            for (i = 0; i < arr2.length; i++)a.addRule(arr2[i], arr[1]);
        }
    }
}

// Function to bind clicks for displaying certain columns
function column(area) {

    var $ = jQuery,
        base = $('#' + area),
        show_area = base[0].id.replace('show_', '');

    //

    if (base.is(':checked')) {
        // Insert the new rules
        header_refs[show_area].style['display'] = 'table-cell';
        row_refs[show_area].style['display'] = 'table-cell';
        addRule('div#com_joomcrm th.' + show_area + '{ display:table-cell; }');
        addRule('div#com_joomcrm td.' + show_area + '{ display:table-cell; }');
    } else {
        // Insert the new rules
        header_refs[show_area].style['display'] = 'none';
        row_refs[show_area].style['display'] = 'none';
        addRule('div#com_joomcrm th.' + show_area + '{ display:none; }');
        addRule('div#com_joomcrm td.' + show_area + '{ display:none; }');
    }

    // Make ajax call to update our database information and session information
    var dataString = "loc=" + loc + "&column=" + show_area;
    $.ajax({
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&task=ajax.updateColumns&format=raw&tmpl=component',
        type: 'post',
        data: dataString,
        success: function (data) {

        }
    });
}

// Show the respected columns relating to user session and database information
function showColumns() {
    var $ = jQuery,
        colFilter = $('#column_filter');

    // Loop through the column filter inputs to determine which ones are checked or not to display the areas
    colFilter.find('input[type="checkbox"]').each(function (index, el) {
        initColumn(el.id);
    });
}

// Filter through deal stages
function dealStage(stage) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=deals&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'stage=' + stage +'&ajaxLimit='+limit+'&layout=list',
        dataType: 'html',
        success: function (data) {
            var dealsDiv = $('#deals');

            // Assign new html
            $.when(dealsDiv.empty()).then(function () {
                dealsDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });

}

// Filter through deal types
function dealType(type) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=deals&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'type=' + type +'&ajaxLimit='+limit+'&layout=list',
        dataType: 'html',
        success: function (data) {
            var dealsDiv = $('#deals');

            // Assign new html
            $.when(dealsDiv.empty()).then(function () {
                dealsDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });

}

// Filter through deal users
function dealUser(user, team) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    var search_id = ( user ) ? user : team;
    var dataString = ( user ) ? 'user=' + search_id : 'team_id=' + search_id;

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=deals&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: dataString+'&ajaxLimit='+limit+'&layout=list',
        dataType: 'html',
        success: function (data) {

            var dealsDiv = $('#deals');

            // Assign new html
            $.when(dealsDiv.empty()).then(function () {
                dealsDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });

}

// Filter through deal closing times
function dealClose(time) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=deals&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'close=' + time+'&ajaxLimit='+limit+'&layout=list',
        dataType: 'html',
        success: function (data) {
            var dealsDiv = $('#deals');

            // Assign new html
            $.when(dealsDiv.empty()).then(function () {
                dealsDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Filter through company types
function companyType(type) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }


    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=companies&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'type=' + type +'&ajaxLimit='+limit+'&layout=list',
        dataType: 'html',
        success: function (data) {
            var companiesDiv = $('#companies');

            // Assign new html
            $.when(companiesDiv.empty()).then(function () {
                companiesDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Filter through company categories
function companyCategory(categoryId) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }


    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=companies&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'category_id=' + categoryId+'&layout=list',
        dataType: 'html',
        success: function (data) {
            var companiesDiv = $('#companies');

            // Assign new html
            $.when(companiesDiv.empty()).then(function () {
                companiesDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}



// Filter through company by user permissions
function companyUser(user, team) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }


    var search_id = ( user ) ? user : team;
    var dataString = ( user ) ? 'user=' + user : 'team_id=' + team;

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=companies&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: dataString+'&ajaxLimit='+limit+'&layout=list',
        dataType: 'html',
        success: function (data) {
            var companiesDiv = $('#companies');

            // Assign new html
            $.when(companiesDiv.empty()).then(function () {
                companiesDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Update event listings
function updateEventList(user, team, clickedElem) {

    var $ = jQuery;



    ajaxLoadingToggle()

    // Make ajax call for new event listings
    var search_event_id = ( user ) ? user : team;
    var dataString = "";

    if (user) {
        dataString += 'assignee_id=' + search_event_id + "&assignee_filter_type=individual";
    } else {
        dataString += 'assignee_id=' + search_event_id + "&assignee_filter_type=team";
    }

    if (typeof loc !== 'undefined' && typeof id !== 'undefined') {
        dataString += "&association_type=" + loc + "&association_id=" + id;
    }

    $.ajax({
        async: true,
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=events&layout=event_listings&format=raw&tmpl=component',
        data: dataString,
        dataType: 'html',
        success: function (data) {
            $('#task_list').empty().html(data);

            // update current selected item
            $('#event_user_link').text($(clickedElem).text());


            ajaxLoadingToggle(true);
        }
    });

}

// Filter people types
function peopleType(type) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=people&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'type=' + type + '&layout=list'+'&ajaxLimit='+limit,
        dataType: 'html',
        success: function (data) {
            var peopleDiv = $('#people');

            // Assign new html
            $.when(peopleDiv.empty()).then(function () {
                peopleDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Filter people by user
function peopleUser(user, team) {
    var $ = jQuery;



    ajaxLoadingToggle();

    var search_id = ( user ) ? user : team;
    var dataString = ( user ) ? 'user=' + search_id : 'team_id=' + search_id;

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=people&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: dataString+'&layout=list'+'&ajaxLimit='+limit,
        dataType: 'html',
        success: function (data) {
            var peopleDiv = $('#people');

            // Assign new html
            $.when(peopleDiv.empty()).then(function () {
                peopleDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Filter people by stage// Updates
function peopleUpdated(stage) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&task=ajax.filterPeople&view=people&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'stage=' + stage+'&layout=list'+'&ajaxLimit='+limit,
        dataType: 'html',
        success: function (data) {
            var peopleDiv = $('#people');

            // Assign new html
            $.when(peopleDiv.empty()).then(function () {
                peopleDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Filter people status
function peopleStatus(status) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&task=ajax.filterPeople&view=people&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'status=' + status+'&layout=list'+'&ajaxLimit='+limit,
        dataType: 'html',
        success: function (data) {
            var peopleDiv = $('#people');

            // Assign new html
            $.when(peopleDiv.empty()).then(function () {
                peopleDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Filter documents based on association
function documentAssoc(assoc){
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=documents&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'assoc=' + assoc +'&layout=list'+'&ajaxLimit='+limit,
        dataType: 'html',
        success: function (data) {
            var docsDiv = $('#documents');

            // Assign new html
            $.when(docsDiv.empty()).then(function () {
                docsDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Filter through document categories
function documentCategory(categoryId) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=documents&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'category_id=' + categoryId +'&layout=list'+'&ajaxLimit='+limit,
        dataType: 'html',
        success: function (data) {
            var documentsDiv = $('#documents');

            // Assign new html
            $.when(documentsDiv.empty()).then(function () {
                documentsDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Filter documents based on ownership
function documentUser(user, team) {
    var $ = jQuery;



    ajaxLoadingToggle();

    var search_id = ( user ) ? user : team;
    var dataString = ( user ) ? 'user=' + search_id : 'team_id=' + search_id;

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=documents&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: dataString +'&layout=list'+'&ajaxLimit='+limit,
        dataType: 'html',
        success: function (data) {
            var docsDiv = $('#documents');

            // Assign new html
            $.when(docsDiv.empty()).then(function () {
                docsDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });

}

// Filter documents based on type
function documentType(type) {
    var $ = jQuery;



    ajaxLoadingToggle();

    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Make ajax call
    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=documents&format=raw&tmpl=component&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: 'type=' + type +'&layout=list'+'&ajaxLimit='+limit,
        dataType: 'html',
        success: function (data) {
            var docsDiv = $('#documents');

            // Assign new html
            $.when(docsDiv.empty()).then(function () {
                docsDiv.html(data);
            });

            ajaxLoadingToggle();
        }
    });
}

// Function for filtering sales dashboard graphs
function salesDashboardFilter(member, team) {



    var data = "";

    // If we are searching for a specific member
    if (parseInt(member) > 0) {
        // graphFilterType = 'member_'+member;
        graphFilterType = 'member';
        graphFilterId = member;
        data = 'filter=member&id=' + member;
    }
    // If we are searching for a specific team
    if (parseInt(team) > 0) {
        // graphFilterType = 'team_'+team;
        graphFilterType = 'team';
        graphFilterId = team;
        data = 'filter=team&id=' + team;
    }
    // If we are searching for the company
    if (parseInt(member) == 0 && parseInt(team) == 0) {
        graphFilterType = 'company';
        data = 'filter=company';
    }

    jQuery.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&task=graphs.getGraphData&format=raw&tmpl=component',
        data: data,
        dataType: 'json',
        success: function (data) {
            // Update our graph data
            // Redisplay the charts
            showAllCharts(data);
        }
    });
}

// Sort all tables by ascending or descending
function sortTable(column, ele) {
    var $ = jQuery,
        orderData = {};

    ajaxLoadingToggle();


    // Hide the image for the last column sorted
    $('div.sort_order').each(function (index, ele) {
        var base = $(ele);

        base.removeClass('order_asc');
        base.removeClass('order_desc');
    });

    // Set page globals
    if (order_col == column) {
        order_dir = ( order_dir == 'desc' ) ? 'asc' : 'desc';
    } else {
        order_col = column;
        order_dir = 'asc';
    }



    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }


    // Construct sort string
    var orderData = {
        filter_order: column,
        filter_order_Dir: order_dir,
        ajaxLimit : limit
    };


    // Get the table we wish to display the results in
    var table_ele = (typeof window.loc !== 'undefined' && window.loc.length > 0) ? "#" + window.loc : $('tbody.results');

    if (window.loc == "custom_report") {
        table_ele = $('tbody.results');
    }



    console.log(order_url)

    // Make ajax call
    $.ajax({
        type: 'post',
        url: order_url+'&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
        data: orderData,
        dataType: 'html',
        success: function (data) {
            $(table_ele).html(data);

            assignFilterOrder();
            bindDropdownItems();
            ajaxLoadingToggle();
        }
    });

}

function assignFilterOrder() {
    var $ = jQuery;

    $('div.sort_order').children('a').each(function () {
        var base = $(this);

        if (base.hasClass(order_col)) {
            base.parent('div').addClass('order_' + order_dir);
        }
    });
}

// Filter a table by certain values or dropdowns
function filterTable(val, col, ele) {

    //

    ajaxLoadingToggle();


    // get pagination limit
    let limit = jQuery('input[name="ajaxLimit"]').val();
    if(limit == undefined){
        limit = jQuery('select[name="limit"] option:selected').val()
    }

    // Construct query string
    var order_string = col + "=" + val;

    // Get the table we wish to display the results in

    if (window.loc == "custom_report") {
        var table_ele = jQuery('tbody.results');
    }
    else if (window.loc == "custom_report") {
        var table_ele = jQuery('tbody.results');
    } else {
        var table_ele = (typeof window.loc !== 'undefined' && window.loc.length > 0) ? "#" + window.loc : jQuery('tbody.results, #com_joomcrm div.results');
    }

    // Determine any other variables we might need to pass such as team or member types
    var input_type = jQuery(ele).attr('type');
    if (input_type != 'text') {

        var selected = jQuery("option:selected", ele);
        if (jQuery(selected.parent()[0]).attr('class') == "member") {
            order_string += "&owner_type=member";

        } else if (jQuery(selected.parent()[0]).attr('class') == "team") {
            order_string += "&owner_type=team";

        }

        order_string += "&loc=" + window.loc + "&format=raw&tmpl=component";

        // Make ajax call
        jQuery.ajax({
            type: 'post',
            url: order_url+'&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
            data: order_string+'&ajaxLimit='+limit,
            dataType: 'html',
            success: function (data) {

                ajaxLoadingToggle(true);

                // Update table html

                jQuery(table_ele).html(data);

                bindFilters();
                bindDropdownItems();

                if (window.loc == "custom_report") {
                    updateReportFooter();
                }

            }
        })

    } else {

        order_string += "&loc=" + window.loc + "&format=raw&tmpl=component";

        // Make ajax call
        jQuery.ajax({
            type: 'post',
            url: order_url+'&Itemid='+Joomla.getOptions('com_joomcrm.urlParams').Itemid,
            data: order_string+'&ajaxLimit='+limit,
            dataType: 'html',
            success: function (data) {

                // Update table html
                jQuery(table_ele).html(data);

                bindFilters();
                bindDropdownItems();

                ajaxLoadingToggle(true);

            }
        })

    }
}


function updateReportFooter() {
    var $ = jQuery;

    $.ajax({
        type: 'post',
        url: Joomla.getOptions('system.paths').rootFull + 'index.php?option=com_joomcrm&view=reports&layout=custom_report_footer&format=raw&tmpl=component',
        data: {id: id},
        dataType: 'html',
        success: function (html) {
            $(".results").next('tfoot').html(html);
        }
    });
}

function toggleCheckbox(checkboxId) {
    // @Jan: it is not necesary to trigger change event. It is triggered anyway.
    // var $ = jQuery,
    //     base = $('#' + checkboxId);

    // if (base.length > 0) {
    //     base.prop('checked', !base[0].checked);
    //     base.trigger('change');
    // }
}