<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;

class JoomcrmModelAvatar extends JoomcrmModel
{
	public $image;

	public $image_type;

	/**
	 * Save user avatars
	 *
	 * @return  [type]  [description]
	 */
	public function saveAvatar()
	{
		// This is the name of the field in the html form, filedata is the default name for swfupload
		// So we will leave it as that
		$fieldName = 'avatar';
		// Any errors the server registered on uploading
		$fileError = $_FILES[$fieldName]['error'];

		if ($fileError > 0)
		{
			switch ($fileError)
			{
				case 1:
					echo Text::_('FILE TO LARGE THAN PHP INI ALLOWS');

					return;
				case 2:
					echo Text::_('FILE TO LARGE THAN HTML FORM ALLOWS');

					return;
				case 3:
					echo Text::_('ERROR PARTIAL UPLOAD');

					return;
				case 4:
					echo Text::_('ERROR NO FILE');

					return;
			}
		}

		// Check the file extension is ok
		$fileName = $_FILES[$fieldName]['name'];
		$fileTemp = $_FILES[$fieldName]['tmp_name'];

		$uploadedFileNameParts = explode('.', $fileName);
		$uploadedFileExtension = array_pop($uploadedFileNameParts);

		$validFileExts = explode(',', 'jpeg,jpg,png,gif,bmp');

		// Assume the extension is false until we know its ok
		$extOk = false;

		// Go through every ok extension, if the ok extension matches the file extension (case insensitive)
		// Then the file extension is ok
		foreach ($validFileExts as $key => $value)
		{
			if (preg_match("/$value/i", $uploadedFileExtension))
			{
				$extOk = true;
			}
		}

		if ($extOk == false)
		{
			echo Text::_('INVALID EXTENSION');

			return;
		}

		// Data generation
		$date         = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));
		$hashFilename = md5($fileName . $date) . "." . $uploadedFileExtension;

		// Lose any special characters in the filename
		$fileName = preg_replace("/[^A-Za-z0-9.]/", "-", $fileName);

		// Always use constants when making file paths, to avoid the possibilty of remote file inclusion
		$uploadPath = JPATH_SITE . '/components/com_joomcrm/media/avatars/' . $hashFilename;

		if (!\Joomla\CMS\Filesystem\File::upload($fileTemp, $uploadPath))
		{
			echo Text::_('ERROR MOVING FILE');

			return;
		}

		$image = new \Joomla\CMS\Image\Image();
		$image->loadFile($uploadPath);
		$image->resize(250, 250, false);
		$image->toFile($uploadPath);

		$item_type = $this->app->input->get('item_type','','word');
		$item_id   = $this->app->input->getInt('item_id');

		$data = array('id' => $item_id, 'avatar' => $hashFilename);

		$this->deleteOldAvatar($item_id, $item_type);

		switch ($item_type)
		{
			case "people":
				$model_name = "People";
				break;
			case "companies":
				$model_name = "Company";
				break;
		}

		$model = BaseDatabaseModel::getInstance($model_name, 'JoomcrmModel');
		$model->store($data);

		return Uri::base() . 'components/com_joomcrm/media/avatars/' . $hashFilename;

	}

	public function deleteOldAvatar($item_id, $item_type)
	{
		$avatar = $this->getAvatar($item_id, $item_type);

		if ($avatar)
		{
			\Joomla\CMS\Filesystem\File::delete(JPATH_SITE . '/components/com_joomcrm/media/avatars/' . $avatar);
		}
	}

	public function getAvatar($item_id, $item_type)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->select("avatar")->from("#__joomcrm_" . $item_type)
				->where('id = ' . (int) $item_id);

		return $db->setQuery($query)->loadResult();
	}
}
