<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

class JoomcrmModelCommission extends JoomcrmModel
{
	public $start_date = null;
	public $end_date = null;

	/**
	 * Get Monthly Commission
	 *
	 * @param $access_type we wish to filter by 'member','team','company'
	 * @param $access_id   the id of the $access_type we wish to filter by
	 *
	 * @return mixed $results
	 */
	public function getMonthlyCommission($access_type = null, $access_id = null)
	{
		// Get member data
		if ($access_type == 'member')
		{
			return $this->getMonthlyCommissionData($access_id);
		}
		else
		{
			// Array to assign our data
			$members = array();
			$results = array();

			// Get team data
			if ($access_type == 'team')
			{
				// Get team members
				$team_members = JoomcrmHelperUsers::getTeamUsers($access_id);
				foreach ($team_members as $key => $member)
				{
					$members[] = $this->getMonthlyCommissionData($member['id']);
				}
			}

			// Get company data
			if ($access_type == 'company')
			{
				// Get company users
				$company_members = JoomcrmHelperUsers::getCompanyUsers();
				foreach ($company_members as $key => $member)
				{
					$members[] = $this->getMonthlyCommissionData($member['id']);
				}
			}

			// Combine data
			foreach ($members as $key => $member)
			{
				foreach ($member as $date_key => $data)
				{
					if (array_key_exists($date_key, $results))
					{
						$results[$date_key]['y'] += $data['y'];
					}
					else
					{
						$results[$date_key]['y'] = $data['y'];
					}
				}
			}

			return $results;

		}
	}

	/**
	 * Get Yearly Commission
	 *
	 * @param $access_type we wish to filter by 'member','team','company'
	 * @param $access_id   the id of the $access_type we wish to filter by
	 *
	 * @return mixed $results
	 */
	public function getYearlyCommission($access_type = null, $access_id = null)
	{
		// Get member data
		if ($access_type == 'member')
		{
			return $this->getYearlyCommissionData($access_id);
		}
		else
		{
			// Array to assign our data
			$members = array();
			$results = array();

			// Get team data
			if ($access_type == 'team')
			{
				// Get team members
				$team_members = JoomcrmHelperUsers::getTeamUsers($access_id);
				foreach ($team_members as $key => $member)
				{
					$members[] = $this->getYearlyCommissionData($member['id']);
				}
			}

			// Get company data
			if ($access_type == 'company')
			{
				// Get company users
				$company_members = JoomcrmHelperUsers::getCompanyUsers();
				foreach ($company_members as $key => $member)
				{
					$members[] = $this->getYearlyCommissionData($member['id']);
				}
			}

			// Combine data
			foreach ($members as $key => $member)
			{
				foreach ($member as $date_key => $data)
				{
					if (array_key_exists($date_key, $results))
					{
						$results[$date_key]['y'] += $data['y'];
					}
					else
					{
						$results[$date_key]['y'] = $data['y'];
					}
				}
			}

			return $results;
		}
	}

	/**
	 * Get monthly commission data for member
	 *
	 * @param $id to search for
	 *
	 * @return mixed $results
	 */
	public function getMonthlyCommissionData($id)
	{
		// Get db
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Get current month
		$current_month = JoomcrmHelperDate::formatDBDate(date('Y-m-01 00:00:00'));

		// Get weeks in month
		$weeks = JoomcrmHelperDate::getWeeksInMonth($current_month);

		// Get stage id to filter deals by
		$won_stage_ids = JoomcrmHelperDeal::getWonStages();

		// Gen query
		$results = array();
		foreach ($weeks as $week)
		{
			$start_date = $week['start_date'];
			$end_date   = $week['end_date'];

			// Flush query
			$query->clear()->select("d.owner_id, SUM(d.amount) AS y")->from("#__joomcrm_deals AS d")
			->where("d.stage_id IN (" . implode(',', $won_stage_ids) . ")")->where("d.actual_close >= '$start_date'")
			->where("d.actual_close < '$end_date'")->where("d.actual_close IS NOT NULL")
			->where("d.owner_id = " . (int) $id)->group("d.owner_id")->where("d.published>0")->where("d.archived=0");

			$results[] = $db->setQuery($query)->loadAssoc();
		}

		// Clean data for commission rate
		foreach ($results as $key => $result)
		{
			if(is_null($result)){
				$results[$key]['y'] = 0;
				continue;
			}
			$commission_rate    = JoomcrmHelperUsers::getCommissionRate($result['owner_id']);
			$results[$key]['y'] = (int) $result['y'] * ($commission_rate / 100);
		}



		return $results;
	}

	/**
	 * Get yearly commission data for user
	 *
	 * @param int $id to search for
	 *
	 * @return mixed $results
	 */
	public function getYearlyCommissionData($id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Get current year and months to loop through
		$current_year = JoomcrmHelperDate::formatDBDate(date('Y-01-01 00:00:00'));
		$months       = JoomcrmHelperDate::getMonthDates($this->start_date, $this->end_date);

		// Get stage id to filter deals by
		$won_stage_ids = JoomcrmHelperDeal::getWonStages();

		// Gen query
		$results = array();
		foreach ($months as $month)
		{
			$start_date = $month['date'];
			$end_date   = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime("$start_date + 1 months")));

			// Flush the query
			$query->clear()->select("d.owner_id,d.actual_close,SUM(d.amount) AS y")->from("#__joomcrm_deals AS d")
			->where("d.stage_id IN (" . implode(',', $won_stage_ids) . ")")->where("d.actual_close >= '$start_date'")
			->where("d.actual_close < '$end_date'")->where("d.actual_close IS NOT NULL")
			->where("d.owner_id = " . (int) $id)->group("d.owner_id")->where("d.published>0")->where("d.archived=0");

			$results[] = $db->setQuery($query)->loadAssoc();
		}

		// Clean data for commission rate
		foreach ($results as $key => $result)
		{
			if(is_null($result)){
				$results[$key]['y'] = 0;
				continue;
			}
			$commission_rate    = JoomcrmHelperUsers::getCommissionRate($result['owner_id']);
			$results[$key]['y'] = (int) $result['y'] * ($commission_rate / 100);
		}

		return $results;
	}

}
