<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;

class JoomcrmModelCompany extends JoomcrmModel
{
	public $_total;
	public $_user;
	public $_team;
	public $_id;
	public $_type;
	public $_category;
	public $show_filters;
	public $published = 1;

	/**
	 * Method to store a record
	 *
	 * @return    boolean    True on success
	 */
	public function store($data = null)
	{

		$session 	= Factory::getSession();
		$input 		= $this->app->input;

		// Load Tables
		$row    = new JoomcrmTableCompany;
		$oldRow = new JoomcrmTableCompany;

		if ($data == null)
		{
			$data = $input->post->getArray();
		}

		if (array_key_exists('name', $data))
		{
			$data['name'] = trim($data['name']);
		}

		// Date generation
		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

		// Detect if we are importing a company
		$importing = array_key_exists('import', $data);


		if (!array_key_exists('id', $data) || $data['id'] == "")
		{
			if (empty($data['created']))
			{
				$data['created']     = $date;
			}

			$status           = 'created';
			$data['owner_id'] = (array_key_exists('owner_id', $data) && ($data['owner_id']>0)) ? $data['owner_id'] : JoomcrmHelperUsers::getUserId();

			if (!$importing)
			{
				$companyExists = $this->checkCompanyName($data['name']);

				if ($companyExists != "")
				{
					$session->set("Company.error", true);
					$session->set("Company.error_message", Text::_('COM_JOOMCRM_DUPLICATE_COMPANY'));
					$session->set("Company.editing", $data);

					return false;
				}
			}

		}
		else
		{
			$row->load($data['id']);
			$oldRow->load($data['id']);
			$status = 'updated';

			// Do not change owner_id on exit
			if (isset($data['owner_id']) && !$data['owner_id'])
			{
				unset($data['owner_id']);
			}
		}

		if (empty($data['modified']))
		{
			$data['modified'] = $date;
		}


        // category import
		if(isset($data['category_id']) && !empty($data['category_id']) && $importing){

			// update category for import
			$newCatId = JoomcrmHelperCompany::updateCategoryForImport($data['category_id']);

			if(!$newCatId)
				unset($data['category_id']);
			else
				$data['category_id'] = $newCatId;
		}




		// Generate custom field string
		$customArray = array();
		foreach ($data as $name => $value)
		{
			if (strstr($name, 'custom_') && !strstr($name, '_input') && !strstr($name, "_hidden"))
			{
				$id            = str_replace('custom_', '', $name);
				$customArray[] = array('custom_field_id' => $id, 'custom_field_value' => $value);
				unset($data[$name]);
			}
		}

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());
			$session->set("Company.error", true);
			$session->set("Company.error_message", Text::_('COM_JOOMCRM_ERROR_SAVING'));
			$session->set("Company.editing", $data);

			return false;
		}

		//$row->custom_fields = $row->id > 0 ? JoomcrmHelperCustom::getCustomCSVData($row->id, "company") : array();
		$dispatcher         = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onBeforeCompanySave', array(&$row));

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());
			$session->set("Company.error", true);
			$session->set("Company.error_message", Text::_('COM_JOOMCRM_ERROR_SAVING'));
			$session->set("Company.editing", $data);

			return false;
		}


		if (!$row->store())
		{
			$this->setError($row->getError());
			$session->set("Company.error", true);
			$session->set("Company.error_message", Text::_('COM_JOOMCRM_ERROR_SAVING').': '.$row->getError());
			$session->set("Company.editing", $data);

			return false;
		}



		// email notifications
		if (!array_key_exists('id', $data) || $data['id'] == ""){

			// send email notification of new company created
			JoomcrmHelperEmail::sendCompanyEmail($row,'add');

			// send email to owner of new company
			if(	isset($data['owner_id']) &&	!empty($data['owner_id'])){
				JoomcrmHelperEmail::sendCompanyEmail($row,'associate',true);
			}

		}
		else{

			// send email notification if owner was updated in company
			if(
				isset($oldRow->owner_id) && // old owner id exist
				!empty($oldRow->owner_id) && // old owner id not empty
				$oldRow->owner_id > 0 && // old owner id > 0
				isset($data['owner_id']) && // new owner id exist
				!empty($data['owner_id']) && // new owner id not empty
				$oldRow->owner_id != $data['owner_id'] // new owner id not equal to old owner id
			){
					JoomcrmHelperEmail::sendCompanyEmail($row,'associate',true);
			}

			// send email notification when copmany edited
			JoomcrmHelperEmail::sendCompanyEmail($row,'edit');

		}

		$id = array_key_exists('id', $data) ? $data['id'] : $this->getDbo()->insertId();


		// update note for import
		if($importing)
			JoomcrmHelperNote::updateNoteForImport($data['note'],'company',$id,$row->owner_id);

		// activity log
		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'company', $status);

		// If we receive no custom post data do not modify the custom fields
		if (count($customArray) > 0)
		{
			JoomcrmHelperJoomcrm::storeCustomCf($row->id, $customArray, 'company');
		}

		$row->custom_fields = $row->id > 0 ? JoomcrmHelperCustom::getCustomCSVData($row->id, "company") : array();
		$dispatcher         = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onAfterCompanySave', array(&$row));

		// Assign any default template systems ( events )
		if ($status == "created")
		{
			$templateModel = new JoomcrmModelTemplate;
			$templateModel->processDefaults($row->id, "company");
		}

		return $row;
	}

	/**
	 * Build our db query object
	 */
	public function _buildQuery()
	{
		/** Large SQL Selections **/
		$db    = $this->getDbo();
		$query = $db->getQuery(true);
		$isDatatableQuery = $this->app->input->get('datatableQuery',0,'int');
		$params = \Joomla\CMS\Component\ComponentHelper::getParams('com_joomcrm');

		$db->setQuery("SET SQL_BIG_SELECTS=1")->execute();

		$user     = $this->_user;
		$team     = $this->_team;
		$category = $this->_category;
		$id       = $this->_id;

		$type     = $this->_type;

		if (!$id)
		{
			$session = Factory::getSession();
			$name    = $this->app->input->getString('company_name', null);

			if(!$isDatatableQuery){ // don't filter if datatable

				if ($name !== null)
				{
					$session->set('company_name_filter', $name);
				}
				else {
					$name = $session->get('company_name_filter');
				}

				// Determine whether we are searching for a team or user
				if ($user)
				{
					$session->set('company_team_filter', null);
				}

				if ($team)
				{
					$session->set('company_user_filter', null);
				}

				// Set user session data
				if ($type != null)
				{
					$session->set('company_type_filter', $type);
				}
				else
				{
					$sess_type = $session->get('company_type_filter',$params->get('companies_type_filter','all'));
					$type      = $sess_type;
				}
				if ($user != null)
				{
					$session->set('company_user_filter', $user);
				}
				else
				{
					$sess_user = $session->get('company_user_filter',$params->get('companies_user_filter',1));
					$user      = $sess_user;
				}
				if ($team != null)
				{
					$session->set('company_team_filter', $team);
				}
				else
				{
					$sess_team = $session->get('company_team_filter');
					$team      = $sess_team;
				}

				if ($category != null)
				{
					$session->set('company_category_filter', $category);
				}
				else
				{
					$sess_category = $session->get('company_category_filter',$params->get('companies_category_filter',0));
					$category      = $sess_category;
				}
			}



		}

		// Generate query for base companies
		$export = $this->app->input->get('export');

		if ($export)
		{
			$select_string = 'DISTINCT(c.id),c.name,c.description,c.address_1,c.address_2,c.address_city,';
			$select_string .= 'c.address_state,c.address_zip,c.address_country,c.website,c.created,c.modified,';
			$select_string .= 'c.phone,c.fax,c.email,c.twitter_user,c.facebook_url,c.flickr_url,c.youtube_url,linkedin_url, c.public,';
			$select_string .= 'CONCAT(u.first_name, " ", u.last_name) AS owner_name,c.owner_id,';
			$select_string .= 'cat.name AS category_name,';
			$select_string .= 'event.id AS event_id,event.type AS event_type,event.name AS event_name,IF(event.type="task",event.due_date,event.start_time) AS event_due_date';

			$query
				->clear()
				->select($select_string)
				->from("#__joomcrm_companies as c")
				->leftJoin("#__joomcrm_users AS u on u.id = c.owner_id")
				->leftJoin("#__joomcrm_company_categories as cat on cat.id = c.category_id")
				->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=c.id AND shared.item_type='company'");
		}
		else
		{
			$query
				->clear()
				->select('DISTINCT(c.id),c.*,cat.name AS category_name,event.id AS event_id,event.type AS event_type,event.name AS event_name,IF(event.type="task",event.due_date,event.start_time) AS event_due_date')
				->select("u.first_name AS owner_first_name,u.last_name AS owner_last_name")
				->from("#__joomcrm_companies as c")
				->leftJoin("#__joomcrm_users AS u on u.id = c.owner_id")
				->leftJoin("#__joomcrm_company_categories as cat on cat.id = c.category_id")
				->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=c.id AND shared.item_type='company'");
		}

		if (!$id)
		{
			// Get current date
			$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00'));

			// Filter for type
			if ($type != null && $type != "all")
			{
				// Filter for deals// Tasks due today
				if ($type == 'today')
				{
					$today    = JoomcrmHelperDate::formatDBDate(date('Y-m-d'), false) . " 00:00:00";
					$tomorrow = JoomcrmHelperDate::formatDBDate(date('Y-m-d', strtotime($today . " +1 days")), false) . " 00:00:00";
					$query
						->leftJoin("#__joomcrm_events_cf AS ecf ON ecf.association_type='company' AND ecf.association_id=c.id")
						->leftJoin("#__joomcrm_events AS event ON event.id=ecf.event_id")
						->where("( event.due_date > '$today' AND event.due_date < '$tomorrow' )")
						->where("event.published>0 AND event.completed=0");
				}

				// Filter for companies and deals// Tasks due tomorrow
				else
				{
					if ($type == "tomorrow")
					{
						$tomorrow         = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() + (1 * 24 * 60 * 60)));
						$dayAfterTomorrow = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00", strtotime($tomorrow . " +1 days")));
						$query
							->leftJoin("#__joomcrm_events_cf AS ecf ON ecf.association_type='company' AND ecf.association_id=c.id")
							->leftJoin("#__joomcrm_events AS event ON event.id=ecf.event_id")
							->where("event.due_date >'" . $tomorrow . "' AND event.due_date < '$dayAfterTomorrow'")
							->where("event.published>0 AND event.completed=0");
					}
					else
					{
						// Get next event
						$query->leftJoin("#__joomcrm_events AS event ON event.id = (
										SELECT event2.id
										FROM #__joomcrm_events_cf AS ecf2
										LEFT JOIN #__joomcrm_events AS event2 ON
											event2.id=ecf2.event_id AND
											ecf2.association_type='company'
										WHERE ecf2.association_id=c.id AND
											  ecf2.association_type='company' AND
											  event2.completed=0
										ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
												 CASE WHEN event2.type='event' THEN start_Time END ASC
										LIMIT 1 )");

					}
				}

				// Filter for companies updated in the last 30 days
				if ($type == "updated_thirty")
				{
					$last_thirty_days = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() - (30 * 24 * 60 * 60)));
					$query->where("c.modified >'$last_thirty_days'");
				}

				// Filter for past companies// last contacted 30 days ago or longer
				if ($type == "past")
				{
					$last_thirty_days = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() - (30 * 24 * 60 * 60)));
					$query->where("c.modified <'$last_thirty_days'");
				}

				// Filter for recent companies
				if ($type == "recent")
				{
					$last_thirty_days = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() - (30 * 24 * 60 * 60)));
					$query->where("c.modified >'$last_thirty_days'");
				}

				$query->group("c.id");

			}
			else
			{
				// Get next event
				$query->leftJoin("#__joomcrm_events AS event ON event.id = (
									SELECT event2.id
									FROM #__joomcrm_events_cf AS ecf2
									LEFT JOIN #__joomcrm_events AS event2 ON
										event2.id=ecf2.event_id AND
										ecf2.association_type='company'
									WHERE ecf2.association_id=c.id AND
										  ecf2.association_type='company' AND
										  event2.completed=0
									ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
											 CASE WHEN event2.type='event' THEN start_Time END ASC
									LIMIT 1 )");

			}

			$session = Factory::getSession()->get('registry');

			/** company name filter **/
			if ($name != null)
			{
				$query->where("( c.name LIKE '%" . $db->escape($name) . "%' )");
				$this->show_filters = 1;
			}

			/** company phone filter **/
			$company_phone = $session->get('Company.' . $this->view . '_phone');
			if ($company_phone != null)
			{
				$query->where("( c.phone LIKE '%" . $db->escape($company_phone) . "%' )");
				$this->show_filters = 1;
			}

			/** company fax filter **/
			$company_fax = $session->get('Company.' . $this->view . '_fax');
			if ($company_fax != null)
			{
				$query->where("( c.fax LIKE '%" . $db->escape($company_fax) . "%' )");
				$this->show_filters = 1;
			}

			// Company email filter
			$company_email = $session->get('Company.' . $this->view . '_email');
			if ($company_email != null)
			{
				$query->where("( c.email LIKE '%" . $db->escape($company_email) . "%' )");
				$this->show_filters = 1;
			}

			// Company address filter
			$company_address = $session->get('Company.' . $this->view . '_address');
			if ($company_address != null)
			{
				$query->where("(    ( c.address_1 LIKE '%" . $db->escape($company_address) . "%' ) OR
									( c.address_2 LIKE '%" . $db->escape($company_address) . "%' ) OR
									( c.address_city LIKE '%" . $db->escape($company_address) . "%' ) OR
									( c.address_state LIKE '%" . $db->escape($company_address) . "%' ) OR
									( c.address_zip LIKE '%" . $db->escape($company_address) . "%' ) ) ");
				$this->show_filters = 1;
			}

			// Company country filter
			$company_country = $session->get('Company.' . $this->view . '_country');
			if ($company_country != null)
			{
				$query->where("( c.address_country LIKE '%" . $db->escape($company_country) . "%' )");
				$this->show_filters = 1;
			}

			// Company description filter
			$company_description = $session->get('Company.' . $this->view . '_description');
			if ($company_description != null)
			{
				$query->where("( c.description LIKE '%" . $db->escape($company_description) . "%' )");
				$this->show_filters = 1;
			}

		}
		else
		{
			// Get next event
			$query->leftJoin("#__joomcrm_events AS event ON event.id = (
								SELECT event2.id
								FROM #__joomcrm_events_cf AS ecf2
								LEFT JOIN #__joomcrm_events AS event2 ON
									event2.id=ecf2.event_id AND
									ecf2.association_type='company'
								WHERE ecf2.association_id=c.id AND
									  ecf2.association_type='company' AND
									  event2.completed=0
								ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
										 CASE WHEN event2.type='event' THEN start_Time END ASC
								LIMIT 1 )");

		}

		// Search for specific companies
		if ($id != null)
		{
			if (is_array($id))
			{
				$query->where("c.id IN (" . implode(',', $id) . ")");
			}
			else
			{
				$query->where("c.id=$id");
			}
		}

		/** ---------------------------------------------------------------
		 * Filter data using member role permissions
		 */
		$member_id   = JoomcrmHelperUsers::getUserId();
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();
		$team_access = JoomcrmHelperUsers::getTeamAccess($member_id);

		if ($id > 0)
		{


			if ($member_role == "manager" || (isset($team_id) && (int) $team_id > 0 && $team_access) )
			{
				$query->where('( c.owner_id = ' . $member_id . ' OR u.team_id = ' . $team_id . " OR shared.user_id=" . $member_id . " OR c.public=1 )");
			}
			else
			{
				if ($member_role == "basic")
				{
					$query->where(array('( c.owner_id = ' . $member_id . " OR shared.user_id=" . $member_id . " OR c.public=1 )"));
				}
			}
		}
		else
		{
			if ((isset($user) && $user == "all"))
			{
				if ($member_role != 'exec')
				{
					// Manager filter
					if ($member_role == 'manager' || (isset($team_id) && $team_access) )
					{
						$query->where('( c.owner_id = ' . $member_id . ' OR u.team_id = ' . $team_id . " OR shared.user_id=" . $member_id . " OR c.public=1 )");
					}
					else
					{
						// Basic user filter
						$query->where('( c.owner_id = ' . $member_id . ' OR shared.user_id=' . $member_id . '  OR c.public=1 )');
					}
				}
			}
			else
			{
				if ($team)
				{
					$query->where("u.team_id=$team");
				}
				else
				{
					if ($user && $user != "all")
					{
						$query->where("c.owner_id=" . $user);
					}
					else
					{
						if (!(isset($user)))
						{
							$query->where('( c.owner_id = ' . $member_id . ' OR shared.user_id=' . $member_id . '  OR c.public=1 )');
						}
					}
				}
			}
		}

		// Filter based on category
		if ($category && ($category != "all" && $category != "any") && $category > 0)
		{
			$query->where("c.category_id=" . $category);
		}


		if ($this->app->input->get('advanced_filter'))
		{
			$this->show_filters = 1;
		}

		// Set user state requests
		$query
			->order($this->getState('Company.filter_order') . ' ' . $this->getState('Company.filter_order_Dir'))
			->where('c.published = ' . $this->published);

		return $query;
	}

	public function getCompany($id)
	{
		$company = $this->getCompanies($id);
		return $company[0];
	}

	/**
	 * [getCompanies description]
	 *
	 * @param   [type]  $id    [description]
	 * @param   [type]  $type  [description]
	 * @param   [type]  $user  [description]
	 * @param   [type]  $team  [description]
	 *
	 * @return  [type]         [description]
	 */
	public function getCompanies($id = null, $type = null, $user = null, $team = null)
	{
		$this->_id   = $id ? $id : $this->_id;
		$this->_type = $type;
		$this->_user = $user;
		$this->_team = $team;

		// Get session data
		$session = Factory::getSession();
		$db      = $this->getDbo();

		// Get query string
		$query = $this->_buildQuery();

		/** ------------------------------------------
		 * Set query limits and load results
		 */
		$limit_config = ComponentHelper::getParams('com_joomcrm')->get('companies_limit',10);
		$limit      = null;
		$limitStart = null;

		if (!$this->_id)
		{

			$limit      = $this->getState($this->_view . '_limit');

			$limitStart = $this->getState($this->_view . '_limitstart');

			if (!$this->_id && $limit != 0)
			{
				if ($limitStart >= $this->getTotal())
				{
					$limitStart = 0;
					$limit      = $limit_config;
					$limitStart = ($limit != 0) ? (floor($limitStart / $limit) * $limit) : 0;

					$this->setState($this->_view . '_limit');
					$this->setState($this->_view . '_limitstart', $limitStart);
				}
			}
		}

		// for export - no limit
		if ($this->get('export_limit') !== null)
		{
			$limit = $this->get('export_limit');
		}

		$companies = $db->setQuery($query, $limitStart, $limit)->loadAssocList();

		// Generate query to join people
		if (count($companies))
		{
			$export = $this->app->input->get('export');

			if (!$export)
			{
				foreach ($companies as $key => $company)
				{
					if ($this->_id)
					{
						// Generate people query
						$contactModel = new JoomcrmModelContacts;
						$contactModel->set('company_id', $company['id']);
						$contactModel->view2 = 'documents';
						$companies[$key]['people'] = $contactModel->getContacts();
						$companies[$key]['people_pagination'] = $contactModel->getPagination();
						$companies[$key]['people_pagination']->setAdditionalUrlParam('company_id', $this->_id);
						$companies[$key]['people_pagination']->setAdditionalUrlParam('format', 'raw');

						// Generate deal query
						$dealModel = new JoomcrmModelDeal;
						$dealModel->set('company_id', $company['id']);
						$dealModel->set('user_override', 'all');
						$deals     = $dealModel->getDeals();
						$companies[$key]['deals_pagination'] = $dealModel->getPagination();
						$companies[$key]['deals_pagination']->setAdditionalUrlParam('company_id', $this->_id);
						$companies[$key]['deals_pagination']->setAdditionalUrlParam('format', 'raw');
						$companies[$key]['pipeline']  = $this->calculatePipeline($this->_id);
						$companies[$key]['won_deals'] = 0;
						for ($i = 0; $i < count($deals); $i++)
						{
							$deal = $deals[$i];
							// $companies[$key]['pipeline'] += $deal['amount'];
							if ($deal['percent'] == 100)
							{
								$companies[$key]['won_deals'] += $deal['amount'];
							}
						}
						$companies[$key]['deals'] = $deals;

						// Get Associated Notes
						$notesModel = new JoomcrmModelNote(array('ignore_request' => true));
						// I sure hope no one has more than 100 notes associated w/a company. pls.
						// NOTE: This is NOT the right way to do it. But, oh well. The current structure is limiting.
						$notesModel->setState('companies_limit', 100);
						$notesModel->setState('companies_limistart', 0);
						$notesModel->public_notes = true;
						$notesModel->filters = false;
						$companies[$key]['notes'] = $notesModel->getNotes($company['id'], 'company');

						// Get Associated Documents
						$documentModel = new JoomcrmModelDocument;
						$documentModel->set('company_id', $company['id']);
						$documentModel->filters = false;
						$documentModel->view2 = 'documents';
						$companies[$key]['documents'] = $documentModel->getDocuments();
						$companies[$key]['documents_pagination'] = $documentModel->getPagination();
						$companies[$key]['documents_pagination']->setAdditionalUrlParam('company_id', $this->_id);
						$companies[$key]['documents_pagination']->setAdditionalUrlParam('format', 'raw');
						$companies[$key]['documents_pagination']->setAdditionalUrlParam('filters', '0');

					}

					$companies[$key]['address_formatted'] = (strlen($company['address_1']) > 0) ? $company['address_1'] . $company['address_2'] . ", " . $company['address_city'] . ' ' . $company['address_state'] . ', ' . $company['address_zip'] . ' ' . $company['address_country'] : "";
				}

			}

		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onCompanyLoad', array(&$companies));

		return $companies;
	}

	public function calculatePipeline($companyId)
	{
		$activeStages = JoomcrmHelperDeal::getActiveStageIds();

		if (count($activeStages) > 0)
		{
			$db    = $this->getDbo();
			$query = $db->getQuery(true);

			// Get primary company amounts
			$query
				->clear()
				->select("deal.amount,deal.id")
				->from("#__joomcrm_deals AS deal")
				->where("deal.company_id=" . (int) $companyId)
				->where("deal.stage_id IN(" . implode(',', $activeStages) . ")")
				->where("deal.published=1")
				->where("deal.archived=0");

			$pcAmounts = $db->setQuery($query)->loadAssocList('id');

			$sum = 0;

			if (is_array($pcAmounts) && count($pcAmounts) > 0)
			{
				foreach ($pcAmounts as $a)
				{
					$sum += $a['amount'];
				}
			}

			return $sum;
		}
		else
		{
			return 0;
		}
	}

	/**
	 * method to get list of companies
	 *
	 * @param   [type]  $company_name  [description]
	 *
	 * @return  [type]                 [description]
	 */
	public function getCompanyList($company_name = null)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("DISTINCT(c.id), c.name, c.id")
			->from("#__joomcrm_companies AS c")
			->leftJoin("#__joomcrm_users AS u on u.id = c.owner_id")
			->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=c.id AND shared.item_type='company'");

		// Member access roles
		$member_id   = JoomcrmHelperUsers::getUserId();
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();
		$team_access = JoomcrmHelperUsers::getTeamAccess($member_id);

		if ($member_role == "manager" || (isset($team_id) && $team_access) )
		{
			$query->where('( u.team_id = ' . $team_id . " OR shared.user_id=" . $member_id . " OR c.public=1 )");
		}
		else
		{
			if ($member_role == "basic")
			{
				$query->where(array('(c.owner_id = ' . $member_id . " OR shared.user_id=" . $member_id . " OR c.public=1  )"));
			}
		}

		if ($company_name)
		{
			$company_name = ucwords($company_name);
			$query->where("LOWER(c.name) LIKE '%" . $db->escape($company_name) . "%'");
		}

		$query->where("c.published=" . $this->published);

		return $db->setQuery($query)->loadAssocList();
	}

	/**
	 * Get Company Names
	 *
	 * @param   boolean $json  [description]
	 *
	 * @return  [type]          [description]
	 */
	public function getCompanyNames($json = false)
	{
		$names  = $this->getCompanyList();
		$return = array();

		if (count($names) > 0)
		{
			foreach ($names as $key => $name)
			{
				$return[] = array('label' => $name['name'], 'value' => $name['id']);
			}
		}

		return $json ? json_encode($return) : $return;
	}

	/**
	 * Checks for existing company by name
	 *
	 * @param  [var] $name company name to check
	 *
	 * @return [int]       ID of existing company
	 */
	public function checkCompanyName($name)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('c.id')
			->from('#__joomcrm_companies AS c')
			->where('LOWER(c.name) = ' . $db->quote(mb_strtolower($name,'UTF-8')));

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * @param   mixed  $idOrName  Name or ID to load
	 *
	 * @return  mixed
	 */
	public function getCompanyName($idOrName)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('c.name')
			->from('#__joomcrm_companies AS c')
			->where('c.id = ' . $db->quote($idOrName), 'OR')
			->where('c.name = ' . $db->quote($db->escape($idOrName)));

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Get total number of rows for pagination
	 */
	public function getTotal()
	{
		if (empty ($this->_total))
		{
			$query        = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
			$this->_total = $this->_total ? $this->_total : 0;
		}

		return $this->_total;
	}

	/**
	 * Generate pagination
	 */
	public function getPagination()
	{
		$limit_config = ComponentHelper::getParams('com_joomcrm')->get('companies_limit',10);

		// Lets load the content if it doesn't already exi•st
		// if (empty($this->_pagination)) {

		$total             = $this->getTotal();
		$total             = $total ? $total : 0;
		$this->_pagination = new JoomcrmPagination(
			$total,
			$this->getState($this->_view . '_limitstart'),
			$this->getState($this->_view . '_limit',$limit_config)
		);

		// }
		return $this->_pagination;
	}

	public function getEmail($company_id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('c.email')
			->from('#__joomcrm_companies AS c')
			->where('c.id=' . (int) $company_id);

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Populate user state requests
	 */
	public function populateState()
	{
		// Get pagination request variables
		$limit_config = ComponentHelper::getParams('com_joomcrm')->get('companies_limit',10);
		$limit      = $this->app->getUserStateFromRequest($this->view . '_limit', 'limit', $limit_config);
		$limitstart = $this->app->getUserStateFromRequest($this->view . '_limitstart', 'limitstart', 0);

		// In case limit has been changed, adjust it
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		$input = \Joomla\CMS\Factory::getApplication()->input;

		// if ajax request (when using filters, pagination limit and start should work good)
		if($input->get('format','','word') == 'raw'){

			$limit = $input->get('ajaxLimit',$limit,'int');
			$limitstart = 0; // should begin from start
		}

		$this->setState($this->view . '_limit', $limit);
		$this->setState($this->view . '_limitstart', $limitstart);

		// Set default filter states for reports
		$filter_order     = $this->app->getUserStateFromRequest('Company.filter_order', 'filter_order', 'c.name');
		$filter_order_Dir = $this->app->getUserStateFromRequest('Company.filter_order_Dir', 'filter_order_Dir', 'asc');

		$filters = array('name' => 'company_name', 'phone' => 'company_phone', 'fax' => 'fax', 'email' => 'email', 'address' => 'address', 'country' => 'country', 'description' => 'description');
		$reset   = $this->app->input->getString('reset_filters', false);

		foreach ($filters as $stateVar => $postVar)
		{
			if ($reset)
			{
				$this->app->setUserState('Company.' . $this->view . '_' . $stateVar, null);
			}
			else
			{
				$val = $this->app->getUserStateFromRequest('Company.' . $this->view . '_' . $stateVar, $postVar, null);
				$this->app->setUserState('Company.' . $this->view . '_' . $stateVar, $val);
			}
		}

		// Set states for reports
		$this->setState('Company.filter_order', $filter_order);
		$this->setState('Company.filter_order_Dir', $filter_order_Dir);
	}
}
