<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;
use Joomla\CMS\Categories\Categories;
use Joomla\CMS\Pagination\Pagination;

class JoomcrmModelContacts extends JoomcrmModel
{
	public $person_id;
	public $deal_id;
	public $company_id;
	public $event_id;
	public $query;

	/**
	* While displaying documents at other views (person, deal, company),
	* it is not wanted to use limit and limit_start from these views for document listing.
	*/
	public $view2 = '';

	public function buildSelect()
	{
		$this->query->select("DISTINCT(p.id),p.*,
							u.first_name as owner_first_name,u.last_name as owner_last_name,
							c.id as company_id,c.name as company_name,IF(p.id=d2.primary_contact_id, 1, 0) AS is_primary_contact")
		->from("#__joomcrm_people AS p")
		->leftJoin("#__joomcrm_people_cf as cf ON cf.person_id = p.id")
		->leftJoin("#__joomcrm_companies AS c ON c.id = p.company_id")
		->leftJoin("#__joomcrm_deals AS d ON d.id = cf.association_id AND cf.association_type = 'deal'")
		->leftJoin("#__joomcrm_deals AS d2 ON d2.primary_contact_id = p.id")
		->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=p.id AND shared.item_type='person'")
		->leftJoin("#__joomcrm_users AS u ON u.id = p.owner_id")
		->leftJoin("#__joomcrm_users AS u2 ON u2.id = p.assignee_id");
	}

	public function buildWhere()
	{
		if ($this->deal_id)
		{
			$this->query->where("(cf.association_id = " . $this->deal_id . " OR d2.id = " . $this->deal_id . ")");
		}

		if ($this->event_id)
		{
			$event_model = new JoomcrmModelEvent;
			$event       = $event_model->getEvent($this->event_id);

			if (array_key_exists('association_type', $event) && $event['association_type'] != null)
			{
				switch ($event['association_type'])
				{
					case "person":
						$this->query->where("p.id=" . $event['association_id']);
						break;
					case "deal":
						$this->query->where("cf.association_id=" . $event['association_id']);
						$this->query->where("cf.association_type='deal'");
						break;
					case "company":
						$this->query->where("p.company_id=" . $event['association_id']);
						break;
				}
			}
			else
			{
				return false;
			}
		}

		if ($this->company_id)
		{
			$this->query->where("p.company_id=" . $this->company_id);
		}

		// Filter based on member access roles
		$user_id     = JoomcrmHelperUsers::getUserId();
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();

		if ($member_role != 'exec')
		{
			if ($member_role == 'manager')
			{
				$this->query->where("(p.owner_id=$user_id OR p.assignee_id=$user_id OR u.team_id=$team_id OR u2.team_id=$team_id OR shared.user_id=$user_id)");
			}
			else
			{
				$this->query->where("(p.owner_id=$user_id OR p.assignee_id=$user_id OR shared.user_id=$user_id)");
			}

		}

		$this->query->where("p.published>0");

		return true;
	}

	public function buildOrder()
	{
		$this->query->order("is_primary_contact DESC");
	}

	public function getContacts($contact = null)
	{
		$db          = $this->getDbo();
		$this->query = $this->_buildQuery($contact);

		return $this->query ? $db->setQuery($this->query)->loadAssocList() : [];
	}

	public function _buildQuery($contact = null)
	{
		$db          = $this->getDbo();
		$this->query = $db->getQuery(true);

		$this->buildSelect();

		if (!$this->buildWhere())
		{
			return false;
		}

		$this->buildOrder();

		if ($contact !== null)
		{
			$this->query->where('(p.first_name LIKE "%' . $db->escape($contact) . '%" OR p.last_name LIKE "%' . $db->escape($contact) . '%")');
		}



		return $this->query;
	}

	/**
	 * Get total number of rows for pagination
	 */
	public function getTotal()
	{
		if (empty ($this->_total))
		{
			$query        = $this->_buildQuery();

			$this->_total = $this->_getListCount($query);

		}

		return $this->_total;
	}

	/**
	 * Generate pagination
	 */
	public function getPagination()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			$this->_pagination = new Pagination($this->getTotal(), $this->getState($this->_view . $this->view2 . '_limitstart'), $this->getState($this->_view . $this->view2 . '_limit'));
		}

		return $this->_pagination;
	}

	/**
	 * Populate user state requests
	 */
	public function populateState()
	{
		// Get pagination request variables
		$limit      = $this->app->getUserStateFromRequest($this->view . $this->view2 . '_limit', 'limit', 10);
		$limitstart = $this->app->getUserStateFromRequest($this->view . $this->view2 . '_limitstart', 'limitstart', 0);
		
		$this->setState($this->view . $this->view2 . '_limit', $limit);
		$this->setState($this->view . $this->view2 . '_limitstart', $limitstart);
	}
}
