<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

class JoomcrmModelConversation extends JoomcrmModel
{
	public $published = 1;
	public $deal_id;

	/**
	 * Method to store a record
	 *
	 * @return    boolean    True on success
	 */
	public function store()
	{
		// Load Tables
		$row    = new JoomcrmTableConversation;
		$oldRow = new JoomcrmTableConversation;
		$data   = \Joomla\CMS\Factory::getApplication()->input->post->getArray();

		// Date generation
		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

		if (!array_key_exists('id', $data))
		{
			$data['created'] = $date;
			$status          = "created";
		}
		else
		{
			$row->load($data['id']);
			$oldRow->load($data['id']);
			$status = "updated";
		}

		$data['modified'] = $date;
		$data['author']   = JoomcrmHelperUsers::getUserId();

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		$id = array_key_exists('id', $data) ? $data['id'] : $this->getDbo()->insertId();

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'conversation', $status);

		return $id;
	}

	public function getConversations()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("c.*, u.first_name as owner_first_name, u.last_name as owner_last_name")
			->from("#__joomcrm_conversations AS c")
			->leftJoin("#__joomcrm_users as u on u.id = c.author")
			->where("c.deal_id=" . $this->deal_id)
			->where("c.published>0")
			->order("c.modified DESC");

		return $db->setQuery($query)->loadAssocList();
	}

	/*
	 * Method to access conversations
	 *
	 * @return array
	 */
	public function getConversation($id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("c.*, u.first_name as owner_first_name, u.last_name as owner_last_name")
			->from("#__joomcrm_conversations as c")
			->where("c.id=" . (int) $id)
			->where("c.published=" . $this->published)
			->leftJoin("#__joomcrm_users AS u ON u.id = c.author");

		$results = $db->setQuery($query)->loadAssocList();

		// Clean results
		if (count($results) > 0)
		{
			foreach ($results as $key => $convo)
			{
				$results[$key]['created_formatted'] = JoomcrmHelperDate::formatDate($convo['created']);
			}
		}

		return $results;
	}

}
