<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Component\ComponentHelper;

class JoomcrmModelDeal extends JoomcrmModel
{
	public $_total;
	public $_pagination;
	public $_typ;
	public $_data;
	public $_id;
	public $_type;
	public $_user;
	public $_stage;
	public $_close;
	public $_team;
	public $_status;
	public $_source;
	public $_modified;
	public $_created;
	public $_session;
	public $_user_id;
	public $recent;
	public $person_id;
	public $company_id;
	public $ordering;
	public $archived;
	public $limitOverride;
	public $completed;
	public $user_override;
	public $start_date;
	public $end_date;
	public $limit = 1;
	public $published = 1;

	/**
	 * Method to store a record
	 *
	 * @return    boolean    True on success
	 */
	public function store($data = null, $returnRow = false)
	{
		// Load Tables
		$row    = new JoomcrmTableDeal;
		$oldRow = new JoomcrmTableDeal;

		if ($data == null)
		{
			$data = \Joomla\CMS\Factory::getApplication()->input->post->getArray();
		}

		// Detect if we are importing a company
		$importing = array_key_exists('import', $data);

		if (array_key_exists('name', $data))
		{
			$data['name'] = trim($data['name']);
		}

		// Date generation
		$date = JoomcrmHelperDate::formatDBDate();

		// Assign the creation date
		if (!array_key_exists('id', $data) || !$data['id'])
		{
			if (empty($data['created']))
			{
				$data['created'] = $date;
			}

			$status          	= "created";
			// Assign the owner id
			$data['owner_id'] 	= array_key_exists('owner_id', $data) ? $data['owner_id'] : JoomcrmHelperUsers::getUserId();
		}
		else
		{
			$row->load($data['id']);
			$oldRow->load($data['id']);
			$status = "updated";
		}

		if (array_key_exists('amount', $data))
		{
			$data['amount'] = str_replace(',', '', $data['amount']);
		}

		// Update our modified date
		if (empty($data['modified']))
		{
			$data['modified'] = $date;
		}

		// Generate custom field string
		$customArray = array();
        foreach ($data as $name => $value)
        {
            if (strstr($name, 'custom_') && !strstr($name, '_input') && !strstr($name, "_hidden"))
            {
                $id            = str_replace('custom_', '', $name);
                $customArray[] = array('custom_field_id' => $id, 'custom_field_value' => $value);
                unset($data[$name]);
            }
        }

		if ((array_key_exists('company_name', $data) && strlen(preg_replace('/\s+/', '', $data['company_name'])) > 0 && $data['company_name'] != "") || (array_key_exists('company', $data) && $data['company'] != ""))
		{
			$company_name = array_key_exists('company_name', $data) ? $data['company_name'] : $data['company'];

			$companyModel    = new JoomcrmModelCompany;
			$existingCompany = $companyModel->checkCompanyName($company_name);

			if ($existingCompany == "")
			{
				$cdata              = array();
				$cdata['name']      = $company_name;
				$data['company_id'] = $companyModel->store($cdata)->id;
			}
			else
			{
				$data['company_id'] = $existingCompany;
			}
		}

		if (array_key_exists('company_id', $data) && is_array($data['company_id']))
		{
			$company_name    = $data['company_id']['value'];
			$companyModel    = new JoomcrmModelCompany;
			$existingCompany = $companyModel->checkCompanyName($company_name);
			if ($existingCompany == "")
			{
				$cdata              = array();
				$cdata['name']      = $company_name;
				$data['company_id'] = $companyModel->store($cdata)->id;
			}
			else
			{
				$data['company_id'] = $existingCompany;
			}
		}

		// Deal was closed
		$closedStages = $this->getClosedStages();

		if (array_key_exists('stage_id', $data)
			&& in_array($data['stage_id'], $closedStages)
			&& !array_key_exists('actual_close', $data))
		{
			$data['actual_close'] = $date;
		}
		elseif (array_key_exists('actual_close', $data)
			&& $data['actual_close'] != ""
			&& $data['actual_close'] != "0000-00-00 00:00:00"
			&& !is_null($data['actual_close']))
		{
			$data['actual_close'] = JoomcrmHelperDate::formatDBDate($data['actual_close']);
		}

		/** check for and automatically associate and create primary contacts or people **/
		if (array_key_exists('person_name', $data)
			&& strlen(preg_replace('/\s+/', '', $data['person_name'])) > 0
			&& $data['person_name'] != "" && $data['person_name'] != " "
			&& !is_null($data['person_name']))
		{
			$peopleModel    = new JoomcrmModelPeople;
			$existingPerson = $peopleModel->checkPersonName($data['person_name']);

			if ($existingPerson == "")
			{
				$pdata               = array();
				$name                = explode(" ", $data['person_name']);
				$pdata['first_name'] = $name[0];
				$pdata['last_name']  = array_key_exists(1, $name) ? $name[1] : "";
				if (array_key_exists('company_id', $data))
				{
					$pdata['company_id'] = $data['company_id'];
				}
				$data['person_id'] = $peopleModel->store($pdata);
			}
			else
			{
				$data['person_id'] = $existingPerson;
			}

		}

		if (array_key_exists('primary_contact_name', $data)
			&& strlen(preg_replace('/\s+/', '', $data['primary_contact_name'])) > 0
			&& $data['primary_contact_name'] != ""
			&& $data['primary_contact_name'] != " "
			&& !is_null($data['primary_contact_name']))
		{
			$peopleModel    = new JoomcrmModelPeople;
			$existingPerson = $peopleModel->checkPersonName($data['primary_contact_name']);

			if ($existingPerson == "")
			{
				$pdata               = array();
				$name                = explode(" ", $data['primary_contact_name']);
				$pdata['first_name'] = $name[0];
				$pdata['last_name']  = array_key_exists(1, $name) ? $name[1] : "";
				if (array_key_exists('company_id', $data))
				{
					$pdata['company_id'] = $data['company_id'];
				}
				$person = $peopleModel->store($pdata);
				$data['primary_contact_id'] = $person->id;
			}
			else
			{
				$data['primary_contact_id'] = $existingPerson;
			}

		}

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		$row->custom_fields = $row->id > 0 ? JoomcrmHelperCustom::getCustomCSVData($row->id, "deal") : array();
		$dispatcher         = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onBeforeDealSave', array(&$row));

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

            return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		$deal_id = (!empty($data['id'])) ? $data['id'] : $this->getDbo()->insertId();

		// update note for import
		if($importing)
			JoomcrmHelperNote::updateNoteForImport($data['note'],'deal',$deal_id,$row->owner_id);

		// activity log
		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'deal', $status);

		// If we receive no custom post data do not modify the custom fields
		if (count($customArray) > 0)
		{
			JoomcrmHelperJoomcrm::storeCustomCf($deal_id, $customArray, 'deal');
		}

		if ((array_key_exists('primary_contact_id', $data) && $data['primary_contact_id'] > 0) || (array_key_exists('person_id', $data) && $data['person_id'] > 0))
		{
			$contactId = array_key_exists('primary_contact_id', $data) ? $data['primary_contact_id'] : $data['person_id'];
			$this->storeContact($deal_id, $contactId);
		}

		$closed_stages = JoomcrmHelperDeal::getClosedStages();

		$row->closed                   = in_array($row->stage_id, $closed_stages) ? true : false;
		$row->actual_close_formatted   = isset($row->actual_close) ? JoomcrmHelperDate::formatDate($row->actual_close) : JoomcrmHelperDate::formatDate(date("Y-m-d"));
		$row->expected_close_formatted = isset($row->expected_close) ? JoomcrmHelperDate::formatDate($row->expected_close) : JoomcrmHelperDate::formatDate(date("Y-m-d"));

		$row->custom_fields = $row->id > 0 ? JoomcrmHelperCustom::getCustomCSVData($row->id, "deal") : array();
		$dispatcher         = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onAfterDealSave', array(&$row));

		// Assign any default template systems ( events )
		if ($status == "created")
		{
			$templateModel = new JoomcrmModelTemplate;
			$templateModel->processDefaults($deal_id, "deal");
		}

		// Return success
		if ($returnRow)
		{

			return $row;
		}
		else
		{
			return $deal_id;
		}
	}

	public function _buildQuery()
	{
		/** Large SQL Selections **/
		$db    = $this->getDbo();
		$query = $db->getQuery(true);
		$isDatatableQuery = $this->app->input->get('datatableQuery',0,'int');

		$db->setQuery("SET SQL_BIG_SELECTS=1")->execute();

		// Set defaults
		$id       = $this->_id;
		$type     = $this->_type;
		$user     = $this->_user;
		$stage    = $this->_stage;
		$close    = $this->_close;
		$team     = $this->_team;
		$status   = $this->_status;
		$source   = $this->_source;
		$modified = $this->_modified;
		$created  = $this->_created;
		$session  = $this->_session;
		$user_id  = $this->_user_id;
		$session  = Factory::getSession();
		$name     = $this->app->input->getString('deal_name', null);

		if ($name !== null)
		{
			$session->set('deal_name_filter', $name);
		}
		else {
			$name = $session->get('deal_name_filter');
		}

		// Determine if we are sorting// Searching for a team or user
		if ($team)
		{
			$session->set('deal_user_filter', null);
		}
		if ($user)
		{
			$session->set('deal_team_filter', null);
		}

		/** -----------------------------
		 * Session data for the default deals page
		 */
		// Set user session data
		if ($this->view != "reports")
		{

			if(!$isDatatableQuery){
				if ($type != null)
				{
					$session->set('deal_type_filter', $type);
				}
				else
				{
					$sess_type = $session->get('deal_type_filter');
					$type      = $sess_type;
				}
				if ($user != null)
				{
					$session->set('deal_user_filter', $user);
				}
				else
				{
					$sess_user = $session->get('deal_user_filter');
					$user      = $sess_user;
				}
				if ($stage != null)
				{
					$session->set('deal_stage_filter', $stage);
				}
				else
				{
					$sess_stage = $session->get('deal_stage_filter');
					$stage      = $sess_stage;
				}
				if ($close != null)
				{
					$session->set('deal_close_filter', $close);
				}
				else
				{
					$sess_close = $session->get('deal_close_filter');
					$close      = $sess_close;
				}
				if ($team != null)
				{
					$session->set('deal_team_filter', $team);
				}
				else
				{
					$sess_team = $session->get('deal_team_filter');
					$team      = $sess_team;
				}
			}


		}

		// Construct query string
		$export = $this->app->input->get('export');

		if ($export)
		{
			$queryString = 'DISTINCT(d.id),d.id,d.name,d.summary,d.probability,d.amount,d.actual_close,d.archived,';
			$queryString .= 'd.modified,d.category,d.expected_close,d.created,d.amount AS filtered_total,';
			$queryString .= '( d.amount * ( d.probability / 100 )) AS forecast,';
			$queryString .= 'c.name as company_name,';
			$queryString .= 'stat.name as status_name,';
			$queryString .= 'source.name as source_name,';
			$queryString .= 'stage.name as stage_name,stage.percent, stage.color as stage_color,';
			$queryString .= 'p.first_name as primary_contact_first_name,p.last_name as primary_contact_last_name,d.primary_contact_id,';
			$queryString .= 'CONCAT(user.first_name, " ", user.last_name) as owner_name,d.owner_id,';
			$queryString .= "p.email,p.phone,";
			$queryString .= "event.id AS event_id,event.type AS event_type,event.name AS event_name,IF(event.type='task',event.due_date,event.start_time) AS event_due_date";

			// Select
			$query->clear()->select($queryString)->from('#__joomcrm_deals as d')
			->leftJoin('#__joomcrm_companies AS c ON c.id = d.company_id AND c.published>0')
			->leftJoin('#__joomcrm_deal_status AS stat ON stat.id = d.status_id')
			->leftJoin('#__joomcrm_sources AS source ON source.id = d.source_id')
			->leftJoin('#__joomcrm_stages AS stage on stage.id = d.stage_id')
			->leftJoin("#__joomcrm_people AS p ON p.id = d.primary_contact_id AND p.published>0")
			->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=d.id AND shared.item_type='deal'")
			->leftJoin('#__joomcrm_users AS user ON user.id = d.owner_id');
		}
		else
		{
			$queryString = 'd.*,';

			if (!$id)
			{
				$queryString .= "SUM(d.amount) AS filtered_total,";
			}

			$queryString .= '( d.amount * ( d.probability / 100 )) AS forecast,';
			$queryString .= 'c.name as company_name,';
			$queryString .= 'stat.name as status_name,IF(stat.id > 0,stat.color,"ffffff") AS status_color,';
			$queryString .= 'source.name as source_name,';
			$queryString .= 'stage.name as stage_name,stage.percent,stage.color as stage_color,';
			$queryString .= 'user.first_name as owner_first_name, user.last_name as owner_last_name,';
			$queryString .= 'p.first_name as primary_contact_first_name,p.last_name as primary_contact_last_name,';
			$queryString .= "p.email,p.phone,";
			$queryString .= "event.id AS event_id,event.type AS event_type,event.name AS event_name,IF(event.type='task',event.due_date,event.start_time) AS event_due_date";

			$query->clear()->select($queryString)->from('#__joomcrm_deals as d')
			->leftJoin('#__joomcrm_companies AS c ON c.id = d.company_id AND c.published>0')
			->leftJoin('#__joomcrm_deal_status AS stat ON stat.id = d.status_id')
			->leftJoin('#__joomcrm_sources AS source ON source.id = d.source_id')
			->leftJoin('#__joomcrm_stages AS stage on stage.id = d.stage_id')
			->leftJoin("#__joomcrm_events_cf AS event_cf ON event_cf.association_id = d.id AND event_cf.association_type ='deal' ")
			->leftJoin('#__joomcrm_users AS user ON user.id = d.owner_id')
			->leftJoin("#__joomcrm_people AS p ON p.id = d.primary_contact_id AND p.published>0")
			->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=d.id AND shared.item_type='deal'");
		}

		if (!$id)
		{
			/** --------------------
			 * Sort data for reports pages
			 */
			if ($this->view == 'reports')
			{
				// Name
				if ($name)
				{
					$query->where("d.name LIKE '%" . $db->escape($name) . "%'");
				}
				// Owner
				$owner_filter = $this->getState('Deal.' . $this->layout . '_owner_id');
				if ($owner_filter != null AND $owner_filter != 'all')
				{
					$owner_type = $this->getState('Deal.' . $this->layout . '_owner_type');
					if ($owner_type == 'member' OR $owner_type == null)
					{
						$query->where("d.owner_id=" . $owner_filter);

					}
					if ($owner_type == 'team')
					{
						// Get team members
						$team_members = JoomcrmHelperUsers::getTeamUsers($owner_filter);
						// Filter by results having team ids
						$ids = "0,";
						for ($i = 0; $i < count($team_members); $i++)
						{
							$member = $team_members[$i];
							$ids .= $member['id'] . ",";
						}
						$ids = substr($ids, 0, -1);
						$query->where("d.owner_id IN(" . $ids . ")");
					}
				}
				// Amount
				$amount_filter = $this->getState('Deal.' . $this->layout . '_amount');
				if ($amount_filter != null AND $amount_filter != 'all')
				{
					if ($amount_filter == 'small')
					{
						$query->where("d.amount <= " . Text::_('COM_JOOMCRM_SMALL_DEAL_AMOUNT'));
					}
					if ($amount_filter == 'medium')
					{
						$query->where("d.amount > " . Text::_('COM_JOOMCRM_SMALL_DEAL_AMOUNT') . " AND d.amount <= " . Text::_('COM_JOOMCRM_MEDIUM_DEAL_AMOUNT'));
					}
					if ($amount_filter == 'large')
					{
						$query->where("d.amount > " . Text::_('COM_JOOMCRM_LARGE_DEAL_AMOUNT'));
					}
				}
				// Source
				$source_filter = $this->getState('Deal.' . $this->layout . '_source_id');
				if ($source_filter != null AND $source_filter != 'all')
				{
					$source = $source_filter;
				}
				// Stage
				$stage_filter   = $this->getState('Deal.' . $this->layout . '_stage_id');
				$stage_override = $this->app->input->getInt('stage_id');

				if ($stage_override && JoomcrmHelperDeal::checkStage($stage_override))
				{
					$stage = $stage_override;
				}
				else
				{
					if ($stage_filter != null AND $stage_filter != 'all')
					{
						$stage = $stage_filter;
					}
				}
				// Status
				$status_filter = $this->getState('Deal.' . $this->layout . '_status_id');
				if ($status_filter != null AND $status_filter != 'all')
				{
					$status = $status_filter;
				}
				// Expected close
				$expected_close_filter = $this->getState('Deal.' . $this->layout . '_expected_close');
				if ($expected_close_filter != null AND $expected_close_filter != 'all')
				{
					$close = $expected_close_filter;
				}

				// Modified
				$modified_filter = $this->getState('Deal.' . $this->layout . '_modified');
				if ($modified_filter != null AND $modified_filter != 'all')
				{
					$modified = $modified_filter;
				}
				// Created
				$created_filter = $this->getState('Deal.' . $this->layout . '_created');
				if ($created_filter != null AND $created_filter != 'all')
				{
					$created = $created_filter;
				}
			}

			// Get current date to use for all date filtering
			$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00'));

			/** ------------------------------------------
			 * Here we filter for diferent types of deals
			 */
			if ($type != null && $type != 'all')
			{
				// Filter for companies with tasks due today
				if ($type == 'today')
				{
					$today    = JoomcrmHelperDate::formatDBDate(date('Y-m-d'), false) . " 00:00:00";
					$tomorrow = JoomcrmHelperDate::formatDBDate(date('Y-m-d', strtotime($today . " +1 days")), false) . " 00:00:00";
					$query
					->leftJoin("#__joomcrm_events_cf AS ecf ON ecf.association_type='deal' AND ecf.association_id=d.id")
					->leftJoin("#__joomcrm_events AS event ON event.id=ecf.event_id")
					->where("event.due_date > '$today' AND event.due_date < '$tomorrow'")
					->where("event.published>0 AND event.completed=0");
				}

				// Filter for companies and deals// Tasks due tomorrow
				else
				{
					if ($type == "tomorrow")
					{
						$tomorrow         = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() + (1 * 24 * 60 * 60)));
						$dayAfterTomorrow = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00", strtotime($tomorrow . " +1 days")));
						$query
						->leftJoin("#__joomcrm_events_cf AS ecf ON ecf.association_type='deal' AND ecf.association_id=d.id")
						->leftJoin("#__joomcrm_events AS event ON event.id=ecf.event_id")
						->where("event.due_date >'" . $tomorrow . "' AND event.due_date < '$dayAfterTomorrow'")
						->where("event.published>0 AND event.completed=0");
					}
					else
					{
						// Get next event
						$query->leftJoin("#__joomcrm_events AS event ON event.id = (
										SELECT event2.id
										FROM #__joomcrm_events_cf AS ecf2
										LEFT JOIN #__joomcrm_events AS event2 ON
											event2.id=ecf2.event_id AND
											ecf2.association_type='deal'
										WHERE ecf2.association_id=d.id AND
											  ecf2.association_type='deal' AND
												event2.completed=0
										ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
												 CASE WHEN event2.type='event' THEN start_Time END ASC
										LIMIT 1 )");
					}
				}

				// Filter for deals updated in the last 30 days
				if ($type == "updated_thirty")
				{
					$last_thirty_days = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() - (30 * 24 * 60 * 60)));
					$query->where("d.modified >'$last_thirty_days'");
				}

				// Filter for most valuable deals
				if ($type == "valuable")
				{
					$query->order('d.amount DESC');
				}

				// Filter for past deals
				if ($type == "past")
				{
					$query->where("event.due_date < '$date'");
					$query->where("event.published>0");
				}

				// Filter for deals not updated in the last 30 days
				if ($type == "not_updated_thirty")
				{
					$last_thirty_days = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() - (30 * 24 * 60 * 60)));
					$query->where("d.modified < '$last_thirty_days'");
				}

				// Filter for shared deals
				if ($type == "shared")
				{
					$query->where("shared.item_id IS NOT NULL");
				}

				// Filter for archived deals
				if ($type == "archived" && is_null($this->archived))
				{
					$query->where("d.archived=1");
				}

			}
			else
			{
				$query->where("d.archived=0");
				// Get next event
				$query->leftJoin("#__joomcrm_events AS event ON event.id = (
									SELECT event2.id
									FROM #__joomcrm_events_cf AS ecf2
									LEFT JOIN #__joomcrm_events AS event2 ON
										event2.id=ecf2.event_id AND
										ecf2.association_type='deal'
									WHERE ecf2.association_id=d.id AND
										  ecf2.association_type='deal' AND
											event2.completed=0
									ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
											 CASE WHEN event2.type='event' THEN start_Time END ASC
									LIMIT 1 )");
			}

			/** --------------------------------------------
			 * Search for closing deal filters
			 */
			if ($close != null && $close != "any")
			{
				if ($close == "this_week")
				{
					$date_info         = getDate(strtotime($date));
					$today             = $date_info['wday'];
					$days_to_remove    = -1 + $today;
					$days_to_add       = 7 - $today;
					$beginning_of_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - $days_to_remove days")));
					$end_of_week       = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " + $days_to_add days")));
					$query->where("d.expected_close >= '$beginning_of_week'")
					->where("d.expected_close < '$end_of_week'");
				}

				if ($close == "next_week")
				{
					$date_info        = getDate(strtotime($date));
					$today            = $date_info['wday'];
					$days_to_add_this = 7 - $today;
					$days_to_add_next = 14 - $today;
					$end_of_this_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " + $days_to_add_this days")));
					$end_of_next_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " + $days_to_add_next days")));
					$query->where("d.expected_close >= '$end_of_this_week'")
					->where("d.expected_close < '$end_of_next_week'");
				}

				if ($close == "this_month")
				{
					$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$next_month = date('Y-m-1 00:00:00', strtotime($this_month . "+1 month"));
					$query->where("d.expected_close >= '$this_month'")->where("d.expected_close < '$next_month'");
				}

				if ($close == "next_month")
				{
					$this_month      = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$next_month      = date('Y-m-1 00:00:00', strtotime($this_month . "+1 month"));
					$next_next_month = date('Y-m-1 00:00:00', strtotime($next_month . "+1 month"));
					$query->where("d.expected_close >= '$next_month'")->where("d.expected_close < '$next_next_month'");
				}

			}

			/** --------------------------------------------
			 * Search for modified deal filters
			 */
			if ($modified != null && $modified != "any")
			{
				if ($modified == "this_week")
				{
					$date_info         = getDate(strtotime($date));
					$today             = $date_info['wday'];
					$days_to_remove    = -1 + $today;
					$days_to_add       = 7 - $today;
					$beginning_of_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - $days_to_remove days")));
					$end_of_week       = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " + $days_to_add days")));
					$query->where("d.modified >= '$beginning_of_week'")->where("d.modified < '$end_of_week'");
				}

				if ($modified == "last_week")
				{
					$date_info              = getDate(strtotime($date));
					$today                  = $date_info['wday'];
					$days_to_remove         = -1 + $today;
					$beginning_of_this_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - $days_to_remove days")));
					$beginning_of_last_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - 7 days")));
					$query->where("d.modified >= '$beginning_of_last_week'")
					->where("d.modified < '$beginning_of_this_week'");
				}

				if ($modified == "this_month")
				{
					$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$next_month = date('Y-m-1 00:00:00', strtotime($this_month . "+1 month"));
					$query->where("d.modified >= '$this_month'")->where("d.modified < '$next_month'");
				}

				if ($modified == "last_month")
				{
					$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$last_month = date('Y-m-1 00:00:00', strtotime($this_month . " -1 month"));
					$query->where("d.modified >= '$last_month'")->where("d.modified < '$this_month'");
				}

			}

			/** --------------------------------------------
			 * Search for created deal filters
			 */
			if ($created != null && $created != "any")
			{
				if ($created == "this_week")
				{
					$date_info         = getDate(strtotime($date));
					$today             = $date_info['wday'];
					$days_to_remove    = -1 + $today;
					$days_to_add       = 7 - $today;
					$beginning_of_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - $days_to_remove days")));
					$end_of_week       = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " + $days_to_add days")));
					$query->where("d.modified >= '$beginning_of_week'")->where("d.modified < '$end_of_week'");
				}

				if ($created == "last_week")
				{
					$date_info              = getDate(strtotime($date));
					$today                  = $date_info['wday'];
					$days_to_remove         = -1 + $today;
					$beginning_of_this_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - $days_to_remove days")));
					$beginning_of_last_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - 7 days")));
					$query->where("d.created >= '$beginning_of_last_week'")
					->where("d.created < '$beginning_of_this_week'");
				}

				if ($created == "this_month")
				{
					$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$next_month = date('Y-m-1 00:00:00', strtotime($this_month . " +1 month"));
					$query->where("d.created >= '$this_month'")->where("d.created < '$next_month'");
				}

				if ($created == "last_month")
				{
					$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$last_month = date('Y-m-1 00:00:00', strtotime($this_month . " -1 month"));
					$query->where("d.created >= '$last_month'")->where("d.created < '$this_month'");
				}

				if ($created == "today")
				{
					$today    = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00"));
					$tomorrow = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+1 day"));
					$query->where("d.created >= '$today'")->where("d.created < '$tomorrow'");
				}

				if ($created == "yesterday")
				{
					$today     = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00"));
					$yesterday = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "-1 day"));
					$query->where("d.created >= '$yesterday'")->where("d.created < '$today'");
				}

			}

			/** ------------------------------------------
			 * Search for status
			 */
			if ($status != null AND $status != 'all')
			{
				$query->where("d.status_id=" . $status);
			}

			/** -------------------------
			 * Search for sources
			 */
			if ($source != null AND $source != 'all')
			{
				$query->where('d.source_id=' . $source);
			}

			/** ----------------------------------------------------------------
			 * Filter for stage id associations
			 */
			$stage_override = $this->app->input->getInt('stage_id');

			if ($stage_override && JoomcrmHelperDeal::checkStage($stage_override))
			{
				$query->where('d.stage_id = ' . $stage_override);
			}
			else
			{
				if ($stage != null && $stage != 'all' && !$id)
				{
					// If we want active deals we must retrieve the active stage ids to filter by
					if ($stage == 'active')
					{
						// Filter by results having team ids
						$stages = array();

						// Get stage ids
						$stages = JoomcrmHelperDeal::getActiveStages(TRUE);

						if (count($stages) > 0)
						{
							$stages = implode(',', array_keys($stages));
							$query->where("d.stage_id IN(" . $stages . ")");
						}
					}
					else
					{
						// Else filter by the stage id
						$query->where("d.stage_id='" . $stage . "'");
					}
				}
			}

			/** --------------------------------------------------------
			 * Filter data for the sources page
			 */
			// Source view
			if ($this->view == "reports" && $this->layout == "source_report")
			{
				// Filter by active and closed stages
				$active_and_closed_stages = JoomcrmHelperDeal::getNonInactiveStages();
				$query->where("d.stage_id IN(" . implode(',', $active_and_closed_stages) . ")");

				// Filter by deals that are associated to sources
				$query->where("d.source_id <> 0");
			}


			if($this->view == 'deals' && $this->layout == 'list'){ // cuz list is called by ajax, sho should use same deal name state as default layout

				$deal_filter = $this->getState('Deal.' . $this->view . '.default.name');

			}else{
				$deal_filter = $this->getState('Deal.' . $this->view . '.' . $this->layout . '.name');
			}

			if ($deal_filter != null)
			{
				$query->where("d.name LIKE '%" . $db->escape($deal_filter) . "%'");
			}

			/** --------------------
			 * Grab only recently accessed deals
			 */

			if ($this->recent)
			{
				$past = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s') . " - 30 days");
				$query->where('d.last_viewed > ' . $db->Quote($past));
			}

			/**---------------------
			 * Group by deal id
			 */
			$query->group('d.id');

			/**
			 * Set our sorting direction if set via post
			 */
			// Default deals view
			//
			if ($this->ordering)
			{
				$query->order($this->ordering);
			}
			else
			{
				if ($this->view == "reports")
				{
					$query->order($this->getState('Deal.' . $this->layout . '_filter_order') . ' ' . $this->getState('Deal.' . $this->layout . '_filter_order_Dir'));
				}
				else
				{
					$query->order($this->getState('Deal.filter_order') . ' ' . $this->getState('Deal.filter_order_Dir'));
				}

			}

		}
		else
		{
			// Get next event
			$query->leftJoin("#__joomcrm_events AS event ON event.id = (
								SELECT event2.id
								FROM #__joomcrm_events_cf AS ecf2
								LEFT JOIN #__joomcrm_events AS event2 ON
									event2.id=ecf2.event_id AND
									ecf2.association_type='deal'
								WHERE ecf2.association_id=d.id AND
									  ecf2.association_type='deal' AND
										event2.completed=0
								ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
										 CASE WHEN event2.type='event' THEN start_Time END ASC
								LIMIT 1 )");
		}

		/** ---------------------
		 * Filter by id
		 */
		if ($id)
		{
			if (is_array($id))
			{
				$query->where("d.id IN (" . implode(',', $id) . ")");
			}
			else
			{
				$query->where("d.id=$id");
			}
		}
		/** or team **/
		if ($team)
		{
			$query->where("user.team_id=$team");
		}

		/** ---------------------------------------------------------------
		 * Filter data using member role permissions
		 */
		$member_id   = JoomcrmHelperUsers::getUserId();
		$team_access = JoomcrmHelperUsers::getTeamAccess($member_id);
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();



		$user       = $this->user_override ? $this->user_override : $user;
		$company_id = $this->app->input->getInt('company_id', $this->company_id);
		$person_id  = $this->app->input->getInt('person_id', $this->person_id);

		if ($id > 0)
		{
			// this fix an issue if current user role is executive and team access enabled, should be always disabled.
			if($member_role == 'exec') $team_access = 0;

			if ($member_role == "manager" || (isset($team_id) && (int) $team_id > 0 && $team_access) )
			{

				$query->where('( d.owner_id = ' . $member_id . ' OR user.team_id = ' . $team_id . " OR shared.user_id=" . $member_id . " )");
			}
			else
			{
				if ($member_role == "basic")
				{
					$query->where(array('(d.owner_id = ' . $member_id . " OR shared.user_id=" . $member_id . " )"));
				}
			}
		}
		else
		{
			if ((isset($company_id) && $company_id > 0) || (isset($person_id) && $person_id > 0) || (isset($user) && $user == "all") || (isset($owner_filter) && $owner_filter == "all"))
			{
				if ($member_role != 'exec')
				{
					// Manager filter
					if ($member_role == 'manager' || (isset($team_id) && $team_access) )
					{
						$query->where('( d.owner_id = ' . $member_id . ' OR user.team_id = ' . $team_id . " OR shared.user_id=" . $member_id . " )");
					}
					else
					{
						// Basic user filter
						$query->where(array('(d.owner_id = ' . $member_id . " OR shared.user_id=" . $member_id . " )"));
					}
				}
			}
			else
			{
				if ($team)
				{
					$query->where("user.team_id=$team");
				}
				else
				{
					if ($user && $user != "all")
					{
						$query->where("(d.owner_id=" . $user . " OR shared.user_id=" . $user . ")");
					}
					else
					{
						if (!(isset($owner_filter)))
						{
							$query->where("( d.owner_id=" . JoomcrmHelperUsers::getLoggedInUser()->id . " OR shared.user_id=" . JoomcrmHelperUsers::getLoggedInUser()->id . " )");
						}
					}
				}
			}
		}

		/** company **/
		if ($this->company_id)
		{
			$query->where("d.company_id=" . $this->company_id);
		}
		/** people **/
		if ($this->person_id)
		{
			$query->leftJoin("#__joomcrm_people_cf AS dpcf ON dpcf.association_id = d.id AND dpcf.association_type='deal'");
			$query->where("( dpcf.person_id=" . $this->person_id . " OR d.primary_contact_id = " . $this->person_id . " )");
		}
		/** archived **/
		if (!is_null($this->archived))
		{
			$query->where("d.archived=" . $this->archived);
		}
		/** published **/
		$query->where("d.published=" . $this->published);

		return $query;
	}

	/**
	 * Method to access deals
	 *
	 * @param $id    to search for
	 * @param $type  to filter by
	 * @param $user  to filter by
	 * @param $stage to filter by
	 * @param $close date to filter by
	 * @param $team  to filter by
	 *
	 * @return $results
	 */
	public function getDeals($id = null, $type = null, $user = null, $stage = null, $close = null, $team = null)
	{
		// Set defaults
		$this->_id       = ($id) ? $id : $this->_id;
		$this->_type     = $type;
		$this->_user     = $user;
		$this->_stage    = $stage;
		$this->_close    = $close;
		$this->_team     = $team;
		$this->_status   = null;
		$this->_source   = null;
		$this->_modified = null;
		$this->_created  = null;

		// Get session data
		$this->_session = Factory::getSession();
		$this->_user_id = JoomcrmHelperUsers::getUserId();

		// Build our db query
		$db    = $this->getDbo();
		$query = $this->_buildQuery();

		/** ------------------------------------------
		 * Set query limits and load results
		 */
		$limit      = $this->getState('Deal.' . $this->view . '.' . $this->layout . '.limit');
		$limitStart = $this->getState('Deal.' . $this->view . '.' . $this->layout . '.limitstart');

		if (!$this->_id && $limit != 0 && $this->limit == 1)
		{
			if ($limitStart >= $this->getTotal())
			{
				$limitStart = 0;
				$limit      = ComponentHelper::getParams('com_joomcrm')->get('deals_limit',10);
				$limitStart = ($limit != 0) ? (floor($limitStart / $limit) * $limit) : 0;
				$this->setState('Deal.' . $this->view . '.' . $this->layout . '.limit', $limit);
				$this->setState('Deal.' . $this->view . '.' . $this->layout . '.limitstart', $limitStart);
			}
			$query .= " LIMIT " . ($limit) . " OFFSET " . ($limitStart);
		}

		if ($this->_id && !is_array($this->_id))
		{
			$query .= " LIMIT 1";
		}

		if ($this->limitOverride != null)
		{
			$query .= " Limit " . $this->limit;
		}
		$db->setQuery($query);
		$deals = $db->loadAssocList();

		/**------------------------------------------
		 * Generate queries to join essential data
		 */

		if (count($deals))
		{
			$closed_stages = JoomcrmHelperDeal::getClosedStages();

			$export = $this->app->input->get('export');

			if (!$export)
			{
				$eventModel = new JoomcrmModelEvent;

				/** ------------------------------------------
				 *  Get data
				 */
				foreach ($deals as $key => $deal)
				{
					$deals[$key]['closed']      = in_array($deal['stage_id'], $closed_stages) ? true : false;
					$deals[$key]['status_name'] = $deal['status_id'] > 0 ? $deals[$key]['status_name'] : Text::_('COM_JOOMCRM_NONE_STATUS');

					if ($this->_id)
					{
						/** ------------------------------------------
						 *  Join contacts
						 */

						$contacts_model = new JoomcrmModelContacts;
						$contacts_model->set('deal_id', $deal['id']);
						$people = $contacts_model->getContacts();
						// Assign results to company
						$deals[$key]['people'] = $people;

						/** ------------------------------------------
						 *  Join conversations
						 */

						$convoModel = new JoomcrmModelConversation;
						$convoModel->set('deal_id', $deal['id']);
						$conversations                = $convoModel->getConversations();
						$deals[$key]['conversations'] = $conversations;

						/** ------------------------------------------
						 *  Join notes
						 */

						$notesModel           = new JoomcrmModelNote;
						$notesModel->public_notes = true;
						$notesModel->filters = false;
						$deals[$key]['notes'] = $notesModel->getNotes($deal['id'], 'deal');

						/** ------------------------------------------
						 *  Join documents
						 */
						$docModel = new JoomcrmModelDocument;
						$docModel->set('deal_id', $deal['id']);
						$docModel->filters = false;
						$docModel->view2 = 'documents';
						$docs                     = $docModel->getDocuments();
						$deals[$key]['documents'] = $docs;

						/** ------------------------------------------
						 *  Join tasks & events
						 */
						$eventModel->set('deal_id', $deal['id']);
						if ($this->completed != null)
						{
							$eventModel->set('completed', $this->completed);
						}
						$deals[$key]['events'] = $eventModel->getEvents();

						/** ------------------------------------------
						 * Update last access for each deal
						 */
						$now   = JoomcrmHelperDate::formatDBDate(date("Y-m-d H:i:s"));
						$query = $db->getQuery(true)->set("last_viewed=" . $db->quote($now))->update("#__joomcrm_deals")
								 ->where("id=" . (int) $deal['id']);

						$db->setQuery($query)->execute();
					}
				}

			}
		}

		/** ------------------------------------------
		 *  Return results
		 */

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onDealLoad', array(&$deals));

		return $deals;
	}

	/**
	*	Prepares Items data for the jQuery dataTable
	*
	*	@param 	array of items from DB
	*	@return array of modified items
	*/
	public function dataTablefy($items)
	{
		$dataTable = array();
		$user = Factory::getApplication()->getIdentity();
		$config     = ComponentHelper::getParams('com_joomcrm');

		if (is_array($items))
		{
			$count = count($items);

			for ($i = 0; $i < $count; $i++)
			{
				$dealLink = '<a href="' . Route::_('index.php?option=com_joomcrm&view=deals&layout=deal&id=' . $items[$i]['id']) . '">';
				$dealLink .= $items[$i]['name'];
				$dealLink .= '</a>';

				$dataTable[$i] = array();
				$dataTable[$i][] = $dealLink;
				$dataTable[$i][] = $items[$i]['owner_first_name'] . ' ' . $items[$i]['owner_last_name'];
				$dataTable[$i][] = '<div class="status-dot" style="background-color:#' . $items[$i]['status_color'] . ' !important;"></div>' . $items[$i]['status_name'];
				$dataTable[$i][] = '<span class="amount">' . JoomcrmHelperUsers::formatAmount($items[$i]['amount']) . '</span>';
			}
		}

		return $dataTable;
	}

	/**
	*	Match column name from jQuery dataTable with actual
	* 	name of the column from database to make ordering.
	*
	*	@param 	integer $columnNumber
	*	@return string
	*/
	public function getColumnName($columnNumber)
	{
		$columnName = '';

		if ($columnNumber == 0) $columnName = 'd.name';
		if ($columnNumber == 1) $columnName = 'user.first_name';
		if ($columnNumber == 2) $columnName = 'stat.name';
		if ($columnNumber == 3) $columnName = 'd.amount';

		return $columnName;
	}

	/**
	 * Get deals that are active for the reports page
	 *
	 * @param none
	 *
	 * @return mixed $results
	 */
	public function getReportDeals($ids = null)
	{
		// Get filter
		$session = Factory::getSession();
		$filter  = $session->get('deal_stage_filter');
		// Get deals
		$deals = $this->getDeals($ids, null, null, 'active');
		// Reset filter
		$session->set('deal_stage_filter', $filter);

		// Return deals
		return $deals;
	}

	/**
	 * Method to get list of deals
	 */
	public function getDealList()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->select("DISTINCT(d.id),d.name,d.id")->from("#__joomcrm_deals AS d")
				 ->leftJoin("#__joomcrm_people_cf AS pcf ON pcf.association_id = d.id AND pcf.association_type='deal'")
				 ->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=d.id AND shared.item_type='deal'")
				 ->leftJoin('#__joomcrm_users AS user ON user.id = d.owner_id');

		/** ---------------------------------------------------------------
		 * Filter data using member role permissions
		 */
		$member_id   = JoomcrmHelperUsers::getUserId();
		$team_access = JoomcrmHelperUsers::getTeamAccess($member_id);
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();

		if ($member_role != 'exec')
		{
			// Manager filter
			if ($member_role == 'manager' || (isset($team_id) && $team_access) )
			{
				$query->where('( d.owner_id = ' . $member_id . ' OR user.team_id = ' . $team_id . " OR shared.user_id=" . $member_id . " )");
			}
			else
			{
				// Basic user filter
				$query->where(array('(d.owner_id = ' . $member_id . " OR shared.user_id=" . $member_id . " )"));
			}
		}

		$query->where("d.published=" . $this->published);

		$associationType = $this->app->input->get('association');
		$associationId   = $this->app->input->getInt('association_id');

		if ($associationType == "company")
		{
			$query->where('d.company_id = ' . $associationId);
		}

		if ($associationType == "person")
		{
			$query->where('pcf.person_id = ' . $associationId);
		}

		$row = $db->setQuery($query)->loadAssocList();

		if (count($row) == 0)
		{
			$row = array();
		}

		$blank = array(array('name' => Text::_('COM_JOOMCRM_NONE'), 'id' => 0));

		return array_merge($blank, $row);
	}

	/**
	 * Method to get graph deal information
	 *
	 * @param $type        type of deal to filter by, values 'stage','status'
	 * @param $access_type to search by 'company','team','member'
	 * @param $access_id   the id of the $member_type to search by
	 */
	public function getGraphDeals($type = null, $access_type = null, $access_id = null)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Search by type
		if ($type == 'stage')
		{
			$query->select("d.stage_id,count(*) AS y, stage.name AS name,stage.id,SUM(d.amount) AS amount")
			->from("#__joomcrm_deals AS d")->leftJoin("#__joomcrm_stages AS stage ON stage.id=d.stage_id");
		}
		if ($type == 'status')
		{
			$query->select("d.status_id,count(*) AS y,status.name AS name,status.id,SUM(d.amount) AS amount")
			->from("#__joomcrm_deals AS d")->leftJoin("#__joomcrm_deal_status AS status ON status.id=d.status_id");
		}

		// If user is not an executive then there are limitations
		if ($access_type != 'company')
		{
			// Team sorting
			if ($access_type == 'team')
			{
				// Get team members
				$team_members = JoomcrmHelperUsers::getTeamUsers($access_id);
				$members      = array();
				$members[]    = 0;
				foreach ($team_members as $key => $member)
				{
					$members[] = $member['id'];
				}
				$query->where("d.owner_id IN (" . implode(",", $members) . ")");
			}

			// Member sorting
			if ($access_type == 'member')
			{
				$query->where("d.owner_id=$access_id");
			}

		}

		// Grouping
		if ($type == 'stage')
		{
			$query->where("d.stage_id<>0 AND d.stage_id=stage.id")->group("d.stage_id");
		}
		if ($type == 'status')
		{
			$query->where("d.status_id<>0 AND d.status_id=status.id")->group("d.status_id");
		}

		if (!is_null($this->archived))
		{
			$query->where("d.archived=" . $this->archived);
		}

		// Filter by start and end dates ( expected_close )
		$query->where("( d.created >='" . $this->start_date . "' AND d.created <= '" . $this->end_date . "' )");

		// Only view published deals
		$query->where("d.published=" . $this->published)->order("ordering ASC");

		$results = $db->setQuery($query)->loadAssocList();

		$total = 0;

		if (count($results) > 0)
		{
			foreach ($results as $key => $stage)
			{
				$total += $stage['amount'];
			}
		}

		// Clean results and force datatypes for graph rendering
		if (count($results) > 0)
		{
			foreach ($results as $key => $stage)
			{
				$results[$key]['id']         = (int) $stage['id'];
				$results[$key]['y']          = (int) $stage['y'];
				$results[$key]['data']       = array((int) $stage['y']);
				$results[$key]['deal_total'] = $total;
			}
		}

		return $results;
	}

	/**
	 * Get lead sources from database where we have associated won deals and grab amount from deals
	 *
	 * @param $access_type to filter by 'member','team','company'
	 * @param $access_id   id of $access_type to filter by
	 *
	 * @return mixed $results
	 */
	public function getLeadSources($access_type = null, $access_id = null)
	{
		// Get won stage id so we know what stage to filter by for the deals
		$won_stage_ids = JoomcrmHelperDeal::getWonStages();

		// Get database
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("s.name,SUM(d.amount) as y")
			->from("#__joomcrm_people AS p")
			->leftJoin("#__joomcrm_sources AS s ON s.id = p.source_id")
			->leftJoin("#__joomcrm_people_cf AS dpcf ON dpcf.person_id = p.id AND dpcf.association_type='deal'")
			->leftJoin("#__joomcrm_deals AS d ON d.id = dpcf.association_id AND d.published>0")
			->where("p.source_id <> 0")
			->where("p.type='Lead'")
			->where("d.stage_id IN(" . implode(',', $won_stage_ids) . ")")
			->where("p.published=" . $this->published)
			->where("( p.created >='" . $this->start_date . "' AND p.created <= '" . $this->end_date . "' )");

		// Filter by access type
		if ($access_type != 'company')
		{
			// Team sorting
			if ($access_type == 'team')
			{
				// Get team members
				$team_members = JoomcrmHelperUsers::getTeamUsers($access_id);
				$query .= " AND d.owner_id IN (";
				// Loop to make string
				$query .= "0,";

				foreach ($team_members as $key => $member)
				{
					$query .= "'" . $member['id'] . "',";
				}

				$query = substr($query, 0, -1);
				$query .= ") ";
			}

			// Member sorting
			if ($access_type == 'member')
			{
				$query .= " AND d.owner_id=$access_id ";
			}

		}

		// Group by source ids
		$query .= " GROUP BY s.id";

		$results = $db->setQuery($query)->loadAssocList();

		if (count($results) > 0)
		{
			foreach ($results as $key => $source)
			{
				$results[$key]['y']    = (int) $source['y'];
				$results[$key]['data'] = array((int) $source['y']);
			}
		}

		return $results;
	}

	/**
	 * Get total number of rows for pagination
	 */
	public function getTotal()
	{
		if (empty ($this->_total))
		{
			$query        = $this->_buildQuery();

			$this->_total = $this->_getListCount($query);

		}

		return $this->_total;
	}

	/**
	 * Generate pagination
	 */
	public function getPagination()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			$limit      = $this->getState('Deal.' . $this->view . '.' . $this->layout . '.limit');
			$limitstart = $this->getState('Deal.' . $this->view . '.' . $this->layout . '.limitstart');
			$layout = $this->_layout;

			if ($layout == 'list')
			{
				$layout = 'default';
			}
			
			$link 		= Route::_('index.php?option=com_joomcrm&view=' . $this->_view . '&layout=' . $layout);
			$this->_pagination = new JoomcrmPagination($this->getTotal(), $limitstart, $limit);
		}

		return $this->_pagination;
	}

	/**
	 * Populate user state requests
	 */
	public function populateState()
	{
		$limit_config = ComponentHelper::getParams('com_joomcrm')->get('deals_limit',10);
		$limit      = $this->app->getUserStateFromRequest('Deal.' . $this->view . '.' . $this->layout . '.limit', 'limit',$limit_config);
		$limitstart = $this->app->getUserStateFromRequest('Deal.' . $this->view . '.' . $this->layout . '.limitstart', 'limitstart', 0);

		// In case limit has been changed, adjust it
		$limitstart = ($limit != 0) ? (floor($limitstart / $limit) * $limit) : 0;

		$input = \Joomla\CMS\Factory::getApplication()->input;


		// if ajax request (when using filters, pagination limit and start should work good)
		if($input->get('format','','word') == 'raw'){
			$limit = $input->get('ajaxLimit',$limit,'int');
			//$limitstart = 0; // should begin from start
		}


		$this->setState('Deal.' . $this->view . '.' . $this->layout . '.limit', $limit);
		$this->setState('Deal.' . $this->view . '.' . $this->layout . '.limitstart', $limitstart);

		/** --------------------------------------
		 * Filter data for different views
		 */

		switch ($this->view)
		{
			case "reports" :
				// Set default filter states for reports
				$filter_order          = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_filter_order', 'filter_order', 'd.name');
				$filter_order_Dir      = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_filter_order_Dir', 'filter_order_Dir', 'asc');
				$deal_filter           = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_name', 'deal_name', null);
				$owner_filter          = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_owner_id', 'owner_id', 'all');
				$owner_type_filter     = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_owner_type', 'owner_type', null);
				$amount_filter         = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_amount', 'deal_amount', null);
				$source_filter         = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_source_id', 'source_id', null);
				$stage_filter          = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_stage_id', 'stage_id', 'active');
				$status_filter         = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_status_id', 'status_id', null);
				$expected_close_filter = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_expected_close', 'expected_close', null);
				$modified_filter       = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_modified', 'modified', null);
				$created_filter        = $this->app->getUserStateFromRequest('Deal.' . $this->layout . '_created', 'created', null);

				// Set states for reports
				$this->setState('Deal.' . $this->layout . '_filter_order', $filter_order);
				$this->setState('Deal.' . $this->layout . '_filter_order_Dir', $filter_order_Dir);
				$this->setState('Deal.' . $this->layout . '_name', $deal_filter);
				$this->setState('Deal.' . $this->layout . '_owner_id', $owner_filter);
				$this->setState('Deal.' . $this->layout . '_owner_type', $owner_type_filter);
				$this->setState('Deal.' . $this->layout . '_amount', $amount_filter);
				$this->setState('Deal.' . $this->layout . '_source_id', $source_filter);
				$this->setState('Deal.' . $this->layout . '_stage_id', $stage_filter);
				$this->setState('Deal.' . $this->layout . '_status_id', $status_filter);
				$this->setState('Deal.' . $this->layout . '_expected_close', $expected_close_filter);
				$this->setState('Deal.' . $this->layout . '_modified', $modified_filter);
				$this->setState('Deal.' . $this->layout . '_created', $created_filter);

				break;
				break;

			case "deals" :
				// Set defaults
				$filter_order     = $this->app->getUserStateFromRequest('Deal.filter_order', 'filter_order', 'd.name');
				$filter_order_Dir = $this->app->getUserStateFromRequest('Deal.filter_order_Dir', 'filter_order_Dir', 'asc');
				$deal_filter      = $this->app->getUserStateFromRequest('Deal.' . $this->view . '.default.name', 'deal_name', null);



				// Set states
				$this->setState('Deal.filter_order', $filter_order);
				$this->setState('Deal.filter_order_Dir', $filter_order_Dir);
				$this->setState('Deal.' . $this->view . '.default.name', $deal_filter);
				break;
			default:
				// Set defaults
				$filter_order     = $this->app->getUserStateFromRequest('Deal.' . $this->view . '.' . $this->layout . '.filter_order', 'filter_order', 'd.name');
				$filter_order_Dir = $this->app->getUserStateFromRequest('Deal.' . $this->view . '.' . $this->layout . '.filter_order_Dir', 'filter_order_Dir', 'asc');
				$deal_filter      = $this->app->getUserStateFromRequest('Deal.' . $this->view . '.' . $this->layout . '.name', 'deal_name', null);

				// get column name form column number sent by jQuery dataTable
				if (is_numeric($filter_order))
				{
					$filter_order = $this->getColumnName($filter_order);
				}

				// Set states
				$this->setState('Deal.filter_order', $filter_order);
				$this->setState('Deal.filter_order_Dir', $filter_order_Dir);
				$this->setState('Deal.' . $this->view . '.' . $this->layout . '.name', $deal_filter);
		}

	}

	/**
	 * Store contacts to CF tables
	 *
	 * @param  [type] $deal_id    [description]
	 * @param  [type] $contact_id [description]
	 *
	 * @return [type]             [description]
	 */
	public function storeContact($deal_id, $contact_id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("COUNT(*)")
			->from("#__joomcrm_people_cf")
			->where("association_id=" . $deal_id)
			->where("association_type='deal'")
			->where("person_id=" . $contact_id);

		$contacts = $db->setQuery($query)->loadResult();

		if ($contacts == 0)
		{
			$created = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

			$data = array($deal_id . ",'deal'," . $contact_id . ",'" . $created . "'");

			$query->clear()->insert('#__joomcrm_people_cf')
			->columns('association_id, association_type, person_id, created')->values($data);

			$db->setQuery($query)->execute();
		}

	}

	/**
	 * Checks for existing company by name
	 *
	 * @param  [var] $name company name to check
	 *
	 * @return [int]       ID of existing company
	 */
	function checkDealName($name)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('d.id')
			->from('#__joomcrm_deals AS d')
			->where('LOWER(d.name) = ' . $db->quote($db->escape(mb_strtolower($name))));

		return $db->setQuery($query)->loadResult();
	}

	public function getClosedStages()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('s.id')
			->from('#__joomcrm_stages AS s')
			->where("s.percent=100");

		return $db->setQuery($query)->loadColumn();
	}

	public function getDealNames($json = false)
	{
		$names  = $this->getDealList();
		$return = array();
		if (count($names) > 0)
		{
			foreach ($names as $key => $deal)
			{
				$return[] = array('label' => $deal['name'], 'value' => $deal['id']);
			}
		}

		return $json ? json_encode($return) : $return;

	}
}
