<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Categories\Categories;
jimport('joomla.filesystem.file');

class JoomcrmModelDocument extends JoomcrmModel
{
	public $company_id;
	public $deal_id;
	public $person_id;
	public $published = 1;
	public $filters = true;
	public $_category;

	/**
	 * Method to store a record
	 *
	 * @return    boolean    True on success
	 */
	public function store($data = null)
	{
		// Import joomlas filesystem functions, we will do all the filewriting with joomlas functions,
		// So if the ftp layer is on, joomla will write with that, not the apache user, which might
		// Not have the correct permissions
		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');
		$config     = ComponentHelper::getParams('com_joomcrm');

		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

		$data = $data ? $data : null;

		if ($data && !array_key_exists("id", $data))
		{
			$data = (array) $data;

			$_FILES                = array();
			$_FILES['document']    = $data;
			$_FILES['tmp_name']    = $data['attachment'];
			$fileName              = $data['value'];
			$fileTemp              = $data['attachment'];
			$association_id        = $data['association_id'];
			$association_type      = $data['association_type'];
			$uploadedFileExtension = substr(strrchr($fileName, '.'), 1);
			$data['is_attachment'] = 1;
			$data['email']         = 1;

		}
		else
		{
			if (is_array($_FILES) && count($_FILES) > 0)
			{
				$association_id   = $_POST['association_id'];
				$association_type = $_POST['association_type'];

				// This is the name of the field in the html form, filedata is the default name for swfupload
				// So we will leave it as that
				$fieldName = 'document';

				// Any errors the server registered on uploading
				$fileError = $_FILES[$fieldName]['error'];
				if ($fileError > 0)
				{
					switch ($fileError)
					{
						case 1:
							$this->setError(Text::_('FILE TO LARGE THAN PHP INI ALLOWS'));

							return false;

						case 2:
							$this->setError(Text::_('FILE TO LARGE THAN HTML FORM ALLOWS'));

							return false;

						case 3:
							$this->setError(Text::_('ERROR PARTIAL UPLOAD'));



							return false;

						case 4:
							$this->setError(Text::_('ERROR NO FILE'));

							return false;
					}
				}

				// Check the file extension is ok
				$fileName = $_FILES[$fieldName]['name'];
				$fileTemp = $_FILES[$fieldName]['tmp_name'];

			}

		}

		$data = $data ? $data : \Joomla\CMS\Factory::getApplication()->input->post->getArray();

		if (is_array($_FILES) && count($_FILES) > 0)
		{
			$uploadedFileNameParts = explode('.', $fileName);
			$uploadedFileExtension = array_pop($uploadedFileNameParts);

			$validFileExts = explode(',', $config->get('allowed_documents','png,jpg,jpeg,mp4,mp3,pdf,txt,doc,docx,xsl,xsls,zip'));

			// Assume the extension is false until we know its ok
			$extOk = false;

			// Go through every ok extension, if the ok extension matches the file extension (case insensitive)
			// Then the file extension is ok
			foreach ($validFileExts as $key => $value)
			{
				if (preg_match("/$value/i", $uploadedFileExtension))
				{
					$extOk = true;
				}
			}

			if ($extOk == false)
			{
				$this->setError(Text::_('INVALID EXTENSION'));

				return false;
			}

			// Data generation
			$hashFilename = md5($fileName . $date) . "." . $uploadedFileExtension;

			// Lose any special characters in the filename
			$fileName = preg_replace("[^A-Za-z0-9.]", "-", $fileName);

			// Always use constants when making file paths, to avoid the possibilty of remote file inclusion
			$uploadPath = JPATH_SITE . '/components/com_joomcrm/documents/' . $hashFilename;

			if (array_key_exists('is_attachment', $data) && $data['is_attachment'])
			{
				if (!\Joomla\CMS\Filesystem\File::write($uploadPath, $fileTemp))
				{
					$this->setError(Text::_('ERROR MOVING FILE'));

					return false;
				}
			}
			else
			{
				if (!\Joomla\CMS\Filesystem\File::upload($fileTemp, $uploadPath))
				{
					$this->setError(Text::_('ERROR MOVING FILE'));

					return false;
				}
			}

			$fileSize = filesize($uploadPath);

			// Update the database
			$newData = array(
				'name' => $fileName,
				'filename' => $hashFilename,
				'association_id' => $association_id,
				'association_type' => $association_type,
				'filetype' => $uploadedFileExtension,
				'category_id' => (int) $data['category_id'],
				'size' => $fileSize / 1024,
				'created' => $date
			);


			if (array_key_exists('email', $data) && $data['email'])
			{
				$newData['email'] = 1;
			}

			$newData['is_image'] = (int) is_array(getimagesize($uploadPath));
			$newData['owner_id'] = JoomcrmHelperUsers::getUserId();

		}
		else
		{
			$newData = $data;
		}

		// Load Tables
		$row    = new JoomcrmTableDocument;
		$oldRow = new JoomcrmTableDocument;

		// Date generation
		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

		if (!array_key_exists('id', $newData))
		{
			$newData['created'] = $date;
			$status             = "uploaded";
		}
		else
		{
			$row->load($data['id']);
			$oldRow->load($data['id']);
			$status = "updated";
		}

		$newData['modified'] = $date;


		// Bind the form fields to the table
		if (!$row->bind($newData))
		{
			$this->setError($row->getError());

			return false;
		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onBeforeDocumentSave', array(&$row));

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		$id = (array_key_exists('id', $data)) ? $data['id'] : $this->getDbo()->insertId();

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'document', $status);

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onAfterDocumentSave', array(&$row));

		return $id;
	}

	public function _buildQuery($id = null)
	{

		if ($this->app->input->getInt('filters', 1) == 0)
		{
			$this->filters = false;
		}

		$category = $this->_category;
		$isDatatableQuery = $this->app->input->get('datatableQuery',0,'int');

		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			 ->select("d.*,cat.name AS category_name, c.name as company_name, deal.name as deal_name, p.first_name as first_name, p.last_name as last_name, CONCAT(u.first_name,' ',u.last_name) AS owner_name")
			 ->from("#__joomcrm_documents AS d")
			 ->leftJoin("#__joomcrm_companies AS c ON d.association_type = 'company' AND d.association_id = c.id")
			 ->leftJoin("#__joomcrm_deals AS deal ON d.association_type = 'deal' AND d.association_id = deal.id")
			 ->leftJoin("#__joomcrm_sources AS source on source.id=deal.source_id")
			 ->leftJoin("#__joomcrm_stages AS stage on stage.id=deal.stage_id")
			 ->leftJoin("#__joomcrm_people AS p ON d.association_type ='person' AND d.association_id = p.id")
			 ->leftJoin("#__joomcrm_users AS u ON u.id = d.owner_id")
			 ->leftJoin("#__joomcrm_document_categories as cat on cat.id = d.category_id");

		// Get user data
		$member_type = JoomcrmHelperUsers::getRole();
		$member_id   = JoomcrmHelperUsers::getUserId();
		$team_id     = JoomcrmHelperUsers::getTeamId();

		// Get session data
		$session = Factory::getSession();

		// Get post data
		$assoc = $this->app->input->getString('assoc');
		$user  = $this->app->input->getString('user');
		$type  = $this->app->input->getString('type');
		$team  = $this->app->input->getInt('team_id');


		if(!$isDatatableQuery){
			// Determine if we are searching for a team or a user
			if ($team)
			{
				$session->set('document_user_filter', null);
			}

			if ($user)
			{
				$session->set('document_team_filter', null);
			}

			// Set user session data
			if ($assoc != null)
			{
				$session->set('document_assoc_filter', $assoc);
			}
			else
			{
				$sess_assoc = $session->get('document_assoc_filter', 'all');
				$assoc      = $sess_assoc;
			}
			if ($user != null)
			{
				$session->set('document_user_filter', $user);
				$session->set('document_team_filter', null);
			}
			else
			{
				$sess_user = $session->get('document_user_filter', JoomcrmHelperUsers::getUserId());
				$user      = $sess_user;
			}
			if ($type != null)
			{
				$session->set('document_type_filter', $type);
			}
			else
			{
				$sess_type = $session->get('document_type_filter');
				$type      = $sess_type;
			}
			if ($team != null)
			{
				$session->set('document_team_filter', $team);
				$session->set('document_user_filter', null);
			}
			else
			{
				$sess_team = $session->get('document_team_filter');
				$team      = $sess_team;
			}

			// filter category
			if(!$id){
				if ($category != null)
				{
					$session->set('document_category_filter', $category);
				}
				else
				{
					$sess_category = $session->get('document_category_filter');
					$category      = $sess_category;
				}
			}

		}






		// in some cases filters are not wanted
		if ($this->filters)
		{
			// Filter based on category
			if ($category && $category != "all" && $category > 0)
			{
				$query->where("d.category_id=" . $category);
			}

			// Filter for team
			if ($team && $this->deal_id == null && $this->person_id == null)
			{
				$query->where('u.team_id = ' . $team);
			}
			
			// Filter for user or document uploaded from administration where owner_id = 0
			if (($user && $user != "all") && $this->deal_id == null && $this->person_id == null)
			{
				$query->where('(d.owner_id = ' . $user . ' OR d.owner_id = 0)');
			}

			// Filter data
			if ($assoc AND $assoc != 'all')
			{
				switch ($assoc)
				{
					case "deals":
						$query->where("d.association_type='deal'");
						break;
					case "people":
						$query->where("d.association_type='person'");
						break;
					case "companies":
						$query->where("d.association_type='company'");
						break;
					case "emails":
						$query->where("d.email=1");
						break;
					case "shared":
						$query->where("d.shared=1");
						break;
				}
			}

			// Set user filter states
			$order 		= $this->getState('Document.' . $this->view . '.' . $this->layout . '.filter_order');
			$orderDir 	= $this->getState('Document.' . $this->view . '.' . $this->layout . '.filter_order_Dir');
			$query->order($order . ' ' . $orderDir);

			// Filter for types
			if ($type AND $type != 'all')
			{
				$doc_types = array();
				switch ($type)
				{
					case "spreadsheets":
						$doc_types = array('xlr', 'xls', 'xlsx');
						break;
					case "images":
						$doc_types = array('bmp', 'gif', 'jpg', 'jpeg', 'png', 'psd', 'pspimage', 'thm', 'tif', 'yuv');
						break;
					case "documents":
						$doc_types = array();
						break;
					case "pdfs":
						$doc_types = array('pdf');
						break;
					case "presentations":
						$doc_types = array('pps', 'ppt', 'pptx', 'key', 'odp');
						break;
					case "others":
						$doc_types = array();
						break;
				}
				if (count($doc_types))
				{
					$queryString = '';
					foreach ($doc_types as $key => $type)
					{
						if ($key)
						{
							$queryString .= " OR d.filetype='" . $type . "'";
						}
						else
						{
							$queryString .= " d.filetype='" . $type . "'";
						}
					}
					$query->where('(' . $queryString . ')');
				}
			}

			// Sort depending on member role
			if ($this->deal_id == null && $this->person_id == null /* && $this->company_id == null */)
			{
				if ($member_type != 'exec')
				{
					$shared = "OR d.shared=1";
					if ($member_type == 'manager')
					{
						$query->where('(u.team_id=' . $team_id . " $shared)");
					}
					else
					{
						$query->where('(d.owner_id=' . $member_id . " $shared)");
					}
				}
			}
		}


		if ($id)
		{
			$query->where("d.id=" . $id);
		}

		if ($this->company_id)
		{
			$query->where("(d.association_type='company' AND d.association_id=" . $this->company_id . ')');
		}

		if ($this->deal_id)
		{
			$query->where("(d.association_type='deal' AND d.association_id=" . $this->deal_id . ')');
		}

		if ($this->person_id)
		{
			$query->where("(d.association_type='person' AND d.association_id=" . $this->person_id . ')');
		}

		if ($this->published)
		{
			$query->where("d.published=" . $this->published);
		}

		$document_filter = $this->getState('Document.' . $this->view . '.' . $this->layout . '.name');
		if ($document_filter != null)
		{
			$query->where("d.name LIKE '%" . $db->escape($document_filter) . "%'");
		}

		return $query;
	}

	/**
	 * Method to retrieve documents
	 *
	 * @param $id specific id to retrieve, if null all are returned
	 */
	public function getDocuments($id = null)
	{
		$db    = $this->getDbo();
		$query = $this->_buildQuery($id);


		/** ------------------------------------------
		 * Set query limits and load results
		 */
		$limit      = $this->getState('Document.' . $this->view . '.' . $this->layout . '.limit');
		$limitStart = $this->getState('Document.' . $this->view . '.' . $this->layout . '.limitstart');

		if (!$id && $limit != 0 && $this->view == "documents")
		{
			if ($limitStart >= $this->getTotal())
			{
				$limitStart = 0;
				$limit      = 10;
				$limitStart = ($limit != 0) ? (floor($limitStart / $limit) * $limit) : 0;
				$this->setState($this->view . '_limit', $limit);
				$this->setState($this->view . '_limitstart', $limitStart);
			}

			if ($limit > 0)
			{
				//$query .= " LIMIT " . ($limit) . " OFFSET " . ($limitStart);
			}
		}

		$docViewType = ComponentHelper::getParams('com_joomcrm')->get('documents_view_type','datatable');

		if($docViewType == 'tree'){
			$limit = 0;
			$limitStart = 0;
		}


		$results = $db->setQuery($query, $limitStart, $limit)->loadAssocList();

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onDocumentLoad', array(&$results));

		return $results;
	}

	public function getDocument($id = null)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->select("*")->from("#__joomcrm_documents");

		if (!$id)
		{
			$document_hash = $this->app->input->getString('document');
			$query->where('filename = ' . $db->quote($document_hash));
		}
		else
		{
			$query->where("id=" . $id);
		}

		$document = $db->setQuery($query)->loadObject();

		$document->path = getcwd() . '/components/com_joomcrm/documents/' . $document->filename;

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onDocumentLoad', array(&$document));

		return $document;
	}

	/**
	 * Method to delete a record
	 *
	 * @return boolean True on success
	 *
	 * @todo  Look at the $oldRow var why it's loading the JoomcrmTableEvent
	 */
	public function deleteDocument($id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('d.filename')
			->from('#__joomcrm_documents as d')
			->where('d.id = ' . (int) $id);

		$filename = $db->setQuery($query)->loadResult();

		// Load Tables
		$oldRow = new JoomcrmTableEvent;
		$oldRow->load($id);

		// Delete
		$query
			->clear()
			->update("#__joomcrm_documents")
			->set("published=-1")
			->where('id = ' . (int) $id);

		$db->setQuery($query)->execute();

		// if ( !unlink(JPATH_COMPONENT.'/documents/'.$filename) ) return false;

		$row = new JoomcrmTableDocument;
		$row->load($id);
		$status = "deleted";

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'document', $status);

		return true;
	}

	/**
	 * Populate user state requests
	 */
	public function populateState()
	{
		// Get states
		$filter_order     = $this->app->getUserStateFromRequest('Document.' . $this->view . '.' . $this->layout . '.filter_order', 'filter_order', 'd.filename');
		$filter_order_Dir = $this->app->getUserStateFromRequest('Document.' . $this->view . '.' . $this->layout . '.filter_order_Dir', 'filter_order_Dir', 'asc');
		$limit            = $this->app->getUserStateFromRequest('Document.' . $this->view . '.' . $this->layout . '.limit', 'limit', 10);
		$limitstart       = $this->app->getUserStateFromRequest('Document.' . $this->view . '.' . $this->layout . '.limitstart', 'limitstart', 0);
		$document_filter  = $this->app->getUserStateFromRequest('Document.' . $this->view . '.' . $this->layout . '.name', 'name', null);

		// In case limit has been changed, adjust it
		$limitstart = ($limit != 0) ? (floor($limitstart / $limit) * $limit) : 0;

		// get column name form column number sent by jQuery dataTable
		if (is_numeric($filter_order))
		{
			$filter_order = $this->getColumnName($filter_order);
		}

		$input = \Joomla\CMS\Factory::getApplication()->input;

		// if ajax request (when using filters, pagination limit and start should work good)
		if($input->get('format','','word') == 'raw'){
			$limit = $input->get('ajaxLimit',$limit,'int');
			$limitstart = 0; // should begin from start
		}

		// Set states
		$this->setState('Document.' . $this->view . '.' . $this->layout . '.filter_order', $filter_order);
		$this->setState('Document.' . $this->view . '.' . $this->layout . '.filter_order_Dir', $filter_order_Dir);
		$this->setState('Document.' . $this->view . '.' . $this->layout . '.limit', $limit);
		$this->setState('Document.' . $this->view . '.' . $this->layout . '.limitstart', $limitstart);
		$this->setState('Document.' . $this->view . '.' . $this->layout . '.name', $document_filter);
	}

	/**
	 * Get total number of rows for pagination
	 */
	public function getTotal()
	{
		if (empty ($this->_total))
		{
			$query        = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
		}

		return $this->_total;
	}

	/**
	 * Generate pagination
	 */
	public function getPagination()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			$this->_pagination = new Pagination($this->getTotal(),
				$this->getState('Document.' . $this->view . '.' . $this->layout . '.limitstart'),
				$this->getState('Document.' . $this->view . '.' . $this->layout . '.limit')
				);
		}

		return $this->_pagination;
	}

	/**
	*	Prepares Items data for the jQuery dataTable
	*
	*	@param 	array of items from DB
	*	@return array of modified items
	*/
	public function dataTablefy($items)
	{
		$dataTable = array();
		$user = Factory::getApplication()->getIdentity();

		if (is_array($items))
		{
			$count = count($items);

			for ($i = 0; $i < $count; $i++)
			{
				$imagePath = '/components/com_joomcrm/media/images/';

				if (\Joomla\CMS\Filesystem\File::exists(JPATH_BASE . $imagePath . $items[$i]['filetype'] . '.png'))
				{
					$imageFile = Uri::base() . $imagePath . $items[$i]['filetype'] . '.png';
				}
				else
				{
					$imageFile = Uri::base() . $imagePath . 'generic_file.png';
				}

				$dropdown = Joomla\CMS\Layout\LayoutHelper::render('document.dropdown',['document' => $items[$i]],null,['component' => 'com_joomcrm','client' => 'site']);

				
				$hidden = '<div class="document_edit_menu" id="document_' . $items[$i]['id'] . '">';
				$hidden .= '<input type="hidden" name="document_' . $items[$i]['id'] . '_hash" id="document_' . $items[$i]['id'] . '_hash" value="' . $items[$i]['filename'] . '" />';

				$dataTable[$i] = array();
				$dataTable[$i][] = '<div class="bg-light p-1 m-1 d-inline-block text-center"><img class="d-block" width="30px" height="30px" src="' . $imageFile . '" alt="' . $items[$i]['filetype'] . '" /><small class="text-muted"><strong>' . strtoupper($items[$i]['filetype']) . '</strong></small></div>';
				$dataTable[$i][] = $dropdown;
				$dataTable[$i][] = $items[$i]['owner_name'];
				$dataTable[$i][] = $items[$i]['category_name'];
				$dataTable[$i][] = $items[$i]['size'];
				$dataTable[$i][] = JoomcrmHelperDate::formatDate($items[$i]['created']) . $hidden;
			}
		}

		return $dataTable;
	}

	/**
	*	Match column name from jQuery dataTable with actual
	* 	name of the column from database to make ordering.
	*
	*	@param 	integer $columnNumber
	*	@return string
	*/
	public function getColumnName($columnNumber)
	{
		$columnName = '';

		if ($columnNumber == 0) $columnName = 'd.filetype';
		if ($columnNumber == 1) $columnName = 'd.name';
		if ($columnNumber == 2) $columnName = 'owner_name';
		if ($columnNumber == 3) $columnName = 'd.size';
		if ($columnNumber == 4) $columnName = 'd.created';

		return $columnName;
	}
}
