<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
class JoomcrmModelEvent extends JoomcrmModel
{
	public $_id;
	public $deal_id;
	public $start_date;
	public $end_date;
	public $loc;
	public $filter_order;
	public $filter_order_Dir;
	public $completed;
	public $limit;
	public $company_id;
	public $person_id;
	public $public;
	public $current_events = false;
	public $published = 1;

	/**
	 * Method to store a record
	 *
	 * @return    boolean    True on success
	 */
	public function store($data = null)
	{
		$row    = new JoomcrmTableEvent;
		$oldRow = new JoomcrmTableEvent;

		$data = ($data == null) ? Factory::getApplication()->input->post->getArray() : $data;


		// Date generation
		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

		if (array_key_exists('name', $data))
		{
			$data['name'] = trim($data['name']);
		}

		if (!array_key_exists('id', $data))
		{
			$data['created'] = $date;
			$status          = "created";
		}
		else
		{
			$row->load($data['id']);
			$oldRow->load($data['id']);
			$status = "updated";
		}

		if (array_key_exists('completed', $data) && $data['completed'] == 1)
		{
			$status               = "completed";
			$data['actual_close'] = $date;
		}

		if (array_key_exists('edit_screen', $data))
		{
			if (array_key_exists('public', $data))
			{
				$type           = ($data['public'] == 1) ? 1 : 0;
				$data['public'] = $type;
			}
			else
			{
				$data['public'] = 0;
			}
		}

		$data['modified']    = $date;
		$data['owner_id']    = array_key_exists('owner_id', $data) ? $data['owner_id'] : JoomcrmHelperUsers::getUserId();
		$data['assignee_id'] = array_key_exists('assignee_id', $data) ? $data['assignee_id'] : JoomcrmHelperUsers::getUserId();

		// Clear any data we dont want to bind
		if (array_key_exists('associate_name', $data))
		{
			unset($data['associate_name']);
		}

		// Create event dates
		if (array_key_exists('type', $data) && $data['type'] == "task" && array_key_exists('due_date_hour', $data))
		{
			$date             = explode(" ", $data['due_date']);
			$data['due_date'] = array_key_exists(0, $date) ? $date[0] . " " . $data['due_date_hour'] : $data['due_date'] . " " . $data['due_date_hour'];
		}
		if (array_key_exists('type', $data) && $data['type'] == "event" && array_key_exists('start_time_hour', $data))
		{
			$date               = explode(" ", $data['start_time']);
			$data['start_time'] = array_key_exists(0, $date) ? $date[0] . " " . $data['start_time_hour'] : $data['start_time'] . " " . $data['start_time_hour'];
		}
		if (array_key_exists('type', $data) && $data['type'] == "event" && array_key_exists('end_time_hour', $data))
		{
			$date             = explode(" ", $data['end_time']);
			$data['end_time'] = array_key_exists(0, $date) ? $date[0] . " " . $data['end_time_hour'] : $data['end_time'] . " " . $data['end_time_hour'];
		}

		if (array_key_exists('due_date', $data) && $data['due_date'] != "" && $data['due_date'] != "0000-00-00 00:00:00" && !is_null($data['due_date']))
		{
			$data['due_date'] = JoomcrmHelperDate::formatDBDate($data['due_date']);
		}

		if (array_key_exists('repeat_end', $data) && $data['repeat_end'] != "" && $data['repeat_end'] != "0000-00-00 00:00:00" && !is_null($data['repeat_end']))
		{
			$data['repeat_end'] = JoomcrmHelperDate::formatDBDate($data['repeat_end']);
		}

		if (array_key_exists('start_date', $data) && $data['start_date'] != "" && $data['start_date'] != "0000-00-00 00:00:00" && !is_null($data['start_date']))
		{
			$data['start_date'] = JoomcrmHelperDate::formatDBDate($data['start_date']);
		}

		if (array_key_exists('end_date', $data) && $data['end_date'] != "" && $data['end_date'] != "0000-00-00 00:00:00" && !is_null($data['end_date']))
		{
			$data['end_date'] = JoomcrmHelperDate::formatDBDate($data['end_date']);
		}

		// All day events
		$data['all_day'] = (array_key_exists('all_day', $data) && $data['all_day'] == true) ? 1 : 0;

		if (array_key_exists('update_future_events', $data) && $data['update_future_events'] == 0 && array_key_exists('parent_id', $data) && $data['parent_id'] != 0)
		{
			if (array_key_exists('due_date', $data))
			{
				$date = JoomcrmHelperDate::formatDBDate($data['due_date']);
			}
			if (array_key_exists('start_time', $data))
			{
				$date = $data['start_time'];
			}

			$this->addExcludes($data['parent_id'], $date);
			unset($data['id']);
			unset($data['repeats']);
		}

		if (array_key_exists('start_time', $data) && $data['start_time'] != "" && $data['start_time'] != "0000-00-00 00:00:00" && !is_null($data['start_time']))
		{
			$data['start_time'] = JoomcrmHelperDate::formatDBDate($data['start_time']);
		}

		if (array_key_exists('end_time', $data) && $data['end_time'] != "" && $data['end_time'] != "0000-00-00 00:00:00" && !is_null($data['end_time']))
		{
			$data['end_time'] = JoomcrmHelperDate::formatDBDate($data['end_time']);
		}


		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		$dispatcher = Factory::getApplication();
		$dispatcher->triggerEvent('onBeforeEventSave', array(&$row));

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		if ($oldRow->id && $row->type == "task" && ($row->due_date > $oldRow->due_date))
		{
			$status = "postponed";
		}

		if ($oldRow->id && $row->type == "event" && ($row->start_time > $oldRow->start_time))
		{
			$status = "postponed";
		}

		// send notification emails
		if (!array_key_exists('id', $data) || $data['id'] == ""){

			// send email to assignee of new event
			if(	isset($data['assignee_id']) &&	!empty($data['assignee_id'])){
				JoomcrmHelperEmail::{'send'.$data['type'].'Email'}($row,'assigned',true);
			}

		}else{

			// send email notification if asignee was updated in event
			if(
				isset($oldRow->assignee_id) &&
				!empty($oldRow->assignee_id) &&
				$oldRow->assignee_id > 0 &&
				isset($data['assignee_id']) &&
				!empty($data['assignee_id']) &&
				$oldRow->assignee_id != $data['assignee_id']
			){
				JoomcrmHelperEmail::{'send'.$data['type'].'Email'}($row,'assigned',true);
			}

		}

		// Event id
		$event_id = array_key_exists('id', $data) ? $data['id'] : $row->id;

		$row->load($event_id);

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'event', $status);

		// If we receive information concerning cf tables
		if (array_key_exists('association_id', $data))
		{
			$postcfdata = array('association_id' => $data['association_id'], 'event_id' => $event_id);
			$this->eventsCf($postcfdata, $data['association_type']);
		}

		$dispatcher = Factory::getApplication();
		$dispatcher->triggerEvent('onAfterEventSave', array(&$row));

		return $event_id;

	}

	/**
	 * Method to access events
	 *
	 * @param   [type]  $loc          [description]
	 * @param   [type]  $user         [description]
	 * @param   [type]  $association  [description]
	 *
	 * @return  array                 [description]
	 */
	public function getEvents($loc = null, $user = null, $association = null)
	{
		if (is_null($loc))
		{
			$loc = $this->loc;
		}

		// detect ajax mode
		$isAjax = Factory::getApplication()->input->get('isAjax',0,'int');

		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select(array(
				'e.*',
				'a.*',
				'ci.name AS category_name',
				'c.name as company_name',
				'c.id as company_id',
				'd.name as deal_name',
				'd.id as deal_id',
				'p.first_name as person_first_name',
				'p.last_name as person_last_name',
				'p.id as person_id',
				'assignee.color AS assignee_color',
				'assignee.first_name AS assignee_first_name',
				'assignee.last_name AS assignee_last_name',
				'owner.first_name as owner_first_name',
				'owner.last_name as owner_last_name'))
			->from('#__joomcrm_events AS e')
			->leftJoin('#__joomcrm_events_categories AS ci ON ci.id = e.category_id')
			->leftJoin('#__joomcrm_events_cf AS a ON e.id = a.event_id')
			->leftJoin('#__joomcrm_companies AS c ON a.association_type = "company" AND a.association_id = c.id AND c.published > 0')
			->leftJoin('#__joomcrm_deals AS d ON a.association_type = "deal" AND a.association_id = d.id AND d.published > 0')
			->leftJoin('#__joomcrm_people AS p ON a.association_type = "person" AND a.association_id = p.id AND p.published > 0')
			->leftJoin('#__joomcrm_users AS assignee ON assignee.id = e.assignee_id')
			->leftJoin('#__joomcrm_users AS owner ON owner.id = e.owner_id');



		if (!$this->public)
		{
			$user_role 		= JoomcrmHelperUsers::getRole();
			$user_id   		= JoomcrmHelperUsers::getUserId();
			$team_id   		= JoomcrmHelperUsers::getTeamId();
			$team_access 	= JoomcrmHelperUsers::getTeamAccess($user_id);

			// Filter based on user role
			if ($user_role != 'exec' && $this->view != "print")
			{
				// Manager filter
				if ($user_role == 'manager' || (isset($team_id) && $team_access) )
				{
					$query->where('(assignee.team_id = ' . $team_id . ' OR owner.team_id = ' . $team_id . ")");
				}
				// Basic user filter
				else
				{
					$query->where("(e.assignee_id = " . $user_id . " OR e.owner_id =" . $user_id . ")");
				}
			}

			// Search for certain user events
			if ($user && $this->view != "print")
			{
				if ($user == $user_id)
				{
					$query->where("(e.assignee_id=" . $user_id . ' OR e.owner_id=' . $user_id . ')');
				}
				elseif ($user != 'all')
				{
					$query->where(array("e.assignee_id=" . $user));
				}
			}



			if (!$association)
			{
				// this one make an issue
				//$association = $this->app->input->getString('association_id', $this->app->input->getString('id'));
				$association = $this->app->input->get('association_id', 0, 'int');
			}



			$association_type  = $this->app->input->getString('association_type', $this->app->input->getString('layout'));
			$association_types = array('company', 'deal', 'person');



			if ($association)
			{
				$association_type = $association_type ? $association_type : $loc;

				if ($association_type == 'company')
				{
					if (is_array($association))
					{
						$query->where("(p.company_id=" . $association . " OR d.company_id=" . $association . " OR ( a.association_type=" . $db->quote("company") . " AND a.association_id IN(" . implode(",", $association) . ") ))");
					}
					else
					{
						$query->where("(p.company_id=" . $association . " OR d.company_id=" . $association . " OR ( a.association_type=" . $db->quote("company") . " AND a.association_id=" . $association . " ))");
					}
				}
				else
				{


					if (is_array($association))
					{
						$query->where("a.association_id IN(" . implode(",", $association) . ")");
					}
					else
					{
						$query->where("a.association_id=" . $association);
					}
					$query->where("a.association_type=" . $db->quote($association_type));
				}
			}
			else
			{
				if ($association_type && in_array($association_type, $association_types))
				{
					$query->where("a.association_type=" . $db->Quote($association_type));
				}
				else
				{
					/** hide events associated with archived deals **/
					$query->where("(d.archived=0 OR d.archived IS NULL)");
				}
			}

			if ($this->_id != null)
			{
				if (is_array($this->_id))
				{
					$query->where("e.id IN (" . implode(',', $this->_id) . ")");
				}
				else
				{
					$query->where("e.id=$this->_id");
				}
			}

			if ($this->current_events)
			{
				$now = JoomcrmHelperDate::formatDBDate(date('Y-m-d'));
				$query->where('e.due_date != "0000-00-00 00:00:00" AND e.due_date >="' . $now . '"');
			}

			/** Filter by status **/
			$status_filter = $this->getState('Event.' . $this->view . '_' . $this->layout . '_status');
			if ($status_filter != null && $this->view != "print")
			{
				$query->where("e.completed=$status_filter");
			}
			else
			{
				if ($this->completed != null)
				{
					if ($this->completed == 'true')
					{
						$query->where("e.completed=1");
					}
					else
					{
						if ($this->completed != 'false')
						{
							$query->where("e.completed=" . $this->completed);
						}
					}
				}
				else
				{
					$query->where("e.completed=0");
				}
			}

			/** Filter by type **/
			$type_filter = $this->getState('Event.' . $this->view . '_' . $this->layout . '_type');
			if ($type_filter != null && $type_filter != "all" && $this->view != "print")
			{
				$query->where("e.type='$type_filter'");
			}

			/** Filter by category **/
			$category_filter = $this->getState('Event.' . $this->view . '_' . $this->layout . '_category');
			if ($category_filter != null && $category_filter != "any" && $this->view != "print")
			{
				$query->where("e.category_id=$category_filter");
			}

			/** Filter by due date **/
			$due_date_filter = $this->getState('Event.' . $this->view . '_' . $this->layout . '_due_date');
			if ($due_date_filter != null && $due_date_filter != "any" && $this->view != "print")
			{
				$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00'));
				switch ($due_date_filter)
				{
					case "today":
						$tomorrow = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() + (1 * 24 * 60 * 60)));
						$query->where("((e.due_date >= '$date' AND e.due_date < '$tomorrow') OR (e.start_time >= '$date' AND e.start_time < '$tomorrow'))");
						break;
					case "tomorrow":
						$tomorrow           = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() + (1 * 24 * 60 * 60)));
						$day_after_tomorrow = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() + (2 * 24 * 60 * 60)));
						$query->where("((e.due_date >= '$tomorrow' AND e.due_date < '$day_after_tomorrow') OR (e.start_time >= '$tomorrow' AND e.start_time < '$day_after_tomorrow'))");
						break;
					case "this_week":
						$date_info         = getDate(strtotime($date));
						$today             = $date_info['wday'];
						$days_to_remove    = -1 + $today;
						$days_to_add       = 7 - $today;
						$beginning_of_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - $days_to_remove days")));
						$end_of_week       = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " + $days_to_add days")));
						$query->where("((e.due_date >= '$beginning_of_week' AND e.due_date < '$end_of_week') OR (e.start_time >= '$beginning_of_week' AND e.start_time < '$end_of_week'))");
						break;
					case "past_due":
						$query->where("((e.due_date < '$date' AND e.due_date != '0000-00-00 00:00:00') OR (e.start_time < '$date' AND e.start_time != '0000-00-00 00:00:00'))");
						break;
					case "not_past_due":
						$query->where("((e.due_date >= '$date' AND e.due_date != '0000-00-00 00:00:00') OR (e.start_time >= '$date' AND e.start_time != '0000-00-00 00:00:00'))");
						break;
				}
			}

			/** Filter by assignee id **/
			$assignee_id_filter   = $this->getState('Event.' . $this->view . '_' . $this->layout . '_assignee_id');
			$assignee_filter_type = $this->getState('Event.' . $this->view . '_' . $this->layout . '_assignee_filter_type');
			if ($loc != "calendar" && $assignee_id_filter != null && $assignee_id_filter != 'all' && $this->view != "print")
			{
				if ($assignee_filter_type == "team")
				{
					$team_members = JoomcrmHelperUsers::getTeamUsers($assignee_id_filter, true);
					$query->where("e.assignee_id IN(" . implode(',', $team_members) . ")");
				}
				else
				{
					$query->where("e.assignee_id=$assignee_id_filter");
				}
			}

			/** Filter by association type **/
			$association_type_filter = $this->getState('Event.' . $this->view . '_' . $this->layout . '_association_type');
			if ($association_type_filter != null && $association_type_filter != "any" && !$association && $assignee_id_filter != 'all' && $this->view != "print" /*&& !$assignee_id_filter */)
			{
				$query->where("a.association_type='" . $association_type_filter . "'");
			}

			if ($this->start_date)
			{
				$query->where("(e.due_date >= '" . $this->start_date . "' OR e.start_time >= '" . $this->start_date . "' OR e.repeats != 'none' )");
			}

			if ($this->end_date)
			{
				$query->where("(e.due_date < '" . $this->end_date . "' OR e.end_time < '" . $this->end_date . "' OR e.repeats != 'none' )");
			}

			if ($this->deal_id > 0)
			{
				$query->where("(a.association_id=" . $this->deal_id . " AND a.association_type='deal')");
			}
		}
		else
		{
			$query->where("e.public=1");
			/** hide events associated with archived deals **/
			$query->where("(d.archived=0 OR d.archived IS NULL)");
			$query->where("e.completed=0");
		}

		$query->where("e.published=" . $this->published);

		$this->filter_order     = $this->getState('Event.' . $this->view . '_' . $this->layout . '_filter_order');
		$this->filter_order_Dir = $this->getState('Event.' . $this->view . '_' . $this->layout . '_filter_order_Dir');
		$query->order($this->filter_order . ' ' . $this->filter_order_Dir);

		/** ------------------------------------------
		 * Set query limits and load results
		 */
		if ($this->limit != null)
		{
			$query .= " LIMIT " . $this->limit;
		}
		else
		{
			if ($this->getState("Event." . $this->view . '_' . $this->layout . '_limit') != 0)
			{
				$query .= " LIMIT " . ($this->getState("Event." . $this->view . '_' . $this->layout . '_limit')) . " OFFSET " . ($this->getState("Event." . $this->view . '_' . $this->layout . '_limitstart'));
			}
		}

		$rows = $db->setQuery($query)->loadAssocList();



		/**
		 * VIRTUAL EVENT GENERATION -----------------------------------------
		 *
		 * Only generate virtual events for the current month // year
		 * Recurring events past the specified dates will have to be loaded via ajax on the clients end
		 * While generating dates if an event is in the excludes field we will not include that in the generation
		 * This accounts for deleted and/or unique modified events that expand from the recurring series, OR
		 * assign the event and event_parent field and dont display if we find an event_parent filled
		 */

		$rowsToAdd = array();

		if (count($rows) > 0)
		{
			foreach ($rows as $key => $event)
			{

				$rows[$key]['association_link']      = $association_link = "";
				$rows[$key]['association_link_lang'] = $association_link_lang = "";

				if ($event['association_type'] != null)
				{
					switch ($event['association_type'])
					{
						case "company":
							$view = "companies";
							break;
						case "deal":
							$view = "deals";
							break;
						case "person":
							$view = "people";
							break;
					}
					$rows[$key]['association_link']      = $association_link = Route::_('index.php?option=com_joomcrm&view=' . $view . "&layout=" . $event['association_type'] . "&id=" . $event['association_id']);
					$rows[$key]['association_link_lang'] = $association_link_lang = ucwords(Text::_("COM_JOOMCRM_ASSOCIATION_LINK_" . strtoupper($event['association_type'])));
				}




				// fix timezone in ajax mode , temporary solution
				$event['due_date']      =   JoomcrmHelperDate::formatDate($event['due_date'],false,false);
				$event['start_time'] =  JoomcrmHelperDate::formatDate($event['start_time'],false,false);
				$event['end_time']      =   JoomcrmHelperDate::formatDate($event['end_time'],false,false);
				$event['repeat_end']    =   JoomcrmHelperDate::formatDate($event['repeat_end'],false,false);
				$event['actual_close']    =   JoomcrmHelperDate::formatDate($event['actual_close'],false,false);


				$rows[$key]['start_time'] = $event['start_time'];
				$rows[$key]['end_time'] = $event['end_time'];
				$rows[$key]['due_date'] = $event['due_date'];
				$rows[$key]['repeat_end'] = $event['repeat_end'];
				$rows[$key]['actual_close'] = $event['actual_close'];

				if ($event['type'] == "event")
				{
					$stime                         = explode(" ", $event['start_time']);
					$rows[$key]['start_time_hour'] = $stime[1];
					$etime                         = explode(" ", $event['end_time']);
					$rows[$key]['end_time_hour']   = $etime[1];
				}
				else
				{
					if(!empty($event['due_date'])){
						$dtime                       = explode(" ", $event['due_date']);
						$rows[$key]['due_date_hour'] = $dtime[1];
					}else{
						$rows[$key]['due_date_hour'] = "00:00:00";
					}
				}

				// Append server flag
				$rows[$key]['server'] = true;



				// Determine if event is recurring
				if (array_key_exists('repeats', $event) && $event['repeats'] !== 'none')
				{
					// Get current date so we know when to stop looping for event virtualization
					$date        = JoomcrmHelperDate::formatDBDate(date("Y-m-d H:i:s"));
					$start_month = $this->start_date != null ? $this->start_date : JoomcrmHelperDate::formatDBDate(date("Y-m-1 00:00:00"));
					$end_month   = $this->end_date != null ? $this->end_date : date("Y-m-1 00:00:00", strtotime($date . " +1 month"));

					$dates = array();

					// Get excluded dates
					$excludes = (array_key_exists('excludes', $event)) ? unserialize((string)$event['excludes']) : null;
					$excludes = (count((array) $excludes) > 0 && $excludes != null) ? $excludes : array();

					// Determine which field we should increment and base repetitions off of
					$event['start_time'] = ($event['type'] == 'task') ? $event['due_date'] : $event['start_time'];
					$event['end_time']   = ($event['type'] == 'task') ? $event['due_date'] : $event['end_time'];

					$initialTime = $event['start_time'];
					$initialKey  = $key;

					if (strtotime($initialTime) > 0)
					{
						// Determine virtual events to generate
						switch ($event['repeats'])
						{
							// Daily: only show yesterday, today and tomorrow.
							case "daily":
								$now       = time();
								// Beginning of the day yesterday
								$yesterday = strtotime(date('Y-m-d 00:00:00', strtotime('-1 day', $now)));
								// End of the day tomorrow
								$tomorrow  = strtotime(date('Y-m-d 23:59:59', strtotime('+1 day', $now)));

								// Remove the daily repeating event from the return if it's before yesterday.
								if (strtotime($event['start_time']) < $yesterday)
								{
									// unset($rows[$key]);
								}

								if (isset($event['repeat_end']) && strtotime($event['repeat_end']) > 0)
								{

									while ((strtotime($event['start_time']) < $tomorrow || strtotime($event['repeat_end']) > strtotime($event['end_time'])) && strtotime($event['start_time']) < strtotime($event['repeat_end']))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 days");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 days");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;

										if (!in_array($event['start_time'], $excludes) && !$past_due && strtotime($event['start_time']) >= $yesterday)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								else
								{
									while (strtotime($event['start_time']) < $tomorrow || strtotime($event['repeat_end']) > strtotime($event['end_time']))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 days");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 days");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;

										if (!in_array($event['start_time'], $excludes) && !$past_due && strtotime($event['start_time']) >= $yesterday)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								break;
							//Weekdays
							case "weekdays":
								$days = array(1, 2, 3, 4, 5);
								if (isset($event['repeat_end']) && strtotime($event['repeat_end']) > 0)
								{
									$event['repeat_end'] = JoomcrmHelperDate::formatDate($event['repeat_end'], false, false);
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])) && (isset($event['repeat_end']) && strtotime($event['start_time']) < strtotime($event['repeat_end'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 days");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 days");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (in_array(date('w', strtotime($event['start_time'])), $days) && !in_array($event['start_time'], $excludes))
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								else
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 days");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 days");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (in_array(date('w', strtotime($event['start_time'])), $days) && !in_array($event['start_time'], $excludes))
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								break;
							//Weekly
							case "weekly":
								if (isset($event['repeat_end']) && strtotime($event['repeat_end']) > 0)
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])) && (isset($event['repeat_end']) && strtotime($event['start_time']) < strtotime($event['repeat_end'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 week");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 week");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (!in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								else
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 week");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 week");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (!in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								break;
							//Weekly Monday Wednesday and Friday
							case "weekly-mwf":
								$days = array(1, 3, 5);
								if (isset($event['repeat_end']) && strtotime($event['repeat_end']) > 0)
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])) && (isset($event['repeat_end']) && strtotime($event['start_time']) < strtotime($event['repeat_end'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 day");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 day");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (in_array(date('w', strtotime($event['start_time'])), $days) && !in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								else
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 day");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 day");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (in_array(date('w', strtotime($event['start_time'])), $days) && !in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								break;
							//Weekly Tuesday Thursday
							case "weekly-tr":
								$days = array(2, 4);
								if (isset($event['repeat_end']) && strtotime($event['repeat_end']) > 0)
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])) && (isset($event['repeat_end']) && strtotime($event['start_time']) < strtotime($event['repeat_end'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 day");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 day");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (in_array(date('w', strtotime($event['start_time'])), $days) && !in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								else
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 day");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 day");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (in_array(date('w', strtotime($event['start_time'])), $days) && !in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								break;
							// Monthly
							case "monthly":
								if (isset($event['repeat_end']) && strtotime($event['repeat_end']) > 0)
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])) && (isset($event['repeat_end']) && strtotime($event['start_time']) < strtotime($event['repeat_end'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 month");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 month");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (!in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								else
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 month");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 month");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (!in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								break;
							// Yearly
							case "yearly":
								if (isset($event['repeat_end']) && strtotime($event['repeat_end']) > 0)
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])) && (isset($event['repeat_end']) && strtotime($event['start_time']) < strtotime($event['repeat_end'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 year");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 year");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (!in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								else
								{
									while ((strtotime($event['start_time']) < strtotime($end_month) || strtotime($event['repeat_end']) > strtotime($event['end_time'])))
									{
										$event['start_time'] = $this->offsetTime($event['start_time'], "+1 year");
										$event['end_time']   = $this->offsetTime($event['end_time'], "+1 year");
										$past_due            = $due_date_filter == "past_due" ? strtotime($event['start_time']) >= strtotime($date) : false;
										if (!in_array($event['start_time'], $excludes) && !$past_due)
										{
											$dates[] = array('start' => $event['start_time'], 'end' => $event['end_time']);
										}
									}
								}
								break;
						}

					}

					// Once we have generated the event dates for the virtual events, then add them to the object as new "unique" events
					if (count($dates) > 0)
					{
						$i = 0;
						foreach ($dates as $key => $date)
						{
							if (isset($this->limit) && $i == $this->limit)
							{
								break;
							}
							$new_event = array('assignee_color' => $event['assignee_color'], // 'id'                 => null,
								// 'id'                 => $event['id'],
								               'id'             => $event['id'] . "-" . $i, 'owner_id' => $event['owner_id'], 'name' => $event['name'], 'description' => $event['description'], 'created' => $event['created'], 'type' => $event['type'], 'assignee_id' => $event['assignee_id'], 'due_date' => $date['start'], 'repeats' => $event['repeats'], 'repeat_end' => $event['repeat_end'], 'start_time' => $date['start'], 'end_time' => $date['end'], 'all_day' => $event['all_day'], 'category_id' => $event['category_id'], 'category_name' => $event['category_name'], 'modified' => $event['modified'], 'completed' => 0, 'actual_close' => $event['actual_close'], 'company_name' => $event['company_name'], 'company_id' => $event['company_id'], 'deal_name' => $event['deal_name'], 'deal_id' => $event['deal_id'], 'person_first_name' => $event['person_first_name'], 'person_last_name' => $event['person_last_name'], 'person_id' => $event['person_id'], 'parent_id' => $event['id'], 'server' => true, 'clone' => true, 'association_type' => $event['association_type'], 'association_link' => $association_link, 'association_link_lang' => $association_link_lang, 'owner_first_name' => $event['owner_first_name'], 'owner_last_name' => $event['owner_last_name'], 'assignee_first_name' => $event['assignee_first_name'], 'assignee_last_name' => $event['assignee_last_name'], 'end_date' => $event['end_date'],);
							if ($event['type'] == "event")
							{
								$new_event['start_time']      = JoomcrmHelperDate::formatDate($new_event['start_time'], false, false);
								$stime                        = explode(" ", $new_event['start_time']);
								$new_event['start_time_hour'] = $stime[1];
								$etime                        = explode(" ", $new_event['end_time']);
								$new_event['end_time_hour']   = $etime[1];
							}
							else
							{
								$dtime                      = explode(" ", $new_event['due_date']);
								$new_event['due_date_hour'] = $dtime[1];
							}
							$rowsToAdd[] = $new_event;
							$i++;
						}
					}

					if (in_array($initialTime, $excludes))
					{
						if ($this->completed != null)
						{
							if ($event['completed'] != $this->completed)
							{
								unset($rows[$initialKey]);
							}
						}
						else
						{
							if (isset($status_filter) && $status_filter != null)
							{
								if ($event['completed'] != $status_filter)
								{
									unset($rows[$initialKey]);
								}
							}
							else
							{
								unset($rows[$initialKey]);
							}
						}
					}

				}

			}
		}

		if (is_array($rows) && is_array($rowsToAdd))
		{
			$rows = array_merge($rows, $rowsToAdd);
		}

		// remove parent event if in excludes dates
		foreach ($rows as $krow => $row){

			$excludes = array_key_exists('excludes', $event) && !empty($event['excludes']) ? unserialize($event['excludes']) : null;
			$excludes = (count((array) $excludes) > 0 && $excludes != null) ? $excludes : array();

			if(count($excludes) == 0) continue;

			$startTime = Factory::getDate($row['start_time'])->format('Y-m-d');

			foreach ($excludes as $exclude){

				if($startTime == trim($exclude)){
					unset($rows[$krow]);
				}

			}

		}

		// Filter data for calendar rendering
		if ($loc == 'calendar' || $this->loc == "calendar")
		{
			if (count($rows) > 0)
			{
				foreach ($rows as $key => $row)
				{
					$rows[$key]['title']  = $row['name'];
					$rows[$key]['allDay'] = $row['all_day'];

					// Determine event type
					if ($row['type'] == 'event')
					{
						$rows[$key]['start'] = $row['start_time'];
						$rows[$key]['end']   = $row['end_time'];
					}

					if ($row['type'] == 'task')
					{
						$rows[$key]['start'] = $row['due_date'];
					}

				}
			}
		}

		/** Sort events after the virtual events have been created **/
		if (is_array($rows) && count($rows) > 0)
		{
			switch ($this->filter_order)
			{
				case "e.due_date":
					usort($rows, self::class . '::dateSort');
					break;
				case "e.category_id":
					usort($rows, self::class . '::typeSort');
					break;
				default:
					usort($rows, self::class . '::dateSort');
					break;
			}
		}

		// filter events after event generation
		if (is_array($rows) && count($rows) > 0)
		{
			if (!$this->completed)
			{
				if ($this->completed != null)
				{
					foreach ($rows as $key => $row)
					{
						if ($row['completed'] != $this->completed)
						{
							unset($rows[$key]);
						}
					}
				}
				else
				{
					if (isset($status_filter) && $status_filter != null)
					{
						foreach ($rows as $key => $row)
						{
							if ($row['completed'] != $status_filter)
							{
								unset($rows[$key]);
							}
						}
					}
					else
					{
						foreach ($rows as $key => $row)
						{
							if ($row['completed'] == 1)
							{
								unset($rows[$key]);
							}
						}
					}
				}
			}
		}

		$dispatcher = Factory::getApplication();
		$dispatcher->triggerEvent('onEventLoad', array(&$rows));

		// Return results
		return $rows;

	}

	function dateSort($a, $b)
	{
		$date1 = $a['type'] == "event" ? $a['start_time'] : $a['due_date'];
		$date2 = $b['type'] == "event" ? $b['start_time'] : $b['due_date'];
		if (strtotime($date1) == strtotime($date2))
		{
			return 0;
		}
		if ($this->filter_order == "e.due_date")
		{
			if ($this->filter_order_Dir == "asc")
			{
				return (strtotime($date1) < strtotime($date2)) ? -1 : 1;
			}
			else
			{
				return (strtotime($date1) > strtotime($date2)) ? -1 : 1;
			}
		}
		else
		{
			return (strtotime($date1) < strtotime($date2)) ? -1 : 1;
		}
	}

	function typeSort($a, $b)
	{
		if ($a['category_id'] == $b['category_id'])
		{
			return 0;
		}
		if ($this->filter_order_Dir == "asc")
		{
			return $a['category_id'] < $b['category_id'] ? -1 : 1;
		}
		else
		{
			return $a['category_id'] > $b['category_id'] ? -1 : 1;
		}
	}

	/**
	 * Method to access an event
	 *
	 * @param   [type]   $id          [description]
	 * @param   boolean $formatTime  [description]
	 *
	 * @return  mixed                 [description]
	 */
	public function getEvent($id = null, $formatTime = true)
	{
		$db = $this->getDbo();

		if (!$id)
		{
			return new JoomcrmTableEvent;
		}

		$data = Factory::getApplication()->input->post->getArray();

		// Determine if we are trying to retrieve a virtual event, if so retrieve its parent instead
		$query = $db->getQuery(true)
			->select('id')
			->from("#__joomcrm_events")
			->where('id = ' . (int) $id);

		$result = $db->setQuery($query)->loadAssocList();

		// If we dont find any results use the parent id instead
		if (count($result) == 0 || count(explode("-", $id)) > 1)
		{
			$id = $this->app->input->getString('parent_id');
		}

		// Gen query
		$query
			->clear()
			->select("e.*,ecf.association_id,ecf.association_type,ecf.event_id,owner.first_name as owner_first_name, owner.last_name as owner_last_name,
			d.name as deal_name, d.amount as deal_amount, CONCAT(p.first_name,' ',p.last_name) AS person_name, c.name as company_name, c.id as company_id,
			c.address_1 as company_address_1, c.address_city as company_address_city, c.address_state as company_address_state, c.address_zip as company_address_zip,
			c.phone as company_phone, c.website as company_website,assignee.first_name AS assignee_first_name,assignee.last_name AS assignee_last_name
			")
			->from("#__joomcrm_events as e")
			->leftJoin('#__joomcrm_events_cf AS ecf ON ecf.event_id = e.id')
			->leftJoin('#__joomcrm_users AS owner ON owner.id = e.owner_id')
			->leftJoin('#__joomcrm_deals as d ON ecf.association_id = d.id AND ecf.association_type = "deal" AND d.published>0')
			->leftJoin('#__joomcrm_people as p ON ecf.association_id = p.id AND ecf.association_type = "person" AND p.published>0')
			->leftJoin('#__joomcrm_companies as c ON ecf.association_id = c.id AND ecf.association_type = "company" AND c.published>0')
			->leftJoin('#__joomcrm_users AS assignee ON assignee.id = e.assignee_id')
			->where("e.id=" . $id);

		$user = JoomcrmHelperUsers::getLoggedInUser();

		if (!$user)
		{
			$query->where("e.public=1");
		}

		$results = $db->setQuery($query)->loadAssocList();

		// Clean results
		// Assign the parent id if needed
		if (count($result) == 0 && is_array($results) && array_key_exists(0, $results))
		{
			$results[0]['parent_id'] = $results[0]['id'];
			unset($results[0]['id']);
		}

		if (is_array($results) && array_key_exists(0, $results))
		{
			$formatted = false;
			// If the event is repeating then we assign the post data as the "virtual" date
			if ($results[0]['repeats'] != "none" && $this->app->input->getString('date') && count($results) > 0)
			{
				if (array_key_exists('type', $results[0]) && $results[0]['type'] == "event")
				{
					$results[0]['start_time']		      = JoomcrmHelperDate::formatDate($results[0]['start_time'], false, false);
					$stime                                = explode(" ", $results[0]['start_time']);
					$results[0]['start_time_hour']        = $stime[1];
					$etime                                = explode(" ", $results[0]['end_time']);
					$results[0]['end_time_hour']          = $etime[1];
					$results[0]['start_time']             = (int) $this->app->input->getString('date');
					$results[0]['end_time']               = (int) $this->app->input->getString('date');
					$results[0]['start_time_unformatted'] = date("Y-m-d", (int) $this->app->input->getString("date"));
					$results[0]['end_time_unformatted']   = date("Y-m-d", (int) $this->app->input->getString("date"));
				}
				else
				{
					if (array_key_exists('due_date', $results[0]))
					{
						$results[0]['due_date_unformatted'] = date("Y-m-d", (int) $this->app->input->getString("date"));
						$dtime                              = explode(" ", $results[0]['due_date']);
						$results[0]['due_date_hour']        = $dtime[1];
						$results[0]['due_date']             = (int) $this->app->input->getString('date');
					}
				}
				// Otherwise the event is not virtual
			}
			else
			{
				$formatted = true;
				if (array_key_exists('type', $results[0]) && $results[0]['type'] == "event")
				{
					$results[0]['start_time_unformatted'] = date("Y-m-d", strtotime($results[0]['start_time']));
					$results[0]['end_time_unformatted']   = date("Y-m-d", strtotime($results[0]['end_time']));
					$results[0]['start_time']             = JoomcrmHelperDate::formatDate($results[0]['start_time'], false, false);
					$stime                                = explode(" ", $results[0]['start_time']);
					$results[0]['start_time_hour']        = $stime[1];
					$results[0]['end_time']               = JoomcrmHelperDate::formatDate($results[0]['end_time'], false, false);
					$etime                                = explode(" ", $results[0]['end_time']);
					$results[0]['end_time_hour']          = $etime[1];
				}
				else
				{
					if (array_key_exists('due_date', $results[0]))
					{
						$results[0]['due_date_unformatted'] = date("Y-m-d", strtotime($results[0]['due_date']));
						$dtime                              = explode(" ", JoomcrmHelperDate::formatDate($results[0]['due_date'], false, false));
						if(isset($dtime[1])) {
							$results[0]['due_date_hour']        = $dtime[1];
						}
					}
				}
			}

			// If we want to format the time then that happens here
			if ($formatTime && !$this->app->input->getString('calendar_filter'))
			{
				if (array_key_exists('created', $results[0]))
				{
					$originalDate                    = $results[0]['created'];
					$results[0]['created_formatted'] = array_key_exists('created', $results[0]) ? JoomcrmHelperDate::formatDate($originalDate) : "";
					$results[0]['created']           = JoomcrmHelperDate::formatDate($originalDate, true, false);
				}
				if (array_key_exists('repeat_end', $results[0]))
				{
					$results[0]['repeat_end_formatted'] = JoomcrmHelperDate::formatDate($results[0]['repeat_end']);
					// $results[0]['repeat_end']           = JoomcrmHelperDate::formatDate($results[0]['repeat_end'], true, false);
				}
				if (array_key_exists('modified', $results[0]))
				{
					$results[0]['modified_formatted'] = JoomcrmHelperDate::formatDate($results[0]['modified']);
					$results[0]['modified']           = JoomcrmHelperDate::formatDate($results[0]['modified'], true, false);
				}
				if (array_key_exists('actual_close', $results[0]))
				{
					$results[0]['actual_close_formatted'] = JoomcrmHelperDate::formatDate($results[0]['actual_close']);
					$results[0]['actual_close']           = JoomcrmHelperDate::formatDate($results[0]['actual_close'], true, false);
				}
				if (array_key_exists('due_date', $results[0]))
				{
					$results[0]['due_date_formatted'] = $this->app->input->getString('date') || $formatted ? JoomcrmHelperDate::formatDateString($results[0]['due_date']) : JoomcrmHelperDate::formatDate($results[0]['due_date']);
					//$results[0]['due_date']           = $this->app->input->getString('date') || $formatted ? JoomcrmHelperDate::formatDateString($results[0]['due_date']) : JoomcrmHelperDate::formatDate($results[0]['due_date'], true, false);
				}
				if (array_key_exists('end_date', $results[0]))
				{
					$results[0]['end_date_formatted'] = $this->app->input->getString('date') ? JoomcrmHelperDate::formatDateString($results[0]['end_date']) : JoomcrmHelperDate::formatDate($results[0]['end_date']);
					$results[0]['end_date']           = JoomcrmHelperDate::formatDate($results[0]['end_date'], false, false);
				}
				if (array_key_exists('start_time', $results[0]))
				{
					$results[0]['start_time_formatted'] = $this->app->input->getString('date') || $formatted ? JoomcrmHelperDate::formatDateString($results[0]['start_time']) : JoomcrmHelperDate::formatDate($results[0]['start_time']);
					$results[0]['start_time']           = $this->app->input->getString('date') || $formatted ? JoomcrmHelperDate::formatDateString($results[0]['start_time']) : JoomcrmHelperDate::formatDate($results[0]['start_time'], true, false);
				}
				if (array_key_exists('end_time', $results[0]))
				{
					$results[0]['end_time_formatted'] = $this->app->input->getString('date') || $formatted ? JoomcrmHelperDate::formatDateString($results[0]['end_time']) : JoomcrmHelperDate::formatDate($results[0]['end_time']);
					$results[0]['end_time']           = $this->app->input->getString('date') || $formatted ? JoomcrmHelperDate::formatDateString($results[0]['end_time']) : JoomcrmHelperDate::formatDate($results[0]['end_time'], true, false);
				}

				if (array_key_exists(0, $results) && !$this->app->input->getString('date') && !$formatted)
				{
					if (array_key_exists('type', $results[0]) && $results[0]['type'] == "event")
					{
						$results[0]['start_time']		   = JoomcrmHelperDate::formatDate($results[0]['start_time'], false, false);
						$stime                         = explode(" ", $results[0]['start_time']);
						$results[0]['start_time_hour'] = array_key_exists(1, $stime) ? $stime[1] : "";
						$etime                         = explode(" ", $results[0]['end_time']);
						$results[0]['end_time_hour']   = array_key_exists(1, $etime) ? $etime[1] : "";
						$results[0]['start_time']      = $stime[0];
						$results[0]['end_time']        = $etime[0];
					}
					else
					{
						if (array_key_exists('due_date', $results[0]))
						{
							$dtime                       = explode(" ", $results[0]['due_date']);
							$results[0]['due_date_hour'] = array_key_exists(1, $dtime) ? $dtime[1] : "";
							$results[0]['due_date']      = $dtime[0];
						}
					}
				}

			}

			// Filter results for calendar display
			if ($this->app->input->getString('calendar_filter'))
			{
				$results[0]['title']  = $results[0]['name'];
				$results[0]['allDay'] = $results[0]['all_day'];

				// Determine event type
				if ($results[0]['type'] == 'event')
				{
					$results[0]['start'] = $results[0]['start_time'];
					$results[0]['end']   = $results[0]['end_time'];
				}

				if ($results[0]['type'] == 'task')
				{
					$results[0]['start'] = $results[0]['due_date'];
				}
			}

			// Join any association data
			if (array_key_exists('association_id', $results[0]) AND !is_null($results[0]['association_id']) AND $results[0]['association_id'] != 0)
			{
				$query->clear();
				switch ($results[0]['association_type'])
				{
					case 'company':
						$query->select("name as association_name, id as association_id")->from("#__joomcrm_companies");
						break;
					case 'deal':
						$query->select("name as association_name, id as association_id")->from("#__joomcrm_deals");
						break;
					case 'person':
						$query
							->select("first_name as association_first_name, last_name as association_last_name, id as association_id")
							->from("#__joomcrm_people");
						break;
				}
				$query->where("published>0")->where("id=" . $results[0]['association_id']);

				$data = $db->setQuery($query)->loadAssocList();

				if (!empty($data))
				{
					$results[0] = array_merge($results[0], $data[0]);

					if ($results[0]['association_type'] == 'person')
					{
						$results[0]['association_name'] = $data[0]['association_first_name'] . " " . $data[0]['association_last_name'];
					}
				}
			}

			$dispatcher = Factory::getApplication();
			$dispatcher->triggerEvent('onEventLoad', array(&$results[0]));
		}

		// Return results
		if (is_array($results) && array_key_exists(0, $results))
		{
			return $results[0];
		}
		else
		{
			return new JoomcrmTableEvent;
		}
	}

	/*
	 * Method to link events and cf tables
	 */
	public function eventsCf($cfdata, $type)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Search for existing associations
		$query
			->select('*')
			->from("#__joomcrm_events_cf")
			->where("event_id = " . (int) $cfdata['event_id']);

		$search = $db->setQuery($query)->loadObjectList();

		// Insert or update data depending on if we find any results
		if (count($search) > 0)
		{
			$query
				->clear()
				->update('#__joomcrm_events_cf')
				->set(array('association_id = ' . (int) $cfdata['association_id'], 'association_type = ' . $db->quote($type), 'event_id = ' . (int) $cfdata['event_id']))
				->where('event_id = ' . (int) $cfdata['event_id']);
		}
		else
		{
			$query
				->clear()
				->insert('#__joomcrm_events_cf')
				->set(array('association_id = ' . (int) $cfdata['association_id'], 'association_type = ' . $db->quote($type), 'event_id = ' . (int) $cfdata['event_id']));
		}

		$db->setQuery($query)->execute();

		return true;
	}

	/**
	 * Add our excluded dates to the parent
	 *
	 * @param int $parent_id parent id to update
	 * @param     datetime   timestamp $date date to exclude
	 */
	public function addExcludes($parent_id, $date)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('excludes')
			->from('#__joomcrm_events')
			->where('id = ' . (int) $parent_id);

		$result = $db->setQuery($query)->loadResult();

		// Unserialize excludes
		$result = unserialize($result);
		$result = is_array($result) ? $result : array();

		// Add new dates to excludes
		$result[] = $date;

		// Serialize excludes
		$result = serialize($result);

		//write new information to database
		$query
			->clear()
			->update('#__joomcrm_events')
			->set('excludes = ' . $db->quote($result))
			->where("id = " . $parent_id);

		$db->setQuery($query)->execute();
	}

	/**
	 * Manually update parent events to reflect changes on virtual generated events
	 *
	 * @param int   $parent_id the id of the parent event to update
	 * @param mixed $data      the array of data to update the parent with
	 */
	public function updateEvent($parent_id, $data)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Unset values
		unset($data['due_date']);
		unset($data['start_time']);
		unset($data['end_time']);
		unset($data['id']);
		unset($data['update_future_events']);
		unset($data['parent_id']);
		unset($data['task']);
		unset($data['created']);
		unset($data['calendar_filter']);

		// Construct values
		$values = array();

		foreach ($data as $key => $info)
		{
			$values[] = $key . ' = ' . $db->quote($info);
		}

		$query
			->update("#__joomcrm_events")
			->set($values)
			->where('id = ' . (int) $parent_id);

		$db->setQuery($query)->execute();
	}

	/**
	 * Remove an event or series of events
	 * This will either :
	 * Remove the parent OR
	 * Add excluded dates to the parent id OR
	 * If removing the parent event ONLY we will change the event date to the next respective date in the series
	 *
	 * @param int    $id   of the REAL event we wish to modify
	 * @param string $type 'single','series'
	 */
	function removeEvent($id = null, $type = null)
	{
		$type       = ($type == null) ? $this->app->input->getString('type') : $type;
		$date       = $this->app->input->getString('date');
		$repeats    = $this->app->input->getString('repeats');
		$event_type = $this->app->input->getString('event_type', $this->app->input->getString('type'));
		$data       = Factory::getApplication()->input->post->getArray();
		if ($id != null)
		{
			$data['event_id'] = $id;
		}

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		// Remove an entire series
		if ($type == 'series')
		{
			$id = $this->app->input->getString('parent_id',0);
			if ($id == 0)
			{
				$id = $this->app->input->getString('event_id');
			}
			$query
				->update('#__joomcrm_events')
				->set("published=-1")
				->where('id=' . $id . " OR parent_id=" . $id);
			// Run database query
			$db->setQuery($query)->execute();
		}

		// Load Tables
		$rowId  = $id ? $id : $data['event_id'];
		$oldRow = new JoomcrmTableEvent;
		$oldRow->load($rowId);

		// Remove individual events
		if ($type == 'single')
		{
			// If we are receiving the parent id
			if ($data['parent_id'] == 0)
			{
				if (!array_key_exists('repeats', $data) || $data['repeats'] == "" || $data['repeats'] == "none")
				{
					// Delete event entry
					$query
						->update('#__joomcrm_events')
						->set("published=-1")
						->where('id = ' . (int) $data['event_id']);

					$db->setQuery($query)->execute();

					// Delete from cf table
					// $query->clear();
					// $query->delete("#__joomcrm_events_cf")->where('event_id='.$data['event_id']);
					// $db->setQuery($query);
					// $db->execute();

				}
				else
				{
					// Determine virtual events to generate
					switch ($data['repeats'])
					{
						// Daily
						case "daily":
							if ($event_type == 'event')
							{
								$start_time = $this->offsetTime($data['start_time'], "+1 days");
								$end_time   = $this->offsetTime($data['end_time'], "+1 days");
							}
							else
							{
								$due_date = $this->offsetTime((string)$data['due_date'], "+1 days");
							}
							break;
						//Weekdays
						case "weekdays":
							$days = array(1, 2, 3, 4, 5);
							if ($event_type == 'event')
							{
								$start_time = $this->offsetTime($data['start_time'], "+1 days");
								$end_time   = $this->offsetTime($data['end_time'], "+1 days");

								while (!in_array(date('w', strtotime($start_time)), $days))
								{
									$start_time = $this->offsetTime($start_time, "+1 days");
									$end_time   = $this->offsetTime($end_time, "+1 days");
								}
							}
							else
							{
								$due_date = $this->offsetTime($data['due_date'], "+1 days");

								while (!in_array(date('w', strtotime($due_date)), $days))
								{
									$due_date = $this->offsetTime($due_date, "+1 days");
								}
							}
							break;
						//Weekly
						case "weekly":
							if ($event_type == 'event')
							{
								$start_time = $this->offsetTime($data['start_time'], "+1 week");
								$end_time   = $this->offsetTime($data['end_time'], "+1 week");
							}
							else
							{
								$due_date = $this->offsetTime($data['due_date'], "+1 week");
							}
							break;
						//Weekly Monday Wednesday and Friday
						case "weekly-mwf":
							$days = array(1, 3, 5);
							if ($event_type == 'event')
							{
								$start_time = $this->offsetTime($data['start_time'], "+1 days");
								$end_time   = $this->offsetTime($data['end_time'], "+1 days");

								while (!in_array(date('w', strtotime($start_time)), $days))
								{
									$start_time = $this->offsetTime($start_time, "+1 days");
									$end_time   = $this->offsetTime($end_time, "+1 days");
								}
							}
							else
							{
								$due_date = $this->offsetTime($data['due_date'], "+1 days");

								while (!in_array(date('w', strtotime($due_date)), $days))
								{
									$due_date = $this->offsetTime($due_date, "+1 days");
								}
							}
							break;
						//Weekly Tuesday Thursday
						case "weekly-tr":
							$days = array(2, 4);
							if ($event_type == 'event')
							{
								$start_time = $this->offsetTime($data['start_time'], "+1 days");
								$end_time   = $this->offsetTime($data['end_time'], "+1 days");

								while (!in_array(date('w', strtotime($start_time)), $days))
								{
									$start_time = $this->offsetTime($start_time, "+1 days");
									$end_time   = $this->offsetTime($end_time, "+1 days");
								}
							}
							else
							{
								$due_date = $this->offsetTime($data['due_date'], "+1 days");

								while (!in_array(date('w', strtotime($due_date)), $days))
								{
									$due_date = $this->offsetTime($due_date, "+1 days");
								}
							}
							break;
						// Monthly
						case "monthly":
							if ($event_type == 'event')
							{
								$start_time = $this->offsetTime($data['start_time'], "+1 month");
								$end_time   = $this->offsetTime($data['end_time'], "+1 month");
							}
							else
							{
								$due_date = $this->offsetTime($data['due_date'], "+1 month");
							}
							break;
						// Yearly
						case "yearly":
							if ($event_type == 'event')
							{
								$start_time = $this->offsetTime($data['start_time'], "+1 year");
								$end_time   = $this->offsetTime($data['end_time'], "+1 year");
							}
							else
							{
								$due_date = $this->offsetTime($data['due_date'], "+1 year");
							}
							break;
					}
					$query->update("#__joomcrm_events");
					if ($event_type == 'event')
					{
						$query->set(array("start_time='" . $start_time . "'", "end_time='" . $end_time . "'"));
					}
					else
					{
						$query->set(array("due_date='" . $due_date . "'"));
					}
					$query->set('completed = 0')->where('id = ' . (int) $data['event_id']);

					$db->setQuery($query)->execute();
				}
			}
			else
			{
				if (array_key_exists('event_id', $data) && $data['event_id'] != 0)
				{
					$query->update('#__joomcrm_events')->set("published=-1")->where('id = ' . (int) $data['event_id']);

					$db->setQuery($query)->execute();
				}
				else
				{
					// If we are receiving a virtual event
					$this->addExcludes($data['parent_id'], $date);
				}
			}
		}

		$row = new JoomcrmTableEvent;
		$row->load($rowId);
		$status = "deleted";

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'event', $status);
	}

	/**
	 * Get the specified offset from the given time.
	 *
	 * Offset should be passed as "+1 week" or "+3 days"
	 *
	 * @param   string $time    [description]
	 * @param   string $offset  [description]
	 *
	 * @return  string           Formatted date string.
	 */
	public function offsetTime($time, $offset)
	{
		return date("Y-m-d H:i:s", strtotime(date("Y-m-d H:i:s", strtotime((string)$time)) . $offset));
	}

	/**
	 * Mark events as completed
	 *
	 * @return  [type]  [description]
	 */
	public function markComplete()
	{
		// Determine if we are editing a series of events of a single event
		$event_id   = $this->app->input->getString('event_id');
		$parent_id  = $this->app->input->getString('parent_id');
		$date       = $this->app->input->getString('due_date', $this->app->input->getString('date'));
		$event_type = $this->app->input->getString('event_type');
		$repeats    = $this->app->input->getString("repeats");
		$completed  = $this->app->input->getString('completed') != "" ? $this->app->input->getString('completed') : 1;
		// echo $this->app->input->getString('completed');

		// Load Tables
		$oldRow = new JoomcrmTableEvent;
		$oldRow->load($event_id);

		//We are only editing a single event entry OR a parent entry
		if ($repeats == 'none' || $repeats == "" /*|| $parent_id == 0*/)
		{
			$db    = $this->getDbo();
			$query = $db->getQuery(true);
			$date  = JoomcrmHelperDate::formatDBDate(date("Y-m-d H:i:s"));
			$query->update("#__joomcrm_events")->set(array('completed=' . $completed, 'actual_close="' . $date . '"'))
				->where('id = ' . (int) $event_id);

			$db->setQuery($query)->execute();

			//We are dealing with a virtually generated event
		}
		else
		{
			$id = $parent_id > 0 ? $parent_id : $event_id;

			// Clone the event and give it the correct // new unique completion date
			$event = $this->getEvent($id, false);

			// Add the event to the parent exclusion
			$exp1        = explode(" ", $date);
			$append      = $event['type'] == "task" ? $event['due_date'] : $event['start_time'];
			$exp2        = explode(" ", $append);
			@$excludeDate = $exp1[0] . " " . $exp2[1];

			$this->addExcludes($id, $excludeDate);

			// Merge arrays
			$data              = Factory::getApplication()->input->post->getArray();
			$data['completed'] = $completed;
			$new_data          = array_merge($event, $data);

			if ($event['type'] == "task")
			{
				$new_data['due_date'] = JoomcrmHelperDate::formatDate($excludeDate, false, false);
				$new_data['type']     = "task";
			}
			else
			{
				$new_data['start_time'] = JoomcrmHelperDate::formatDate($excludeDate, false, false);
				$new_data['end_time']   = JoomcrmHelperDate::formatDate($excludeDate, false, false);
				$new_data['type']       = "event";
			}

			unset($new_data['id']);
			unset($new_data['repeats']);
			unset($new_data['excludes']);
			// unset($new_data['parent_id']);
			foreach ($new_data as $key => $data)
			{
				if (is_null($data))
				{
					$new_data[$key] = 0;
				}
			}


			// Store new data
			$id    = $this->store($new_data);
			$event = $this->getEvent($id);
			// Echo json_encode($event);

		}

		$row = new JoomcrmTableEvent;
		$row->load($event_id);
		$status = "completed";

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'event', $status);

	}

	/**
	 * Mark events incomplete
	 */
	public function markIncomplete()
	{
		// Determine if we are editing a series of events of a single event
		$event_id   = $this->app->input->getString('event_id');
		$parent_id  = $this->app->input->getString('parent_id');
		$date       = $this->app->input->getString('date');
		$event_type = $this->app->input->getString('event_type');
		$repeats    = $this->app->input->getString("repeats");

		$db    = $this->getDbo();
		$query = $db->getQuery(true)->update("#__joomcrm_events")
			->set(array('completed=0', 'actual_close="0000-00-00 00:00:00"'))->where("id=" . $event_id);

		$db->setQuery($query)->execute();
	}

	/**
	 * Postpone events
	 *
	 * @param   [type]  $days      [description]
	 * @param   [type]  $event_id  [description]
	 *
	 * @return  [type]             [description]
	 */
	public function postponeEvent($days = null, $event_id = null)
	{
		$event_id = ($event_id == null) ? $this->app->input->getString('event_id') : $event_id;
		$days     = ($days == null) ? $this->app->input->getString("days") : $days;

		$db    = $this->getDbo();
		$query = $db->getQuery(true)->select("e.type,e.due_date,e.start_time,e.end_time")->from("#__joomcrm_events AS e")
			->where("e.id=" . $event_id);

		$dates = $db->setQuery($query)->loadObjectList();

		// Load Tables
		$oldRow = new JoomcrmTableEvent;
		$oldRow->load($event_id);

		if (count($dates) > 0)
		{
			foreach ($dates as $date)
			{
				$query->clear();
				$query->update('#__joomcrm_events');
				if ($date->type == "task")
				{
					$due_date = $this->offsetTime($date->due_date, "+" . $days . " days");
					$query->set(array("due_date='" . $due_date . "'"));
				}
				else
				{
					$start_time = $this->offsetTime($date->start_time, "+" . $days . " days");
					$end_time   = $this->offsetTime($date->end_time, "+" . $days . " days");
					$query->set(array("start_time='" . $start_time . "'", "end_time='" . $end_time . "'"));
				}
				$query->where("id=" . $event_id);

				$db->setQuery($query)->execute();
			}
		}

		$row = new JoomcrmTableEvent;
		$row->load($event_id);
		$status = "postponed";

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'event', $status);

		return true;

	}

	public function getNextEvent()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->select("event.*,ecf.*")->from("#__joomcrm_events_cf AS ecf");

		if ($this->company_id)
		{
			$query->leftJoin("#__joomcrm_events AS event ON event.id=ecf.event_id AND ecf.association_type='company'")
				->where("ecf.association_id=" . $this->company_id)->where("ecf.association_type='company'");
		}
		else
		{
			if ($this->person_id)
			{
				$query->leftJoin("#__joomcrm_events AS event ON event.id=ecf.event_id AND ecf.association_type='person'")
					->where("ecf.association_id=" . $this->person_id)->where("ecf.association_type='person'");
			}
			else
			{
				if ($this->deal_id)
				{
					$query
						->leftJoin("#__joomcrm_events AS event ON event.id=ecf.event_id AND ecf.association_type='deal'")
						->where("ecf.association_id=" . $this->deal_id)->where("ecf.association_type='deal'");
				}
			}
		}

		$query->order(" CASE WHEN event.type='task' THEN due_date END ASC,
						CASE WHEN event.type='event' THEN start_time END ASC  ");

		$query->where("event.completed=0");

		$event = $db->setQuery($query, 0, 1)->loadObject();

		if ($event && $event->type == "event")
		{
			$event->due_date = $event->start_time;
		}

		return $event;
	}

	/**
	 * Populate user state requests
	 */
	public function populateState()
	{
		// if ( $view == "events" && ( $layout == "default" || $layout == "list" || $layout == null ) ) {

		// Get pagination request variables
		$limit      = $this->app->getUserStateFromRequest("Event." . $this->view . '_' . $this->layout . '_limit', 'limit', 10);
		$limitstart = $this->app->getUserStateFromRequest("Event." . $this->view . '_' . $this->layout . '_limitstart', 'limitstart', 0);

		// In case limit has been changed, adjust it
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		$input = Factory::getApplication()->input;


		$this->setState("Event." . $this->view . '_limit', $limit);
		$this->setState("Event." . $this->view . '_limitstart', $limitstart);

		// Set default filter states for reports
		$filterOrder             = "CASE e.type WHEN 'event' THEN e.start_time WHEN 'task' THEN e.due_date ELSE e.due_date END";
		$filterOrderDir          = "ASC";
		$filter_order            = $this->app->getUserStateFromRequest('Event.' . $this->view . '_' . $this->layout . '_filter_order', 'filter_order', $filterOrder);
		$filter_order_Dir        = $this->app->getUserStateFromRequest('Event.' . $this->view . '_' . $this->layout . '_filter_order_Dir', 'filter_order_Dir', $filterOrderDir);
		$status_filter           = $this->app->getUserStateFromRequest('Event.' . $this->view . '_' . $this->layout . '_status', 'status', 0);
		$type_filter             = $this->app->getUserStateFromRequest('Event.' . $this->view . '_' . $this->layout . '_type', 'type', 'all');
		$category_filter         = $this->app->getUserStateFromRequest('Event.' . $this->view . '_' . $this->layout . '_category', 'category', 'any');
		$due_date_filter         = $this->app->getUserStateFromRequest('Event.' . $this->view . '_' . $this->layout . '_due_date', 'due_date', 'any');
		$association_type_filter = $this->app->getUserStateFromRequest('Event.' . $this->view . '_' . $this->layout . '_association_type', 'association_type', 'any');
		$assignee_id_filter      = $this->app->getUserStateFromRequest('Event.' . $this->view . '_' . $this->layout . '_assignee_id', 'assignee_id', JoomcrmHelperUsers::getUserId());
		$assignee_filter_type    = $this->app->getUserStateFromRequest('Event.' . $this->view . '_' . $this->layout . '_assignee_filter_type', 'assignee_filter_type', 'individual');

		// Set states for reports
		$this->setState('Event.' . $this->view . '_' . $this->layout . '_filter_order', $filter_order);
		$this->setState('Event.' . $this->view . '_' . $this->layout . '_filter_order_Dir', $filter_order_Dir);
		$this->setState('Event.' . $this->view . '_' . $this->layout . '_status', $status_filter);
		$this->setState('Event.' . $this->view . '_' . $this->layout . '_type', $type_filter);
		$this->setState('Event.' . $this->view . '_' . $this->layout . '_category', $category_filter);
		$this->setState('Event.' . $this->view . '_' . $this->layout . '_due_date', $due_date_filter);
		$this->setState('Event.' . $this->view . '_' . $this->layout . '_association_type', $association_type_filter);
		$this->setState('Event.' . $this->view . '_' . $this->layout . '_assignee_id', $assignee_id_filter);
		$this->setState('Event.' . $this->view . '_' . $this->layout . '_assignee_filter_type', $assignee_filter_type);

	}

	/**
	 * Get the iCal feed for a specific user
	 * @param  int $uid The ID of the user
	 * @return string
	 */
	function getIcal($uid)
	{
		$events = $this->getEvents();

		$ical = "BEGIN:VCALENDAR
VERSION:2.0
PRODID:-//JoomCRM/User Calendar//v1.0//EN
";

		foreach ($events as $event)
		{
			if ($event['type'] == 'task')
			{
				if ($event['due_date'] == '0000-00-00 00:00:00')
				{
					// do not export tasks without due date
					continue;
				}
				$dtstart = date("Ymd\THi00", strtotime($event["due_date"]));
				$dtend = date("Ymd\THi00", strtotime($event["due_date"]));
			}
			else
			{
				$dtstart = date("Ymd\THi00", strtotime($event["start_time"]));
				$dtend = date("Ymd\THi00", strtotime($event["end_time"]));
			}

			$ical .= "BEGIN:VEVENT
DTSTART:".$dtstart."
DTEND:".$dtend."
SUMMARY:".$event['name']."
DESCRIPTION:".$event['description']."
END:VEVENT
";
		}

		$ical .= "END:VCALENDAR";

		return $ical;
	}

}
