<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;
class JoomcrmModelExport extends JoomcrmModel
{
	/**
	 * Dynamically download CSV files based on type requested
	 *
	 * @return  [type]  [description]
	 */
	public function getCsv()
	{
		$download_type = $this->app->input->get('list_type');
		$data          = $this->getCsvData($download_type);

		return $this->generateCsv($data['header'], $data['rows']);
	}

	/**
	 * Get CSV data
	 *
	 * @param   [type]  $data_type  [description]
	 *
	 * @return  [type]  [description]
	 */
	public function getCsvData($data_type)
	{
		$data = array();

		$export_ids = $this->app->input->get('ids', '', 'RAW');

		switch ($data_type)
		{
			case "deals":
				$model = new JoomcrmModelDeal;
				$model->set("_id", $export_ids);
				$model->set('limit', 0);
				$model->set('view', $data_type);
				$data = $model->getDeals($export_ids);
				break;
			case "companies":
				$model = new JoomcrmModelCompany;
				$model->set('export_limit', 0);
				$data  = $model->getCompanies($export_ids);
				break;
			case "people":
				$model = new JoomcrmModelPeople;
				$model->set("_id", $export_ids);
				$model->set('limit', 0);
				$data = $model->getPeople();
				break;
			case "sales_pipeline":
				$model = new JoomcrmModelDeal;
				$model->set("_id", $export_ids);
				$model->set('limit',0);
				$data = $model->getReportDeals($export_ids);
				break;
			case "source_report":
				$model = new JoomcrmModelDeal;
				$model->set("_id", $export_ids);
				$model->set('limit',0);
				$data = $model->getDeals($export_ids);
				break;
			case "roi_report":
				$model = new JoomcrmModelSource;
				$data  = $model->getRoiSources($export_ids);
				break;
			case "notes":
				$model = new JoomcrmModelNote;
				$data  = $model->getNotes(null, null, false, 1000);
				break;
			case "custom_report":
				$model = new JoomcrmModelReport;
				$data  = $model->getCustomReportData( \Joomla\CMS\Factory::getApplication()->input->get('report_id'));
				break;
		}

		$data_type       = $data_type == "custom_report" ? "deals" : $data_type;
		$getCustomFields = array("deals", "companies", "people");


		if (in_array($data_type, $getCustomFields))
		{
			if (count($data) > 0)
			{
				foreach ($data as $key => $item)
				{
					$customData = JoomcrmHelperCustom::getCustomCsvData($item['id'], $data_type);

					foreach ($customData as $name => $value)
					{
						$name = str_replace('.','',$name);

						$data[$key][$name . "_custom"] = $value;
					}
					unset($data[$key]['id']);
				}
			}
		}

		$header = array();
		if (count($data))
		{
			$header = array_keys($data[0]);
		}

		return array('header' => $header, 'rows' => $data);
	}

	/**
	 * Generate CSV
	 *
	 * @param   [type]  $header  [description]
	 * @param   [type]  $data    [description]
	 *
	 * @return  [type]  [description]
	 */
	public function generateCsv($header, $data)
	{
		$str = "";

		if (count($header) > 0)
		{
			$str .= implode(',', $header) . "\r\n";
		}

		if (count($data))
		{
			foreach ($data as $row)
			{
				$arr = array();

				foreach ($row as $cell)
				{
					$arr[] = '"' . $cell . '"';
				}

				$str .= implode(",", $arr) . "\r\n";
			}
		}

		return $str;
	}

	/**
	 * Generate vcards for people
	 *
	 * @return  [type]  [description]
	 */
	public function getVcard()
	{
		$person_id = $this->app->input->getInt('person_id');

		$model = new JoomcrmModelPeople;

		$person = $model->getPerson($person_id);
		$person = $person[0];

		$str = "";

		$str .= "BEGIN:VCARD\r\n";
		$str .= "VERSION:4.0\r\n";
		$str .= "N:" . $person['last_name'] . ";" . $person['first_name'] . ";;;\r\n";
		$str .= "FN: " . $person['first_name'] . " " . $person['last_name'] . "\r\n";
		$str .= "ORG:" . $person['company_name'] . "\r\n";
		$str .= "TITLE:" . $person['position'] . "\r\n";
		$str .= 'TEL;TYPE="work,voice";VALUE=uri:tel:+' . $person['phone'] . "\r\n";
		$str .= 'TEL;TYPE="mobile,voice";VALUE=uri:tel:+' . $person['mobile_phone'] . "\r\n";
		$str .= 'ADR;TYPE=work:;;' . $person['work_address_1'] . " " . $person['work_address_2'] . ';' . $person['work_city'] . ';' . $person['work_state'] . ';' . $person['work_zip'] . ';' . $person['work_country'] . "\r\n";
		$str .= "EMAIL:" . $person['email'] . "\r\n";
		$str .= "END:VCARD\r\n";

		return $str;
	}

}
