<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;
class JoomcrmModelGoal extends JoomcrmModel
{
	public $published = 1;

	/**
	 * Method to store a record
	 *
	 * @return    boolean    True on success
	 */
	public function store()
	{
		// Load Tables
		$row    = new JoomcrmTableGoal;
		$oldRow = new JoomcrmTableGoal;

		$data = \Joomla\CMS\Factory::getApplication()->input->post->getArray();

		if (array_key_exists('name', $data))
		{
			$data['name'] = trim($data['name']);
		}

		// Date generation
		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

		if (!array_key_exists('id', $data))
		{
			$data['created'] = $date;
			$status          = "created";
		}
		else
		{
			$row->load($data['id']);
			$oldRow->load($data['id']);
			$status = "updated";
		}

		// Assign checkboxes
		if (array_key_exists('leaderboard', $data))
		{
			$data['leaderboard'] = 1;
		}
		else
		{
			$data['leaderboard'] = 0;
		}

		// Assign owner id
		$data['owner_id'] = JoomcrmHelperUsers::getUserId();

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'goal', $status);

		return true;
	}

	public function getGoal($goalId = null)
	{
		if (is_null($goalId))
		{
			$goalId = $this->app->input->get('id');
		}

		$db    = $this->getDbo();
		$query = $db->getQuery(true)->select("g.*")->from("#__joomcrm_goals AS g")->where("g.id=" . $goalId);

		return $db->setQuery($query)->loadAssoc();
	}

	/**
	 * Get Individual Goals
	 *
	 * @param int $id specific individual to search for
	 *
	 * @return mixed $results goals matched
	 */
	public function getIndividualGoals($id = null)
	{
		// Load database
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Load goals associated with an individual
		$member_id = JoomcrmHelperUsers::getUserId();

		$query->select("g.*, u.first_name, u.last_name")->from("#__joomcrm_goals AS g");

		// If we are seaching for a specific individual
		if ($id)
		{
			$query->where("g.assigned_type='member' AND g.assigned_id=$id");
		}
		else
		{
			// Else get logged in member
			$query->where("g.assigned_type='member' AND g.assigned_id=$member_id");
		}

		$query->leftJoin("#__joomcrm_users AS u ON u.id = g.assigned_id AND u.published=1")
		->where("g.published = " . (int) $this->published);

		$results = $db->setQuery($query)->loadAssocList();

		// Get essential data
		$this->won_stage_ids = JoomcrmHelperDeal::getWonStages();

		if (count($results) > 0)
		{
			foreach ($results as $key => $goal)
			{
				$results[$key]['goal_info'] = $this->goalInfo($goal);
			}
		}

		return $results;
	}

	/**
	 * Get individual goals for executives
	 *
	 * @param   none
	 *
	 * @return  mixed  $results
	 *
	 * @todo    Refactor this into the method above with an addiational "assigned_type" filter.
	 */
	public function getExecIndividualGoals()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Load goals associated with an individual
		$member_id = JoomcrmHelperUsers::getUserId();

		$query->select("g.*, u.first_name, u.last_name")->from("#__joomcrm_goals AS g");

		// If we are seaching for a specific individual
		$query->where("g.assigned_type = 'member'")
		->leftJoin("#__joomcrm_users AS u ON u.id = g.assigned_id AND u.published=1")
		->where("g.published=" . $this->published);

		$results = $db->setQuery($query)->loadAssocList();

		// Get essential data
		$this->won_stage_ids = JoomcrmHelperDeal::getWonStages();
		if (count($results) > 0)
		{
			foreach ($results as $key => $goal)
			{
				$results[$key]['goal_info'] = $this->goalInfo($goal);
			}
		}

		return $results;
	}

	/**
	 * Get individual goals for managers
	 *
	 * @param none
	 *
	 * @return mixed $results
	 */
	public function getManagerIndividualGoals()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Load goals associated with an individual
		$member_id = JoomcrmHelperUsers::getUserId();
		$team_id   = JoomcrmHelperUsers::getTeamId($member_id);

		$query->select("g.*, u.first_name, u.last_name")->from("#__joomcrm_goals AS g");

		// Get team members
		$team_members = JoomcrmHelperUsers::getTeamUsers($team_id, true);

		// Filter by results having team ids
		$query->where("g.owner_id IN (" . implode(",", $team_members) . ")")->where("g.assigned_type = 'member'");

		// Left join users names
		$query->leftJoin("#__joomcrm_users AS u ON u.id = g.assigned_id AND u.published=1")
		->where("g.published=" . $this->published);

		$results = $db->setQuery($query)->loadAssocList();

		// Get essential data
		$this->won_stage_ids = JoomcrmHelperDeal::getWonStages();

		if (count($results) > 0)
		{
			foreach ($results as $key => $goal)
			{
				$results[$key]['goal_info'] = $this->goalInfo($goal);
			}
		}

		return $results;
	}

	/**
	 * Get Team Goals
	 *
	 * @param int $id specific team to search for
	 *
	 * @return mixed $results goals matched
	 */
	public function getTeamGoals($id = null)
	{
		// Get user team id
		$team_id = JoomcrmHelperUsers::getTeamId();

		// Load database
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Load goals associated with team id
		$query
		->select("g.*, u.first_name, u.last_name, IF(t.name!='', t.name, CONCAT(u.first_name, ' ', u.last_name)) AS team_name")
		->from("#__joomcrm_goals AS g");

		// If we are searching for a specific team
		if ($id)
		{
			$query->where("g.assigned_type='team' AND g.assigned_id=$id");
		}
		// Else load associated team if any
		else
		{
			$query->where("g.assigned_type='team' AND g.assigned_id=$team_id");
		}

		// Left join managers name
		$query->leftJoin("#__joomcrm_teams AS t ON t.team_id = g.assigned_id")
		->leftJoin("#__joomcrm_users AS u on u.id = t.leader_id AND u.published=1")
		->where("g.published=" . $this->published);

		$results = $db->setQuery($query)->loadAssocList();

		// Get essential data
		$this->won_stage_ids = JoomcrmHelperDeal::getWonStages();

		if (count($results) > 0)
		{
			foreach ($results as $key => $goal)
			{
				$results[$key]['goal_info'] = $this->goalInfo($goal);
			}
		}

		return $results;
	}

	/**
	 * Get Team Goals for Executives
	 *
	 * @param none
	 *
	 * @return mixed $results
	 */
	public function getExecTeamGoals()
	{
		// Load database
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Load goals associated with team id
		$query
		->select("g.*, u.first_name, u.last_name, IF(t.name!='', t.name, CONCAT(u.first_name, ' ', u.last_name)) AS team_name")
		->from("#__joomcrm_goals AS g");

		$db->setQuery($query);

		$query->where("g.assigned_type='team'")->leftJoin("#__joomcrm_teams AS t ON t.team_id = g.assigned_id")
		->leftJoin("#__joomcrm_users AS u on u.id = t.leader_id AND u.published=1")
		->where('g.published = ' . $this->published);

		$results = $db->setQuery($query)->loadAssocList();

		// Get essential data
		$this->won_stage_ids = JoomcrmHelperDeal::getWonStages();

		if (count($results) > 0)
		{
			foreach ($results as $key => $goal)
			{
				$results[$key]['goal_info'] = $this->goalInfo($goal);
			}
		}

		return $results;
	}

	/**
	 * Get Company Goals
	 *
	 * @param none
	 *
	 * @return mixed $results goals matched
	 */
	public function getCompanyGoals()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)->select("g.*")->from("#__joomcrm_goals AS g")->where("g.assigned_type = 'company'")
				 ->where('g.published = ' . (int) $this->published);

		$results = $db->setQuery($query)->loadAssocList();

		// Get essential data
		$this->won_stage_ids = JoomcrmHelperDeal::getWonStages();

		if (count($results) > 0)
		{
			foreach ($results as $key => $goal)
			{
				$results[$key]['goal_info'] = $this->goalInfo($goal);
			}
		}

		return $results;
	}

	/**
	 * Get a leaderboard entry
	 *
	 * @param int $id id of leaderboard to get
	 *
	 * @return mixed $results db results of leaderboard info
	 */
	public function getLeaderBoards($id = null)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("g.*")
			->from("#__joomcrm_goals AS g")
			->leftJoin("#__joomcrm_users AS u ON u.id = g.assigned_id AND u.published = 1")
			->where("g.leaderboard = 1");

		if ($id)
		{
			$query->where('g.id = ' . (int) $id);
		}
		else
		{
			// Filter based on member access roles
			$user_id     = JoomcrmHelperUsers::getUserId();
			$member_role = JoomcrmHelperUsers::getRole();
			$team_id     = JoomcrmHelperUsers::getTeamId();

			// more info about commenting this out at CRM-327
			// if ($member_role != 'exec')
			// {
			// 	if ($member_role == 'manager')
			// 	{
			// 		$query->where('u.team_id = ' . (int) $team_id);
			// 	}
			// 	else
			// 	{
			// 		$query->where('g.owner_id = ' . (int) $user_id);
			// 	}
			// }

		}

		$query->where('g.published = ' . (int) $this->published);

		$results = $db->setQuery($query)->loadAssocList();

		// Get data for leaderboard
		$count = count($results);
		for ($i = 0; $i < $count; $i++)
		{
			$results[$i]['members'] = $this->leaderboardInfo($results[$i]);
		}

		return $results;
	}

	/**
	 * Get Goal Information
	 *
	 * @param $goal mixed array of goal information
	 *
	 * @return $results int information relating to goal
	 */
	public function goalInfo($goal)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);
		$role  = JoomcrmHelperUsers::getRole();

		//win_cash
		if ($goal['goal_type'] == 'win_cash')
		{
			$query->select("SUM(amount)");
			$query->from("#__joomcrm_deals");
			// Filter based on company
			if ($goal['assigned_type'] != 'company')
			{
				// Filter by member
				if ($goal['assigned_type'] == 'member')
				{
					$query->where("owner_id=" . $goal['assigned_id']);
				}
				// Filter by team
				if ($goal['assigned_type'] == 'team')
				{
					// Get team members
					$team_members = JoomcrmHelperUsers::getTeamUsers($goal['assigned_id'], true);
					// Filter by results having team ids

					if (!$team_members && $role != 'exec')
					{
						$query->where("owner_id IN(" . implode(",", $team_members) . ")");
					}
				}
				$query->where("stage_id IN (" . implode(',', $this->won_stage_ids) . ")");
			}
			else
			{
				$query->where("stage_id IN (" . implode(',', $this->won_stage_ids) . ")");
			}
			// Filter by start and end date
			$query->where('modified >= ' . $db->quote($goal['start_date']))
			->where('modified <= ' . $db->quote($goal['end_date']));
		}

		//win_deals
		if ($goal['goal_type'] == 'win_deals')
		{
			$query->select("COUNT(id)")->from("#__joomcrm_deals");

			// Filter based on company
			if ($goal['assigned_type'] != 'company')
			{
				// Filter by member
				if ($goal['assigned_type'] == 'member')
				{
					$query->where("owner_id=" . $goal['assigned_id']);
				}
				// Filter by team
				if ($goal['assigned_type'] == 'team')
				{
					// Get team members
					$team_members = JoomcrmHelperUsers::getTeamUsers($goal['assigned_id'], true);
					// Filter by results having team ids
					$query->where("owner_id IN(" . implode(",", $team_members) . ")");
				}
				$query->where("stage_id IN (" . implode(',', $this->won_stage_ids) . ")");
			}
			else
			{
				$query->where("stage_id IN (" . implode(',', $this->won_stage_ids) . ")");
			}
			// Filter by start and end date
			$query->where('modified >= ' . $db->quote($goal['start_date']))
			->where('modified <= ' . $db->quote($goal['end_date']));
		}

		// Move_deals
		if ($goal['goal_type'] == 'move_deals')
		{
			$query->select("COUNT(id)")->from("#__joomcrm_deals");

			// Filter based on company
			if ($goal['assigned_type'] != 'company')
			{
				// Filter based on member
				if ($goal['assigned_type'] == 'member')
				{
					$query->where("owner_id=" . $goal['assigned_id']);
				}
				// Filter based on team
				if ($goal['assigned_type'] == 'team')
				{
					// Get team members
					$team_members = JoomcrmHelperUsers::getTeamUsers($goal['assigned_id'], true);
					// Filter by results having team ids
					$query->where("owner_id IN(" . implode(",", $team_members) . ")");
				}
				$query->where("stage_id=" . $goal['stage_id']);
			}
			else
			{
				$query->where("stage_id=" . $goal['stage_id']);
			}
			// Filter by start and end date
			$query->where('modified >= ' . $db->quote($goal['start_date']))
			->where('modified <= ' . $db->quote($goal['end_date']));
		}

		// Complete_tasks
		if ($goal['goal_type'] == 'complete_tasks')
		{
			$query->select("COUNT(id)")->from("#__joomcrm_events");

			// Filter based on company
			if ($goal['assigned_type'] != 'company')
			{
				// Filter based on member
				if ($goal['assigned_type'] == 'member')
				{
					$query->where("assignee_id=" . $goal['assigned_id']);
				}
				// Filter based on team
				if ($goal['assigned_type'] == 'team')
				{
					// Get team members
					$team_members = JoomcrmHelperUsers::getTeamUsers($goal['assigned_id'], true);
					// Filter by results having team ids
					$query->where("assignee_id IN(" . implode(",", $team_members) . ")");
				}
				if ($goal['category_id'] > 0)
				{
					$query->where("category_id=" . $goal['category_id']);
				}
				$query->where("completed=1");
			}
			else
			{
				if ($goal['category_id'] > 0)
				{
					$query->where("category_id=" . $goal['category_id']);
				}
				$query->where("completed=1");
			}
			// Filter by start and end date
			$query->where('modified >= ' . $db->quote($goal['start_date']))
			->where('modified <= ' . $db->quote($goal['end_date']));
		}

		//write_notes
		if ($goal['goal_type'] == 'write_notes')
		{
			$query->select("COUNT(id)")->from("#__joomcrm_notes");

			// Filter by company
			if ($goal['assigned_type'] != 'company')
			{
				// Filter based on member
				if ($goal['assigned_type'] == 'member')
				{
					$query->where("owner_id=" . $goal['assigned_id']);
				}
				// Filter based on team
				if ($goal['assigned_type'] == 'team')
				{
					// Get team members
					$team_members = JoomcrmHelperUsers::getTeamUsers($goal['assigned_id'], true);
					// Filter by results having team ids
					$query->where("owner_id IN(" . implode(",", $team_members) . ")");
				}
			}

			if ($goal['category_id'])
			{
				$query->where("category_id=" . (int) $goal['category_id']);
			}
			// Filter by start and end date
			$query
				->where('modified >= ' . $db->quote($goal['start_date']))
				->where('modified <= ' . $db->quote($goal['end_date']));
		}

		// Create_deals
		if ($goal['goal_type'] == 'create_deals')
		{
			$query->select("COUNT(id)")->from("#__joomcrm_deals");

			// Filter by company
			if ($goal['assigned_type'] != 'company')
			{
				// Filter based on member
				if ($goal['assigned_type'] == 'member')
				{
					$query->where("owner_id=" . $goal['assigned_id']);
				}
				// Filter based on team
				if ($goal['assigned_type'] == 'team')
				{
					// Get team members
					$team_members = JoomcrmHelperUsers::getTeamUsers($goal['assigned_id'], true);
					// Filter by results having team ids
					$query->where("owner_id IN(" . implode(",", $team_members) . ")");
				}
			}
			// Filter by start and end date
			$query->where('modified >= ' . $db->quote($goal['start_date']))
			->where('modified <= ' . $db->quote($goal['end_date']));
		}

		$query->where("published=" . $this->published);

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Get leaderboard info
	 *
	 * @param   mixed $leaderboard  leaderboard object
	 *
	 * @return  mixed  $results      leaderboard information
	 */
	public function leaderboardInfo($leaderboard)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Get won stage id
		$won_stage_ids = JoomcrmHelperDeal::getWonStages();

		// Assign start_date and end_date
		$start_date = $leaderboard['start_date'];
		$end_date   = $leaderboard['end_date'];

		// Select members from database and join essential data
		// Get members
		$query->select("u.id, u.first_name, u.last_name")->from("#__joomcrm_users AS u");

		// Left join data
		//win_cash
		if ($leaderboard['goal_type'] == 'win_cash')
		{
			$query->select("SUM(d.amount) AS cash_won")
			->leftJoin("#__joomcrm_deals AS d ON d.owner_id = u.id AND d.stage_id IN (" . implode(",", $won_stage_ids) . ") AND d.modified >= '$start_date' AND d.modified <= '$end_date' AND d.published>0")
			->order('SUM(d.amount) desc');
		}
		//win_deals
		if ($leaderboard['goal_type'] == 'win_deals')
		{
			$query->select("COUNT(d.id) AS deals_won")
			->leftJoin("#__joomcrm_deals AS d ON d.owner_id = u.id AND d.stage_id IN (" . implode(",", $won_stage_ids) . ") AND d.modified >= '$start_date' AND d.modified <= '$end_date' AND d.published>0")
			->order('COUNT(d.id) desc');
		}
		// Move_deals
		if ($leaderboard['goal_type'] == 'move_deals')
		{
			$query->select("COUNT(d.id) AS deals_moved")
			->leftJoin("#__joomcrm_deals AS d ON d.owner_id = u.id AND d.stage_id=" . $leaderboard['stage_id'] . " AND d.modified >= '$start_date' AND d.modified <= '$end_date' AND d.published>0")
			->order('COUNT(d.id) desc');
		}
		// Complete_tasks
		if ($leaderboard['goal_type'] == 'complete_tasks')
		{
			$extra = $leaderboard['category_id'] > 0 ? "AND e.category_id=" . $leaderboard['category_id'] : "";

			$query->select("COUNT(e.id) AS tasks_completed")
			->leftJoin("#__joomcrm_events AS e ON e.assignee_id = u.id AND e.completed=1 " . $extra . " AND e.modified >= '$start_date' AND e.modified <= '$end_date' AND e.published>0")
			->order('COUNT(e.id) desc');
		}
		//write_notes
		if ($leaderboard['goal_type'] == 'write_notes')
		{
			$query->select("COUNT(n.id) AS notes_written")
			->leftJoin("#__joomcrm_notes AS n ON n.owner_id = u.id AND n.category_id=" . $leaderboard['category_id'] . " AND n.created >= '$start_date' AND n.created <= '$end_date' AND n.published>0")
			->order('COUNT(n.id) desc');
		}
		// Create_deals
		if ($leaderboard['goal_type'] == 'create_deals')
		{
			$query->select("COUNT(d.id) AS deals_created")
			->leftJoin("#__joomcrm_deals AS d ON d.owner_id = u.id AND d.created >= '$start_date' AND d.created <= '$end_date' AND d.published>0")
			->order('COUNT(d.id) desc');
		}

		// Switch depending on leaderboard type
		switch ($leaderboard['assigned_type'])
		{
			case "team":
				$query->where("u.team_id=" . $leaderboard['assigned_id']);
				break;
			case "member":
				$query->where("u.id=" . $leaderboard['assigned_id']);
				break;
		}

		$query->where('u.published = 1')->group('u.id');

		return $db->setQuery($query)->loadAssoclist();
	}

}
