<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;



JLoader::register('JoomcrmHelperUsers', JPATH_COMPONENT . '/helpers/users.php');
BaseDatabaseModel::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_joomcrm/model', 'JoomcrmModel');

class JoomcrmModelMyteam extends JoomcrmModel
{
	/*
	 * Save invite in database
	 */
	public function saveInvite($data)
	{

		$invite            = new stdClass();
		$invite->team_id   = $data['team_id'];
		$invite->email     = $data['email'];
		$invite->token     = $data['token'];
		$invite->sent_date = Factory::getDate()->toSql();
		$invite->params    = $data['params'];

		$result = Factory::getDbo()->insertObject('#__joomcrm_invites', $invite);

		return $result;

	}

	/*
	 * Check if email already invited
	 */
	public function alreadyInvited($email)
	{

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$query
			->select("email")
			->from("#__joomcrm_invites")
			->where("email = '$email'");

		$db->setQuery($query);

		$result = $db->loadResult();

		return $result;

	}

	/*
	 * Check of user given id has a team
	 */
	public static function isTeamOwner($id)
	{

		return JoomcrmHelperUsers::isTeamOwner($id);
	}

	/*
	 * Get invite object
	 */
	public function getInvite($token, $id)
	{

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$query
			->select("*")
			->from("#__joomcrm_invites")
			->where("token = '$token' AND id = $id");

		$db->setQuery($query);

		$result = $db->loadObject();

		return !is_null($result) ? $result : false;
	}

	/*
	 * Check if email has a team
	 */
	public function  hasTeam($email)
	{
		// get user id from email
		$userId = JoomcrmHelperUsers::getUserIdFromEmail($email);

		if(is_null($userId) || empty($userId)) return false;

		$joomcrm_user_id = JoomcrmHelperUsers::getUserId($userId);

		if(is_null($joomcrm_user_id) || empty($joomcrm_user_id)) return false;

		$db = Factory::getDbo();

		$query = $db->getQuery(true);
		$query->select('*');
		$query->from($db->quoteName('#__joomcrm_users'));

		// where clause
		$query->where("id = $joomcrm_user_id AND published = 1");

		$db->setQuery($query);

		$result = $db->loadObject();

		if(is_null($result) || empty($result)){
			return false;
		}

		return true;
	}

	/*
	 * Add user to Joomla and JoomCRM users tables using email and team id
	 */
	public function addUserToTeam($invite)
	{

		// check if user already exist in joomla users table
		$juserid = JoomcrmHelperUsers::getUserIdFromEmail($invite->email);

		// create joomla user based on email only if not found in users table
		if (is_null($juserid))
		{

			$juser = $this->registerJUser($invite->email);

			if (!$juser)
			{
				return false;
			}
		}


		// get joomla user id by email
		$juserid = JoomcrmHelperUsers::getUserIdFromEmail($invite->email);


		// check if user already exist in JoomCRM database and unpublished
		if($this->userExistInTeam($juserid,$invite->team_id,'uid')){



			// joomcrm user id
			$joomcrm_user_id = JoomcrmHelperUsers::getUserId($juserid);

			$member = new stdClass();

			$member->id = $joomcrm_user_id;
			$member->published = 1; // publish user again
			$member->team_id = $invite->team_id; // update team user

			$result = Factory::getDbo()->updateObject('#__joomcrm_users', $member, 'id');

			return true;
		}


		// laod user object
		$user           = Factory::getUser($juserid);
		$invite->params = json_decode($invite->params);

		// create joomcrm user based on joomla user id and team id
		$usersModel = BaseDatabaseModel::getInstance('Users', 'JoomcrmModel');


		// create joomcrm user
		$usersModel->store([
			'uid'        => $user->id,
			'first_name' => $user->name,
			'last_name'  => '',
			'role_type'  => 'basic',
			'team_id'    => $invite->team_id,
			'color'      => ''
		]);

		return true;

	}

	/*
	 * Get current user team
	 */
	public function getMyTeam(){

		return Joomla\Utilities\ArrayHelper::toObject(JoomcrmHelperUsers::getTeamUsers());

	}

	// get current user Invites
	public function getMyInvites(){

		$team_id = JoomcrmHelperUsers::getTeamId();

		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$query
			->select("*")
			->from("#__joomcrm_invites")
			->where("team_id = $team_id");

		$db->setQuery($query);

		$result = $db->loadObjectList();


		return $result;

	}


	/*
	 * Check if exist in my team
	 */
	public function userExistInTeam($id, $team_id,$column = 'id'){
		$db = Factory::getDbo();

		$query = $db->getQuery(true);
		$query->select('*');
		$query->from($db->quoteName('#__joomcrm_users'));

		// where clause
		$query->where("$column = $id AND team_id = $team_id");

		$db->setQuery($query);

		$result = $db->loadObject();

		if(is_null($result) || empty($result)){
			return false;
		}

		return $result;
	}

	/*
	 * Check if given invite owner by given team
	 */
	public function isMyInvite($id,$team_id){
		$db = Factory::getDbo();

		$query = $db->getQuery(true);
		$query->select('*');
		$query->from($db->quoteName('#__joomcrm_invites'));

		// where clause
		$query->where("id = $id AND team_id = $team_id");

		$db->setQuery($query);

		$result = $db->loadObject();

		if(is_null($result) || empty($result)){
			return false;
		}

		return $result;
	}

	/*
	 * Remove invite by given id from invites table.
	 */
	public function removeInvite($id,$team_id = null)
	{

		$db = Factory::getDbo();

		$query = $db->getQuery(true);

		$query->delete($db->quoteName('#__joomcrm_invites'));

		// where clause
		$query->where("id = $id");
		if(!is_null($team_id)){
			$query->where("team_id = $team_id");
		}

		$db->setQuery($query);

		$result = $db->execute();

		return $result;

	}

	/*
	 * Register a new Joomla user using email only, password is auto generated and sent by email, username = email
	 */
	public function registerJUser($email)
	{

		// load language file
		$lang      = Factory::getLanguage();
		$extension = 'com_users';
		$lang->load($extension);

		// get global config
		$config = Factory::getConfig();

		// get component params
		$params = ComponentHelper::getParams('com_users');

		// get default user group after registration
		$system = $params->get('new_usertype', 2);

		// build data
		$data['groups'][] = $system;
		$data['name']     = explode('@', $email)[0];
		$data['email']    = $email;
		$data['username'] = $email;

		// get user instance
		$user = JUser::getInstance();

		// Bind the data.
		if (!$user->bind($data))
		{
			$this->setError(Text::sprintf('COM_USERS_REGISTRATION_BIND_FAILED', $user->getError()));

			return false;
		}

		// Load the users plugin group.
		PluginHelper::importPlugin('user');

		// Store the data.
		if (!$user->save())
		{
			$this->setError(Text::sprintf('COM_USERS_REGISTRATION_SAVE_FAILED', $user->getError()));

			return false;
		}

		// Compile the notification mail values.
		$data             = $user->getProperties();
		$data['fromname'] = $config->get('fromname');
		$data['mailfrom'] = $config->get('mailfrom');
		$data['sitename'] = $config->get('sitename');
		$data['siteurl']  =Uri::root();

		// email subject
		$emailSubject = Text::sprintf(
			'COM_USERS_EMAIL_ACCOUNT_DETAILS',
			$data['name'],
			$data['sitename']
		);

		// email body
		$emailBody = Text::sprintf(
			'COM_USERS_EMAIL_REGISTERED_BODY',
			$data['name'],
			$data['sitename'],
			$data['siteurl'],
			$data['username'],
			$data['password_clear']
		);

		// Send the registration email.
		Factory::getMailer()->sendMail($data['mailfrom'], $data['fromname'], $data['email'], $emailSubject, $emailBody);

		return true;

	}


	public function deleteMember($id){

		$db = Factory::getDbo();

		$query = $db->getQuery(true);

		$conditions = array(
			$db->quoteName('id') . ' = '.$id,
		);

		$query->delete($db->quoteName('#__joomcrm_users'));
		$query->where($conditions);

		$db->setQuery($query);

		$result = $db->execute();

		return $result;

	}



}
