<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Categories\Categories;
class JoomcrmModelNote extends JoomcrmModel
{
	public $_id = null;
	public $published = 1;
	public $public_notes;
	public $filters = true;
	public $_view = "Report.notes";
	public $view = "Report.notes";

	/**
	 * Method to store a record
	 *
	 * @return    boolean    True on success
	 */
	public function     store($data = null)
	{
		// Load Tables
		$row    = new JoomcrmTableNote;
		$oldRow = new JoomcrmTableNote;

		if ($data == null)
		{
			$data = \Joomla\CMS\Factory::getApplication()->input->post->getArray();
		}

		if (array_key_exists('name', $data))
		{
			$data['name'] = trim($data['name']);
		}

		if (array_key_exists('is_email', $data))
		{
			$model        = new JoomcrmModelMail;
			$email        = $model->getEmail($data['email_id']);
			$data['note'] = $email->getMessageBody();
		}

		/** check for and automatically associate and create primary contacts or people **/
		if (array_key_exists('person_name', $data) && $data['person_name'] != "")
		{
			$peopleModel    = new JoomcrmModelPeople;
			$existingPerson = $peopleModel->checkPersonName($data['person_name']);

			if ($existingPerson == "")
			{

				$pdata               = array();
				$name                = explode(" ", $data['person_name']);
				$pdata['first_name'] = array_key_exists(0, $name) ? $name[0] : "";
				$pdata['last_name']  = array_key_exists(1, $name) ? $name[1] : "";


				$data['person_id']   = $peopleModel->store($pdata);

				if(is_object($data['person_id']))
					$data['person_id'] = $data['person_id']->id;
			}
			else
			{
				$data['person_id'] = $existingPerson;
			}

		}

		/** check for and automatically associate and create deals **/
		if (array_key_exists('deal_name', $data) && $data['deal_name'] != "")
		{
			$dealModel    = new JoomcrmModelDeal;
			$existingDeal = $dealModel->checkDealName($data['deal_name']);

			if ($existingDeal == "")
			{
				$pdata           = array();
				$pdata['name']   = $data['deal_name'];
				$data['deal_id'] = $dealModel->store($pdata);
				if(is_object($data['deal_id']))
					$data['deal_id'] = $data['deal_id']->id;
			}
			else
			{
				$data['deal_id'] = $existingDeal;
			}

		}

		// Date generation
		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

		if (!array_key_exists('id', $data))
		{
			$data['created'] = $date;
			$status          = "created";
		}
		else
		{
			$row->load($data['id']);
			$oldRow->load($data['id']);
			$status = "updated";
		}

		$data['modified'] = $date;
		$data['owner_id'] = JoomcrmHelperUsers::getUserId();

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onBeforeNoteSave', array(&$row));

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		if (array_key_exists('id', $data))
		{
			$id = $data['id'];
		}
		else
		{
			$id = $this->getDbo()->insertId();
		}

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'note', $status);

		// Store email attachments
		if (array_key_exists('is_email', $data))
		{
			$model = new JoomcrmModelMail;
			$email = $model->getEmail($data['email_id']);
			$attachments = $email->getAttachments();

			if (!empty($attachments))
			{
				$model->storeAttachments($attachments, $data['person_id']);
			}
		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onAfterNoteSave', array(&$row));

		return $id;
	}

	/*
	 * Method to access a note
	 *
	 * @return array
	 */
	public function getNote($id)
	{
		$db = $this->getDBO();
		$query = $db->getQuery(true)
			->select("n.*, cat.name as category_name, company.name as company_name, deal.name as deal_name, person.first_name as person_first_name,person.last_name as person_last_name, owner.first_name as owner_first_name, owner.last_name as owner_last_name")
			->from("#__joomcrm_notes as n")
			->leftJoin("#__joomcrm_notes_categories AS cat ON cat.id = n.category_id")
			->leftJoin("#__joomcrm_companies AS company ON company.id = n.company_id AND company.published>0")
			->leftJoin("#__joomcrm_deals AS deal ON deal.id = n.deal_id AND deal.published>0")
			->leftJoin("#__joomcrm_people AS person on person.id = n.person_id AND person.published>0")
			->leftJoin('#__joomcrm_users AS owner ON owner.id = n.owner_id')
			->where("n.id = " . (int) $id)
			->where("n.published=" . (int) $this->published);

		$results = $db->setQuery($query)->loadAssocList();

		// Clean results
		if (count($results) > 0)
		{
			foreach ($results as $key => $note)
			{
				$results[$key]['created_formatted'] = JoomcrmHelperDate::formatDate($note['created']);
			}
		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onNoteLoad', array(&$results));

		// Return results
		return $results;
	}

	public function _buildQuery($object_id = null, $type = null, $display = true)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("n.*,cat.name as category_name,company.name as company_name,
							company.id as company_id,deal.name as deal_name,deal.id as deal_id,
							person.id as person_id,person.first_name as person_first_name,
							person.last_name as person_last_name,
							event.name as event_name, event.id as event_id,
							owner.last_name as owner_last_name, owner.first_name as owner_first_name")
			->from("#__joomcrm_notes as n")
			->leftJoin("#__joomcrm_notes_categories AS cat ON cat.id = n.category_id")
			->leftJoin("#__joomcrm_companies AS company ON company.id = n.company_id AND company.published>0")
			->leftJoin("#__joomcrm_events AS event ON event.id = n.event_id AND event.published>0")
			->leftJoin("#__joomcrm_deals AS deal ON deal.id = n.deal_id AND deal.published>0")
			->leftJoin("#__joomcrm_people AS person on n.person_id = person.id AND person.published>0")
			->leftJoin("#__joomcrm_users AS owner ON n.owner_id = owner.id");

		if ($this->filters)
		{
			$company_filter = $this->getState('company_name');
			if ($company_filter != null)
			{
				$query->where("company.name LIKE '%" . $company_filter . "%'");
			}
			// Deal
			$deal_filter = $this->getState('deal_name');
			if ($deal_filter != null)
			{
				$query->where("deal.name LIKE '%" . $deal_filter . "%'");
			}
			// Person
			$person_filter = $this->getState('person_name');
			if ($person_filter != null)
			{
				$query->where("person.first_name LIKE '%" . $person_filter . "%' OR person.last_name LIKE '%" . $person_filter . "%'");
			}

			// Owner
			$owner_filter = $this->getState('owner_id');
			if ($owner_filter != null && $owner_filter != "all")
			{
				$owner_type = $this->getState('owner_type');
				switch ($owner_type)
				{
					case "team":
						$team_member_ids = JoomcrmHelperUsers::getTeamUsers($owner_filter, true);
						if ($team_member_ids)
						{
							$query->where("n.owner_id IN (" . implode(',', $team_member_ids) . ")");
						}
						break;
					case "member":
						$query->where("n.owner_id=" . $owner_filter);
						break;
				}
			}

			// Created
			$created_filter = $this->getState('created');
			if ($created_filter != null)
			{
				$date = date("Y-m-d H:i:s");
				if ($created_filter == "this_week")
				{
					$date_info         = getDate(strtotime($date));
					$today             = $date_info['wday'];
					$days_to_remove    = -1 + $today;
					$days_to_add       = 7 - $today;
					$beginning_of_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - $days_to_remove days")));
					$end_of_week       = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " + $days_to_add days")));
					$query->where("n.modified >= '$beginning_of_week'");
					$query->where("n.modified < '$end_of_week'");
				}

				if ($created_filter == "last_week")
				{
					$date_info              = getDate(strtotime($date));
					$today                  = $date_info['wday'];
					$days_to_remove         = -1 + $today;
					$beginning_of_this_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - $days_to_remove days")));
					$beginning_of_last_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime($date . " - 7 days")));
					$query->where("n.created >= '$beginning_of_last_week'");
					$query->where("n.created < '$beginning_of_this_week'");
				}

				if ($created_filter == "this_month")
				{
					$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$next_month = date('Y-m-1 00:00:00', strtotime($date . " +1 month"));
					$query->where("n.created >= '$this_month'");
					$query->where("n.created < '$next_month'");
				}

				if ($created_filter == "last_month")
				{
					$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$last_month = date('Y-m-1 00:00:00', strtotime($date . " -1 month"));
					$query->where("n.created >= '$last_month'");
					$query->where("n.created < '$this_month'");
				}

				if ($created_filter == "today")
				{
					$today    = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00"));
					$tomorrow = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+1 day"));
					$query->where("n.created >= '$today'");
					$query->where("n.created < '$tomorrow'");
				}

				if ($created_filter == "yesterday")
				{
					$today     = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00"));
					$yesterday = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "-1 day"));
					$query->where("n.created >= '$yesterday'");
					$query->where("n.created < '$today'");
				}

			}

			// Category
			$category_filter = $this->getState('category_id');
			if ($category_filter != null)
			{
				$query->where("n.category_id=" . $category_filter);
			}

			if ($this->getState('filter_order'))
			{
				$query->order($this->getState('filter_order') . ' ' . $this->getState('filter_order_Dir'));
			}

		}

		if ($this->_id)
		{
			if (is_array($this->_id))
			{
				$query->where("n.id IN (" . implode(',', $this->_id) . ")");
			}
			else
			{
				$query->where('n.id= ' . (int) $this->_id);
			}
		}

		if ($object_id)
		{
			switch ($type)
			{
				case 'person':
					$query->where('n.person_id =' . $object_id);
					break;

				case 'company':
					$query->where('(n.company_id =' . $object_id . ' OR deal.company_id = ' . $object_id . ' OR person.company_id = ' . $object_id . ")");
					break;

				case 'deal':
					$query->where('n.deal_id=' . $object_id);
					break;
				case "event":
					$query->where("n.event_id=$object_id");
					break;
			}
		}

		/** ---------------------------------------------------------------
		 * Filter data using member role permissions
		 */
		$member_id   = JoomcrmHelperUsers::getUserId();
		$team_access = JoomcrmHelperUsers::getTeamAccess($member_id);
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();

		if ($this->public_notes != true)
		{
			if ($member_role != 'exec')
			{
				// Manager filter
				if ($member_role == 'manager' || (isset($team_id) && $team_access) )
				{
					$query->where('owner.team_id = ' . (int) $team_id);
				}
				else
				{
					// Basic user filter
					$query->where('n.owner_id = ' . (int) $member_id);
				}
			}
		}

		$query
			->where("n.published = " . $this->published)
			->order("n.modified DESC");

		return $query;
	}

	/*
	 * Method to access notes
	 *
	 * @return array
	 */
	public function getNotes($object_id = null, $type = null, $display = true, $forceLimit = null)
	{
		// if dataTable call, return only items
		if ($this->app->input->get('task') == 'getdata')
		{
			$display = false;
		}

		$db = $this->getDbo();
		$query = $this->_buildQuery($object_id, $type, $display);

		$limitStart = 0;
		$limit = 0;

		if ($this->filters)
		{
			$limit      = $this->getState('limit');
			$limitStart = $this->getState('limitstart');

			if (!$this->_id && $limit != 0)
			{
				if ($limitStart >= $this->getTotal())
				{
					$limitStart = 0;
					$limit      = 0;
					$limitStart = ($limit != 0) ? (floor($limitStart / $limit) * $limit) : 0;
					$this->setState('limit', $limit);
					$this->setState('limitstart', $limitStart);
				}
			}
		}

		if ($this->view == 'print')
		{
			$limit = 1000;
		}

		if ($forceLimit)
		{
			$limit = $forceLimit;
		}

		if ($query !== false)
		{
			$results = $db->setQuery($query, $limitStart, $limit)->loadAssocList();
		}
		else
		{
			$results = array();
		}

		// Clean results
		if (count($results) > 0)
		{
			foreach ($results as $key => $note)
			{
				$results[$key]['created_formatted'] = JoomcrmHelperDate::formatDate($note['created']);
			}
		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onNoteLoad', array(&$results));

		if (!$display)
		{
			return $results;
		}
		else
		{
			$config    = array('name' => 'note');
			$notesView = new JoomcrmHelperView($config);
			$notesView->notes = $results;

			return $notesView;
		}
	}

	/*
	 * method to get list of deals
	 */
	public function getNoteCategories()
	{
		$db = $this->getDbo();
		$query = $db->getQuery(true)
			->select("name,id FROM #__joomcrm_notes_categories")
			->order('ordering');

		$row    = $db->setQuery($query)->loadAssocList();
		$blank  = array(array('name' => Text::_('COM_JOOMCRM_NONE'), 'id' => 0));

		return array_merge($blank, $row);
	}

	public function populateState()
	{
		// HTTP request key => default value
		$filters = array(
			'limit' => 10,
			'limitstart' => 0,
			'filter_order' => 'company.name',
			'filter_order_Dir' => 'asc',
			'company_name' => null,
			'deal_name' => null,
			'person_name' => null,
			'owner_id' => null,
			'owner_type' => null,
			'created' => null,
			'category_id' => null
		);

		$datatableQuery = $this->app->input->get('datatableQuery', false);

		// In case limit has been changed, adjust it
		$filters['limitstart'] = ($filters['limit'] != 0 ? (floor($filters['limitstart'] / $filters['limit']) * $filters['limit']) : 0);

		foreach ($filters as $key => $default)
		{
			$uniqueName = 'Note.' . $this->view . '.' . $this->layout . '.' . $key;

			$requestVal = $this->app->getUserStateFromRequest($uniqueName, $key, $default);	

			if ($datatableQuery && $key == 'filter_order')
			{
				$requestVal = $this->getColumnName($requestVal);
			}

			$this->setState($key, $requestVal);
		}

		if ($datatableQuery)
		{
			$columns = $this->getColumns();
			$this->initColumnSearch($columns);
		}
	}

	/**
	 * Get total number of rows for pagination
	 */
	public function getTotal()
	{
		if (empty ($this->_total))
		{
			$query        = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
			$this->_total = $this->_total ? $this->_total : 0;
		}

		return $this->_total;
	}

	public function getPagination()
	{
		// Lets load the content if it doesn't already exist
		// if (empty($this->_pagination)) {
		$total             = $this->getTotal();
		$total             = $total ? $total : 0;
		$this->_pagination = new Pagination($total, $this->getState('limitstart'), $this->getState('limit'));

		// }
		return $this->_pagination;
	}

	/**
	*	Prepares Items data for the jQuery dataTable
	*
	*	@param 	array of items from DB
	*	@return array of modified items
	*/
	public function dataTablefy($items)
	{
		$dataTable = array();
		$user = Factory::getApplication()->getIdentity();

		if (is_array($items))
		{
			$count = count($items);

			for ($i = 0; $i < $count; $i++)
			{
				$personLink = '';
				$dealLink = '';
				$companyLink = '';

				if ($items[$i]['person_id'])
				{
					$personLink = '<a href="' . Route::_('index.php?option=com_joomcrm&view=people&layout=person&id=' . $items[$i]['person_id']) . '">';
					$personLink .= $items[$i]['person_first_name'] . ' ' . $items[$i]['person_last_name'];
					$personLink .= '</a>';
				}

				if ($items[$i]['company_id'])
				{
					$companyLink = '<a href="' . Route::_('index.php?option=com_joomcrm&view=companies&layout=company&id=' . $items[$i]['company_id']) . '">';
					$companyLink .= $items[$i]['company_name'];
					$companyLink .= '</a>';
				}

				if ($items[$i]['deal_id'])
				{
					$dealLink = '<a href="' . Route::_('index.php?option=com_joomcrm&view=deals&layout=deal&id=' . $items[$i]['deal_id']) . '">';
					$dealLink .= $items[$i]['deal_name'];
					$dealLink .= '</a>';
				}

				$dataTable[$i] = array();
				$dataTable[$i][] = '<input type="checkbox" name="ids[]" value="' . $items[$i]['id'] . '"/>';
				$dataTable[$i][] = $items[$i]['company_id'] ? $companyLink : '<div class="text-center muted">-</div>';
				$dataTable[$i][] = $items[$i]['deal_id'] ? $dealLink : '<div class="text-center muted">-</div>';
				$dataTable[$i][] = $items[$i]['person_id'] ? $personLink : '<div class="text-center muted">-</div>';
				$dataTable[$i][] = $items[$i]['owner_first_name'] . ' ' . $items[$i]['owner_last_name'];
				$dataTable[$i][] = JoomcrmHelperDate::formatDate($items[$i]['created']);
				$dataTable[$i][] = $items[$i]['category_name'];

				if (isset($items[$i]['name']) && $items[$i]['name'])
				{
					$dataTable[$i][] = "<b>" . $items[$i]['name'] . '</b><br />';
				}
				else
				{
					$note = $items[$i]['note'];

					// TODO it would be better to make extra long notes shorter with possibility to see the whole note
					// if (strlen($note) > 100)
					// {
					// 	$note = substr($note, 0, 70) . '...';
					// }

					$dataTable[$i][] = nl2br($note);
				}
			}
		}

		return $dataTable;
	}

	/**
	*	Match column name from jQuery dataTable with actual
	* 	name of the column from database to make ordering.
	*
	*	@param 	integer $columnNumber
	*	@return string
	*/
	public function getColumnName($columnNumber)
	{
		$columnName = 'n.id';
		$columns = $this->getColumns();

		if (isset($columns[$columnNumber]['name']))
		{
			$columnName = $columns[$columnNumber]['name'];
		}

		return $columnName;
	}

	/**
	*	Defines columns for dataTable
	*
	*	@return array
	*/
	public function getColumns()
	{
		$columns = array(
			array(
				'name' => 'n.id',
				'filter' => 'id'
			),
			array(
				'name' => 'company.name',
				'filter' => 'company_name'
			),
			array(
				'name' => 'deal.name',
				'filter' => 'deal_name'
			),
			array(
				'name' => 'person.last_name',
				'filter' => 'person_name'
			),
			array(
				'name' => 'owner.last_name',
				'filter' => 'owner_id'
			),
			array(
				'name' => 'n.created',
				'filter' => 'created'
			),
			array(
				'name' => 'cat.name',
				'filter' => 'category_id'
			)
		);

		return $columns;
	}

	/**
	*	Looks at the request and if search query for a column exists, 
	* 	it'll add it to the columns array.
	* 	Method also sets status of model and user state.
	*
	*	@param 	array $columns
	*	@return void
	*/
	public function initColumnSearch(&$columns)
	{
		$requestColumns = $this->app->input->get('columns', array(), 'ARRAY');

		foreach ($columns as $key => &$column)
		{
			$column['search'] = false;

			// proceed only if there is some value to set
			if (isset($requestColumns[$key]['search']['value']) && $requestColumns[$key]['search']['value'])
			{
				$column['search'] = $requestColumns[$key]['search']['value'];

				// if filter should be cleared
				if ($column['search'] == '__clear')
				{
					$column['search'] = '';
				}

				if ($column['filter'] == 'owner_id')
				{
					$values = explode(':', $column['search']);
					if (isset($values[1]))
					{
						$column['search'] = $values[0];
						$this->setState('owner_type', $values[1]);
					}
				}

				$uniqueName = 'Note.' . $this->view . '.' . $this->layout . '.' . $column['filter'];
				$this->setState($column['filter'], $column['search']);
				$this->app->setUserState($uniqueName, $column['search']);
			}
		}
	}
}
