<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Table\Table;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

class JoomcrmModelPeople extends JoomcrmModel
{
	public $_total = null;

	public $_id = null;

	public $person = null;

	public $recent = false;

	public $published = 1;

	public $company_id = null;

	public $event_id = null;

	public $type = null;

	public $limit = 1;

	public $limitStart = null;

	public $order = null;

	public $_pagination = null;

	public $filters = true;

	/**
	 * Constructor
	 */
	public function __construct()
	{
		parent::__construct();
		$this->_id  = $this->app->input->getInt('id');
	}

	/**
	 * Method to store a record
	 *
	 * @return    boolean    True on success
	 */
	public function store($data = null)
	{

		// Load Tables
		$row    = new JoomcrmTablePeople;
		$oldRow = new JoomcrmTablePeople;



		if ($data == null)
		{
			$data = \Joomla\CMS\Factory::getApplication()->input->post->getArray();
		}

		// Detect if we are importing a company
		$importing = array_key_exists('import', $data);

		if (array_key_exists('first_name', $data))
		{
			$data['first_name'] = trim($data['first_name']);
		}

		if (array_key_exists('last_name', $data))
		{
			$data['last_name'] = trim($data['last_name']);
		}

		if (array_key_exists('edit_screen', $data))
		{
			if (array_key_exists('type', $data))
			{
				$data['type'] = $data['type'] == 0 ? 'contact' : 'lead';
			}
		}


		// Date generation
		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));

		if (!array_key_exists('id', $data) && !array_key_exists('item_id', $data))
		{
			if (empty($data['created']))
			{
				$data['created']     = $date;
			}

			$data['owner_id']    = array_key_exists('owner_id', $data) && $data['owner_id'] > 0 ? $data['owner_id'] : JoomcrmHelperUsers::getUserId();
			$data['assignee_id'] = array_key_exists('assignee_id', $data) ? $data['assignee_id'] : JoomcrmHelperUsers::getUserId();
			$status              = "created";
		}
		else
		{

			$row->load($data['id']);
			$oldRow->load($data['id']);

			$status = "updated";
		}

		if (empty($data['modified']))
		{
			$data['modified'] = $date;
		}

		// change assignee_id the same as owner_id [CRM-314]
		if (isset($data['owner_id']) && $data['owner_id'])
		{
			$data['assignee_id'] = $data['owner_id'];
		}

		// Generate custom field string
		$customArray = array();
		foreach ($data as $name => $value)
		{

            if (strstr($name, 'custom_') && !strstr($name, '_input') && !strstr($name, "_hidden"))
            {
                $id            = str_replace('custom_', '', $name);
                $customArray[] = array('custom_field_id' => $id, 'custom_field_value' => $value);
                unset($data[$name]);
            }
		}



		if (!empty($data['company_name']) || !empty($data['company']))
		{

			$company_name = isset($data['company_name']) ? $data['company_name'] : $data['company'];
			$companyModel    = new JoomcrmModelCompany;
			$existingCompany = $companyModel->checkCompanyName($company_name);

			if (!$existingCompany)
			{
				$cdata              = array();

                // attach company custom fields if available
                if(isset($data['company_cf'])){

                    $cdata = $data['company_cf'];

                }

                $cdata['name']      = $company_name;

                // add also phone and email
                if(isset($data['email']) && !empty($data['email'])){
                    $cdata['email'] = $data['email'];
                }

                if(isset($data['phone']) && !empty($data['phone'])){
                    $cdata['phone'] = $data['phone'];
                }


				$data['company_id'] = $companyModel->store($cdata)->id;



			}
			else
			{
				$data['company_id'] = $existingCompany;
			}
		}

		if (array_key_exists('company_id', $data) && is_array($data['company_id']))
		{
			$company_name    = $data['company_id']['value'];
			$companyModel    = new JoomcrmModelCompany;
			$existingCompany = $companyModel->checkCompanyName($company_name);
			if ($existingCompany == "")
			{
				$cdata              = array();
				$cdata['name']      = $company_name;
				$data['company_id'] = $companyModel->store($cdata)->id;
			}
			else
			{
				$data['company_id'] = $existingCompany;
			}
		}

		/** retrieving joomla user id **/
		if (array_key_exists('email', $data))
		{
			$data['uid'] = self::associateJoomlaUser($data['email']);
		}

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		$row->custom_fields = $row->id > 0 ? JoomcrmHelperCustom::getCustomCSVData($row->id, "people") : array();
		$dispatcher         = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onBeforePersonSave', array(&$row));

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		$person_id = array_key_exists('id', $data) ? $data['id'] : $this->getDbo()->insertId();

		// update note for import
		if($importing)
			JoomcrmHelperNote::updateNoteForImport($data['note'],'person',$person_id,$row->owner_id);


		/** Updating the joomla user **/
		if (array_key_exists('uid', $data) && $data['uid'] != "")
		{
			$this->updateJoomlaUser($data);
		}

		JoomcrmHelperActivity::saveActivity($oldRow, $row, 'person', $status);

		// If we receive no custom post data do not modify the custom fields

		if (count($customArray) > 0)
		{
			JoomcrmHelperJoomcrm::storeCustomCf($person_id, $customArray, 'people');
		}

		// Bind to cf tables for deal & person association
		if (array_key_exists('deal_id', $data) && $data['deal_id'] != 0)
		{
			$data['id'] = $row->id;
			$deal       = array('association_id = ' . $data['deal_id'], 'association_type="deal"', 'person_id = ' . $row->id, "created = '$date'");

			if (!$this->dealsPeople($deal, $data))
			{
				return false;
			}
		}

		// Pass Status to plugin & form ID if available
		$row->status = $status;

		if (isset($data) && is_array($data) && array_key_exists('form_id', $data))
		{
			$row->form_id = $data['form_id'];
		}
		else
		{
			$row->form_id = '';
		}

		$row->custom_fields = $row->id > 0 ? JoomcrmHelperCustom::getCustomCSVData($row->id, "people") : array();

		\Joomla\CMS\Plugin\PluginHelper::importPlugin('user','joomcrm');

		$dispatcher         = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onAfterPersonSave', array(&$row));

		// Assign any default template systems ( events )
		if ($status == "created")
		{
			$templateModel = new JoomcrmModelTemplate;
			$templateModel->processDefaults($person_id, "person");
		}

		return $row;
	}

	public function updateJoomlaUser($data)
	{
		$name = $data['first_name'] . ' ' . $data['last_name'];

		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->update("#__users")
			->set(array(
				"email=" . $db->quote($data['email']),
				"name=" . $db->quote($name)
			))
			->where("id=" . $data['uid']);

		$db->setQuery($query)->execute();
	}

	/**
	 * Get ID from Joomla user which has provided email adress
	 * AND isn't associated with any JoomCRM user yet.
	 *
	 * @param string $email address
	 * @return integer
	 */
	public function associateJoomlaUser($email)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("u.id")
			->from("#__users u")
			->leftJoin('#__joomcrm_people p ON p.uid = u.id')
			->where("u.email = " . $db->quote($email))
			->where("p.uid = 0");

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Method to link deals and people in cf tables
	 *
	 * @param $cfdata
	 * @param $data
	 *
	 * @return bool
	 */
	public function dealsPeople($cfdata, $data)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("count(*)")
			->from("#__joomcrm_people_cf")
			->where("association_id=" . (int) $data['deal_id'])
			->where("association_type='deal'")
			->where("person_id=" . (int) $data['id']);

		$result = $db->setQuery($query)->loadResult();

		if ($result > 0)
		{
			return true;
		}
		else
		{
			$query
				->clear()
				->insert('#__joomcrm_people_cf')
				->set($cfdata);

			$db->setQuery($query);

			return (bool) $db->execute();
		}
	}

	/**
	 * Build our query
	 */
	public function _buildQuery()
	{
		/** Large SQL Selections **/
		$db    = $this->getDbo();
		$query = $db->getQuery(true);
		$isDatatableQuery = $this->app->input->get('datatableQuery',0,'int');
		$config = \Joomla\CMS\Component\ComponentHelper::getParams('com_joomcrm');

		$db->setQuery("SET SQL_BIG_SELECTS=1")->execute();

		// Retrieve person id
		if (!$this->_id)
		{
			// Get filters
			$type   = $this->app->input->getString('type', $this->type);
			$user   = $this->app->input->getString('user');
			$stage  = $this->app->input->getString('stage');
			$tag    = $this->app->input->getString('tag');
			$status = $this->app->input->getString('status');
			$name   = $this->app->input->getString('name', null);
			$team   = $this->app->input->getInt('team_id');

			// Get session data
			$session = Factory::getSession();


			if(!$isDatatableQuery){ // if not datatables query

				// Determine whether we are filtering with a team or user
				if ($team)
				{
					$session->set('people_user_filter', null);
				}
				if ($user)
				{
					$session->set('people_team_filter', null);
				}

				if ($name !== null)
				{
					$session->set('people_name_filter', $name);
				}
				else {
					$name = $session->get('people_name_filter');
				}


				// Set user session data
				if ($type != null)
				{
					$session->set('people_type_filter', $type);
				}
				else
				{
					$sess_type = $session->get('people_type_filter',$config->get('people_type_filter','all'));
					$type      = $sess_type;
				}
				if ($user != null)
				{
					$session->set('people_user_filter', $user);
				}
				else
				{
					$sess_user = $session->get('people_user_filter',$config->get('people_user_filter',1));
					$user      = $sess_user;
				}
				if ($stage != null)
				{
					$session->set('people_stage_filter', $stage);
				}
				else
				{
					$sess_stage = $session->get('people_stage_filter',$config->get('people_updated_filter','past_thirty'));
					$stage      = $sess_stage;
				}
				if ($tag != null)
				{
					$session->set('people_tag_filter', $tag);
				}
				else
				{
					$sess_tag = $session->get('people_tag_filter');
					$tag      = $sess_tag;
				}
				if ($status != null)
				{
					$session->set('people_status_filter', $status);
				}
				else
				{
					$sess_status = $session->get('people_status_filter',$config->get('people_status_filter','any'));
					$status      = $sess_status;
				}
				if ($team != null)
				{
					$session->set('people_team_filter', $team);
				}
				else
				{
					$sess_team = $session->get('people_team_filter');
					$team      = $sess_team;
				}
			}

		}

		$export = $this->app->input->getString('export');

		if ($export)
		{
			$query
				->select('p.id,p.first_name,p.last_name,p.position,p.phone,p.email,p.home_address_1,
					p.home_address_2,' . 'p.home_city,p.home_state,p.home_zip,p.home_country,p.fax,p.website,p.facebook_url,
					p.twitter_user,' . 'p.linkedin_url,p.created,p.type,p.info,p.modified,p.work_address_1,
					p.work_address_2,' . 'p.work_city,p.work_state,p.work_zip,p.work_country,
					p.assignment_note,p.mobile_phone,p.home_email,' . 'p.other_email,p.home_phone,
					c.name as company_name, CONCAT(u2.first_name," ",u2.last_name) AS assignee_name,
					' . 'CONCAT(u.first_name, " ", u.last_name) AS owner_name,p.owner_id, stat.name as status_name,
					' . 'source.name as source_name,' . "event.id AS event_id,event.type AS event_type,
					event.name AS event_name,IF(event.type='task',event.due_date,event.start_time) AS event_due_date")
				->from('#__joomcrm_people AS p')
				->leftJoin('#__joomcrm_companies AS c ON c.id = p.company_id AND c.published=1')
				->leftJoin('#__joomcrm_people_status AS stat ON stat.id = p.status_id')
				->leftJoin('#__joomcrm_sources AS source ON source.id = p.source_id')
				->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=p.id AND shared.item_type='person'")
				->leftJoin("#__joomcrm_users AS u ON u.id = p.owner_id")
				->leftJoin("#__joomcrm_users AS u2 ON u2.id = p.assignee_id");

		}
		else
		{
			$query
				->select('p.*,c.name as company_name, CONCAT(u2.first_name," ",u2.last_name) AS assignee_name,u.first_name AS owner_first_name,
							u.last_name AS owner_last_name, stat.name as status_name,stat.color as status_color,
							source.name as source_name,event.id as event_id,
							event.name as event_name, event.type as event_type,
							IF(event.type="task",event.due_date,event.start_time) AS event_due_date,event.description as event_description')
				->from('#__joomcrm_people AS p')
				->leftJoin('#__joomcrm_companies AS c ON c.id = p.company_id AND c.published=1')
				->leftJoin('#__joomcrm_people_status AS stat ON stat.id = p.status_id')
				->leftJoin('#__joomcrm_sources AS source ON source.id = p.source_id')
				->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=p.id AND shared.item_type='person'")
				->leftJoin("#__joomcrm_users AS u ON u.id = p.owner_id")
				->leftJoin("#__joomcrm_users AS u2 ON u2.id = p.assignee_id");
		}

		// group ids
		$query->group("p.id");

		/** ---------------------------------------------------------------
		 * Filter data using member role permissions
		 */
		$member_id        = JoomcrmHelperUsers::getUserId();
		$team_access	  = JoomcrmHelperUsers::getTeamAccess($member_id);
		$member_role      = JoomcrmHelperUsers::getRole();
		$team_id          = JoomcrmHelperUsers::getTeamId();
		$this->company_id = $this->app->input->getInt('company_id', $this->company_id);

		if ((isset($this->company_id) && $this->company_id > 0) || (isset($user) && $user == "all") || (isset($owner_filter) && $owner_filter == "all"))
		{
			if ($member_role != 'exec')
			{
				// Manager filter
				if ($member_role == 'manager' || (isset($team_id) && (int) $team_id > 0 && $team_access) )
				{
					$query->where("(u.team_id=$team_id OR u2.team_id=$team_id OR shared.user_id=$member_id)");
				}
				else
				{
					// Basic user filter
					$query->where("(p.owner_id=$member_id OR p.assignee_id=$member_id OR shared.user_id=$member_id)");
				}
			}
		}
		else
		{
			if (isset($team) && $team)
			{
				$query->where("(u.team_id=$team OR u2.team_id=$team)");
			}
			else
			{
				if (isset($user) && $user != "all")
				{
					$query->where("(p.owner_id=$user OR p.assignee_id=$user)");
				}
				else
				{
					if (!(isset($owner_filter)))
					{
						if ($this->_id)
						{
							// this fix an issue if current user role is executive and team access enabled, should be always disabled.
							if($member_role == 'exec') $team_access = 0;


							if ($member_role == "basic")
							{
								$query->where("(p.owner_id=$member_id OR p.assignee_id=$member_id OR shared.user_id=$member_id)");
							}
							if ($member_role == "manager" || (isset($team_id) && $team_access) )
							{
								$team_members = JoomcrmHelperUsers::getTeamUsers($team_id, true);
								$team_members = array_merge($team_members, array(0 => $member_id));
								$query->where("(p.owner_id IN(" . implode(',', $team_members) . ") OR p.assignee_id IN(" . implode(',', $team_members) . ") OR shared.user_id=" . $member_id . ")");
							}
						}
						else
						{
							$query->where("(p.owner_id=$member_id OR p.assignee_id=$member_id OR shared.user_id=$member_id)");
						}
					}
				}
			}
		}

		// Searching for specific person
		if ($this->_id && $this->filters)
		{
			if (is_array($this->_id))
			{
				$query->where("p.id IN (" . implode(',', $this->_id) . ")");
			}
			else
			{
				$query->where("p.id = " . (int) $this->_id);
			}
			// Get next event
			$query->leftJoin("#__joomcrm_events AS event ON event.id = (
									SELECT event2.id
									FROM #__joomcrm_events_cf AS ecf2
									LEFT JOIN #__joomcrm_events AS event2 ON
										event2.id=ecf2.event_id AND
										ecf2.association_type='person'
									WHERE ecf2.association_id=p.id AND
										  ecf2.association_type='person' AND
										  event2.completed=0
									ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
											 CASE WHEN event2.type='event' THEN start_Time END ASC
									LIMIT 1 )");
		}

		if (!$this->_id)
		{
			if (!$export)
			{
				// Filter data
				if ($type AND $type != 'all' && $this->filters)
				{
					switch ($type)
					{
						case 'leads':
							$query->where("p.type='lead'");
							break;
						case 'not_leads':
							$query->where("p.type='contact'");
							break;
						case "shared";
							$query->where("( shared.user_id=" . $member_id . " OR ( p.owner_id=" . $member_id . " AND shared.user_id IS NOT NULL ) )");
							break;
					}
				}

				// Search with status
				if ($status AND $status != 'any' && $this->filters)
				{
					$query->where('p.status_id=' . $status);
				}

				// Get current date
				$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00'));

				// Filter for type
				if ($stage != null && $stage != 'all' && $this->filters)
				{
					// Filter for deals// Tasks due today
					if ($stage == 'today')
					{
						$today    = JoomcrmHelperDate::formatDBDate(date('Y-m-d'), false) . " 00:00:00";
						$tomorrow = JoomcrmHelperDate::formatDBDate(date('Y-m-d', strtotime($today . " +1 days")), false) . " 00:00:00";
						$query
							->leftJoin("#__joomcrm_events_cf AS ecf ON ecf.association_type='person' AND ecf.association_id=p.id")
							->leftJoin("#__joomcrm_events AS event ON event.id=ecf.event_id")
							->where("( event.due_date > '$today' AND event.due_date < '$tomorrow' )")
							->where("event.published>0 AND event.completed=0");
					}

					// Filter for companies and deals// Tasks due tomorrow
					else
					{
						if ($stage == "tomorrow")
						{
							$tomorrow         = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() + (1 * 24 * 60 * 60)));
							$dayAfterTomorrow = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00", strtotime($tomorrow . " +1 days")));
							$query->leftJoin("#__joomcrm_events_cf AS ecf ON ecf.association_type='person' AND ecf.association_id=p.id");
							$query->leftJoin("#__joomcrm_events AS event ON event.id=ecf.event_id");
							$query->where("event.due_date >'" . $tomorrow . "' AND event.due_date < '$dayAfterTomorrow'");
							$query->where("event.published>0 AND event.completed=0");
						}
						else
						{
							// Get next event
							$query->leftJoin("#__joomcrm_events AS event ON event.id = (
												SELECT event2.id
												FROM #__joomcrm_events_cf AS ecf2
												LEFT JOIN #__joomcrm_events AS event2 ON
													event2.id=ecf2.event_id AND
													ecf2.association_type='person'
												WHERE ecf2.association_id=p.id AND
													  ecf2.association_type='person' AND
													  event2.completed=0
												ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
														 CASE WHEN event2.type='event' THEN start_Time END ASC
												LIMIT 1 )");

						}
					}

					// Filter for people updated in the last 30 days
					if ($stage == "past_thirty")
					{
						$last_thirty_days = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() - (30 * 24 * 60 * 60)));
						$query->where("p.modified >'$last_thirty_days'");
					}

					// Filter for recently added people
					if ($stage == "recently_added")
					{
						$last_five_days = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', time() - (5 * 24 * 60 * 60)));
						$query->where("p.modified >'$last_five_days'");
					}

					// Filter for last imported people
					if ($stage == "last_import")
					{
					}

				}
				else
				{
					// Get latest task entry

					if ($this->recent)
					{
						$query->where("( event.due_date IS NULL OR event.due_date=(SELECT MIN(e2.due_date) FROM #__joomcrm_events_cf e2cf " . "LEFT JOIN #__joomcrm_events as e2 on e2.id = e2cf.event_id " . "WHERE e2cf.association_id=p.id AND e2.published>0 AND e2.completed=0) )");
					}

					// Get next event
					$query->leftJoin("#__joomcrm_events AS event ON event.id = (
											SELECT event2.id
											FROM #__joomcrm_events_cf AS ecf2
											LEFT JOIN #__joomcrm_events AS event2 ON
												event2.id=ecf2.event_id AND
												ecf2.association_type='person'
											WHERE ecf2.association_id=p.id AND
												  ecf2.association_type='person' AND
												  event2.completed=0
											ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
													 CASE WHEN event2.type='event' THEN start_Time END ASC
											LIMIT 1 )");
				}

			}
			else
			{
				// Get next event
				$query->leftJoin("#__joomcrm_events AS event ON event.id = (
										SELECT event2.id
										FROM #__joomcrm_events_cf AS ecf2
										LEFT JOIN #__joomcrm_events AS event2 ON
											event2.id=ecf2.event_id AND
											ecf2.association_type='person'
										WHERE ecf2.association_id=p.id AND
											  ecf2.association_type='person' AND
											  event2.completed=0
										ORDER BY CASE WHEN event2.type='task' THEN due_date END ASC,
												 CASE WHEN event2.type='event' THEN start_Time END ASC
										LIMIT 1 )");
			}

			/** company filter **/
			if ($this->company_id)
			{
				$query->where("p.company_id=" . $this->company_id);
			}

			if ($this->event_id)
			{
				$query->where("event.id=" . $this->event_id);
			}

			/** person name filter **/
			if ($name != null && $this->filters)
			{
				$query->where("( p.first_name LIKE '%" . $db->escape($name) . "%' OR p.last_name LIKE '%" . $db->escape($name) . "%' OR CONCAT(p.first_name,' ',p.last_name) LIKE '%" . $db->escape($name) . "%')");
			}

		}

		$query->where("p.published=" . $this->published);

		return $query;
	}

	/*
	 * Method to access people
	 *
	 * @return array
	 */
	public function getPeople($id = null)
	{

		// Get query
		$db    = $this->getDbo();
		$query = $this->_buildQuery();

		/** ------------------------------------------
		 * Set query limits/ordering and load results
		 */
		$limit      = (int) $this->getState('People.' . $this->view . '.' . $this->layout . '.limit');
		$limitStart = (int) $this->getState('People.' . $this->view . '.' . $this->layout . '.limitstart');
		$order 		= $this->getState('People.' . $this->view . '.' . $this->layout . '.filter_order');
		$orderDir 	= $this->getState('People.' . $this->view . '.' . $this->layout . '.filter_order_Dir');


		if ($this->order != null)
		{
			$query->order($this->order);
		}
		else
		{
			$query->order($order . ' ' . $orderDir);
		}


		if (!$this->_id && $limit != 0)
		{


			if ($limitStart >= $this->getTotal())
			{
				$limitStart = 0;
				$limit      = ComponentHelper::getParams('com_joomcrm')->get('people_limit',10);
				$limitStart = ($limit != 0) ? (floor($limitStart / $limit) * $limit) : 0;
				$this->setState('People.' . $this->view . '.' . $this->layout . '.limit', $limit);
				$this->setState('People.' . $this->view . '.' . $this->layout . '.limitstart', $limitStart);
			}
			if ($limit > 0)
			{
				$query .= " LIMIT " . ($limit) . " OFFSET " . ($limitStart);
			}
		}


		$people = $db->setQuery($query)->loadAssocList();

		// Generate query to join deals
		if (count($people) > 0)
		{
			$export =  \Joomla\CMS\Factory::getApplication()->input->get('export');

			if (!$export)
			{
				if ($this->_id)
				{
					// Generate query to join notes
					foreach ($people as $key => $person)
					{
						// Deals
						$dealModel = new JoomcrmModelDeal;
						$dealModel->set('person_id', $person['id']);
						$dealModel->set('user_override', 'all');
						$people[$key]['deals'] = $dealModel->getDeals();;

						// Notes
						$notesModel            = new JoomcrmModelNote;
						$notesModel->public_notes = true;
						$notesModel->filters = false;
						$people[$key]['notes'] = $notesModel->getNotes($person['id'], 'person');

						// Docs
						$docsModel = new JoomcrmModelDocument;
						$docsModel->set('person_id', $person['id']);
						$docsModel->filters = false;
						$people[$key]['documents'] = $docsModel->getDocuments();

						// Calculate total pipeline
						$people[$key]['total_pipeline'] = $this->calculatePipeline($person['id']);

						// Calculate won deal amount
						$people[$key]['won_deal_amount'] = $this->calculateWonDealAmount($person['id']);
					}
				}
			}
		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onPersonLoad', array(&$people));

		return $people;
	}

	/**
	*	Prepares Items data for the jQuery dataTable
	*
	*	@param 	array of items from DB
	*	@return array of modified items
	*/
	public function dataTablefy($items)
	{
		$dataTable = array();
		$user = Factory::getApplication()->getIdentity();

		if (is_array($items))
		{
			$count = count($items);

			for ($i = 0; $i < $count; $i++)
			{
				$personlLink = '<a href="' . Route::_('index.php?option=com_joomcrm&view=people&layout=person&id=' . $items[$i]['id']) . '">';
				$personlLink .= $items[$i]['first_name'] . ' ' . $items[$i]['last_name'];
				$personlLink .= '</a>';

				if (isset($items[$i]['avatar']) && $items[$i]['avatar'])
				{
					$avatar = '<img src="' . Uri::base() . 'components/com_joomcrm/media/avatars/' . $items[$i]['avatar'] . '"/>';
				}
				else
				{
					$avatar = '<img src="' . Uri::base() . 'components/com_joomcrm/media/images/person.png' . '"/>';
				}

				$dataTable[$i] = array();
				$dataTable[$i][] = $avatar;
				$dataTable[$i][] = $personlLink;
				$dataTable[$i][] = $items[$i]['position'];
				$dataTable[$i][] = $items[$i]['phone'];
				$dataTable[$i][] = $items[$i]['owner_first_name'] . ' ' . $items[$i]['owner_last_name'];
				$dataTable[$i][] = ucfirst($items[$i]['type']);
				$dataTable[$i][] = JoomcrmHelperDate::formatDate($items[$i]['modified']);
			}
		}

		return $dataTable;
	}

	/**
	*	Match column name from jQuery dataTable with actual
	* 	name of the column from database to make ordering.
	*
	*	@param 	integer $columnNumber
	*	@return string
	*/
	public function getColumnName($columnNumber)
	{
		$columnName = '';

		if ($columnNumber == 0) $columnName = 'p.avatar';
		if ($columnNumber == 1) $columnName = 'p.first_name';
		if ($columnNumber == 2) $columnName = 'p.position';
		if ($columnNumber == 3) $columnName = 'p.phone';
		if ($columnNumber == 4) $columnName = 'u.first_name';
		if ($columnNumber == 5) $columnName = 'p.type';
		if ($columnNumber == 6) $columnName = 'p.modified';

		return $columnName;
	}

	public function calculatePipeline($personId)
	{
		$activeStages = JoomcrmHelperDeal::getActiveStageIds();

		if (count($activeStages) > 0)
		{
			$db    = $this->getDbo();
			$query = $db->getQuery(true)
				->select("deal.amount,deal.id")
				->from("#__joomcrm_people_cf AS pcf")
				->leftJoin("#__joomcrm_deals AS deal ON ( deal.id = pcf.association_id OR deal.primary_contact_id = pcf.person_id ) AND deal.published=1 AND deal.archived=0 AND deal.stage_id IN (" . implode(',', $activeStages) . ")")
				->where("pcf.person_id = " . (int) $personId)
				->where("pcf.association_type='deal'");

			$cfAmounts = $db->setQuery($query)->loadAssocList('id');

			// Get primary contact amounts
			$query
				->clear()
				->select("deal.amount,deal.id")
				->from("#__joomcrm_deals AS deal")
				->where("deal.primary_contact_id=" . (int) $personId)
				->where("deal.stage_id IN(" . implode(',', $activeStages) . ")")
				->where("deal.published=1")
				->where("deal.archived=0");

			$pcAmounts = $db->setQuery($query)->loadAssocList('id');

			// Merge any unique ids
			$array = $cfAmounts + $pcAmounts;

			$sum = 0;

			if (is_array($array) && count($array) > 0)
			{
				foreach ($array as $a)
				{
					$sum += $a['amount'];
				}
			}

			return $sum;
		}
		else
		{
			return 0;
		}
	}

	public function calculateWonDealAmount($personId)
	{
		$wonStages = JoomcrmHelperDeal::getWonStages();

		if (count($wonStages) > 0)
		{
			$db    = $this->getDbo();
			$query = $db->getQuery(true)
				->select("deal.amount,deal.id")
				->from("#__joomcrm_people_cf AS pcf")
				->leftJoin("#__joomcrm_deals AS deal ON ( deal.id = pcf.association_id OR deal.primary_contact_id = pcf.person_id ) AND deal.published=1 AND deal.archived=0 AND deal.stage_id IN (" . implode(',', $wonStages) . ")")
				->where("pcf.person_id=" . (int) $personId)
				->where("pcf.association_type='deal'");

			$cfAmounts = $db->setQuery($query)->loadAssocList('id');

			// Get primary contact amounts
			$query
				->clear()
				->select("deal.amount,deal.id")
				->from("#__joomcrm_deals AS deal")
				->where("deal.primary_contact_id=" . (int) $personId)
				->where("deal.stage_id IN(" . implode(',', $wonStages) . ")")
				->where("deal.published=1")
				->where("deal.archived=0");

			$pcAmounts = $db->setQuery($query)->loadAssocList('id');

			// Merge any unique ids
			$array = $cfAmounts + $pcAmounts;

			// Return sum
			$sum = 0;

			if (is_array($array) && count($array) > 0)
			{
				foreach ($array as $a)
				{
					$sum += $a['amount'];
				}
			}

			return $sum;
		}
		else
		{
			return 0;
		}
	}

	/*
	 * Method to retrieve person
	 */
	public function getPerson($id = null)
	{
		$app = Factory::getApplication();
		$id = $id ? $id : $app->input->get('id');
		$layout = $app->input->get('layout');


		if ( $id > 0 ){
			$db = Factory::getDbo();
            $query = $this->_buildQuery();

			$person = $db->setQuery($query)->loadAssoc();

			// Generate query to join deals
			/* Deals */
			$dealModel = new JoomcrmModelDeal;
			$dealModel->set('person_id', $person['id']);
			$person['deals'] = $dealModel->getDeals();

			/* Notes */
			$notesModel = new JoomcrmModelNote;
			$notesModel->filters = false;
			$person['notes'] = $notesModel->getNotes($person['id'], 'person');

			/* Docs */
			$docsModel = new JoomcrmModelDocument;
			$docsModel->set('person_id', $person['id']);
			$person['documents'] = $docsModel->getDocuments();

	 	}else{
	         // TODO update things to OBJECTS
	        $person = (array)Table::getInstance('People','JoomcrmTable');
	        $this->person = $person;
		}

		$dispatcher = \Joomla\CMS\Factory::getApplication();
		$dispatcher->triggerEvent('onPersonLoad', array(&$people));

		return $person;
	}

	/*
	 * Method to retrieve list of names and ids
	 */
	public function getPeopleList()
	{
		$db = $this->getDbo();
		$query = $db->getQuery(true)
			->select("DISTINCT(p.id),p.first_name,p.last_name,p.email")
			->from("#__joomcrm_people AS p")
			->leftJoin("#__joomcrm_people_cf AS dcf ON dcf.person_id = p.id AND dcf.association_type='deal'")
			->leftJoin("#__joomcrm_shared AS shared ON shared.item_id=p.id AND shared.item_type='person'")
			->leftJoin("#__joomcrm_users AS u ON u.id = p.owner_id")
			->leftJoin("#__joomcrm_users AS u2 ON u2.id = p.assignee_id");

		// Filter based on member access roles
		$user_id     = JoomcrmHelperUsers::getUserId();
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();
		$team_access = JoomcrmHelperUsers::getTeamAccess($user_id);


		if ($member_role != 'exec')
		{
			if ($member_role == 'manager' || (isset($team_id) && $team_access) )
			{
				$query->where("(u.team_id=$team_id OR u2.team_id=$team_id OR shared.user_id=$user_id)");
			}
			else
			{
				$query->where("(p.owner_id=$user_id OR p.assignee_id=$user_id OR shared.user_id=$user_id)");
			}

		}

		$query->where("p.published=" . $this->published);

		$associationType = $this->app->input->get('association');
		$associationId   = $this->app->input->getInt('association_id');

		if ($associationType == "company")
		{
			$query->where("p.company_id=" . $associationId);
		}
		if ($associationType == "deal")
		{
			$query->where("dcf.association_id=" . $associationId . " AND dcf.association_type='deal'");
		}

		$db->setQuery($query);

		// Load list
		$row    = $db->loadAssocList();
		$blank  = array(array('first_name' => Text::_('COM_JOOMCRM_NONE'), 'last_name' => '', 'id' => 0));

		return array_merge($blank, $row);
	}

	/**
	 * Get total number of rows for pagination
	 */
	public function getTotal()
	{
		if (empty ($this->_total))
		{
			$query        = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
		}

		return $this->_total;
	}

	/**
	 * Generate pagination
	 */
	public function getPagination()
	{
		// Lets load the content if it doesn't already exist

		if (empty($this->_pagination))
		{
			$total             = $this->getTotal();
			$this->_pagination = new JoomcrmPagination($total, $this->getState('People.' . $this->view . '.' . $this->layout . '.limitstart'), $this->getState('People.' . $this->view . '.' . $this->layout . '.limit'));
		}

		return $this->_pagination;
	}

	/**
	 * Populate user state requests
	 */
	public function populateState()
	{

		// TODO add these limits to the switch statement to support multiple pages and layouts

		// Get request variables
		$limit_config = ComponentHelper::getParams('com_joomcrm')->get('people_limit',10);
		$limit      		= $this->app->getUserStateFromRequest('People.' . $this->view . '.' . $this->layout . '.limit', 'limit', $limit_config);
		$limitstart 		= $this->app->getUserStateFromRequest('People.' . $this->view . '.' . $this->layout . '.limitstart', 'limitstart', 0);
		$filter_order     	= $this->app->getUserStateFromRequest('People.' . $this->view . '.' . $this->layout . '.filter_order', 'filter_order', 'p.last_name');
		$filter_order_Dir 	= $this->app->getUserStateFromRequest('People.' . $this->view . '.' . $this->layout . '.filter_order_Dir', 'filter_order_Dir', 'asc');
		$people_filter    	= $this->app->getUserStateFromRequest('People.' . $this->view . '.' . $this->layout . '.name', 'name', null);

		// In case limit has been changed, adjust it
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		// get column name form column number sent by jQuery dataTable
		if (is_numeric($filter_order))
		{
			$filter_order = $this->getColumnName($filter_order);
		}

		$input = \Joomla\CMS\Factory::getApplication()->input;

		// if ajax request (when using filters, pagination limit and start should work good)
		if(
			$input->get('format','','word') == 'raw' &&
			$input->get('task','','word') != 'getdata' // skip for ajax views that use datatables
		){
			$limit = $input->get('ajaxLimit',$limit,'int');
			$limitstart = 0; // should begin from start
		}


		// Set states for reports
		$this->setState('People.' . $this->view . '.' . $this->layout . '.limit', $limit);
		$this->setState('People.' . $this->view . '.' . $this->layout . '.limitstart', $limitstart);
		$this->setState('People.' . $this->view . '.' . $this->layout . '.filter_order', $filter_order);
		$this->setState('People.' . $this->view . '.' . $this->layout . '.filter_order_Dir', $filter_order_Dir);
		$this->setState('People.' . $this->view . '.' . $this->layout . '.name', $people_filter);
	}

	public function getDropdowns()
	{
		$dropdowns = array();

		$this->person['type'] = is_null($this->person) ? '' : $this->person['type'];

		$dropdowns['person_type'] = JoomcrmHelperDropdown::generateDropdown('person_type', $this->person['type']);

		return $dropdowns;
	}

	public function searchForContact($contact_name, $idsOnly = true)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		$select = $idsOnly ? "CONCAT(first_name,' ',last_name) AS label,id AS value" : "*";

		$query
			->select($select)
			->from("#__joomcrm_people")
			->where("published=" . $this->published)
			->where("LOWER(first_name) LIKE '%" . $db->escape(ucwords($contact_name)) . "%' OR LOWER(last_name) LIKE '%" . $db->escape(ucwords($contact_name)) . "%'");

		return $db->setQuery($query)->loadObjectList();
	}

	/**
	 * Checks for existing person by name
	 *
	 * @param  [var] $name person name to check
	 *
	 * @return [int]       ID of existing person
	 */
	public function checkPersonName($name)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('p.id')
			->from('#__joomcrm_people AS p')
			->where('LOWER(CONCAT(p.first_name," ",p.last_name)) = "' . $db->escape(mb_strtolower($name)) . '"');

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * Checks for existing person by email
	 *
	 * @param  [var] $name person name to check
	 *
	 * @return [int]       ID of existing person
	 */
	public function checkPersonEmail($email)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('p.id')
			->from('#__joomcrm_people AS p')
			->where('LOWER(p.email) = "' . $db->escape(mb_strtolower($email)) . '"');

		return $db->setQuery($query)->loadResult();
	}

	public function getPeopleNames($json = false)
	{
		$names  = $this->getPeopleList();
		$return = array();

		if (count($names) > 0)
		{
			foreach ($names as $key => $person)
			{
				$personName = "";
				$personName .= array_key_exists('first_name', $person) ? $person['first_name'] : "";
				$personName .= array_key_exists('last_name', $person) ? " " . $person['last_name'] : "";
				$return[] = array('label' => $personName, 'value' => $person['id']);
			}
		}

		return $json ? json_encode($return) : $return;
	}

	public function getEmail($person_id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('p.email')
			->from('#__joomcrm_people AS p')
			->where('p.id = ' . (int) $person_id);

		return $db->setQuery($query)->loadResult();
	}

	public function getPersonName($person_id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select('CONCAT(first_name," ",last_name)')
			->from('#__joomcrm_people AS p')
			->where('p.id = ' . (int) $person_id);

		return $db->setQuery($query)->loadResult();
	}

}
