<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Router\Route;
class JoomcrmModelReport extends JoomcrmModel
{
	public $published = 1;

	/**
	 * Method to store a record
	 *
	 * @param $_POST data
	 *
	 * @return  boolean  True on success
	 */
	public function store()
	{
		// Load Tables
		$row    = new JoomcrmTableReport;
		$oldRow = new JoomcrmTableReport;

		$data = \Joomla\CMS\Factory::getApplication()->input->post->getArray();

		// Date generation
		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));
		if (!array_key_exists('id', $data))
		{
			$data['created'] = $date;
			$status          = "created";
		}
		else
		{
			$row->load($data['id']);
			$oldRow->load($data['id']);
			$status = "updated";
		}

		// Modified date
		$data['modified'] = $date;

		// Assign owner id
		$data['owner_id'] = JoomcrmHelperUsers::getUserId();

		if (array_key_exists('calculation_names', $data))
		{
			if (count($data['calculation_names']) > 0)
			{
				foreach ($data['calculation_names'] as $key => $name)
				{
					$op                   = $data['fields'][$key];
					$data['fields'][$key] = array('name' => $name, 'operation' => $op);
				}
			}
		}

		// Insert custom field data
		$data['fields'] = serialize($data['fields']);

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		return true;
	}

	/**
	 * Get Custom Reports
	 *
	 * @param   int $id  specific report to search for
	 *
	 * @return  mixed  $results reports matched
	 */
	public function getCustomReports($id = null)
	{
		// Load database
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("report.*")
			->from("#__joomcrm_reports as report")
			->leftJoin("#__joomcrm_users AS u ON u.id = report.owner_id");

		// Search for reports
		if ($id != null)
		{
			$query->where("report.id=$id");
		}

		// Filter based on member access roles
		$user_id     = JoomcrmHelperUsers::getUserId();
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();

		if ($member_role != 'exec')
		{
			if ($member_role == 'manager')
			{
				$query->where("u.team_id=$team_id");
			}
			else
			{
				$query->where("(report.owner_id=$user_id)");
			}
		}

		/**
		 * Set our sorting direction if set via post
		 */
		$layout = str_replace("_filter", "", \Joomla\CMS\Factory::getApplication()->input->getCmd('layout'));
		// Default deals view
		if ($layout == "custom_reports")
		{
			$query->order($this->getState('Report.filter_order') . ' ' . $this->getState('Report.filter_order_Dir'));
		}

		return $db->setQuery($query)->loadAssocList();
	}

	/**
	 * Get data for custom reports
	 *
	 * @param   int     $id     custom id data to retrieve
	 * @param   boolean $count  get only the count
	 *
	 * @return  mixed  $results
	 */
	public function getCustomReportData($id = null, $count = false)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Get the custom report so we know what data to filter and select
		$custom_report        = $this->getCustomReports($id);
		$custom_report        = $custom_report[0];
		$custom_report_fields = unserialize($custom_report['fields']);

		// Gen query
		// Construct query string
		if ($count)
		{
			$queryString = 'COUNT(d.id)';
		}
		else
		{
			$queryString = 'd.*,SUM(d.amount) AS filtered_total,';
			$queryString .= 'c.name as company_name,';
			$queryString .= 'stat.name as status_name,IF(stat.id > 0,stat.color,"ffffff") AS status_color,';
			$queryString .= 'source.name as source_name,';
			$queryString .= 'stage.name as stage_name,stage.color as stage_color,stage.percent,';
			$queryString .= 'user.first_name, user.last_name,';
			$queryString .= 'p.first_name as primary_contact_first_name,p.last_name as primary_contact_last_name,';
			$queryString .= "p.email as primary_contact_email,p.phone as primary_contact_phone,";
			$queryString .= "p.work_city as primary_contact_city,p.work_state as primary_contact_state,";
			$queryString .= "pc.name as primary_contact_company_name,";
			$queryString .= "company_category.name as company_category";
		}

		$query
			->select($queryString)
			->from("#__joomcrm_deals AS d")
			->leftJoin('#__joomcrm_companies AS c ON c.id = d.company_id AND c.published>0')
			->leftJoin("#__joomcrm_company_categories AS company_category ON c.category_id=company_category.id")
			->leftJoin('#__joomcrm_deal_status AS stat ON stat.id = d.status_id')
			->leftJoin('#__joomcrm_sources AS source ON source.id = d.source_id')
			->leftJoin('#__joomcrm_stages AS stage on stage.id = d.stage_id')
			->leftJoin('#__joomcrm_users AS user ON user.id = d.owner_id')
			->leftJoin("#__joomcrm_people AS p ON p.id = d.primary_contact_id AND p.published>0")
			->leftJoin("#__joomcrm_companies AS pc ON pc.id = p.company_id AND pc.published>0");

		if (!$count)
		{
			$query->group("d.id");
		}

		if ($this->view == "print")
		{
			$this->layout = "custom_report";
			$id     = $this->app->input->getString('custom_report');
		}

		$filter_order     = $this->getState('Report.' . $id . '_filter_order');
		$filter_order_Dir = $this->getState('Report.' . $id . '_filter_order_Dir', 'ASC');
		$filter_order     = (strstr($filter_order, "custom_")) ? str_replace("d.", "", $filter_order) : $filter_order;

		if ($filter_order != null && !$count)
		{
			$query->order($filter_order . ' ' . $filter_order_Dir);
		}

		// Assign defaults
		$close    = null;
		$modified = null;
		$created  = null;
		$status   = null;
		$source   = null;
		$stage    = null;

		// Filter by deal names
		$deal_filter = $this->getState('Report.' . $id . '_deal_name');

		if ($deal_filter != null)
		{
			$query->where("d.name LIKE '%" . $deal_filter . "%'");
		}

		// Owner
		$owner_filter = $this->getState('Report.' . $id . '_owner_id');

		if ($owner_filter != null AND $owner_filter != 'all')
		{
			// strip 't:' prefix which diferenciate member id from team id
			$owner_filter = str_replace('t:', '', $owner_filter);

			$owner_type = $this->getState('Report.' . $id . '_owner_type');

			if ($owner_type == 'member')
			{
				$query->where("d.owner_id=" . $owner_filter);

			}
			if ($owner_type == 'team')
			{
				// Get team members
				$team_members = JoomcrmHelperUsers::getTeamUsers($owner_filter);
				// Filter by results having team ids
				$ids = "";

				for ($i = 0; $i < count($team_members); $i++)
				{
					$member = $team_members[$i];
					$ids .= $member['id'] . ",";
				}

				$ids = substr($ids, 0, -1);
				$query->where("d.owner_id IN(" . $ids . ")");
			}
		}

		// Amount
		$amount_filter = $this->getState('Report.' . $id . '_amount');

		if ($amount_filter != null AND $amount_filter != 'all')
		{
			if ($amount_filter == 'small')
			{
				$query->where("d.amount <= 50");
			}
			if ($amount_filter == 'medium')
			{
				$query->where("d.amount > 50 AND d.amount <= 400");
			}
			if ($amount_filter == 'large')
			{
				$query->where("d.amount > 400");
			}
		}

		// Source
		$source_filter = $this->getState('Report.' . $id . '_source_id');

		if ($source_filter != null AND $source_filter != 'all')
		{
			$source = $source_filter;
		}

		// Stage
		$stage_filter = $this->getState('Report.' . $id . '_stage_id');

		if ($stage_filter != null AND $stage_filter != 'all')
		{
			$stage = $stage_filter;
		}

		// Status
		$status_filter = $this->getState('Report.' . $id . '_status_id');

		if ($status_filter != null AND $status_filter != 'all')
		{
			$status = $status_filter;
		}

		// Expected close
		$expected_close_filter = $this->getState('Report.' . $id . '_expected_close');

		if ($expected_close_filter != null AND $expected_close_filter != 'all')
		{
			$expected_close = $expected_close_filter;
		}

		// Actual close
		$actual_close_filter = $this->getState('Report.' . $id . '_actual_close');

		if ($actual_close_filter != null AND $actual_close_filter != 'all')
		{
			$actual_close = $actual_close_filter;
		}

		// Modified
		$modified_filter = $this->getState('Report.' . $id . '_modified');

		if ($modified_filter != null AND $modified_filter != 'all')
		{
			$modified = $modified_filter;
		}

		// Created
		$created_filter = $this->getState('Report.' . $id . '_created');

		if ($created_filter != null AND $created_filter != 'all')
		{
			$created = $created_filter;
		}

		// Filter by primary contact name
		$primary_contact_name = $this->getState('Report.' . $id . '_primary_contact_name');

		if ($primary_contact_name != null)
		{
			$query->where("(p.first_name LIKE '%" . $primary_contact_name . "%' OR p.last_name LIKE '%" . $primary_contact_name . "%')");
		}

		// Filter by primary contact email
		$primary_contact_email = $this->getState('Report.' . $id . '_primary_contact_email');

		if ($primary_contact_email != null)
		{
			$query->where("p.email LIKE '%" . $primary_contact_email . "%'");
		}

		// Filter by primary contact city
		$primary_contact_city = $this->getState('Report.' . $id . '_primary_contact_city');

		if ($primary_contact_city != null)
		{
			$query->where("p.work_city LIKE '%" . $primary_contact_city . "%'");
		}

		// Filter by primary contact state
		$primary_contact_state = $this->getState('Report.' . $id . '_primary_contact_state');

		if ($primary_contact_state != null)
		{
			$query->where("p.work_state LIKE '%" . $primary_contact_state . "%'");
		}

		// Filter by primary contact phone
		$primary_contact_phone = $this->getState('Report.' . $id . '_primary_contact_phone');

		if ($primary_contact_phone != null)
		{
			$query->where("p.phone LIKE '%" . $primary_contact_phone . "%'");
		}

		// Filter by company categories
		$company_category = $this->getState('Report.' . $id . '_company_category');

		if ($company_category != "any" && $company_category != null)
		{
			$query->where('c.category_id=' . $company_category);
		}

		// Get current date to use for all date filtering
		$date = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00'));

		/** --------------------------------------------
		 * Search for expected close date
		 */
		if ($expected_close_filter != null && $expected_close_filter != "all")
		{
			if ($expected_close == "this_week")
			{
				$this_week = null;
				$next_week = null;
				JoomcrmHelperDate::x_week_range($this_week, $next_week, JoomcrmHelperDate::getCurrentTime(true, false));
				$query
					->where("d.expected_close >= '$this_week'")
					->where("d.expected_close < '$next_week'");
			}

			if ($expected_close == "next_week")
			{
				$week_after_next      = null;
				$week_after_next_week = null;
				JoomcrmHelperDate::x_week_range($week_after_next, $week_after_next_week, JoomcrmHelperDate::getCurrentTime(true, false) . " + 1 week");
				$query
					->where("d.expected_close >= '$week_after_next'")
					->where("d.expected_close < '$week_after_next_week'");
			}

			if ($expected_close == "this_month")
			{
				$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-0 00:00:00'));
				$next_month = date('Y-m-0 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+1 month"));
				$query
					->where("d.expected_close >= '$this_month'")
					->where("d.expected_close < '$next_month'");
			}

			if ($expected_close == "next_month")
			{
				$next_month      = date("Y-m-0 00:00:00", strtotime(date("Y-m-d 00:00:00", strtotime($date)) . "+1 month"));
				$next_next_month = date("Y-m-0 00:00:00", strtotime(date("Y-m-d 00:00:00", strtotime($date)) . "+2 months"));
				$query
					->where("d.expected_close >= '$next_month'")
					->where("d.expected_close < '$next_next_month'");
			}

		}

		/** --------------------------------------------
		 * Search for actual close date
		 */
		if ($actual_close_filter != null && $actual_close_filter != "all")
		{
			if ($actual_close == "this_week")
			{
				$this_week = null;
				$next_week = null;
				JoomcrmHelperDate::x_week_range($this_week, $next_week, JoomcrmHelperDate::getCurrentTime(true, false));
				$query
					->where("d.actual_close >= '$this_week'")
					->where("d.actual_close < '$next_week'");
			}

			if ($actual_close == "last_week")
			{
				$last_week      = null;
				$last_last_week = null;
				JoomcrmHelperDate::x_week_range($week_after_next, $week_after_next_week, JoomcrmHelperDate::getCurrentTime(true, false) . " - 1 week");
				$query
					->where("d.actual_close >= '$last_last_week'")
					->where("d.actual_close < '$last_week'");
			}

			if ($actual_close == "last_month")
			{
				$this_month = date("Y-m-0 00:00:00", strtotime(date("Y-m-d 00:00:00", strtotime($date))));
				$last_month = date("Y-m-0 00:00:00", strtotime(date("Y-m-d 00:00:00", strtotime($date)) . "-1 months"));
				$query
					->where("d.actual_close >= '$last_month'")
					->where("d.actual_close < '$this_month'");
			}

		}

			$range = $this->getState('Report.' . $id . '_range');
			/** --------------------------------------------
			 * Search for closing deal filters
			 */
			if ($range != null && $range != "any")
			{

				if ($range == "this_month")
				{
					$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$next_month = date('Y-m-1 00:00:00', strtotime($this_month . "+1 month"));
					$query->where("d.expected_close >= '$this_month'")->where("d.expected_close < '$next_month'");
				}

				if ($range == "this_quarter")
				{
					$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$end_month = date('Y-m-1 00:00:00', strtotime($this_month . "+3 months"));
					$query->where("d.expected_close >= '$this_month'")->where("d.expected_close < '$end_month'");
				}


				if ($range == "next_month")
				{
					$this_month      = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$next_month      = date('Y-m-1 00:00:00', strtotime($this_month . "+1 month"));
					$next_next_month = date('Y-m-1 00:00:00', strtotime($next_month . "+1 month"));
					$query->where("d.expected_close >= '$next_month'")->where("d.expected_close < '$next_next_month'");
				}

				if ($range == "next_quarter")
				{
					$this_month      = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$next_month      = date('Y-m-1 00:00:00', strtotime($this_month . "+3 months"));
					$next_next_month = date('Y-m-1 00:00:00', strtotime($next_month . "+6 months"));
					$query->where("d.expected_close >= '$next_month'")->where("d.expected_close < '$next_next_month'");
				}

				if ($range == "year_remainder")
				{
					$this_month      = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
					$this_year       = JoomcrmHelperDate::formatDBDate(date('Y-12-31 11:59:59'));
					$query->where("d.expected_close >= '$this_month'")->where("d.expected_close < '$this_year'");
				}

			}


		/** --------------------------------------------
		 * Search for modified deal filters
		 */
		if ($modified != null && $modified != "any")
		{
			if ($modified == "this_week")
			{
				$this_week = null;
				$next_week = null;
				JoomcrmHelperDate::x_week_range($this_week, $next_week, JoomcrmHelperDate::getCurrentTime(true, false));
				$query
					->where("d.modified >= '$this_week'")
					->where("d.modified < '$next_week'");
			}

			if ($modified == "last_week")
			{
				$last_week      = null;
				$last_last_week = null;
				JoomcrmHelperDate::x_week_range($week_after_next, $week_after_next_week, JoomcrmHelperDate::getCurrentTime(true, false) . " - 1 week");
				$query
					->where("d.modified >= '$last_last_week'")
					->where("d.modified < '$last_week'");
			}

			if ($modified == "this_month")
			{
				$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
				$next_month = date('Y-m-1 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+1 month"));
				$query
					->where("d.modified >= '$this_month'")
					->where("d.modified < '$next_month'");
			}

			if ($modified == "last_month")
			{
				$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
				$last_month = date('Y-m-1 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "-1 month"));
				$query
					->where("d.modified >= '$last_month'")
					->where("d.modified < '$this_month'");
			}

			if ($modified == "today")
			{
				$today    = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00"));
				$tomorrow = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+1 day"));
				$query
					->where("d.modified >= '$today'")
					->where("d.modified < '$tomorrow'");
			}

			if ($modified == "yesterday")
			{
				$today     = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00"));
				$yesterday = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "-1 day"));
				$query
					->where("d.modified >= '$yesterday'")
					->where("d.modified < '$today'");
			}

		}

		/** --------------------------------------------
		 * Search for created deal filters
		 */
		if ($created != null && $created != "any")
		{
			if ($created == "this_week")
			{
				$this_week = null;
				$next_week = null;
				JoomcrmHelperDate::x_week_range($this_week, $next_week, JoomcrmHelperDate::getCurrentTime(true, false));
				$query
					->where("d.created >= '$this_week'")
					->where("d.created < '$next_week'");
			}

			if ($created == "last_week")
			{
				$last_week      = null;
				$last_last_week = null;
				JoomcrmHelperDate::x_week_range($last_week, $last_last_week, JoomcrmHelperDate::getCurrentTime(true, false) . " - 1 week");
				$query
					->where("d.created >= '$last_last_week'")
					->where("d.created < '$last_week'");
			}

			if ($created == "this_month")
			{
				$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
				$next_month = date('Y-m-1 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+1 month"));
				$query
					->where("d.created >= '$this_month'")
					->where("d.created < '$next_month'");
			}

			if ($created == "last_month")
			{
				$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-1 00:00:00'));
				$last_month = date('Y-m-1 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "-1 month"));
				$query
					->where("d.created >= '$last_month'")
					->where("d.created < '$this_month'");
			}

			if ($created == "today")
			{
				$today    = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00"));
				$tomorrow = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+1 day"));
				$query
					->where("d.created >= '$today'")
					->where("d.created < '$tomorrow'");
			}

			if ($created == "yesterday")
			{
				$today     = JoomcrmHelperDate::formatDBDate(date("Y-m-d 00:00:00"));
				$yesterday = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "-1 day"));
				$query
					->where("d.created >= '$yesterday'")
					->where("d.created < '$today'");
			}

		}

		/** ------------------------------------------
		 * Search for status
		 */
		if ($status != null AND $status != 'all')
		{
			$query->where("d.status_id=" . $status);
		}

		/** -------------------------
		 * Search for sources
		 */
		if ($source != null AND $source != 'all')
		{
			$query->where('d.source_id=' . $source);
		}

		/** ----------------------------------------------------------------
		 * Filter for stage id associations
		 */
		if ($stage != null && $stage != 'all')
		{
			// If we want active deals we must retrieve the active stage ids to filter by
			if ($stage == 'active')
			{
				// Get stage ids
				$stage_ids = JoomcrmHelperDeal::getActiveStages();
				// Filter by results having team ids
				$stages = "";
				for ($i = 0; $i < count($stage_ids); $i++)
				{
					$stage = $stage_ids[$i];
					$stages .= $stage['id'] . ",";
				}
				$stages = substr($stages, 0, -1);
				$query->where("d.stage_id IN(" . $stages . ")");
			}
			else
			{
				// Else filter by the stage id
				$query->where("d.stage_id='" . $stage . "'");
			}
		}

		/** ---------------------------------------------------------------------------------------------------------------
		 * Field for custom field user states
		 */
		// Get custom filters
		$custom_fields = JoomcrmHelperDeal::getUserCustomFields();
		// If the user has defined any custom fields we will left join the associated data here
		if (count($custom_fields) > 0)
		{
			foreach ($custom_fields as $row)
			{
				// Join different data based on type
				if (!$count)
				{
					switch ($row['type'])
					{
						// If the type is forecast we want to calculate the amount
						case "forecast":
							$query->select("( d.amount * ( d.probability / 100 )) as custom_" . $row['id']);
							break;
						// Else join the associated value from the database
						default :
							$query
								->select("custom_" . $row['id'] . ".value as custom_" . $row['id'])
								->leftJoin("#__joomcrm_deal_custom_cf as custom_" . $row['id'] . " on " . "custom_" . $row['id'] . ".deal_id = d.id AND " . "custom_" . $row['id'] . ".custom_field_id = " . $row['id']);
							break;
					}
				}

				// If the user has any associated user state requests set in the model we set the filters here
				$custom_field_filter = $this->getState('Report.' . $id . '_' . $row['id']);
				if ($custom_field_filter != null AND $custom_field_filter != 'all')
				{
					switch ($row['type'])
					{
						case "forecast":
							$query->where("( d.amount * ( d.probability / 100 )) LIKE '%" . $custom_field_filter . "%'");
							break;
						case "date":

							if ($custom_field_filter == "this_week")
							{
								$this_week = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00'));
								$next_week = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+7 days"));
								$query
									->where("custom_" . $row['id'] . ".value >= '$this_week'")
									->where("custom_" . $row['id'] . ".value < '$next_week'");
							}

							if ($custom_field_filter == "next_week")
							{
								$next_week       = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+7 days"));
								$week_after_next = date('Y-m-d 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+14 days"));
								$query
									->where("custom_" . $row['id'] . ".value >= '$next_week'")
									->where("custom_" . $row['id'] . ".value < '$week_after_next'");
							}

							if ($custom_field_filter == "this_month")
							{
								$this_month = JoomcrmHelperDate::formatDBDate(date('Y-m-0 00:00:00'));
								$next_month = date('Y-m-0 00:00:00', strtotime(date("Y-m-d", strtotime($date)) . "+1 month"));
								$query
									->where("custom_" . $row['id'] . ".value >= '$this_month'")
									->where("custom_" . $row['id'] . ".value < '$next_month'");
							}

							if ($custom_field_filter == "next_month")
							{
								$next_month      = date("Y-m-0 00:00:00", strtotime(date("Y-m-d 00:00:00", strtotime($date)) . "+1 month"));
								$next_next_month = date("Y-m-0 00:00:00", strtotime(date("Y-m-d 00:00:00", strtotime($date)) . "+2 months"));
								$query
									->where("custom_" . $row['id'] . ".value >= '$next_month'")
									->where("custom_" . $row['id'] . ".value < '$next_next_month'");
							}

							break;
						default:
							$query->where("custom_" . $row['id'] . ".value LIKE '%" . $custom_field_filter . "%'");
							break;
					}
				}
			}
		}

		// Filter based on member access roles
		$user_id     = JoomcrmHelperUsers::getUserId();
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();

		if ($member_role != 'exec')
		{
			if ($member_role == 'manager')
			{
				$query->where("user.team_id=$team_id");
			}
			else
			{
				$query->where("(d.owner_id=$user_id)");
			}

		}

		$query
			->where("d.published=" . $this->published)
			->where("d.archived=0");

		if (JoomcrmHelperQuote::hasCRMQuote())
		{
			JoomcrmHelperQuote::constructQuery($query);
		}

		// Calculation fields
		if (!$count)
		{
			foreach ($custom_report_fields as $key => $field)
			{
				if (strstr($key, "calculate_field_"))
				{
					$select = $db->escape($field['operation']);
					$str    = "(" . $select . ') AS ' . $key;

					if (strlen($select) > 0 && $this->checkValidity($select))
					{
						$query->select($str);
					}
				}
			}
		}

		if ($count)
		{
			$result = $db->setQuery($query)->loadResult();
		}
		else
		{
			$result = $db->setQuery($query, $this->limitstart, $this->limit)->loadAssocList();
		}

		return $result;
	}

	/**
	 * Get total custom report count
	 */
	public function getTotal($id)
	{
		return $this->getCustomReportData($id, true);
	}

	/**
	 * Generate pagination
	 */
	public function getPagination($id)
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			$link 		= Route::_('index.php?option=com_joomcrm&view=reports&layout=custom_report&id=' . $id);
			$this->_pagination = new Pagination($this->getTotal($id), $this->limitstart, $this->limit);
		}

		return $this->_pagination;
	}

	/**
	 * @param $stringToTest
	 *
	 * @return bool
	 *
	 * @todo  actually check the string?!?
	 */
	private function checkValidity($stringToTest)
	{
		// $expression = JoomcrmHelperJoomcrm::splitExpr($stringToTest);
		// echo '<PRE>';
		// print_r($expression);
		return true;

	}

	/**
	 * Method to delete a record
	 *
	 * @param int $id document id to delete
	 *
	 * @return boolean True on success
	 */
	public function deleteReport($id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		$report = $this->getCustomReports($id);

		if (count($report) > 0)
		{
			$query
				->delete("#__joomcrm_reports")
				->where("id = " . (int) $id);

			$db->setQuery($query)->execute();
		}

		return true;
	}

	/**
	 * Populate user state requests
	 */
	public function populateState()
	{
		$id = $this->app->input->getInt('id', $this->app->input->getString('custom_report'));
		$this->limit      = 10;
		$this->limitstart = $this->app->getUserStateFromRequest('Report.' . $id . '.limit', 'limitstart', 0);

		// Set layout for filter pages
		if ($this->view == "print")
		{
			$id           = $this->app->input->getString('custom_report');
			$this->layout = "custom_report";
		}

		/** --------------------------------------
		 * Filter data for different views
		 */
		switch ($this->layout)
		{
			case "custom_reports"    :
				// Set default filter states for reports
				$filter_order     = $this->app->getUserStateFromRequest('Report.filter_order', 'filter_order', 'report.name');
				$filter_order_Dir = $this->app->getUserStateFromRequest('Report.filter_order_Dir', 'filter_order_Dir', 'asc');
				// Set states for reports
				$this->setState('Report.filter_order', $filter_order);
				$this->setState('Report.filter_order_Dir', $filter_order_Dir);
				break;

			case "custom_report"    :
			case "custom_report_filter" :

				$filters = array(
					'filter_order' => 'd.name',
					'filter_order_Dir' => 'asc',
					'deal_name' => null,
					'owner_id' => JoomcrmHelperUsers::getUserId(),
					'owner_type' => 'member',
					'deal_amount' => null,
					'source_id' => null,
					'stage_id' => null,
					'status_id' => null,
					'expected_close' => null,
					'actual_close' => null,
					'modified' => null,
					'created' => null,
					'primary_contact_name' => null,
					'primary_contact_phone' => null,
					'primary_contact_email' => null,
					'primary_contact_city' => null,
					'primary_contact_state' => null,
					'company_category' => null,
					'range' => null
				);

				// Set default filter states for reports
				foreach ($filters as $filter => $defaultVal)
				{
					$this->setSessionState($id, $filter, $defaultVal);
				}

				// Get custom filters
				$custom_fields = JoomcrmHelperDeal::getUserCustomFields();

				if (count($custom_fields) > 0)
				{
					foreach ($custom_fields as $row)
					{
						$this->setSessionState($id, 'custom_' . $row['id']);
					}
				}

				break;
		}
	}

	public function setSessionState($id, $filter, $defaultVal = null)
	{
		$session = Factory::getSession();
		$uniqueKey = 'Report.' . $id . '_' . $filter;
		$sessionVal = $session->get($uniqueKey);
		$requestVal = $this->app->input->getString($filter);

		if ($requestVal !== null)
		{
			$value = $requestVal;
		}
		elseif ($sessionVal !== null)
		{
			$value = $sessionVal;
		}
		else {
			$value = $defaultVal;
		}

		$session->set($uniqueKey, $value);
		$this->setState($uniqueKey, $value);
	}
}
