<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

class JoomcrmModelRevenue extends JoomcrmModel
{
	public $start_date;
	public $end_date;

	/**
	 * Get Monthly Revenue
	 *
	 * @param $access_type we want to filter by 'member','team','company'
	 * @param $access_id   the id of the $access_type we want to filter by
	 *
	 * @return mixed $results
	 */
	public function getMonthlyRevenue($access_type = null, $access_id = null)
	{
		$db    = $this->getDbo();


		// Get current month
		$current_month = JoomcrmHelperDate::formatDBDate(date('Y-m-01 00:00:00'));

		// Get weeks in month
		$weeks = JoomcrmHelperDate::getWeeksInMonth($current_month);

		// Get stage id to filter deals by
		$won_stage_ids = JoomcrmHelperDeal::getWonStages();

		// Gen query
		$results = array();

		foreach ($weeks as $week)
		{
			$start_date = $week['start_date'];
			$end_date   = $week['end_date'];

			// Gen query string
			$query = $db->getQuery(true);
			$query
				->select("SUM(d.amount) AS y")
				->from("#__joomcrm_deals AS d")
				->where("d.stage_id IN (" . implode(',', $won_stage_ids) . ")")
				->where("d.actual_close >= '$start_date'")
				->where("d.actual_close < '$end_date'")
				->where("d.actual_close IS NOT NULL")
				->where("d.published>0")
				->where("d.archived=0");


			// Filter by owner type
			if ($access_type != 'company')
			{
				// Team sorting
				if ($access_type == 'team')
				{
					// Get team members
					$team_members = JoomcrmHelperUsers::getTeamUsers($access_id);

					if(!empty($team_members)){
						$query .= " AND d.owner_id IN (";
						// Loop to make string
						foreach ($team_members as $key => $member)
						{
							$query .= "'" . $member['id'] . "',";
						}
						$query = substr($query, 0, -1);
						$query .= ") ";
					}

				}

				// Member filter
				if ($access_type == 'member')
				{
					$query->where("d.owner_id=$access_id");
				}
			}

			$totals = $db->setQuery($query)->loadAssoc();



			if (!$totals)
			{
				$totals = array('y' => 0);
			}

			$totals['y'] = (int) $totals['y'];
			$results[]   = $totals;
		}

		return $results;
	}

	/**
	 * Get Yearly Revenue
	 *
	 * @param $access_type we wish to filter by 'member','team','company'
	 * @param $access_id   the id of the $access_type we wish to filter by
	 *
	 * @return mixed $results
	 */
	public function getYearlyRevenue($access_type = null, $access_id = null)
	{
		$db    = $this->getDbo();

		// Get current year and months to loop through
		$current_year = JoomcrmHelperDate::formatDBDate(date('Y-01-01 00:00:00'));
		$months       = JoomcrmHelperDate::getMonthDates($this->start_date, $this->end_date);

		// Get stage id to filter deals by
		$won_stage_ids = JoomcrmHelperDeal::getWonStages();

		// Gen query
		$results = array();

		foreach ($months as $month)
		{
			$start_date = $month['date'];
			$end_date   = JoomcrmHelperDate::formatDBDate(date('Y-m-d 00:00:00', strtotime("$start_date + 1 months")));

			// Generate query string
			$query = $db->getQuery(true);
			$query
				->select("d.actual_close,SUM(d.amount) AS y")
				->from("#__joomcrm_deals AS d")
				->where("d.stage_id IN (" . implode(',', $won_stage_ids) . ")")
				->where("d.actual_close >= '$start_date'")
				->where("d.actual_close < '$end_date'")
				->where("d.actual_close IS NOT NULL")
				->where("d.published>0")
				->where("d.archived=0");

			// Filter by access type
			if ($access_type != 'company')
			{
				// Team sorting
				if ($access_type == 'team')
				{
					// Get team members
					$team_members = JoomcrmHelperUsers::getTeamUsers($access_id);
					$query .= " AND d.owner_id IN (";
					// Loop to make string
					foreach ($team_members as $key => $member)
					{
						$query .= "'" . $member['id'] . "',";
					}
					$query = substr($query, 0, -1);
					$query .= ") ";
				}

				// Member filter
				if ($access_type == 'member')
				{
					$query->where("d.owner_id=$access_id");
				}
			}

			$totals = $db->setQuery($query)->loadAssoc();

			if (!$totals)
			{
				$totals = array('y' => 0);
			}
			$totals['y'] = (int) $totals['y'];
			$results[]   = $totals;
		}

		return $results;
	}

}
