<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

class JoomcrmModelSource extends JoomcrmModel
{
	public $_id = null;

	/**
	 * Get sources for return on investment reports
	 *
	 * @param none
	 *
	 * @return mixed $results
	 */
	public function getRoiSources()
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("s.id,s.name,count(d.id) as number_of_deals,sum(d.amount) as revenue,s.type,s.cost")
			->select("IF ( s.type <> 'per', ( ( ( ( sum(d.amount) - s.cost ) / s.cost ) * 100 ) ), ( ( sum(d.amount) - ( s.cost * count(d.id) ) ) / ( s.cost * count(d.id) ) * 100 ) ) AS roi")
			->from("#__joomcrm_sources AS s");

		$won_stage_ids = JoomcrmHelperDeal::getWonStages();
		$query
			->leftJoin("#__joomcrm_deals AS d ON d.source_id = s.id AND d.stage_id IN (" . implode(',', $won_stage_ids) . ") AND d.published=1 AND d.archived=0")
			->leftJoin("#__joomcrm_users AS u ON u.id = d.owner_id")
			->order($this->getState('Source.filter_order') . ' ' . $this->getState('Source.filter_order_Dir'))
			->group("s.id");

		if ($this->_id)
		{
			if (is_array($this->_id))
			{
				$query->where("s.id IN (" . implode(',', $this->_id) . ")");
			}
			else
			{
				$query->where("s.id=$this->_id");
			}
		}

		// Filter based on member access roles
		$user_id     = JoomcrmHelperUsers::getUserId();
		$member_role = JoomcrmHelperUsers::getRole();
		$team_id     = JoomcrmHelperUsers::getTeamId();

		if ($member_role != 'exec')
		{
			if ($member_role == 'manager')
			{
				$query->where("u.team_id=$team_id");
			}
			else
			{
				$query->where("(d.owner_id=$user_id)");
			}
		}

		return $db->setQuery($query)->loadAssocList();
	}

	/*
	 * Populate the model based on user requests
	 */
	public function populateState()
	{
		$filter_order     = $this->app->getUserStateFromRequest('Source.filter_order', 'filter_order', 's.name');
		$filter_order_Dir = $this->app->getUserStateFromRequest('Source.filter_order_Dir', 'filter_order_Dir', 'asc');

		// Set states
		$this->setState('Source.filter_order', $filter_order);
		$this->setState('Source.filter_order_Dir', $filter_order_Dir);
	}
}
