<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

class JoomcrmModelTemplate extends JoomcrmModel
{
	/**
	 * Run items through template system
	 * @param   $associationId   The ID of the associated item
	 * @param   $assocationType  The type of the association
	 * @return  Boolean  		 True or False
	 */
	public function createTemplate($associationId = null, $associationType = null)
	{
		$template_id      = $this->app->input->getInt('template_id');
		$association_id   = $associationId ? $associationId : $this->app->input->getInt('association_id');
		$association_type = $associationType ? $associationType : $this->app->input->get('association_type');

		$template = $this->getTemplate($template_id);

		$current_date = date("Y-m-d 00:00:00");

		if (count($template) > 0)
		{
			$event_model = new JoomcrmModelEvent;

			foreach ($template as $event)
			{
				unset($event['id']);

				$event['association_id']   = $association_id;
				$event['association_type'] = $association_type;
				$event['category_id']      = $event['type'];
				$event['type']             = "task";

				$event['due_date']      = JoomcrmHelperDate::formatDBDate(date("Y-m-d", strtotime($current_date . " +" . $event['day'] . " days")), false);
				$event['due_date_hour'] = "00:00:00";

				if (!$event_model->store($event))
				{
					return false;
				}
			}
		}

		return true;

	}

	public function processDefaults($associationId = null, $associationType = null)
	{
		$association_id   = $associationId ? $associationId : $this->app->input->getInt('association_id');
		$association_type = $associationType ? $associationType : $this->app->input->get('association_type');
		$owner_id 		  = $this->app->input->get('owner_id', null);

		$templates = $this->getDefaults($associationType);

		$current_date = date("Y-m-d 00:00:00");

		if (count($templates) > 0)
		{
			foreach ($templates as $templateInfo)
			{
				$template = $this->getTemplate($templateInfo['id']);
				$event_model = new JoomcrmModelEvent;
				
				if (is_array($template))
				{
					foreach ($template as $event)
					{
						unset($event['id']);

						$event['association_id']   = $association_id;
						$event['association_type'] = $association_type;
						$event['category_id']      = $event['type'];
						$event['type']             = "task";

						$event['owner_id'] = $event['assignee_id'] = $owner_id ? $owner_id : JoomcrmHelperUsers::getUserId();

						$event['due_date']      = JoomcrmHelperDate::formatDBDate(date("Y-m-d", strtotime($current_date . " +" . $event['day'] . " days")), false);
						$event['due_date_hour'] = "00:00:00";

						if (!$event_model->store($event))
						{
							return false;
						}
					}
				}

			}
		}

		return true;

	}

	public function getDefaults($associationType = null)
	{
		$associationType = $associationType ? $associationType : $this->app->input->get("association_type");
		$db              = $this->getDbo();
		$query           = $db->getQuery(true);

		switch ($associationType)
		{
			case "people":
			case "person":
				$type = "person";
				break;
			case "deal":
			case "deals":
				$type = "deal";
				break;
			case "company":
			case "companies":
				$type = "company";
				break;
		}

		if (isset($type))
		{
			$query
				->select("t.*")
				->from("#__joomcrm_templates AS t")
				->where('t.type = ' . $db->quote($type))
				->where("t.default=1");

			return $db->setQuery($query)->loadAssocList();
		}

		return array();
	}

	/**
	 * Get template events
	 *
	 * @param   $template_id  The ID of the template to get
	 *
	 * @return   Array        The list of template data  
	 */
	public function getTemplate($template_id)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("t.*")
			->from("#__joomcrm_template_data AS t")
			->where("t.template_id = " . (int) $template_id);

		return $db->setQuery($query)->loadAssocList();
	}

}
