<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

class JoomcrmModelUser extends JoomcrmModel
{
	/**
	 * Method to store a record
	 *
	 * @return    boolean    True on success
	 */
	function store($data = null)
	{

		// Load Tables
		$row = new JoomcrmTableUsers;

		if ($data['id'])
		{
			$row->load($data['id']);
		}

		if (!$data)
		{
			$data = Factory::getApplication()->input->post->getArray();
		}

		if (isset($data['fullscreen']))
		{
			$data['fullscreen'] = (int) !$row->fullscreen;
		}

		// Date generation
		$date             = JoomcrmHelperDate::formatDBDate(date('Y-m-d H:i:s'));
		$data['modified'] = $date;

		$data['commission_rate'] = isset($data['commission_rate']) ? (float) $data['commission_rate'] : 0;

		if($data['commission_rate'] > 0){

			$data['commission_rate'] = $data['commission_rate'] > 100 ? 100 : $data['commission_rate'];

		}

		// Update users email address
		if (array_key_exists('email', $data))
		{
			$emails = $data['email'];

			if ($this->updateEmail($data['id'], $emails) !== true)
			{
				return false;
			}

			unset($data['email']);
		}

		if (array_key_exists('document_bypass', $data))
		{
			$data['document_bypass'] = serialize($data['document_bypass']);
		}

		// Bind the form fields to the table
		if (!$row->bind($data))
		{
			$this->setError($row->getError());

			return false;
		}

		// Make sure the record is valid
		if (!$row->check())
		{
			$this->setError($row->getError());

			return false;
		}

		// Store the web link table to the database
		if (!$row->store())
		{
			$this->setError($row->getError());

			return false;
		}

		return true;

	}

	/**
	 * Use post data and update a users email address(es) in the users_email_cf db table
	 *
	 * @param int   $user_id the user id of the user being updated
	 * @param mixed $emails  an array of new email addresses to be associated with the user
	 *
	 * @return void
	 */
	public function updateEmail($user_id, $emails)
	{
		$values = array();
		$emailUsed = null;

		// Build insert list
		foreach ($emails as $email)
		{
			if ($email != null AND $email != '')
			{
				if (!(JoomcrmHelperJoomcrm::checkEmailName($email)))
				{
					$values[] = $user_id . ",'" . $email . "'";
				}
				else
				{
					$emailUsed = $email;
				}
			}
		}

		// if some email address is used, inform user and do nothing.
		if ($emailUsed) 
		{
			$this->setError(Text::_('COM_JOOMCRM_EMAIL_EXISTS') . ' (' . $emailUsed . ')');
			return false;
		}

		// There is no email to save.
		if (empty($values))
		{
			$this->setError(Text::_('COM_JOOMCRM_ERROR'));
			return false;
		}

		// Delete old entries
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->delete('#__joomcrm_users_email_cf')
			->where('member_id = ' . (int) $user_id);

		$db->setQuery($query)->execute();

		// Insert new entries
		$query
			->clear()
			->insert('#__joomcrm_users_email_cf')
			->columns(array('member_id,email'))
			->values($values);

		// Return
		$db->setQuery($query);

		if ($db->execute())
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Update a users database columns for displaying data on individual pages
	 *
	 * @param string $loc    the column in the database to update
	 * @param string $column the column in the serialized array that will be updated
	 */
	public function updateColumns($loc, $column)
	{
		// Get user id
		$user_id = JoomcrmHelperUsers::getUserId();

		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select($loc . "_columns")
			->from("#__joomcrm_users")
			->where("id = " . (int) $user_id);

		$result = unserialize($db->setQuery($query)->loadResult());

		// If we have no data assigned grab the defaults
		if (!is_array($result))
		{
			switch ($loc)
			{
				case "deals":
					$result = JoomcrmHelperDeal::getDefaultColumnFilters();
					break;
				case "people":
					$result = JoomcrmHelperPeople::getDefaultColumnFilters();
					break;
				case "companies":
					$result = JoomcrmHelperCompany::getDefaultColumnFilters();
					break;
			}
		}
		// If we do find the value in the array remove it
		if (in_array($column, $result))
		{
			$key = array_search($column, $result);
			unset($result[$key]);
		}
		else
		{
			// If we dont find the value in the array add it
			$result[] = $column;
		}

		// Serialize the new array
		$result = serialize($result);

		// Update the database
		$query
			->clear()
			->update('#__joomcrm_users')
			->set($loc . '_columns = ' . $db->quote($result))
			->where("id = " . (int) $user_id);

		$db->setQuery($query)->execute();
	}

	public function searchForUser($userName)
	{
		$db    = $this->getDbo();
		$query = $db->getQuery(true)
			->select("c.id")
			->from("#__users AS u")
			->leftJoin("#__joomcrm_users AS c ON c.uid=u.id")
			->where("u.username=" . $db->Quote($userName));

		return $db->setQuery($query)->loadResult();
	}

}
