<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

use Joomcrm\Mailing\Mailing;

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Factory;

class JoomcrmViewCompaniesHtml extends JoomcrmHelperView
{

	public function display($tpl = null)
	{
		$app = \Joomla\CMS\Factory::getApplication();
		$config     = \Joomla\CMS\Component\ComponentHelper::getParams('com_joomcrm');
		$input = \Joomla\CMS\Factory::getApplication()->input;

		// add some data to js for ajax pagination
		\Joomla\CMS\Factory::getDocument()->addScriptOptions('com_joomcrm.urlParams',[
			'Itemid' => JoomcrmHelperRoute::getCompaniesRoute(1)
		]);

		$model = new JoomcrmModelCompany();

		$this->pagination = $model->getPagination();

		$layout = $this->getLayout();

		$state = $model->getState();

		// Session data
		$session     = Factory::getSession();
		$member_role = JoomcrmHelperUsers::getRole();
		$user_id     = JoomcrmHelperUsers::getUserId();
		$team_id     = JoomcrmHelperUsers::getTeamId();
		$company     = $session->get('company_type_filter',$config->get('companies_type_filter','all'));
		$category    = $session->get("company_category_filter",$config->get('companies_category_filter',0));
		$user        = $session->get('company_user_filter',$config->get('companies_user_filter',1));
		$team        = $session->get('company_team_filter');

		// Load java libs
		$doc = Factory::getDocument();
		$doc->addScript(Uri::base() . 'components/com_joomcrm/media/js/company_manager.js');

		// Determine if we are requesting a specific company or all companies
		// If id requested
		$id = $app->input->getInt('id');



		if ($id)
		{
			$companies = $model->getCompanies($id);

			if (is_null($companies[0]['id']))
			{
                $app->enqueueMessage(Text::_('COM_JOOMCRM_NOT_AUTHORIZED'),'warning');
				$app->redirect(Route::_('index.php?option=com_joomcrm&view=companies'), 303);
			}
		}
		else
		{
			// Else load all companies
			if ($layout != 'edit')
			{
				$companies = $model->getCompanies();

			}
		}

		// Get column filters
		$column_filters   = JoomcrmHelperCompany::getColumnFilters();
		$selected_columns = JoomcrmHelperCompany::getSelectedColumnFilters();

		// Get company type filters
		$company_types = JoomcrmHelperCompany::getTypes();
		$company_type  = ($company) ? $company_types[$company] : $company_types['all'];

		// Get company category filters
		$company_categories = JoomcrmHelperCompany::getCategories();
		$company_category   = ($category) ? $company_categories[$category] : 0;

		// Get user filter
		if ($user AND $user != $user_id AND $user != 'all')
		{
			$user_info = JoomcrmHelperUsers::getUsers($user);
			$user_info = $user_info[0];
			$user_name = $user_info['first_name'] . " " . $user_info['last_name'];
		}
		else
		{
			if ($team)
			{
				$team_info = JoomcrmHelperUsers::getTeams($team);
				$team_info = $team_info[0];
				$user_name = $team_info['team_name'] . Text::_('COM_JOOMCRM_TEAM_APPEND');
			}
			else
			{
				if ($user == 'all')
				{
					$user_name = Text::_('COM_JOOMCRM_ALL_USERS');
				}
				else
				{
					$user_name = Text::_('COM_JOOMCRM_ME');
				}
			}
		}

		// Get associated members and teams
		$teams = JoomcrmHelperUsers::getTeams();
		$users = JoomcrmHelperUsers::getUsers();

		// Get total associated companies for count display
		$company_count = JoomcrmHelperUsers::getCompanyCount($user_id, $team_id, $member_role);

		// Load Events & Tasks for person

		if ($layout == "company")
		{

			$eventModel  = new JoomcrmModelEvent;
			$events = $eventModel->getEvents("company", null,  \Joomla\CMS\Factory::getApplication()->input->get('id'));

			$ref = array(array('ref' => 'events', 'data' => $events));
			$this->event_dock = JoomcrmHelperView::getView('events', 'event_dock', $ref);

			$ref = array(array('ref' => 'deals', 'data' => $companies[0]['deals']));

			$this->deal_dock = JoomcrmHelperView::getView('deals', 'deal_dock', $ref);


			$docViewType = ComponentHelper::getParams('com_joomcrm')->get('documents_view_type','datatable');
			$docType = ($docViewType == 'datatable') ? 'document_dock' : 'document_tree';
			$ref = array(array('ref' => 'documents', 'data' => $companies[0]['documents']));
			$this->document_dock = JoomcrmHelperView::getView('documents', $docType, $ref);

			$ref = array(array('ref' => 'people', 'data' => $companies[0]['people']));
			$this->people_dock = JoomcrmHelperView::getView('people', 'people_dock', $ref);

			$custom_fields_view = JoomcrmHelperView::getView('custom', 'default');
			$type               = "company";
			$custom_fields_view->type = $type;
			$custom_fields_view->item = $companies[0];
			$this->custom_fields_view = $custom_fields_view;

			$mailingEnabled = Mailing::loadIntegration() && $config->get('acy_company',1) == 1;

			$this->mailing = Mailing::loadIntegration();

			if ($mailingEnabled)
			{
				$mailing_lists = $this->mailing->lists;

				$newsletters   = array();
				if (is_array($mailing_lists) && array_key_exists(0, $mailing_lists))
				{
					$newsletters = $this->mailing->getNewsletters($mailing_lists[0]->listid);
				}
				$mailing_dock = JoomcrmHelperView::getView('mailing', 'default');
				$mailing_dock->newsletters = $newsletters;
				$mailing_dock->mailing_lists = $mailing_lists;
				$mailing_dock->company = $companies[0];
				$this->mailing_dock = $mailing_dock;
			}

			if (JoomcrmHelperBanter::hasBanter())
			{
				$room_lists  = JoomcrmHelperTranscriptlists::getRooms();
				$transcripts = array();
				if (is_array($room_lists) && count($room_lists) > 0)
				{
					$transcripts = $room_list->getTranscripts($room_lists[0]->id);
				}
				$banter_dock = JoomcrmHelperView::getView('banter', 'default');
				$banter_dock->rooms = $room_lists;
				$banter_dock->transcripts = $transcripts;
				$this->banter_dock = $banter_dock;
			}

			/** get latest activities **/
			$activity               = JoomcrmHelperActivity::getActivity();
			$latest_activities      = JoomcrmHelperView::getView('activities', 'default');
			$latest_activities_view = JoomcrmHelperView::getView('activities', 'latest_activities');
			$latest_activities_view->activity = $activity;

			$latest_activities->latest_activities = $latest_activities_view;
			$this->latest_activities = $latest_activities;
		}

		if ($layout == "default")
		{
			$company_list = JoomcrmHelperView::getView('companies', 'list', array(array('ref' => 'companies', 'data' => $companies)));
			$company_list->show_filters = $model->show_filters;
			$company_list->state = $state;
			$company_list->app = $app;

			$company_list->member_role = $member_role;
			$company_list->user_name = $user_name;
			$company_list->users = $users;
			$company_list->teams = $teams;

			$total = $model->getTotal();
			$company_list->total = $total;
			$this->pagination->setAdditionalUrlParam('Itemid',JoomcrmHelperRoute::getCompaniesRoute(1));
			$company_list->pagination = $this->pagination;
			$this->company_list = $company_list;

			$this->company_filter = $session->get('company_name_filter');

		}

		if ($layout == "edit")
		{
			if ($session->get("Company.error"))
			{
				$item         = $session->get("Company.editing");
				$item['id']   = '';
				$companies[0] = $item;
				$session->set("Company.error", false);
				$session->set("Company.error_message", "");
				$session->set("Company.editing", null);
			}
			else
			{
				$item =  \Joomla\CMS\Factory::getApplication()->input->get('id') && array_key_exists(0, $companies) ? $companies[0] : array('id' => '');
			}
			$edit_custom_fields_view = JoomcrmHelperView::getView('custom', 'edit');
			$edit_custom_fields_view->type = 'company';
			$edit_custom_fields_view->item = $item;

			$this->edit_custom_fields_view = $edit_custom_fields_view;

			$doc->addScriptDeclaration("var loc='company_edit';");
			$companyId = (isset($companies) && is_array($companies) && array_key_exists(0, $companies)) ? $companies[0]['id'] : 0;
			$doc->addScriptDeclaration("var company_id=" . $companyId . ";");
		}

		// Ref assignments
		$this->companies = $companies;
		$this->user_id = $user_id;
		$this->member_role = $member_role;
		$this->teams = $teams;
		$this->users = $users;
		$this->company_types = $company_types;
		$this->company_type = $company_type;
		$this->company_categories = $company_categories;
		$this->company_category = $company_category;
		$this->user_name = $user_name;
		$this->company_count = $company_count;
		$this->state = $state;
		$this->app = $app;
		$this->column_filters = $column_filters;
		$this->selected_columns = $selected_columns;
		$this->show_filters = $model->show_filters;

		parent::display($tpl);
	}

}
