<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

class JoomcrmViewDashboardHtml extends JoomcrmHelperView
{
	public function display($tpl = null)
	{
		$doc = Factory::getDocument();

		$dashFloats        = JoomcrmHelperUsers::getFloats('dashboard');

		$this->dash_floats_left = $this->removeWidgetFromDash($dashFloats['left']);
		$this->dash_floats_right = $this->removeWidgetFromDash($dashFloats['right']);



		/* not sure what this code used for, will be checked in next update
		$peopleModel = new JoomcrmModelPeople;
		$json        = true;
		$peopleNames = $peopleModel->getPeopleNames($json);
		$doc->addScriptDeclaration("var people_names=" . $peopleNames . ";");

		$dealModel = new JoomcrmModelDeal;
		$json      = true;
		$dealNames = $dealModel->getDealNames($json);
		$doc->addScriptDeclaration("var deal_names=" . $dealNames . ";");
		*/


		parent::display($tpl);
	}


	private function widgetsConfigMap(){

		$config = ComponentHelper::getParams('com_joomcrm');

		$isSetInbox = !empty($config->get('imap_host','')) &&
			!empty($config->get('imap_host','imap_user','')) &&
			!empty($config->get('imap_host','imap_pass','')) &&
			!empty($config->get('imap_host','imap_user',''));

		return [
			'tasks_events_float' => $config->get('show_tasks_widget',1),
			'sales_float' => $config->get('show_my_sales_widget',1),
			'deals_float' => $config->get('show_recent_accessed_deals_widget',1),
			'inbox_float' => $config->get('show_inbox_widget',1) && $isSetInbox,
			'latest_float' => $config->get('show_latest_activities_widget',1)
		];



	}

	// remove disabled widgets from config
	private function removeWidgetFromDash($widgets){

		if(empty($widgets) || count($widgets) == 0)
			return [];

		// widgets mapped with config 0 => disabled , 1 => enabled
		$widgetsConfigMap = $this->widgetsConfigMap();

		foreach ($widgets as $k => $name){
			// if widget disabled from config remove it from array
			if(!$widgetsConfigMap[$name])
				unset($widgets[$k]);
		}

		return $widgets;
	}



	public function getDateFormat(){

		$user = JoomCRMHelperUsers::getLoggedInUser();
		$dateFormat = $user->date_format;

		if ($dateFormat == 'd/m/y')
		{
			$dateFormat = 'dd/mm/yy';
		}
		elseif ($dateFormat == 'm/d/y')
		{
			$dateFormat = 'mm/dd/yy';
		}

		return $dateFormat;

	}

	public function getRecentlyAccessedDeals(){
		$dealModel = new JoomcrmModelDeal;
		$dealModel->set('recent', true);
		$dealModel->set("ordering", "d.last_viewed DESC");
		$dealModel->set('archived', 0);
		$dealModel->set("limitOverride", true);
		$dealModel->set("limit", 20);
		$recentDeals = $dealModel->getDeals();
	}

	public function getMySales(){

		// load JavaScript libs
		$doc = Factory::getDocument();
		$doc->addScript(Uri::base() . 'components/com_joomcrm/media/js/highcharts.js');
		$doc->addScript(Uri::base() . 'components/com_joomcrm/media/js/dashboard.js');
		$doc->addScriptDeclaration('var graphType="dashboard";');
		$doc->addScriptDeclaration('var homePageChart="' . JoomcrmHelperUsers::getHomePageChart() . '";');


		// Get data for sales graphs
		$model = new JoomcrmModelGraphs;
		$model->set("graphType", "dashboard");
		return $model->getGraphData();

	}

	public function getEventsTasks(){

		// Get model and retrieve info
		$model = new JoomcrmModelEvent;

		$model->set("limit", "20");
		$model->set('filter_order_Dir', 'asc');
		$events    = $model->getEvents();
		$ref       = array(array('ref' => 'events', 'data' => $events));
		$eventDock = JoomcrmHelperView::getView('events', 'dashboard_event_dock', $ref);

		return $eventDock;

	}

	public function getLatestActivities(){

		$activity               = JoomcrmHelperActivity::getActivity();
		$latest_activities      = JoomcrmHelperView::getView('activities', 'default');
		$latest_activities_view = JoomcrmHelperView::getView('activities', 'latest_activities');
		$latest_activities_view->activity = $activity;
		$latest_activities->latest_activities = $latest_activities_view;

		return $latest_activities;

	}

}
