<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;
;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

class JoomcrmViewDealsHtml extends JoomcrmView
{
	public function display($tpl = null)
	{
		$input = Factory::getApplication()->input;
		// Retrieve deal list from model
		$model    = new JoomcrmModelDeal;
		$state    = $model->getState();
		$dealList = array();
		$deal     = array();
		$doc      = Factory::getDocument();

		// add some data to js for ajax pagination
		\Joomla\CMS\Factory::getDocument()->addScriptOptions('com_joomcrm.urlParams',[
			'Itemid' => JoomcrmHelperRoute::getCompaniesRoute(1)
		]);

		// Session info
		$session     = Factory::getSession();
		$member_role = JoomcrmHelperUsers::getRole();
		$user_id     = JoomcrmHelperUsers::getUserId();
		$team_id     = JoomcrmHelperUsers::getTeamId();

		// Determine if we are requesting a specific deal or all deals
		// If id requested
		if ($input->getInt('id',0))
		{
			$model->set('_id', $input->getInt('id',0));
			$dealList = $model->getDeals();

			if (is_null($dealList[0]['id']))
			{



				$app = Factory::getApplication();
				$app->redirect(Route::_('index.php?option=com_joomcrm&view=deals'), Text::_('COM_JOOMCRM_NOT_AUTHORIZED'));
			}
		}
		else
		{
			// Else load all deals

			$this->pagination = $model->getPagination();

			if ( \Joomla\CMS\Factory::getApplication()->input->get('layout') != 'edit')
			{

				$dealList = $model->getDeals();
			}
		}

		// Determine if we are editing an existing deal entry
		if (count($dealList) == 1)
		{
			// Grab deal object
			$deal           = $dealList[0];
			$deal['header'] = ucwords(Text::_('COM_JOOMCRM_DEAL_EDIT'));
		}
		else
		{
			// Else we are creating a new entry
			$deal               = array();
			$deal['name']       = "";
			$deal['summary']    = "";
			$deal['company_id'] = $input->getInt('company_id',0);
			$deal['person_id']  = $input->getInt('person_id',0);

			// Get company name to prefill data and hidden fields
			if ($deal['company_id'])
			{
				$company              = JoomcrmHelperCompany::getCompany($deal['company_id']);
				$deal['company_name'] = $company[0]['name'];
				$deal['company_id']   = $company[0]['id'];
			}

			// If a person is specified prefill data
			if ($deal['person_id'])
			{
				$person = JoomcrmHelperPeople::getPerson($deal['person_id']);

				$deal['person_name'] = $person[0]['last_name'] . ', ' . $person[0]['first_name'];
				$deal['person_id']   = $person[0]['id'];

				// Assign company if person is associated with company
				if ($person[0]['company_id'])
				{
					$deal['company_id']   = $person[0]['company_id'];
					$deal['company_name'] = $person[0]['company_name'];
				}

			}

			// Assign rest of null data
			$deal['amount']         = "";
			$deal['stage_id']       = 0;
			$deal['source_id']      = 0;
			$deal['probability']    = 0;
			$deal['status_id']      = 0;
			$deal['expected_close'] = "";
			$deal['header']         = ucwords(Text::_('COM_JOOMCRM_DEAL_HEADER'));
		}

		$doc->addScript(Uri::base() . 'components/com_joomcrm/media/js/deal_manager.js');


		// Dropdown info
		// Get deal type filters
		$deal_types     = JoomcrmHelperDeal::getDealTypes();
		$deal_type_name = $session->get('deal_type_filter');
		$deal_type_name = ($deal_type_name) ? $deal_types[$deal_type_name] : $deal_types['all'];

		// Get column filters
		$column_filters   = JoomcrmHelperDeal::getColumnFilters();
		$selected_columns = JoomcrmHelperDeal::getSelectedColumnFilters();

		// Get member access info
		$teams  = JoomcrmHelperUsers::getTeams();
		$users  = JoomcrmHelperUsers::getUsers();
		$stages = JoomcrmHelperDeal::getStages();

		// Get deal stage filters
		$stage_name = $session->get('deal_stage_filter');
		$stage_name = ($stage_name) ? $stages[$stage_name] : $stages['all'];

		// Get session data to prefill filters
		$user_filter = $session->get('deal_user_filter');
		$team_filter = $session->get('deal_team_filter');
		if ($user_filter == "all" && $user_filter != $user_id)
		{
			$user_name = Text::_('COM_JOOMCRM_ALL_USERS');
		}
		else
		{
			if ($user_filter == "all")
			{
				$user_name = Text::_('COM_JOOMCRM_ME');
			}
			else
			{
				if ($user_filter AND $user_filter != $user_id AND $user_filter != 'all')
				{

					$user_info = JoomcrmHelperUsers::getUsers($user_filter);

					$user_info = $user_info[0];
					$user_name = $user_info['first_name'] . " " . $user_info['last_name'];
				}
				else
				{
					if ($team_filter)
					{
						$team_info = JoomcrmHelperUsers::getTeams($team_filter);
						$team      = $team_info[0];
						$user_name = $team['team_name'] . Text::_('COM_JOOMCRM_TEAM_APPEND');
					}
					else
					{
						$user_name = Text::_('COM_JOOMCRM_ME');
					}
				}
			}
		}

		// Get closing time filters
		$closing_names = JoomcrmHelperDeal::getClosing();
		$closing_name  = $session->get('deal_close_filter');
		$closing_name  = ($closing_name) ? $closing_names[$closing_name] : $closing_names['all'];

		// Get total deals associated with user
		$total_deals = JoomcrmHelperUsers::getDealCount($user_id, $team_id, $member_role);

		// Load Events & Tasks for person
		$layout = $this->getLayout();
		if ($layout == "deal")
		{
			$model     = new JoomcrmModelEvent;
			$events    = $model->getEvents("deal", null, $input->getInt('id'));
			$ref       = array(array('ref' => 'events', 'data' => $events));
			$eventDock = JoomcrmHelperView::getView('events', 'event_dock', $ref);
			$this->event_dock = $eventDock;

			$ref                = array(array('ref' => 'contacts', 'data' => $dealList[0]['people']));
			$contact_info       = JoomcrmHelperView::getView('contacts', 'default', $ref);
			$primary_contact_id = JoomcrmHelperDeal::getPrimaryContact($dealList[0]['id']);
			$contact_info->primary_contact_id = $primary_contact_id;
			$this->contact_info = $contact_info;

			$docViewType = ComponentHelper::getParams('com_joomcrm')->get('documents_view_type','datatable');
			$docType = ($docViewType == 'datatable') ? 'document_dock' : 'document_tree';
			$ref = array(array('ref' => 'documents', 'data' => $deal['documents']));
			$this->document_dock = JoomcrmHelperView::getView('documents', $docType, $ref);

			$custom_fields_view = JoomcrmHelperView::getView('custom', 'default');
			$type               = "deal";
			$custom_fields_view->type = $type;
			$custom_fields_view->item = $dealList[0];
			$this->custom_fields_view = $custom_fields_view;

			if (JoomcrmHelperBanter::hasBanter())
			{
				$room_lists  = JoomcrmHelperTranscriptlists::getRooms();
				$transcripts = array();
				if (is_array($room_lists) && count($room_lists) > 0)
				{
					$transcripts = $room_lists->getTranscripts($room_lists[0]->id);
				}
				$banter_dock = JoomcrmHelperView::getView('banter', 'default');
				$banter_dock->rooms = $room_lists;
				$banter_dock->transcripts = $transcripts;
				$this->banter_dock = $banter_dock;
			}

			if (JoomcrmHelperQuote::hasCRMQuote())
			{
				$deal       = & $dealList[0];
				$quote_dock = JoomcrmHelperQuote::getDockView($deal);
				$this->quote_dock = $quote_dock;
			}

			/** get latest activities **/
			$activity               = JoomcrmHelperActivity::getActivity();
			$latest_activities      = JoomcrmHelperView::getView('activities', 'default');
			$latest_activities_view = JoomcrmHelperView::getView('activities', 'latest_activities');
			$latest_activities_view->activity = $activity;
			$latest_activities->latest_activities = $latest_activities_view;
			$this->latest_activities = $latest_activities;
		}

		if ($layout == "default" || $layout == "list")
		{
			$deal_list = JoomcrmHelperView::getView('deals', 'list', array(array('ref' => 'dealList', 'data' => $dealList)));
			$this->total     = $model->getTotal();

			$deal_list->total = $this->total;
			$deal_list->pagination = $this->pagination;
			$this->deal_list = $deal_list;
			$this->state = $state;
			$doc->addScriptDeclaration("
			loc = 'deals';
			order_url = Joomla.getOptions('system.paths').rootFull+'index.php?option=com_joomcrm&view=deals&layout=list&format=raw&tmpl=component';
			order_dir = '" . $state->get('Deal.filter_order_Dir') . "';
			order_col = '" . $state->get('Deal.filter_order') . "';");

			$deal_name = $session->get('deal_name_filter');
			$this->deal_filter = $deal_name;
		}

		if ($layout == "edit")
		{
			$edit_custom_fields_view = JoomcrmHelperView::getView('custom', 'edit');
			$edit_custom_fields_view->type = 'deal';
			$edit_custom_fields_view->item = ( \Joomla\CMS\Factory::getApplication()->input->get('id') && array_key_exists(0, $dealList)) ? $dealList[0] : array('id' => '');
			$this->edit_custom_fields_view = $edit_custom_fields_view;

			$companyModel = new JoomcrmModelCompany;
			$json         = true;
			$companyNames = $companyModel->getCompanyNames($json);
			$doc->addScriptDeclaration("var company_names=" . $companyNames . ";");

			$peopleModel = new JoomcrmModelPeople;
			$json        = true;
			$peopleNames = $peopleModel->getPeopleNames($json);
			$doc->addScriptDeclaration("var people_names=" . $peopleNames . ";");
		}

		$closed_stages = JoomcrmHelperDeal::getClosedStages();

		// Assign results to view
		$this->closed_stages = $closed_stages;
		$this->dealList = $dealList;
		$this->totalDeals = $total_deals;
		$this->deal = $deal;
		$this->deal_types = $deal_types;
		$this->deal_type_name = $deal_type_name;
		$this->user_id = $user_id;
		$this->member_role = $member_role;
		$this->teams = $teams;
		$this->users = $users;
		$this->stages = $stages;
		$this->stage_name = $stage_name;
		$this->user_name = $user_name;
		$this->closing_names = $closing_names;
		$this->closing_name = $closing_name;
		$this->state = $state;
		$this->column_filters = $column_filters;
		$this->selected_columns = $selected_columns;

		parent::display($tpl);
	}

}
