<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;

class JoomcrmViewDocumentsHtml extends JoomcrmView
{
	public function display($tpl = null)
	{
		// Get model
		$model            = new JoomcrmModelDocument;
		$this->pagination = $model->getPagination();
		$this->documents  = $model->getDocuments();
		$this->state      = $model->getState();

		// add some data to js for ajax pagination
		\Joomla\CMS\Factory::getDocument()->addScriptOptions('com_joomcrm.urlParams',[
			'Itemid' => JoomcrmHelperRoute::getCompaniesRoute(1)
		]);


		// Add js
		Factory::getDocument()
			->addScript(Uri::root() . 'administrator/components/com_joomcrm/media/js/document_manager.js')
			->addScriptDeclaration("var loc='documents';
				order_url = Joomla.getOptions('system.paths').rootFull+'index.php?option=com_joomcrm&view=documents&layout=list&format=raw&tmpl=component';
				order_dir = '" . $this->state->get('Document.document.filter_order_Dir') . "';
				order_col = '" . $this->state->get('Document.filter_order') . "';");

		// Session data
		$session           = Factory::getSession();
		$this->member_role = JoomcrmHelperUsers::getRole();

		// Associations
		$assoc             = $session->get('document_assoc_filter');
		$this->assoc_names = JoomcrmHelperDocument::getAssocTypes();
		$this->assoc_name  = ($assoc) ? $this->assoc_names[$assoc] : $this->assoc_names['all'];

		// Users
		$this->user_id = JoomcrmHelperUsers::getUserId();
		$user          = $session->get('document_user_filter');
		$team          = $session->get('document_team_filter');

		// categories
		$category    = $session->get("company_category_filter");


		// Get company category filters
		$document_categories = JoomcrmHelperDocument::getCategories();
		$document_category   = ($category) ? $document_categories[$category] : 0;

		if ($user == "all")
		{
			$this->user_name = Text::_('COM_JOOMCRM_ALL_USERS');
		}
		else
		{
			if ($user && $user != $this->user_id)
			{
				$user_info = JoomcrmHelperUsers::getUser($user);
				$this->user_name = $user_info->first_name . " " . $user_info->last_name;
			}
			else
			{
				if ($team)
				{
					$team_info = JoomcrmHelperUsers::getTeams($team);
					$team_info = $team_info[0];
					$this->user_name = $team_info['team_name'] . Text::_('COM_JOOMCRM_TEAM_APPEND');
				}
				else
				{
					$this->user_name = Text::_('COM_JOOMCRM_ME');
				}
			}
		}

		// Type
		$type             = $session->get('document_type_filter');
		$this->type_names = JoomcrmHelperDocument::getDocTypes();
		$this->type_name  = ($type && array_key_exists($type, $this->type_names)) ? $this->type_names[$type] : $this->type_names['all'];

		// Teams
		$this->teams = JoomcrmHelperUsers::getTeams();
		// Users
		$this->users = JoomcrmHelperUsers::getUsers();

		// List view
		$data = array(
			array('ref' => 'documents', 'data' => $this->documents),
			array('ref' => 'state', 'data' => $this->state),
			array('ref' => 'pagination', 'data' => $this->pagination)
		);
		$this->document_list = JoomcrmHelperView::getView('documents', 'list', $data);

		$this->document_categories = $document_categories;
		$this->document_category = $document_category;


		// Display
		parent::display($tpl);
	}

}
