<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Factory;

JLoader::register('JoomcrmHelperArray', JPATH_ADMINISTRATOR . '/components/com_joomcrm/helper/array.php');

foreach ($this->documents as &$document)
{
	if ($document['category_name'] == null) $document['category_name'] = Text::_('COM_JOOMCRM_NO_CATEGORY');
}

$tree = JoomcrmHelperArray::array_group_by($this->documents, 'category_name');
$i    = 0;

?>

<div class="accordion mt-2" id="accordionCats">
	<?php foreach ($tree as $cat => $documents): ?>
        <div class="card" style="overflow: visible">
            <div class="card-header p-2 bg-white" id="cat-heading-<?php echo $i ?>">
                <h2 class="mb-0">
                    <button class="btn btn-link btn-block text-start" type="button" data-bs-toggle="collapse" data-target="#cat-collapse-<?php echo $i ?>" aria-expanded="true" aria-controls="#cat-collapse-<?php echo $i ?>">
						<i class="far <?php echo $i == 0 ? 'fa-folder-open' : 'fa-folder' ?>"></i> <?php echo $cat ?>

                        <span class="badge bg-dark float-end"><?php echo count($documents) ?></span>
                    </button>

                </h2>
            </div>

            <div id="cat-collapse-<?php echo $i ?>" class="collapse<?php echo $i == 0 ? ' show' : '' ?>" aria-labelledby="cat-heading-<?php echo $i ?>" data-parent="#accordionCats">
                <div class="card-body p-2">
					<table class="table table-borderless m-0">
						<?php foreach ($documents as $document): ?>
							<?php
							$imageFile = \Joomla\CMS\Filesystem\File::exists(JPATH_BASE . '/components/com_joomcrm/media/images/' . $document['filetype'] . '.png') ? Uri::base() . 'components/com_joomcrm/media/images/' . $document['filetype'] . '.png' : Uri::base() . 'components/com_joomcrm/media/images/generic_file.png';

							?>
							<tr>
								<td>
                                    <div class="bg-light p-1 m-1 d-inline-block text-center">
                                        <img class="d-block" width="30px" height="30px" src="<?php echo $imageFile ?>"/>
                                        <small class="text-muted">
                                            <strong><?php echo strtoupper($document['filetype']) ?></strong>
                                        </small>
                                    </div>
                                </td>

                                <td>
                                <td>
									<?php echo LayoutHelper::render('document.dropdown',['document' => $document],null,['component' => 'com_joomcrm','client' => 'site']) ?>
                                </td>
                                </td>

                                <td>
	                                <?php echo $document['owner_name'] ?>
                                </td>

                                <td>
	                                <?php echo $document['size'] . 'Kb' ?>
                                </td>

                                <td>
	                                <?php echo JoomcrmHelperDate::formatDate($document['created']) ?>
                                </td>

                            </tr>
                            <div class="document_edit_menu" id="document_<?php echo $document['id'] ?>">

                                <input
                                        type="hidden"
                                        name="document_<?php echo $document['id'] ?>_hash"
                                        id="document_<?php echo $document['id'] ?>_hash"
                                        value="<?php echo $document['filename'] ?>"
                                />

                            </div>
						<?php endforeach; ?>
                    </table>
                </div>
            </div>
        </div>
		<?php $i++ ?>
	<?php endforeach; ?>
</div>

<script>

    jQuery(document).ready(function($){

        $('#accordionCats').on('shown.bs.collapse', function () {
            $(this).find('.collapse').prev().find('i').removeClass('fa-folder-open').addClass('fa-folder')
            $(this).find('.collapse.show').prev().find('i').removeClass('fa-folder').addClass('fa-folder-open')
        })

        $('#accordionCats').on('hide.bs.collapse', function () {
            $(this).find('.collapse.show').prev().find('i').removeClass('fa-folder-open').addClass('fa-folder')
        })
    });


</script>