<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;

class JoomcrmViewGoalsHtml extends JoomcrmView
{
	public function display($tpl = null)
	{
        $type =  \Joomla\CMS\Factory::getApplication()->input->get('type');
        $header     = "";
        // Edit layout
		if ($this->getLayout() == 'edit')
		{
			// Determine the type of goal we are creating// Editing
			switch ($type)
			{
				case "win_cash":
					$header = ucwords(Text::_('COM_JOOMCRM_WIN_MORE_CASH'));
					break;
				case "win_deals":
					$header = ucwords(Text::_('COM_JOOMCRM_WIN_MORE_DEALS'));
					break;
				case "move_deals";
					$header = ucwords(Text::_('COM_JOOMCRM_MOVE_DEALS_FORWARD'));
					break;
				case "complete_tasks";
					$header = ucwords(Text::_('COM_JOOMCRM_COMPLETE_TASKS'));
					break;
				case "write_notes";
					$header = ucwords(Text::_('COM_JOOMCRM_WRITE_NOTES'));
					break;
				case "create_deals";
					$header = ucwords(Text::_('COM_JOOMCRM_CREATE_DEALS'));
					break;
				default:
					if ( \Joomla\CMS\Factory::getApplication()->input->get('id'))
					{
						$model      = new JoomcrmModelGoal;
						$this->goal = $model->getGoal();
						$this->type = $this->goal['goal_type'];
						$header     = ucwords(Text::_('COM_JOOMCRM_' . strtoupper($this->goal['goal_type'])));
					}
					else
					{
						Factory::getApplication()->redirect('index.php?option=com_joomcrm&view=goals');
					}
					break;
			}

		}
		else
		{
			if ($this->getLayout() != 'add')
			{
				// Load model
				$model = new JoomcrmModelGoal;

				// Get all goals from model depending on user type
				$member_role = JoomcrmHelperUsers::getRole();

				// Basic members
				if ($member_role == 'basic')
				{
					$individual_goals = $model->getIndividualGoals();
					$team_goals       = $model->getTeamGoals();
					$company_goals    = $model->getCompanyGoals();
					$leaderboards     = $model->getLeaderBoards();
				}

				// Managers
				if ($member_role == 'manager')
				{
					// $individual_goals = $model->getManagerIndividualGoals();
					$individual_goals = $model->getIndividualGoals();
					$team_goals       = $model->getTeamGoals();
					$company_goals    = $model->getCompanyGoals();
					$leaderboards     = $model->getLeaderBoards();
				}

				// Executives
				if ($member_role == 'exec')
				{
					// $individual_goals = $model->getExecIndividualGoals();
					$individual_goals = $model->getIndividualGoals();
					// $team_goals = $model->getExecTeamGoals();
					$team_goals    = $model->getTeamGoals();
					$company_goals = $model->getCompanyGoals();
					$leaderboards  = $model->getLeaderBoards();
				}

				// Assign goals to global goal object to pass through to view
				$goals                   = new stdClass();
				$goals->individual_goals = $individual_goals;
				$goals->team_goals       = $team_goals;
				$goals->company_goals    = $company_goals;
				$goals->leaderboards     = $leaderboards;

				// If we get results then load the default goals page else show the add goals page
				$goal_count = false;

				foreach ($goals as $goal_list)
				{
					if (count($goal_list) > 0)
					{
						$goal_count = true;
					}
				}
				if ($goal_count)
				{
					// Set layout
					$this->setLayout('default');
					// Assign view refs
					$this->goals = $goals;
				}
				else
				{
					// Add goal layout
					$this->setLayout('add');
				}
			}
		}

		// Load java libs
		$doc = Factory::getDocument();
		$doc->addScript(Uri::base() . 'components/com_joomcrm/media/js/goal_manager.js');
		$doc->addScriptDeclaration('var loc = "goals";');

		// Get associated members and teams
		$teams       		= JoomcrmHelperUsers::getTeams();
		$users       		= JoomcrmHelperUsers::getUsers();
		$member_role 		= JoomcrmHelperUsers::getRole();
		$user_id     		= JoomcrmHelperUsers::getUserId();
		$team_id     		= JoomcrmHelperUsers::getTeamId();

		// Assign view refs
		$this->type = $type;
		$this->header = $header;
		$this->teams = $teams;
		$this->users = $users;
		$this->user_id = $user_id;
		$this->team_id = $team_id;
		$this->member_role = $member_role;

		$goalFloats        = JoomcrmHelperUsers::getFloats('goal');
		$goal_floats_left  = $goalFloats['left'];
		$goal_floats_right = $goalFloats['right'];
		$this->goal_floats_left = $goal_floats_left;
		$this->goal_floats_right = $goal_floats_right;

		parent::display($tpl);
	}

}
