<?php
/**
 * JoomCRM
 *
 * @author     JoomBoost <support@joomboost.com>
 * @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
 * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Website: https://www.joomboost.com
 */
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Uri\Uri;

// load tab state saver

// load JS File
HTMLHelper::_('script',Uri::root().'/media/com_joomcrm/js/myteam.js');

?>
<div id="jb_template">

    <div class="page-header mb-4">
		<?php if ($this->isTeamOwner): ?>

        <?php if(Factory::getApplication()->getIdentity()->authorise('core.inviteto.own.team','com_joomcrm')): ?>
            <div class="float-end">
				<?php echo LayoutHelper::render('myteam.invitemodal') ?>
            </div>
        <?php endif; ?>

        <h1><?php echo Text::_('COM_JOOMCRM_MANAGE_MYTEAM') ?></h1>
		<?php else: ?>
        <h1><?php echo Text::_('COM_JOOMCRM_MY_TEAM') ?></h1>
        <?php endif; ?>
    </div>

	<?php echo HTMLHelper::_('uitab.startTabSet', 'teamTab', array('active' => 'myTeam')); ?>

	<?php echo HTMLHelper::_('uitab.addTab', 'teamTab', 'myTeam', '<i class="fas fa-users"></i> ' . Text::_('COM_JOOMCRM_MEMBERS')); ?>
    <div class="border rounded mb-3">
        <table class="m-0 table table-hover bg-white">
            <thead class="bg-light">
            <tr>
                <th scope="col" class="border-0"><?php echo Text::_('COM_JOOMCRM_MYTEAM_NAME') ?></th>
                <th scope="col" class="border-0"><?php echo Text::_('COM_JOOMCRM_MYTEAM_USERNAME') ?></th>
                <th scope="col" class="border-0"><?php echo Text::_('COM_JOOMCRM_MYTEAM_EMAIL') ?></th>
                <th scope="col" class="border-0"><?php echo Text::_('COM_JOOMCRM_MYTEAM_ROLE') ?></th>
				<?php if ($this->isTeamOwner): ?>
                    <th scope="col" class="border-0"><?php echo Text::_('COM_JOOMCRM_MYTEAM_ACTIONS') ?></th>
				<?php endif; ?>
            </tr>
            </thead>
            <tbody>
			<?php foreach ($this->myteam as $user):

				// if user in loop is current user and is team owner
				$teamOwner = (Factory::getApplication()->getIdentity()->id == $user->uid) && $this->isTeamOwner;

			    // skip current user info
			    if((Factory::getApplication()->getIdentity()->id == $user->uid) && !$this->isTeamOwner) continue;

				?>
                <tr>
                    <th scope="row">
						<?php echo $user->first_name; ?> <?php if ($teamOwner): ?><small class="badge bg-success"><i
                                class="far fa-star"></i> <?php echo Text::_('COM_JOOMCRM_PEOPLE_OWNER') ?>
                            </small><?php endif; ?>
                    </th>
                    <td><?php echo Factory::getUser($user->uid)->username ?></td>
                    <td><?php echo Factory::getUser($user->uid)->email ?></td>
                    <td><?php echo $user->role_type; ?></td>
					<?php if ($this->isTeamOwner): ?>
                        <td>
							<?php if (!$teamOwner): ?>
                                <a href="<?php echo Route::_('index.php?option=com_joomcrm&task=myteam.deleteMember&' . Session::getFormToken() . '=1&memberId=' . $user->id) ?>" class="btn btn-sm btn-danger">
                                    <i class="fas fa-times"></i> <?php echo Text::_('COM_JOOMCRM_DELETE') ?>
                                </a>
							<?php endif; ?>
                        </td>
					<?php endif; ?>
                </tr>
			<?php endforeach; ?>
        </table>
    </div>
	<?php echo HTMLHelper::_('uitab.endTab'); ?>
	<?php if ($this->isTeamOwner):
		$invites_count = !empty($this->myinvites) ? count($this->myinvites) : 0;
		?>
		<?php echo HTMLHelper::_('uitab.addTab', 'teamTab', 'pendingUsers', '<i class="fas fa-user-clock"></i> ' . Text::_('COM_JOOMCRM_PENDING_INVITATIONS') . "  <span class='badge bg-dark text-white rounded'>" . $invites_count . "</span>"); ?>
		<?php if ($invites_count > 0): ?>
        <div class="border rounded mb-3">
            <table class="m-0 table table-hover bg-white">
                <thead class="bg-light">
                <tr>
                    <th scope="col" class="border-0"><?php echo Text::_('COM_JOOMCRM_MYTEAM_EMAIL') ?></th>
                    <th scope="col" class="border-0"><?php echo Text::_('COM_JOOMCRM_MYTEAM_SENT_AT') ?></th>
                    <th scope="col" class="border-0"
                        width="20%"><?php echo Text::_('COM_JOOMCRM_MYTEAM_ACTIONS') ?></th>
                </tr>
                </thead>
                <tbody>
				<?php foreach ($this->myinvites as $invite): ?>
                    <tr>
                        <th scope="row">
							<?php echo $invite->email ?>
                        </th>
                        <td><?php echo $invite->sent_date ?></td>
                        <td>
                            <a href="<?php echo Route::_('index.php?option=com_joomcrm&task=myteam.resendEmail&' . Session::getFormToken() . '=1&inviteId=' . $invite->id) ?>"
                               class="btn btn-sm btn-light text-dark border-light bg-light"
                            >
                                <i class="far fa-paper-plane"></i> <?php echo Text::_('COM_JOOMCRM_RESEND_EMAIL') ?>
                            </a>

                            <a href="<?php echo Route::_('index.php?option=com_joomcrm&task=myteam.deleteInvite&' . Session::getFormToken() . '=1&inviteId=' . $invite->id) ?>"
                               class="btn btn-sm btn-danger"
                            >
                                <i class="fas fa-times"></i> <?php echo Text::_('COM_JOOMCRM_DELETE') ?>
                            </a>

                        </td>
                    </tr>
				<?php endforeach; ?>
            </table>
        </div>

	<?php else: ?>
        <p class="m-0 alert alert-warning"><?php echo Text::_('COM_JOOMCRM_NO_PENDING_INVITATIONS') ?></p>
	<?php endif; ?>
		<?php echo HTMLHelper::_('uitab.endTab'); ?>
	<?php endif; ?>



	<?php echo HTMLHelper::_('uitab.endTabSet'); ?>

</div>