<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/



defined('_JEXEC') or die;

use Joomcrm\Mailing\Mailing;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

class JoomcrmViewPeopleHtml extends JoomcrmHelperView
{
	public function display($tpl = null)
	{
		/// Retrieve task list from model
		$model = $this->getModel();
		$state = $model->getState();
		$input = Factory::getApplication()->input;
		$config     = ComponentHelper::getParams('com_joomcrm');

		// add some data to js for ajax pagination
		\Joomla\CMS\Factory::getDocument()->addScriptOptions('com_joomcrm.urlParams',[
			'Itemid' => JoomcrmHelperRoute::getCompaniesRoute(1)
		]);

		// Pagination
		$this->pagination = $model->getPagination();

		// Session data
		$session = Factory::getSession();

		$user_id     = JoomcrmHelperUsers::getUserId();
		$team_id     = JoomcrmHelperUsers::getTeamId();
		$member_role = JoomcrmHelperUsers::getRole();

		$people_type_name = $session->get('people_type_filter');
		$user             = $session->get('people_user_filter',$config->get('people_user_filter',1));
		$team             = $session->get('people_team_filter');
		$stage            = $session->get('people_stage_filter',$config->get('people_updated_filter','past_thirty'));
		$tag              = $session->get('people_tag_filter');
		$status           = $session->get('people_status_filter',$config->get('people_status_filter','any'));

		// Load java
		$document = Factory::getDocument();
		$document->addScript(Uri::base() . 'components/com_joomcrm/media/js/people_manager.js');


		$people = $model->getPeople();
		$person = array();

		// Determine if we are editing an existing person entry
		if ( \Joomla\CMS\Factory::getApplication()->input->get('id'))
		{
			// Grab deal object
			$person = $people[0];
			if (is_null($person['id']))
			{
				$app = Factory::getApplication();
				$app->redirect(Route::_('index.php?option=com_joomcrm&view=people'), Text::_('COM_JOOMCRM_NOT_AUTHORIZED'));
			}
			$person['header'] = Text::_('COM_JOOMCRM_EDIT') . ' ' . $person['first_name'] . ' ' . $person['last_name'];
		}
		else
		{
			// Else we are creating a new entry
			$person               = array();
			$person['id']         = '';
			$person['first_name'] = "";
			$person['last_name']  = "";
			$person['company_id'] = ( \Joomla\CMS\Factory::getApplication()->input->get('company_id')) ?  \Joomla\CMS\Factory::getApplication()->input->get('company_id') : null;
			$person['deal_id']    = ( \Joomla\CMS\Factory::getApplication()->input->get('deal_id')) ?  \Joomla\CMS\Factory::getApplication()->input->get('deal_id') : null;

			// Get company name to prefill data on page and hidden fields
			if ($person['company_id'])
			{
				$company                = JoomcrmHelperCompany::getCompany($person['company_id']);
				$person['company_name'] = $company[0]['name'];
				$person['company_id']   = $company[0]['id'];
			}

			// Get deal name to prefill data on page and hidden fields
			if ($person['deal_id'])
			{
				$deal                = JoomcrmHelperDeal::getDeal($person['deal_id']);
				$person['deal_name'] = $deal[0]['name'];
				$person['deal_id']   = $deal[0]['id'];
			}

			// Set null fields
			$person['position']  = "";
			$person['phone']     = "";
			$person['email']     = "";
			$person['type']      = '';
			$person['source_id'] = null;
			$person['status_id'] = null;
			$person['header']    = Text::_('COM_JOOMCRM_PERSON_ADD');
		}

		// Get total people associated with users account
		$total_people = JoomcrmHelperUsers::getPeopleCount($user_id, $team_id, $member_role);

		// Get filter types
		$people_types     = JoomcrmHelperPeople::getPeopleTypes();

		$params = ComponentHelper::getParams('com_joomcrm');

		$people_type_name = ($people_type_name && array_key_exists($people_type_name, $people_types)) ? $people_types[$people_type_name] : $people_types[$params->get('people_type_filter','all')];

		// Get column filters
		$column_filters   = JoomcrmHelperPeople::getColumnFilters();
		$selected_columns = JoomcrmHelperPeople::getSelectedColumnFilters();

		// Get user filter
		// Get associated users// Teams
		$teams = JoomcrmHelperUsers::getTeams();
		$users = JoomcrmHelperUsers::getUsers();

		if ($user AND $user != $user_id AND $user != 'all')
		{
			$user_info = JoomcrmHelperUsers::getUsers($user);
			$user_info = $user_info[0];
			$user_name = $user_info['first_name'] . " " . $user_info['last_name'];
		}
		else
		{
			if ($team)
			{
				$team_info = JoomcrmHelperUsers::getTeams($team);
				$team_info = $team_info[0];
				$user_name = $team_info['team_name'] . Text::_('COM_JOOMCRM_TEAM_APPEND');
			}
			else
			{
				if ($user == 'all')
				{
					$user_name = Text::_('COM_JOOMCRM_ALL_USERS');
				}
				else
				{
					$user_name = Text::_('COM_JOOMCRM_ME');
				}
			}
		}

		// Get stage filter
		$stages      = JoomcrmHelperPeople::getStages();
		$stages_name = ($stage) ? $stages[$stage] : $stages['past_thirty'];

		// Get status filter
		$status_list = JoomcrmHelperPeople::getStatusList();
		for ($i = 0; $i < count($status_list); $i++)
		{
			if ($status_list[$i]['id'] == $status AND $status != 'any')
			{
				$status_name = $status_list[$i]['name'];
				break;
			}
		}
		$status_name = ($status AND $status != 'any') ? $status_name : 'any status';

		$dropdowns = $model->getDropdowns();

		// Load Events & Tasks for person
		$layout = $this->getLayout();
		if ($layout == "person")
		{
			$model     = new JoomcrmModelEvent;
			$events    = $model->getEvents("person", null,  \Joomla\CMS\Factory::getApplication()->input->get('id'));
			$ref       = array(array('ref' => 'events', 'data' => $events));
			$eventDock = JoomcrmHelperView::getView('events', 'event_dock', $ref);
			$this->event_dock = $eventDock;

			$deal_dock = JoomcrmHelperView::getView('deals', 'deal_dock', array(array('ref' => 'deals', 'data' => $person['deals'])));
			$this->deal_dock = $deal_dock;


			$docViewType = ComponentHelper::getParams('com_joomcrm')->get('documents_view_type','datatable');
			$docType = ($docViewType == 'datatable') ? 'document_dock' : 'document_tree';
			$ref = array(array('ref' => 'documents', 'data' => $person['documents']));
			$this->document_dock = JoomcrmHelperView::getView('documents', $docType, $ref);

			$custom_fields_view = JoomcrmHelperView::getView('custom', 'default');
			$custom_fields_view->type = 'people';
			$custom_fields_view->item = $person;
			$this->custom_fields_view = $custom_fields_view;

			$mailingEnabled = Mailing::loadIntegration() && $config->get('acy_person',1) == 1;
			$this->mailing = Mailing::loadIntegration();

			if ($mailingEnabled)
			{
				$mailing_lists = $this->mailing->lists;

				$newsletters   = array();
				if (is_array($mailing_lists) && array_key_exists(0, $mailing_lists))
				{


					$newsletters = $this->mailing->getNewsletters($mailing_lists[0]->listid);
				}
				$mailing_dock = JoomcrmHelperView::getView('mailing', 'default');
				$mailing_dock->newsletters = $newsletters;
				$mailing_dock->mailing_lists = $mailing_lists;
				$this->mailing_dock = $mailing_dock;
			}

			if (JoomcrmHelperBanter::hasBanter())
			{
				$room_lists  = JoomcrmHelperTranscriptlists::getRooms();
				$transcripts = array();
				if (is_array($room_lists) && count($room_lists) > 0)
				{
					$transcripts = $room_lists->getTranscripts($room_lists[0]->id);
				}
				$banter_dock = JoomcrmHelperView::getView('banter', 'default');
				$banter_dock->rooms = $room_lists;
				$banter_dock->transcripts = $transcripts;
				$this->banter_dock = $banter_dock;
			}

			/** get latest activities **/
			$activity               = JoomcrmHelperActivity::getActivity();
			$latest_activities      = JoomcrmHelperView::getView('activities', 'default');
			$latest_activities_view = JoomcrmHelperView::getView('activities', 'latest_activities');
			$latest_activities_view->activity = $activity;
			$latest_activities->latest_activities = $latest_activities_view;
			$this->latest_activities = $latest_activities;
		}

		if ($layout == "default")
		{
			$people_list = JoomcrmHelperView::getView('people', 'list', array(array('ref' => 'people', 'data' => $people)));
			$total       = $model->getTotal();
			$people_list->total = $total;
			$people_list->pagination = $this->pagination;
			$this->people_list = $people_list;

			$people_filter = $session->get('people_name_filter');
			$this->people_filter = $people_filter;
		}

		if ($layout == "edit")
		{
			$edit_custom_fields_view = JoomcrmHelperView::getView('custom', 'edit');
			$edit_custom_fields_view->type = 'people';
			$edit_custom_fields_view->item = ( \Joomla\CMS\Factory::getApplication()->input->get('id') && array_key_exists(0, $people)) ? $people[0] : array('id' => '');
			$this->edit_custom_fields_view = $edit_custom_fields_view;

			$companyModel = new JoomcrmModelCompany;
			$json         = true;
			$companyNames = $companyModel->getCompanyNames($json);
			$document->addScriptDeclaration("var company_names=" . $companyNames . ";");
		}

		if ( \Joomla\CMS\Factory::getApplication()->input->get('id'))
		{
			$add_note = JoomcrmHelperView::getView('note', 'edit');
			$this->add_note = $add_note;

			$add_task         = JoomcrmHelperView::getView('events', 'edit_task');
			$association_type = 'person';
			$association_id   = $input->getInt('id',0);
			$add_task->association_type = $association_type;
			$add_task->association_id = $association_id;

			$this->add_task = $add_task;
		}


		// Assign results to view

		$this->people = $people;
		$this->person = $person;
		$this->totalPeople = $total_people;
		$this->people_type_name = $people_type_name;
		$this->people_types = $people_types;
		$this->user_id = $user_id;
		$this->team_id = $team_id;
		$this->member_role = $member_role;
		$this->user_name = $user_name;
		$this->teams = $teams;
		$this->users = $users;
		$this->stages = $stages;
		$this->stages_name = $stages_name;
		$this->status_list = $status_list;
		$this->status_name = $status_name;
		$this->state = $state;
		$this->column_filters = $column_filters;
		$this->selected_columns = $selected_columns;
		$this->dropdowns = $dropdowns;

		parent::display($tpl);
	}

}
