<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;

class JoomcrmViewPrintHtml extends JoomcrmHelperView
{
	public function display($tpl = null)
	{
		/* Model // function info */
		$model_name =  \Joomla\CMS\Factory::getApplication()->input->get('model');

		$layout = $this->getLayout();
		$this->view =  \Joomla\CMS\Factory::getApplication()->input->get('view');

		$document = Factory::getDocument();

		if (!$model_name)
		{
			switch ($layout)
			{
				case "person":
					$model_name = "people";
					break;
				case "company":
					$model_name = "companies";
					break;
				case "deal":
					$model_name = "deal";
					break;
			}
		}

		if ($layout != "report")
		{
			$model  = BaseDatabaseModel::getInstance($model_name, 'JoomcrmModel');
			$params = null;
			switch ($model_name)
			{
				case "company":
					$func = "companies";
					break;
				case "deal":
					$func = "deals";
					break;
				case "people":
					$func = "people";
					break;
				case "event":
				case "events":
					$func = "events";
					if ($layout == "calendar")
					{
						$params = "calendar";

						// Load js libs
						$document->addScript(Uri::base() . 'components/com_joomcrm/media/js/fullcalendar.js');
						$document->addScript(Uri::base() . 'components/com_joomcrm/media/js/calendar_manager.js');

						// Load required css for calendar
						$document->addStyleSheet(Uri::base() . 'components/com_joomcrm/media/css/fullcalendar.css');
					}
					break;
			}
		}

		switch ($layout)
		{
			case "report":
				$report  =  \Joomla\CMS\Factory::getApplication()->input->get('report');
				$item_id = ( \Joomla\CMS\Factory::getApplication()->input->get('ids')) ?  \Joomla\CMS\Factory::getApplication()->input->get('ids') : null;
				if (is_array($item_id))
				{
					$items = array();
					foreach ($item_id as $key => $item)
					{
						$items[] = $item;
					}
				}
				else
				{
					$items = $item_id;
				}
				switch ($report)
				{
					case "sales_pipeline":
						$model = new JoomcrmModelDeal;
						$model->set('_id', $items);
						$state   = $model->getState();
						$reports = $model->getReportDeals();
						$header  = JoomcrmHelperView::getView('reports', 'sales_pipeline_header', array(array('ref' => 'state', 'data' => $state), array('ref' => 'reports', 'data' => $reports)));
						$table   = JoomcrmHelperView::getView('reports', 'sales_pipeline_filter', array(array('ref' => 'reports', 'data' => $reports)));
						$footer  = JoomcrmHelperView::getView('reports', 'sales_pipeline_footer');
						break;
					case "source_report":
						$model = new JoomcrmModelDeal;
						$model->set('_id', $items);
						$reports = $model->getDeals();
						$state   = $model->getState();
						$header  = JoomcrmHelperView::getView('reports', 'source_report_header', array(array('ref' => 'state', 'data' => $state), array('ref' => 'reports', 'data' => $reports)));
						$table   = JoomcrmHelperView::getView('reports', 'source_report_filter', array(array('ref' => 'reports', 'data' => $reports)));
						$footer  = JoomcrmHelperView::getView('reports', 'source_report_footer');
						break;
					case "roi_report":
						$model = new JoomcrmModelSource;
						$model->set('_id', $items);
						$sources = $model->getRoiSources();
						$header  = JoomcrmHelperView::getView('reports', 'roi_report_header');
						$table   = JoomcrmHelperView::getView('reports', 'roi_report_filter', array(array('ref' => 'sources', 'data' => $sources)));
						$footer  = JoomcrmHelperView::getView('reports', 'roi_report_footer');
						break;
					case "notes":
						$model = new JoomcrmModelNote;
						$model->set('_id', $items);
						$items 		= $model->getNotes(null, null, false);
						$state      = $model->getState();
						$header     = JoomcrmHelperView::getView('reports', 'notes_header', array(
							array('ref' => 'view', 'data' => $this->view)
						));
						$table      = JoomcrmHelperView::getView('reports', 'notes_list', array(
							array('ref' => 'items', 'data' => $items)
						));
						$footer     = JoomcrmHelperView::getView('reports', 'notes_footer', array(
							array('ref' => 'view', 'data' => $this->view)
						));
						break;
					case "custom_report":
						$model       = new JoomcrmModelReport;
						$report      = $model->getCustomReports( \Joomla\CMS\Factory::getApplication()->input->get('custom_report'));
						$report_data = $model->getCustomReportData( \Joomla\CMS\Factory::getApplication()->input->get('custom_report'));
						$state       = $model->getState();
						$data        = array(array('ref' => 'report_data', 'data' => $report_data), array('ref' => 'report', 'data' => $report), array('ref' => 'state', 'data' => $state));
						$header      = JoomcrmHelperView::getView('reports', 'custom_report_header', $data);
						$table       = JoomcrmHelperView::getView('reports', 'custom_report_filter', $data);
						$footer      = JoomcrmHelperView::getView('reports', 'custom_report_footer');
						break;
				}
				$this->header = $header;
				$this->table = $table;
				$this->footer = $footer;
				break;
			default:
				/* Item info */
				$function = "get" . ucwords($func);
				$item_id  = ( \Joomla\CMS\Factory::getApplication()->input->get('item_id')) ?  \Joomla\CMS\Factory::getApplication()->input->get('item_id') : null;
				$ids      = ( \Joomla\CMS\Factory::getApplication()->input->get('ids')) ?  \Joomla\CMS\Factory::getApplication()->input->get('ids') : null;

				$item_id = $item_id ? $item_id : $ids;

				if (is_array($item_id))
				{
					$items = array();
					if ( \Joomla\CMS\Factory::getApplication()->input->get('item_id'))
					{
						foreach ($item_id as $key => $item)
						{
							$items[] = $key;
						}
					}
					else
					{
						foreach ($item_id as $key => $item)
						{
							$items[] = $item;
						}
					}
				}
				else
				{
					$items = $item_id;
				}

				$model->set('_id', $items);
				$model->set("completed", null);
				$info = $model->$function($params);

				/* Event list */
				$model  = new JoomcrmModelEvent;
				$events = $model->getEvents("deal", null, $item_id);
				if (count($events) > 0)
				{
					$ref       = array(array('ref' => 'events', 'data' => $events), array('ref' => 'print', 'data' => true));
					$eventDock = JoomcrmHelperView::getView('events', 'event_dock', $ref);
					$this->event_dock = $eventDock;
				}

				/* Contact info */
				if (is_array($info) && array_key_exists(0, $info) && array_key_exists('people', $info[0]) && count($info[0]['people']) > 0)
				{
					$contactModel = new JoomcrmModelContacts;
					$contactModel->set('deal_id', $info[0]['id']);
					$contacts     = $contactModel->getContacts();
					$ref          = array(array('ref' => 'contacts', 'data' => $contacts), array('ref' => 'print', 'data' => true));
					$contact_info = JoomcrmHelperView::getView('contacts', 'default', $ref);
					$this->contact_info = $contact_info;
				}

				$this->info = $info;
				break;
		}

		$custom_fields = array('deal', 'company', 'person');
		if (in_array($layout, $custom_fields))
		{
			switch ($layout)
			{
				case "company":
					$type = "company";
					break;
				case "deal":
					$type = "deal";
					break;
				case "person":
					$type = "people";
					break;
			}
			$this->custom_fields = JoomcrmHelperView::getView('print', 'custom_fields');
			$this->custom_fields->item_type = $type;
			$this->custom_fields->item = $info[0];
		}

		$js = "jQuery(document).ready(function(){
				window.print();
		})";

		$document->addScriptDeclaration($js);

		parent::display($tpl);
	}

}
