<?php
/**
* JoomCRM
*
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;

class JoomcrmViewReportsHtml extends JoomcrmHelperView
{
	public function display($tpl = null)
	{
		// Load reports menu bar
		$this->menu = JoomcrmHelperTemplate::loadReportMenu();

		$document = Factory::getDocument();
		$document->addScriptDeclaration("var graphType='report';");

		// Determine view layout
		$layout = $this->getLayout();
		$this->view =  Factory::getApplication()->input->get('view');

		if ($layout == 'default')
		{
			// Load javascripts
			$document->addScript(Uri::base() . 'components/com_joomcrm/media/js/highcharts.js');
			$document->addScript(Uri::base() . 'components/com_joomcrm/media/js/sales_dashboard.js');
			$document->addScriptDeclaration("var loc='sales_dashboard';");

			// Get data for sales graphs
			$model      = new JoomcrmModelGraphs;
			$graph_data = $model->getGraphData();

			// Assign results to view
			$this->graph_data = $graph_data;

			$dashFloats              = JoomcrmHelperUsers::getFloats('sales_dashboard');
			$sales_dash_floats_left  = $dashFloats['left'];
			$sales_dash_floats_right = $dashFloats['right'];
			$this->sales_dash_floats_left = $sales_dash_floats_left;
			$this->sales_dash_floats_right = $sales_dash_floats_right;

		}
		// Sales pipeline page
		if ($layout == 'sales_pipeline')
		{
			// Get deals for reports
			$model = new JoomcrmModelDeal;
			$model->set('archived', 0);
			$model->set('limit', 0);
			$reports = $model->getReportDeals();

			// Initialise state variables.
			$state = $model->getState();

			// Info for dropdowns
			$deal_amounts     = JoomcrmHelperDeal::getAmounts();
			$deal_stages      = JoomcrmHelperDeal::getActiveStages(true);
			$deal_statuses    = JoomcrmHelperDeal::getStatuses();
			$deal_close_dates = JoomcrmHelperDeal::getClosing();
			$modified_dates   = JoomcrmHelperDeal::getModified();

			// List view
			$sales_pipeline_header = JoomcrmHelperView::getView('reports', 'sales_pipeline_header', array(array('ref' => 'state', 'data' => $state), array('ref' => 'reports', 'data' => $reports)));
			$sales_pipeline_list   = JoomcrmHelperView::getView('reports', 'sales_pipeline_filter', array(array('ref' => 'reports', 'data' => $reports)));
			$sales_pipeline_footer = JoomcrmHelperView::getView('reports', 'sales_pipeline_footer');

			$sales_pipeline_header->deal_amounts = $deal_amounts;
			$sales_pipeline_header->deal_stages = $deal_stages;
			$sales_pipeline_header->deal_statuses = $deal_statuses;
			$sales_pipeline_header->deal_close_dates = $deal_close_dates;
			$sales_pipeline_header->modified_dates = $modified_dates;
			$sales_pipeline_header->created_dates = JoomcrmHelperDate::getCreatedDates();
			$sales_pipeline_header->team_names = JoomcrmHelperDropdown::getTeamNames();
			$sales_pipeline_header->user_names = JoomcrmHelperDropdown::getUserNames();
			$sales_pipeline_header->state = $state;

			// Assign refs to view
			$this->sales_pipeline_header = $sales_pipeline_header;
			$this->sales_pipeline_list = $sales_pipeline_list;
			$this->sales_pipeline_footer = $sales_pipeline_footer;
			$this->state = $state;
			$this->reports = $reports;

		}
		// Source report page
		if ($layout == 'source_report')
		{
			// Get deals for reports
			$model = new JoomcrmModelDeal;
			$model->set('archived', 0);
			$model->set('limit', 0);
			$reports = $model->getDeals();

			// Get model state
			$state = $model->getState();

			// Info for dropdowns
			$deal_amounts     = JoomcrmHelperDeal::getAmounts();
			$deal_sources     = JoomcrmHelperDeal::getSources();
			$deal_stages      = JoomcrmHelperDeal::getStages();
			$deal_statuses    = JoomcrmHelperDeal::getStatuses();
			$deal_close_dates = JoomcrmHelperDeal::getClosing();
			$modified_dates   = JoomcrmHelperDeal::getModified();

			// List view
			$source_report_header = JoomcrmHelperView::getView('reports', 'source_report_header', array(array('ref' => 'state', 'data' => $state), array('ref' => 'reports', 'data' => $reports)));
			$source_list          = JoomcrmHelperView::getView('reports', 'source_report_filter', array(array('ref' => 'reports', 'data' => $reports)));
			$source_report_footer = JoomcrmHelperView::getView('reports', 'source_report_footer');

			// Assign refs to view
			$source_report_header->deal_amounts = $deal_amounts;
			$source_report_header->deal_sources = $deal_sources;
			$source_report_header->deal_stages = $deal_stages;
			$source_report_header->deal_statuses = $deal_statuses;
			$source_report_header->deal_close_dates = $deal_close_dates;
			$source_report_header->modified_dates = $modified_dates;
			$source_report_header->created_dates = JoomcrmHelperDate::getCreatedDates();
			$source_report_header->team_names = JoomcrmHelperDropdown::getTeamNames();
			$source_report_header->user_names = JoomcrmHelperDropdown::getUserNames();
			$source_report_header->state = $state;

			$this->source_report_header = $source_report_header;
			$this->source_report_list = $source_list;
			$this->source_report_footer = $source_report_footer;
			$this->state = $state;
			$this->reports = $reports;
		}
		// Roi report page
		if ($layout == 'roi_report')
		{
			// Get sources for reports
			$model   = new JoomcrmModelSource;
			$sources = $model->getRoiSources();

			// Initialise state variables.
			$state = $model->getState();

			// List view
			$roi_report_header = JoomcrmHelperView::getView('reports', 'roi_report_header');
			$roi_list          = JoomcrmHelperView::getView('reports', 'roi_report_filter', array(array('ref' => 'sources', 'data' => $sources)));
			$roi_report_footer = JoomcrmHelperView::getView('reports', 'roi_report_footer');

			$roi_report_header->state = $state;

			// Assign refs to view
			$this->roi_report_header = $roi_report_header;
			$this->roi_list = $roi_list;
			$this->roi_report_footer = $roi_report_footer;
			$this->state = $state;
			$this->sources = $sources;
		}
		// Deal milestones page
		if ($layout == 'deal_milestones')
		{
			// Get deals for reports
			$model = new JoomcrmModelDeal;
			$model->set('archived', 0);
			$model->set('completed', 'false');
			$deals = $model->getDeals();

			// Pagination
			$pagination = $model->getPagination();
			$this->pagination = $pagination;

			// Initialise state variables.
			$state = $model->getState();

			// List view
			$deal_milestone_list = JoomcrmHelperView::getView('reports', 'deal_milestones_filter', array(array('ref' => 'deals', 'data' => $deals)));

			// Assign refs to view
			$this->deal_milestone_list = $deal_milestone_list;
			$this->state = $state;
			$this->deals = $deals;
		}
		// Notes report page
		if ($layout == 'notes')
		{
			$model 			  = new JoomcrmModelNote;
			$this->header 	  = JoomcrmHelperView::getView('reports', 'notes_header', array(array('ref' => 'view', 'data' => $this->view)));
			$this->state      = $model->getState();
			$this->team_names = JoomcrmHelperDropdown::getTeamNames();
			$this->user_names = JoomcrmHelperDropdown::getUserNames();
			$this->categories = $model->getNoteCategories();
			$this->created_dates = JoomcrmHelperDate::getCreatedDates();

			$document->addScriptDeclaration("
			loc = 'notes';
			order_url = Joomla.getOptions('system.paths').rootFull+'index.php?option=com_joomcrm&view=deals&layout=list&format=raw&tmpl=component';
			order_dir = '" . $this->state->get('Note.filter_order_Dir') . "';
			order_col = '" . $this->state->get('Note.filter_order') . "';
			var dataTableColumns = [
				{ 'bSortable': false },
				null,
				null,
				null,
				null,
				null,
				null,
				null
				];
			");
		}

		// Custom reports default
		if ($layout == 'custom_reports')
		{
			$document->addScript(Uri::base() . 'components/com_joomcrm/media/js/custom_reports.js');

			// Get info from model
			$model   = new JoomcrmModelReport;
			$reports = $model->getCustomReports();
			$state   = $model->getState();

			// List view
			$custom_reports_list = JoomcrmHelperView::getView('reports', 'custom_reports_filter', array(array('ref' => 'reports', 'data' => $reports)));

			// Assign references
			$this->custom_reports_list = $custom_reports_list;
			$this->reports = $reports;
			$this->state = $state;
		}

		// Individual custom reports
		if ($layout == "custom_report")
		{
			$id =  \Joomla\CMS\Factory::getApplication()->input->get('id');
			// Get model
			$model = new JoomcrmModelReport;
			// Get report
			$report      = $model->getCustomReports($id);
			$report_data = $model->getCustomReportData($id);
			$state       = $model->getState();
			$this->pagination = $model->getPagination($id);
			// Assign refs
			$this->report = $report;
			$this->report_data = $report_data;
			$this->state = $state;

			// Info for dropdowns
			$deal_amounts              = JoomcrmHelperDeal::getAmounts();
			$deal_sources              = JoomcrmHelperDeal::getSources();
			$deal_stages               = JoomcrmHelperDeal::getSourceStages();
			$deal_statuses             = JoomcrmHelperDeal::getStatuses();
			$deal_expected_close_dates = JoomcrmHelperDeal::getExpectedClosingDates();
			$deal_actual_close_dates   = JoomcrmHelperDeal::getActualClosingDates();
			$modified_dates            = JoomcrmHelperDeal::getModified();
			$companyCategories         = JoomcrmHelperCompany::getCategories();

			// List layout
			$data = array(
				array('ref' => 'report_data', 'data' => $report_data),
				array('ref' => 'report', 'data' => $report),
				array('ref' => 'state', 'data' => $state)
			);

			$custom_report_header = JoomcrmHelperView::getView('reports', 'custom_report_header', $data);
			$custom_report_list   = JoomcrmHelperView::getView('reports', 'custom_report_filter', $data);
			$custom_report_footer = JoomcrmHelperView::getView('reports', 'custom_report_footer');

			$custom_report_header->deal_amounts = $deal_amounts;
			$custom_report_header->deal_sources = $deal_sources;
			$custom_report_header->deal_stages = $deal_stages;
			$custom_report_header->deal_statuses = $deal_statuses;
			$custom_report_header->deal_actual_close_dates = $deal_actual_close_dates;
			$custom_report_header->deal_expected_close_dates = $deal_expected_close_dates;
			$custom_report_header->modified_dates = $modified_dates;
			$custom_report_header->created_dates = JoomcrmHelperDate::getCreatedDates();
			$custom_report_header->team_names = JoomcrmHelperDropdown::getTeamNames('t:');
			$custom_report_header->user_names = JoomcrmHelperDropdown::getUserNames();
			$custom_report_header->state = $state;
			$custom_report_header->company_categories = $companyCategories;


			// Get range time filters
			$this->range_names 	= JoomcrmHelperDate::getRangeNames();

			$range_name  		= $state->get('Report.' .  \Joomla\CMS\Factory::getApplication()->input->get('id') . '_' . $layout . '_range');
			$this->range_name  	= ($range_name) ? $this->range_names[$range_name] : $this->range_names['all'];

			// Assign refs to view
			$this->custom_report_header = $custom_report_header;
			$this->custom_report_list = $custom_report_list;
			$this->custom_report_footer = $custom_report_footer;
			$this->state = $state;

			$document->addScriptDeclaration('loc="custom_report";');
			$document->addScriptDeclaration('id=' . $report[0]['id'] . ";");

		}

		// Edit custom reports
		if ($layout == "edit_custom_report")
		{
			$document->addScript(Uri::base() . 'components/com_joomcrm/media/js/custom_reports.js');

			// If we are editing an existing entry
			$id =  \Joomla\CMS\Factory::getApplication()->input->get('id');
			if ($id != null)
			{
				$model  = new JoomcrmModelReport;
				$this->report = $model->getCustomReports($id);
			}

			// Assign references
			$columns = JoomcrmHelperDeal::getAllCustomFields($id);
			if (JoomcrmHelperQuote::hasCrmQuote())
			{
				$columns = $columns + JoomcrmHelperQuote::getColumns();
			}
			$this->columns = $columns;
		}

		// Assign user filter priviliges
		$member_role = JoomcrmHelperUsers::getRole();
		$user     	 = JoomcrmHelperUsers::getLoggedInUser();
		$team_id     = JoomcrmHelperUsers::getTeamId();
		$user_id     = $user->id;
		$dateFormat  = $user->date_format;

		if ($dateFormat == 'd/m/y')
		{
			$this->dateFormat = 'dd/mm/yy';
		}
		elseif ($dateFormat == 'm/d/y')
		{
			$this->dateFormat = 'mm/dd/yy';
		}

		// If the user is not basic then they are able to filter through company/team/user data
		if ($member_role != 'basic')
		{
			// Exec can see teams
			if ($member_role == 'exec')
			{
				$this->teams = JoomcrmHelperUsers::getTeams();
			}

			// Manager and exec users
			$this->users = JoomcrmHelperUsers::getUsers();
		}

		// Assign team names for drop downs
		$this->team_names = JoomcrmHelperDropdown::getTeamNames();

		// Assign user names for drop downs
		$this->user_names = JoomcrmHelperDropdown::getUserNames();

		// Assign categories for drop downs
		$this->categories = JoomcrmHelperNote::getCategories();

		// Assign refs
		$this->member_role = $member_role;
		$this->user_id = $user_id;
		$this->team_id = $team_id;
		$this->layout = $layout;

		parent::display($tpl);
	}

}
