<?php
/**
* JoomCRM
* 
* @author     JoomBoost <support@joomboost.com>
* @copyright  Copyright (C) 2018 Joomboost.com All Rights Reserved.
* @license    http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
* Website: https://www.joomboost.com
*/


defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\MVC\View\HtmlView;

class JoomcrmView extends HtmlView
{
	public function __construct($config = array())
	{
		if (isset($config['name']))
		{
			$this->_name = $config['name'];
		}

		$config['template_path'] = JPATH_COMPONENT . '/view/' . $this->getName() . '/tmpl';


		parent::__construct($config);
	}

	public function display($tpl = null)
	{

		$datatablePagelength = ComponentHelper::getParams('com_joomcrm')->get('datatable_pagelength',10);

		// change datatable page length
		Factory::getDocument()->addScriptOptions('com_joomcrm',['datatable_pagelength' => $datatablePagelength]);

		parent::display($tpl);
	}

	/**
	 * Method to get the view name
	 *
	 * The model name by default parsed using the classname, or it can be set
	 * by passing a $config['name'] in the class constructor
	 *
	 * @return  string  The name of the model
	 *
	 * @since   11.1
	 */
	public function getName()
	{
		if (empty($this->_name))
		{
			$class = get_class($this);
			$pos   = strpos($class, 'View');

			if ($pos === false)
			{
                Factory::getApplication()->enqueueMessage(Text::_('JLIB_APPLICATION_ERROR_VIEW_GET_NAME'),'error');
			}

			$tmp = strtolower(substr($class, ($pos + 4)));

			// Remove the format from the end of the string.
			$this->_name = preg_replace('/(raw|html|json)$/', '', $tmp);
		}

		return $this->_name;
	}
}